@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,81 +14,49 @@
 # limitations under the License.
 ##
 
-import os
 
-from twisted.cred.portal import Portal
 from twisted.internet.defer import inlineCallbacks, maybeDeferred, returnValue
 
-from twext.web2 import http_headers
-from twext.web2 import responsecode
-from twext.web2 import server
-from twext.web2.auth import basic
-from twext.web2.dav import auth
-from twext.web2.dav import davxml
-from twext.web2.http import HTTPError
-from twext.web2.iweb import IResponse
-from twext.web2.test.test_server import SimpleRequest
+from txweb2 import http_headers
+from txweb2 import responsecode
+from txdav.xml import element as davxml
+from txweb2.http import HTTPError
+from txweb2.iweb import IResponse
 
-from twistedcaldav.test.util import TestCase
+from twistedcaldav.test.util import StoreTestCase, SimpleStoreRequest
 from twistedcaldav.directory.principal import DirectoryPrincipalProvisioningResource
-from twistedcaldav.directory.xmlfile import XMLDirectoryService
-from twistedcaldav.directory.test.test_xmlfile import xmlFile, augmentsFile
 
-from calendarserver.provision.root import RootResource
-from twistedcaldav.directory import augment
-from twistedcaldav.config import config
+import calendarserver.provision.root  # for patching checkSACL
 
-class FakeCheckSACL(object):
-    def __init__(self, sacls=None):
-        self.sacls = sacls or {}
 
-    def __call__(self, username, service):
-        if service not in self.sacls:
-            return 1
+TEST_SACLS = {
+    "calendar": [
+        "dreid"
+    ],
+    "addressbook": [
+        "dreid"
+    ],
+}
 
-        if username in self.sacls[service]:
-            return 0
 
-        return 1
+def stubCheckSACL(username, service):
+    if service not in TEST_SACLS:
+        return True
 
-class RootTests(TestCase):
+    if username in TEST_SACLS[service]:
+        return True
 
+    return False
+
+
+
+class RootTests(StoreTestCase):
+
+    @inlineCallbacks
     def setUp(self):
-        super(RootTests, self).setUp()
+        yield super(RootTests, self).setUp()
 
-        self.docroot = self.mktemp()
-        os.mkdir(self.docroot)
-
-        RootResource.CheckSACL = FakeCheckSACL(sacls={"calendar": ["dreid"]})
-
-        directory = XMLDirectoryService(
-            {
-                "xmlFile" : xmlFile,
-                "augmentService" :
-                    augment.AugmentXMLDB(xmlFiles=(augmentsFile.path,))
-            }
-        )
-
-        principals = DirectoryPrincipalProvisioningResource(
-            "/principals/",
-            directory
-        )
-
-        root = RootResource(self.docroot, principalCollections=[principals])
-
-        root.putChild("principals",
-                      principals)
-
-        portal = Portal(auth.DavRealm())
-        portal.registerChecker(directory)
-
-        self.root = auth.AuthenticationWrapper(
-            root,
-            portal,
-            credentialFactories=(basic.BasicCredentialFactory("Test realm"),),
-            loginInterfaces=(auth.IPrincipal,))
-
-        self.site = server.Site(self.root)
+        self.patch(calendarserver.provision.root, "checkSACL", stubCheckSACL)
 
 
 
@@ -104,8 +72,8 @@ class ComplianceTests(RootTests):
         Deferred which will fire with (something adaptable to) an HTTP response
         object.
         """
-        request = SimpleRequest(self.site, method, ("/".join([""] + segments)))
-        rsrc = self.root
+        request = SimpleStoreRequest(self, method, ("/".join([""] + segments)))
+        rsrc = self.actualRoot
         while segments:
             rsrc, segments = (yield maybeDeferred(
                 rsrc.locateChild, request, segments
@@ -127,7 +95,7 @@ class ComplianceTests(RootTests):
 
 
 class SACLTests(RootTests):
-    
+
     @inlineCallbacks
     def test_noSacls(self):
         """
@@ -135,18 +103,12 @@ class SACLTests(RootTests):
 
         should return a valid resource
         """
-        self.root.resource.useSacls = False
+        self.actualRoot.useSacls = False
 
-        request = SimpleRequest(self.site,
-                                "GET",
-                                "/principals/")
+        request = SimpleStoreRequest(self, "GET", "/principals/")
 
         resrc, segments = (yield maybeDeferred(
-            self.root.locateChild, request, ["principals"]
-        ))
-
-        resrc, segments = (yield maybeDeferred(
-            resrc.locateChild, request, ["principals"]
+            self.actualRoot.locateChild, request, ["principals"]
         ))
 
         self.failUnless(
@@ -157,6 +119,7 @@ class SACLTests(RootTests):
 
         self.assertEquals(segments, [])
 
+
     @inlineCallbacks
     def test_inSacls(self):
         """
@@ -165,26 +128,18 @@ class SACLTests(RootTests):
 
         should return a valid resource
         """
-        self.root.resource.useSacls = True
+        self.actualRoot.useSacls = True
 
-        request = SimpleRequest(
-            self.site,
+        principal = yield self.actualRoot.findPrincipalForAuthID("dreid")
+        request = SimpleStoreRequest(
+            self,
             "GET",
             "/principals/",
-            headers=http_headers.Headers({
-                "Authorization": [
-                    "basic",
-                    "%s" % ("dreid:dierd".encode("base64"),)
-                ]
-            })
+            authPrincipal=principal
         )
 
         resrc, segments = (yield maybeDeferred(
-            self.root.locateChild, request, ["principals"]
-        ))
-
-        resrc, segments = (yield maybeDeferred(
-            resrc.locateChild, request, ["principals"]
+            self.actualRoot.locateChild, request, ["principals"]
         ))
 
         self.failUnless(
@@ -196,15 +151,15 @@ class SACLTests(RootTests):
         self.assertEquals(segments, [])
 
         self.assertEquals(
-            request.authzUser,
+            request.authzUser.principalElement(),
             davxml.Principal(
                 davxml.HRef(
-                    "/principals/__uids__/"
-                    "5FF60DAD-0BDE-4508-8C77-15F0CA5C8DD1/"
+                    "/principals/__uids__/5FF60DAD-0BDE-4508-8C77-15F0CA5C8DD1/"
                 )
             )
         )
 
+
     @inlineCallbacks
     def test_notInSacls(self):
         """
@@ -213,31 +168,28 @@ class SACLTests(RootTests):
 
         should return a 403 forbidden response
         """
-        self.root.resource.useSacls = True
+        self.actualRoot.useSacls = True
 
-        request = SimpleRequest(
-            self.site,
+        principal = yield self.actualRoot.findPrincipalForAuthID("wsanchez")
+
+        request = SimpleStoreRequest(
+            self,
             "GET",
             "/principals/",
-            headers=http_headers.Headers({
-                "Authorization": [
-                    "basic",
-                    "%s" % ("wsanchez:zehcnasw".encode("base64"),)
-                ]
-            })
+            authPrincipal=principal
         )
 
-        resrc, segments = (yield maybeDeferred(
-            self.root.locateChild, request, ["principals"]
-        ))
-
         try:
-            resrc, segments = (yield maybeDeferred(
-                resrc.locateChild, request, ["principals"]
+            _ignore_resrc, _ignore_segments = (yield maybeDeferred(
+                self.actualRoot.locateChild, request, ["principals"]
             ))
+            raise AssertionError(
+                "RootResource.locateChild did not return an error"
+            )
         except HTTPError, e:
             self.assertEquals(e.response.code, 403)
 
+
     @inlineCallbacks
     def test_unauthenticated(self):
         """
@@ -247,20 +199,16 @@ class SACLTests(RootTests):
         should return a 401 UnauthorizedResponse
         """
 
-        self.root.resource.useSacls = True
-        request = SimpleRequest(
-            self.site,
+        self.actualRoot.useSacls = True
+        request = SimpleStoreRequest(
+            self,
             "GET",
             "/principals/"
         )
 
-        resrc, segments = (yield maybeDeferred(
-            self.root.locateChild, request, ["principals"]
-        ))
-
         try:
-            resrc, segments = (yield maybeDeferred(
-                resrc.locateChild, request, ["principals"]
+            _ignore_resrc, _ignore_segments = (yield maybeDeferred(
+                self.actualRoot.locateChild, request, ["principals"]
             ))
             raise AssertionError(
                 "RootResource.locateChild did not return an error"
@@ -268,6 +216,7 @@ class SACLTests(RootTests):
         except HTTPError, e:
             self.assertEquals(e.response.code, 401)
 
+
     @inlineCallbacks
     def test_badCredentials(self):
         """
@@ -276,54 +225,30 @@ class SACLTests(RootTests):
 
         should return a 401 UnauthorizedResponse
         """
-        self.root.resource.useSacls = True
+        self.actualRoot.useSacls = True
 
-        request = SimpleRequest(
-            self.site,
+        request = SimpleStoreRequest(
+            self,
             "GET",
             "/principals/",
-            headers=http_headers.Headers({
-                    "Authorization": ["basic", "%s" % (
-                            "dreid:dreid".encode("base64"),)]}))
-
-        resrc, segments = (yield maybeDeferred(
-            self.root.locateChild, request, ["principals"]
-        ))
-
-        try:
-            resrc, segments = (yield maybeDeferred(
-                resrc.locateChild, request, ["principals"]
-            ))
-        except HTTPError, e:
-            self.assertEquals(e.response.code, 401)
-
-    @inlineCallbacks
-    def test_internalAuthHeader(self):
-        """
-        Test the behavior of locateChild when x-calendarserver-internal
-        header is set.
-
-        authnuser and authzuser will be set to the internal principal
-        """
-        self.patch(config.Scheduling.iMIP, "Password", "xyzzy")
-
-        headers = http_headers.Headers({})
-        headers.setRawHeaders("x-calendarserver-internal", ["xyzzy"])
-
-        request = SimpleRequest(
-            self.site,
-            "GET",
-            "/principals/",
-            headers=headers,
+            headers=http_headers.Headers(
+                {
+                    "Authorization": [
+                        "basic", "%s" % ("dreid:dreid".encode("base64"),)
+                    ]
+                }
+            )
         )
 
-        resrc, segments = (yield
-            RootResource.locateChild(self.root.resource, request, ["principals"]
-        ))
-
-        expected = "<?xml version='1.0' encoding='UTF-8'?>\n<principal xmlns='DAV:'>\r\n  <href>/principals/__uids__/%s/</href>\r\n</principal>" % (config.Scheduling.iMIP.GUID,)
-        self.assertEquals(request.authnUser.toxml(), expected)
-        self.assertEquals(request.authzUser.toxml(), expected)
+        try:
+            _ignore_resrc, _ignore_segments = (yield maybeDeferred(
+                self.actualRoot.locateChild, request, ["principals"]
+            ))
+            raise AssertionError(
+                "RootResource.locateChild did not return an error"
+            )
+        except HTTPError, e:
+            self.assertEquals(e.response.code, 401)
 
 
     def test_DELETE(self):
@@ -334,9 +259,10 @@ class SACLTests(RootTests):
                 self.fail("Incorrect response for DELETE /: %s"
                           % (response.code,))
 
-        request = SimpleRequest(self.site, "DELETE", "/")
+        request = SimpleStoreRequest(self, "DELETE", "/")
         return self.send(request, do_test)
 
+
     def test_COPY(self):
         def do_test(response):
             response = IResponse(response)
@@ -345,14 +271,15 @@ class SACLTests(RootTests):
                 self.fail("Incorrect response for COPY /: %s"
                           % (response.code,))
 
-        request = SimpleRequest(
-            self.site,
+        request = SimpleStoreRequest(
+            self,
             "COPY",
             "/",
-            headers=http_headers.Headers({"Destination":"/copy/"})
+            headers=http_headers.Headers({"Destination": "/copy/"})
         )
         return self.send(request, do_test)
 
+
     def test_MOVE(self):
         def do_test(response):
             response = IResponse(response)
@@ -361,16 +288,18 @@ class SACLTests(RootTests):
                 self.fail("Incorrect response for MOVE /: %s"
                           % (response.code,))
 
-        request = SimpleRequest(
-            self.site,
+        request = SimpleStoreRequest(
+            self,
             "MOVE",
             "/",
-            headers=http_headers.Headers({"Destination":"/copy/"})
+            headers=http_headers.Headers({"Destination": "/copy/"})
         )
         return self.send(request, do_test)
 
+
+
 class SACLCacheTests(RootTests):
-    
+
     class StubResponseCacheResource(object):
         def __init__(self):
             self.cache = {}
@@ -381,18 +310,22 @@ class SACLCacheTests(RootTests):
             if str(request) in self.cache:
                 self.cacheHitCount += 1
                 return self.cache[str(request)]
-    
-    
+
+
         def cacheResponseForRequest(self, request, response):
             self.cache[str(request)] = response
             return response
 
-    def setUp(self):
-        super(SACLCacheTests, self).setUp()
-        self.root.resource.responseCache = SACLCacheTests.StubResponseCacheResource()
 
+    @inlineCallbacks
+    def setUp(self):
+        yield super(SACLCacheTests, self).setUp()
+        self.actualRoot.responseCache = SACLCacheTests.StubResponseCacheResource()
+
+
+    @inlineCallbacks
     def test_PROPFIND(self):
-        self.root.resource.useSacls = True
+        self.actualRoot.useSacls = True
 
         body = """<?xml version="1.0" encoding="utf-8" ?>
 <D:propfind xmlns:D="DAV:">
@@ -402,48 +335,45 @@ class SACLCacheTests(RootTests):
 </D:prop>
 </D:propfind>
 """
+        principal = yield self.actualRoot.findPrincipalForAuthID("dreid")
 
-        request = SimpleRequest(
-            self.site,
+        request = SimpleStoreRequest(
+            self,
             "PROPFIND",
             "/principals/users/dreid/",
             headers=http_headers.Headers({
-                    'Authorization': ['basic', '%s' % ('dreid:dierd'.encode('base64'),)],
-                    'Content-Type': 'application/xml; charset="utf-8"',
-                    'Depth':'1',
+                'Depth': '1',
             }),
+            authPrincipal=principal,
             content=body
         )
+        response = yield self.send(request)
+        response = IResponse(response)
 
-        def gotResponse1(response):
-            if response.code != responsecode.MULTI_STATUS:
-                self.fail("Incorrect response for PROPFIND /principals/: %s" % (response.code,))
+        if response.code != responsecode.MULTI_STATUS:
+            self.fail("Incorrect response for PROPFIND /principals/: %s" % (response.code,))
 
-            request = SimpleRequest(
-                self.site,
-                "PROPFIND",
-                "/principals/users/dreid/",
-                headers=http_headers.Headers({
-                        'Authorization': ['basic', '%s' % ('dreid:dierd'.encode('base64'),)],
-                        'Content-Type': 'application/xml; charset="utf-8"',
-                        'Depth':'1',
-                }),
-                content=body
-            )
+        request = SimpleStoreRequest(
+            self,
+            "PROPFIND",
+            "/principals/users/dreid/",
+            headers=http_headers.Headers({
+                'Depth': '1',
+            }),
+            authPrincipal=principal,
+            content=body
+        )
+        response = yield self.send(request)
+        response = IResponse(response)
 
-            d = self.send(request, gotResponse2)
-            return d
+        if response.code != responsecode.MULTI_STATUS:
+            self.fail("Incorrect response for PROPFIND /principals/: %s" % (response.code,))
+        self.assertEqual(self.actualRoot.responseCache.cacheHitCount, 1)
 
-        def gotResponse2(response):
-            if response.code != responsecode.MULTI_STATUS:
-                self.fail("Incorrect response for PROPFIND /principals/: %s" % (response.code,))
-            self.assertEqual(self.root.resource.responseCache.cacheHitCount, 1)
 
-        d = self.send(request, gotResponse1)
-        return d
 
 class WikiTests(RootTests):
-    
+
     @inlineCallbacks
     def test_oneTime(self):
         """
@@ -451,12 +381,9 @@ class WikiTests(RootTests):
         request.checkedWiki will be set to True
         """
 
-        request = SimpleRequest(self.site, "GET", "/principals/")
+        request = SimpleStoreRequest(self, "GET", "/principals/")
 
-        resrc, segments = (yield maybeDeferred(
-            self.root.locateChild, request, ["principals"]
-        ))
-        resrc, segments = (yield maybeDeferred(
-            resrc.locateChild, request, ["principals"]
+        _ignore_resrc, _ignore_segments = (yield maybeDeferred(
+            self.actualRoot.locateChild, request, ["principals"]
         ))
         self.assertTrue(request.checkedWiki)
