@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2007-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2007-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,13 +14,16 @@
 # limitations under the License.
 ##
 
-from calendarserver.tap.util import directoryFromConfig, MemoryLimitService, Stepper
+from calendarserver.tap.util import (
+    MemoryLimitService, Stepper, verifyTLSCertificate
+)
 from twistedcaldav.util import computeProcessCount
 from twistedcaldav.test.util import TestCase
-from twistedcaldav.config import config
-from twistedcaldav.directory.augment import AugmentXMLDB
 from twisted.internet.task import Clock
 from twisted.internet.defer import succeed, inlineCallbacks
+from twisted.python.filepath import FilePath
+from twistedcaldav.config import ConfigDict
+
 
 class ProcessCountTestCase(TestCase):
 
@@ -39,10 +42,10 @@ class ProcessCountTestCase(TestCase):
             (6, 2, 2, 2, 2, 6),
             (1, 2, 1, 4, 99, 8),
 
-            (2, 1, 2, 2, 2, 2), # 2 cores, 2GB = 2
-            (2, 1, 2, 2, 4, 2), # 2 cores, 4GB = 2
-            (2, 1, 2, 8, 6, 8), # 8 cores, 6GB = 8
-            (2, 1, 2, 8, 16, 8), # 8 cores, 16GB = 8
+            (2, 1, 2, 2, 2, 2),   # 2 cores, 2GB = 2
+            (2, 1, 2, 2, 4, 2),   # 2 cores, 4GB = 2
+            (2, 1, 2, 8, 6, 8),   # 8 cores, 6GB = 8
+            (2, 1, 2, 8, 16, 8),  # 8 cores, 16GB = 8
         )
 
         for min, perCPU, perGB, cpu, mem, expected in data:
@@ -55,21 +58,6 @@ class ProcessCountTestCase(TestCase):
 
 
 
-class UtilTestCase(TestCase):
-
-    def test_directoryFromConfig(self):
-        """
-        Ensure augments service is on by default
-        """
-        dir = directoryFromConfig(config)
-        for service in dir._recordTypes.values():
-            # all directory services belonging to the aggregate have
-            # augmentService set to AugmentXMLDB
-            if hasattr(service, "augmentService"):
-                self.assertTrue(isinstance(service.augmentService, AugmentXMLDB))
-
-
-
 # Stub classes for MemoryLimitServiceTestCase
 
 class StubProtocol(object):
@@ -106,10 +94,10 @@ class MemoryLimitServiceTestCase(TestCase):
         """
         data = {
             # PID : (name, resident memory-in-bytes, virtual memory-in-bytes)
-            101 : ("process #1", 10, 1010),
-            102 : ("process #2", 30, 1030),
-            103 : ("process #3", 50, 1050),
-            99  : ("memcached-Default", 10, 1010),
+            101: ("process #1", 10, 1010),
+            102: ("process #2", 30, 1030),
+            103: ("process #3", 50, 1050),
+            99: ("memcached-Default", 10, 1010),
         }
 
         processes = []
@@ -217,8 +205,9 @@ class StepperTestCase(TestCase):
             StepFour(self._record, False)
         )
         result = (yield self.stepper.start("abc"))
-        self.assertEquals(result, "abc") # original result passed through
-        self.assertEquals(self.history,
+        self.assertEquals(result, "abc")  # original result passed through
+        self.assertEquals(
+            self.history,
             ['one success', 'two success', 'three success', 'four success'])
 
 
@@ -228,7 +217,8 @@ class StepperTestCase(TestCase):
         self.stepper.addStep(StepThree(self._record, True))
         self.stepper.addStep(StepFour(self._record, True))
         self.failUnlessFailure(self.stepper.start(), ZeroDivisionError)
-        self.assertEquals(self.history,
+        self.assertEquals(
+            self.history,
             ['one success', 'two failure', 'three failure', 'four failure'])
 
 
@@ -239,6 +229,56 @@ class StepperTestCase(TestCase):
         self.stepper.addStep(StepThree(self._record, True))
         self.stepper.addStep(StepFour(self._record, False))
         result = (yield self.stepper.start("abc"))
-        self.assertEquals(result, None) # original result is gone
-        self.assertEquals(self.history,
+        self.assertEquals(result, None)  # original result is gone
+        self.assertEquals(
+            self.history,
             ['one success', 'two failure', 'three success', 'four failure'])
+
+
+
+class PreFlightChecksTestCase(TestCase):
+    """
+    Verify that missing, empty, or bogus TLS Certificates are detected
+    """
+
+    def test_missingCertificate(self):
+        success, _ignore_reason = verifyTLSCertificate(
+            ConfigDict(
+                {
+                    "SSLCertificate": "missing",
+                }
+            )
+        )
+        self.assertFalse(success)
+
+
+    def test_emptyCertificate(self):
+        certFilePath = FilePath(self.mktemp())
+        certFilePath.setContent("")
+        success, _ignore_reason = verifyTLSCertificate(
+            ConfigDict(
+                {
+                    "SSLCertificate": certFilePath.path,
+                }
+            )
+        )
+        self.assertFalse(success)
+
+
+    def test_bogusCertificate(self):
+        certFilePath = FilePath(self.mktemp())
+        certFilePath.setContent("bogus")
+        keyFilePath = FilePath(self.mktemp())
+        keyFilePath.setContent("bogus")
+        success, _ignore_reason = verifyTLSCertificate(
+            ConfigDict(
+                {
+                    "SSLCertificate": certFilePath.path,
+                    "SSLPrivateKey": keyFilePath.path,
+                    "SSLAuthorityChain": "",
+                    "SSLMethod": "SSLv3_METHOD",
+                    "SSLCiphers": "ALL:!aNULL:!ADH:!eNULL:!LOW:!EXP:RC4+RSA:+HIGH:+MEDIUM",
+                }
+            )
+        )
+        self.assertFalse(success)
