@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 ##
-# Copyright (c) 2006-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
-
+from __future__ import print_function
 from __future__ import with_statement
 
 from getopt import getopt, GetoptError
@@ -31,10 +31,10 @@ import uuid
 import xattr
 import zlib
 
-from twext.python.plistlib import readPlistFromString
+from plistlib import readPlistFromString
 
-from pycalendar.calendar import PyCalendar
-from pycalendar.attribute import PyCalendarAttribute
+from pycalendar.icalendar.calendar import Calendar
+from pycalendar.parameter import Parameter
 
 COPY_CAL_XATTRS = (
     'WebDAV:{DAV:}resourcetype',
@@ -47,26 +47,28 @@ COPY_EVENT_XATTRS = ('WebDAV:{DAV:}getcontenttype',)
 
 def usage(e=None):
     if e:
-        print e
-        print ""
+        print(e)
+        print("")
 
     name = os.path.basename(sys.argv[0])
-    print "usage: %s [options] source destination" % (name,)
-    print ""
-    print "  Anonymizes calendar data"
-    print ""
-    print "  source and destination should refer to document root directories"
-    print ""
-    print "options:"
-    print "  -h --help: print this help and exit"
-    print "  -n --node <node>: Directory node (defaults to /Search)"
-    print ""
+    print("usage: %s [options] source destination" % (name,))
+    print("")
+    print("  Anonymizes calendar data")
+    print("")
+    print("  source and destination should refer to document root directories")
+    print("")
+    print("options:")
+    print("  -h --help: print this help and exit")
+    print("  -n --node <node>: Directory node (defaults to /Search)")
+    print("")
 
     if e:
         sys.exit(64)
     else:
         sys.exit(0)
 
+
+
 def main():
 
     try:
@@ -83,8 +85,6 @@ def main():
     # Get configuration
     #
     directoryNode = "/Search"
-    sourceDirectory = None
-    destDirectory = None
 
     for opt, arg in optargs:
         if opt in ("-h", "--help"):
@@ -107,17 +107,18 @@ def main():
     directoryMap.dumpDsImports(os.path.join(destDirectory, "dsimports"))
 
 
+
 def anonymizeRoot(directoryMap, sourceDirectory, destDirectory):
     # sourceDirectory and destDirectory are DocumentRoots
 
-    print "Anonymizing calendar data from %s into %s" % (sourceDirectory, destDirectory)
+    print("Anonymizing calendar data from %s into %s" % (sourceDirectory, destDirectory))
 
     homes = 0
     calendars = 0
     resources = 0
 
     if not os.path.exists(sourceDirectory):
-        print "Can't find source: %s" % (sourceDirectory,)
+        print("Can't find source: %s" % (sourceDirectory,))
         sys.exit(1)
 
     if not os.path.exists(destDirectory):
@@ -147,10 +148,11 @@ def anonymizeRoot(directoryMap, sourceDirectory, destDirectory):
                         for home in os.listdir(secondPath):
                             record = directoryMap.lookupCUA(home)
                             if not record:
-                                print "Couldn't find %s, skipping." % (home,)
+                                print("Couldn't find %s, skipping." % (home,))
                                 continue
                             sourceHome = os.path.join(secondPath, home)
-                            destHome = os.path.join(destUidHomes,
+                            destHome = os.path.join(
+                                destUidHomes,
                                 record['guid'][0:2], record['guid'][2:4],
                                 record['guid'])
                             homeList.append((sourceHome, destHome, record))
@@ -162,13 +164,13 @@ def anonymizeRoot(directoryMap, sourceDirectory, destDirectory):
                     continue
                 record = directoryMap.lookupCUA(home)
                 if not record:
-                    print "Couldn't find %s, skipping." % (home,)
+                    print("Couldn't find %s, skipping." % (home,))
                     continue
                 sourceHome = os.path.join(sourceUidHomes, home)
                 destHome = os.path.join(destUidHomes, record['guid'])
                 homeList.append((sourceHome, destHome, record))
 
-        print "Processing %d calendar homes..." % (len(homeList),)
+        print("Processing %d calendar homes..." % (len(homeList),))
 
         for sourceHome, destHome, record in homeList:
             quotaUsed = 0
@@ -224,6 +226,10 @@ def anonymizeRoot(directoryMap, sourceDirectory, destDirectory):
 
                     data = anonymizeData(directoryMap, data)
 
+                    if data is None:
+                        # Ignore data we can't parse
+                        continue
+
                     destResource = os.path.join(destCal, resource)
                     with open(destResource, "w") as res:
                         res.write(data)
@@ -240,12 +246,10 @@ def anonymizeRoot(directoryMap, sourceDirectory, destDirectory):
 
                     resources += 1
 
-
                 # Store new ctag on calendar
                 xml = "<?xml version='1.0' encoding='UTF-8'?>\r\n<getctag xmlns='http://calendarserver.org/ns/'>%s</getctag>\r\n" % (str(datetime.datetime.now()),)
                 xattr.setxattr(destCal, "WebDAV:{http:%2F%2Fcalendarserver.org%2Fns%2F}getctag", zlib.compress(xml))
 
-
             # Calendar home quota
             xml = "<?xml version='1.0' encoding='UTF-8'?>\r\n<quota-used xmlns='http://twistedmatrix.com/xml_namespace/dav/private/'>%d</quota-used>\r\n" % (quotaUsed,)
             xattr.setxattr(destHome, "WebDAV:{http:%2F%2Ftwistedmatrix.com%2Fxml_namespace%2Fdav%2Fprivate%2F}quota-used", zlib.compress(xml))
@@ -258,26 +262,32 @@ def anonymizeRoot(directoryMap, sourceDirectory, destDirectory):
             for freeBusy in freeBusies:
                 xml += "<href xmlns='DAV:'>/calendars/__uids__/%s/%s/</href>\n" % (record['guid'], freeBusy)
             xml += "</calendar-free-busy-set>\n"
-            xattr.setxattr(destInbox,
+            xattr.setxattr(
+                destInbox,
                 "WebDAV:{urn:ietf:params:xml:ns:caldav}calendar-free-busy-set",
                 zlib.compress(xml)
             )
 
             if not (homes % 100):
-                print " %d..." % (homes,)
+                print(" %d..." % (homes,))
 
-    print "Done."
-    print ""
+    print("Done.")
+    print("")
+
+    print("Calendar totals:")
+    print(" Calendar homes: %d" % (homes,))
+    print(" Calendars: %d" % (calendars,))
+    print(" Events: %d" % (resources,))
+    print("")
 
-    print "Calendar totals:"
-    print " Calendar homes: %d" % (homes,)
-    print " Calendars: %d" % (calendars,)
-    print " Events: %d" % (resources,)
-    print ""
 
 
 def anonymizeData(directoryMap, data):
-    pyobj = PyCalendar.parseText(data)
+    try:
+        pyobj = Calendar.parseText(data)
+    except Exception, e:
+        print("Failed to parse (%s): %s" % (e, data))
+        return None
 
     # Delete property from the top level
     try:
@@ -295,19 +305,19 @@ def anonymizeData(directoryMap, data):
                     cua = prop.getValue().getValue()
                     record = directoryMap.lookupCUA(cua)
                     if record is None:
-                        # print "Can't find record for", cua
+                        # print("Can't find record for", cua)
                         record = directoryMap.addRecord(cua=cua)
                         if record is None:
                             comp.removeProperty(prop)
                             continue
                     prop.setValue("urn:uuid:%s" % (record['guid'],))
-                    if prop.hasAttribute('X-CALENDARSERVER-EMAIL'):
-                        prop.replaceAttribute(PyCalendarAttribute('X-CALENDARSERVER-EMAIL', record['email']))
+                    if prop.hasParameter('X-CALENDARSERVER-EMAIL'):
+                        prop.replaceParameter(Parameter('X-CALENDARSERVER-EMAIL', record['email']))
                     else:
-                        prop.removeAttributes('EMAIL')
-                        prop.addAttribute(PyCalendarAttribute('EMAIL', record['email']))
-                    prop.removeAttributes('CN')
-                    prop.addAttribute(PyCalendarAttribute('CN', record['name']))
+                        prop.removeParameters('EMAIL')
+                        prop.addParameter(Parameter('EMAIL', record['email']))
+                    prop.removeParameters('CN')
+                    prop.addParameter(Parameter('CN', record['name']))
             except KeyError:
                 pass
 
@@ -334,14 +344,15 @@ def anonymizeData(directoryMap, data):
             except KeyError:
                 pass
 
-    return pyobj.getText(includeTimezones=True)
+    return pyobj.getText(includeTimezones=Calendar.ALL_TIMEZONES)
+
 
 
 class DirectoryMap(object):
 
     def __init__(self, node):
 
-        self.map = { }
+        self.map = {}
         self.byType = {
             'users' : [],
             'groups' : [],
@@ -363,12 +374,12 @@ class DirectoryMap(object):
             'resources' : ('Resources', 'resource'),
         }
 
-        print "Fetching records from directory: %s" % (node,)
+        print("Fetching records from directory: %s" % (node,))
 
-        for internalType, (recordType, friendlyType) in self.strings.iteritems():
-            print " %s..." % (internalType,)
+        for internalType, (recordType, _ignore_friendlyType) in self.strings.iteritems():
+            print(" %s..." % (internalType,))
             child = Popen(
-                args = [
+                args=[
                     "/usr/bin/dscl", "-plist", node, "-readall",
                     "/%s" % (recordType,),
                     "GeneratedUID", "RecordName", "EMailAddress", "GroupMembers"
@@ -390,15 +401,19 @@ class DirectoryMap(object):
                     origRecordNames = record['dsAttrTypeStandard:RecordName']
                     origEmails = record.get('dsAttrTypeStandard:EMailAddress', [])
                     origMembers = record.get('dsAttrTypeStandard:GroupMembers', [])
-                    self.addRecord(internalType=internalType, guid=origGUID,
+                    self.addRecord(
+                        internalType=internalType, guid=origGUID,
                         names=origRecordNames, emails=origEmails,
                         members=origMembers)
 
-        print "Done."
-        print ""
+        print("Done.")
+        print("")
 
-    def addRecord(self, internalType="users", guid=None, names=None,
-        emails=None, members=None, cua=None):
+
+    def addRecord(
+        self, internalType="users", guid=None, names=None,
+        emails=None, members=None, cua=None
+    ):
 
         if cua:
             keys = [self.cua2key(cua)]
@@ -442,7 +457,7 @@ class DirectoryMap(object):
                     name = urllib.quote(name).lower()
                     keys.append(name)
                 except:
-                    # print "Failed to urllib.quote( ) %s. Skipping." % (name,)
+                    # print("Failed to urllib.quote( ) %s. Skipping." % (name,))
                     pass
         if emails:
             for email in emails:
@@ -450,6 +465,7 @@ class DirectoryMap(object):
                 keys.append(email)
         return keys
 
+
     def cua2key(self, cua):
         key = cua.lower()
 
@@ -465,23 +481,25 @@ class DirectoryMap(object):
 
         return key
 
+
     def lookupCUA(self, cua):
         key = self.cua2key(cua)
 
-        if key and self.map.has_key(key):
+        if key and key in self.map:
             return self.map[key]
         else:
             return None
 
 
     def printStats(self):
-        print "Directory totals:"
+        print("Directory totals:")
         for internalType, (recordType, ignore) in self.strings.iteritems():
-            print " %s: %d" % (recordType, self.counts[internalType])
+            print(" %s: %d" % (recordType, self.counts[internalType]))
 
         unknown = self.counts['unknown']
         if unknown:
-            print " Principals not found in directory: %d" % (unknown,)
+            print(" Principals not found in directory: %d" % (unknown,))
+
 
     def dumpDsImports(self, dirPath):
         if not os.path.exists(dirPath):
@@ -559,6 +577,8 @@ class DirectoryError(Exception):
     Error trying to access dscl
     """
 
+
+
 class DatabaseError(Exception):
     """
     Error trying to access sqlite3
@@ -566,8 +586,6 @@ class DatabaseError(Exception):
 
 
 
-
-
 def anonymize(text):
     """
     Return a string whose value is the hex digest of text, repeated as needed
@@ -579,24 +597,23 @@ def anonymize(text):
         try:
             text = text.encode('utf-8')
         except UnicodeEncodeError:
-            print "Failed to anonymize:", text
+            print("Failed to anonymize:", text)
             text = "Anonymize me!"
     h = hashlib.md5(text)
     h = h.hexdigest()
     l = len(text)
-    return (h*((l/32)+1))[:-(32-(l%32))]
+    return (h * ((l / 32) + 1))[:-(32 - (l % 32))]
 
 
 
 nameChars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
 def randomName(length):
     l = []
-    for i in xrange(length):
+    for _ignore in xrange(length):
         l.append(random.choice(nameChars))
     return "".join(l)
 
 
 
-
 if __name__ == "__main__":
     main()
