@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,6 @@
 Shared main-point between utilities.
 """
 
-from calendarserver.tap.caldav import CalDAVServiceMaker, CalDAVOptions
 from calendarserver.tap.util import checkDirectories
 from calendarserver.tools.util import loadConfig, autoDisableMemcached
 
@@ -27,16 +26,22 @@ from twext.python.log import StandardIOObserver
 from twistedcaldav.config import ConfigurationError
 from twisted.internet.defer import inlineCallbacks, succeed
 from twisted.application.service import Service
+from twisted.python.logfile import LogFile
+from twisted.python.log import FileLogObserver
 
 import sys
 from calendarserver.tap.util import getRootResource
 from errno import ENOENT, EACCES
-from twext.enterprise.queue import NonPerformingQueuer
+from twext.enterprise.jobs.queue import NonPerformingQueuer
+from twistedcaldav.timezones import TimezoneCache
 
 # TODO: direct unit tests for these functions.
 
 
-def utilityMain(configFileName, serviceClass, reactor=None, serviceMaker=CalDAVServiceMaker, patchConfig=None, onShutdown=None, verbose=False):
+def utilityMain(
+    configFileName, serviceClass, reactor=None, serviceMaker=None,
+    patchConfig=None, onShutdown=None, verbose=False, loadTimezones=False
+):
     """
     Shared main-point for utilities.
 
@@ -68,6 +73,11 @@ def utilityMain(configFileName, serviceClass, reactor=None, serviceMaker=CalDAVS
         will be imported and used.
     """
 
+    from calendarserver.tap.caldav import CalDAVServiceMaker, CalDAVOptions
+    if serviceMaker is None:
+        serviceMaker = CalDAVServiceMaker
+
+
     # We want to validate that the actual service is always an instance of WorkerService, so wrap the
     # service maker callback inside a function that does that check
     def _makeValidService(store):
@@ -89,6 +99,14 @@ def utilityMain(configFileName, serviceClass, reactor=None, serviceMaker=CalDAVS
 
         checkDirectories(config)
 
+        utilityLogFile = LogFile.fromFullPath(
+            config.UtilityLogFile,
+            rotateLength=config.ErrorLogRotateMB * 1024 * 1024,
+            maxRotatedFiles=config.ErrorLogMaxRotatedFiles
+        )
+        utilityLogObserver = FileLogObserver(utilityLogFile)
+        utilityLogObserver.start()
+
         config.ProcessType = "Utility"
         config.UtilityServiceClass = _makeValidService
 
@@ -107,6 +125,10 @@ def utilityMain(configFileName, serviceClass, reactor=None, serviceMaker=CalDAVS
         if onShutdown is not None:
             reactor.addSystemEventTrigger("before", "shutdown", onShutdown)
 
+        if loadTimezones:
+            TimezoneCache.create()
+
+
     except (ConfigurationError, OSError), e:
         sys.stderr.write("Error: %s\n" % (e,))
         return
