@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- test-case-name: calendarserver.tools.test.test_calverify -*-
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,21 +24,26 @@ of simple commands.
 
 from calendarserver.tools import tables
 from calendarserver.tools.cmdline import utilityMain, WorkerService
-from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.datetime import DateTime
+
 from twext.enterprise.dal.syntax import Select, Parameter, Count, Delete, \
     Constant
+from twext.who.idirectory import RecordType
+
 from twisted.internet.defer import inlineCallbacks, returnValue, succeed
 from twisted.python.filepath import FilePath
 from twisted.python.text import wordWrap
 from twisted.python.usage import Options
+
 from twistedcaldav import caldavxml
 from twistedcaldav.config import config
-from twistedcaldav.datafilters.peruserdata import PerUserDataFilter
 from twistedcaldav.directory import calendaruserproxy
-from twistedcaldav.directory.directory import DirectoryService
-from twistedcaldav.query import calendarqueryfilter
 from twistedcaldav.stdconfig import DEFAULT_CONFIG_FILE
+
+from txdav.caldav.datastore.query.filter import Filter
 from txdav.common.datastore.sql_tables import schema, _BIND_MODE_OWN
+from txdav.who.idirectory import RecordType as CalRecordType
+
 from uuid import UUID
 import os
 import sys
@@ -90,26 +95,28 @@ class DBInspectOptions(Options):
 
 
 
+@inlineCallbacks
 def UserNameFromUID(txn, uid):
-    record = txn.directoryService().recordWithGUID(uid)
-    return record.shortNames[0] if record else "(%s)" % (uid,)
+    record = yield txn.directoryService().recordWithUID(uid)
+    returnValue(record.shortNames[0] if record else "(%s)" % (uid,))
 
 
 
+@inlineCallbacks
 def UIDFromInput(txn, value):
     try:
-        return str(UUID(value)).upper()
+        returnValue(str(UUID(value)).upper())
     except (ValueError, TypeError):
         pass
 
-    record = txn.directoryService().recordWithShortName(DirectoryService.recordType_users, value)
+    record = yield txn.directoryService().recordWithShortName(RecordType.user, value)
     if record is None:
-        record = txn.directoryService().recordWithShortName(DirectoryService.recordType_locations, value)
+        record = yield txn.directoryService().recordWithShortName(CalRecordType.location, value)
     if record is None:
-        record = txn.directoryService().recordWithShortName(DirectoryService.recordType_resources, value)
+        record = yield txn.directoryService().recordWithShortName(CalRecordType.resource, value)
     if record is None:
-        record = txn.directoryService().recordWithShortName(DirectoryService.recordType_groups, value)
-    return record.guid if record else None
+        record = yield txn.directoryService().recordWithShortName(RecordType.group, value)
+    returnValue(record.uid if record else None)
 
 
 
@@ -178,7 +185,7 @@ class CalendarHomes(Cmd):
         table = tables.Table()
         table.addHeader(("Owner UID", "Short Name"))
         for uid in sorted(uids):
-            shortname = UserNameFromUID(txn, uid)
+            shortname = yield UserNameFromUID(txn, uid)
             if shortname.startswith("("):
                 missing += 1
             table.addRow((
@@ -224,7 +231,7 @@ class CalendarHomesSummary(Cmd):
         table.addHeader(("Owner UID", "Short Name", "Calendars", "Resources"))
         totals = [0, 0, 0]
         for uid in sorted(results.keys()):
-            shortname = UserNameFromUID(txn, uid)
+            shortname = yield UserNameFromUID(txn, uid)
             table.addRow((
                 uid,
                 shortname,
@@ -281,7 +288,7 @@ class Calendars(Cmd):
         table = tables.Table()
         table.addHeader(("Owner UID", "Short Name", "Calendar", "Resources"))
         for uid, calname, count in sorted(uids, key=lambda x: (x[0], x[1])):
-            shortname = UserNameFromUID(txn, uid)
+            shortname = yield UserNameFromUID(txn, uid)
             table.addRow((
                 uid,
                 shortname,
@@ -323,7 +330,7 @@ class CalendarsByOwner(Cmd):
     def doIt(self, txn):
 
         uid = raw_input("Owner UID/Name: ")
-        uid = UIDFromInput(txn, uid)
+        uid = yield UIDFromInput(txn, uid)
         uids = yield self.getCalendars(txn, uid)
 
         # Print table of results
@@ -331,7 +338,7 @@ class CalendarsByOwner(Cmd):
         table.addHeader(("Owner UID", "Short Name", "Calendars", "ID", "Resources"))
         totals = [0, 0, ]
         for uid, calname, resid, count in sorted(uids, key=lambda x: x[1]):
-            shortname = UserNameFromUID(txn, uid)
+            shortname = yield UserNameFromUID(txn, uid)
             table.addRow((
                 uid if totals[0] == 0 else "",
                 shortname if totals[0] == 0 else "",
@@ -384,7 +391,7 @@ class Events(Cmd):
         table = tables.Table()
         table.addHeader(("Owner UID", "Short Name", "Calendar", "ID", "Type", "UID"))
         for uid, calname, id, caltype, caluid in sorted(uids, key=lambda x: (x[0], x[1])):
-            shortname = UserNameFromUID(txn, uid)
+            shortname = yield UserNameFromUID(txn, uid)
             table.addRow((
                 uid,
                 shortname,
@@ -478,9 +485,10 @@ class EventDetails(Cmd):
     """
     Base class for common event details commands.
     """
+    @inlineCallbacks
     def printEventDetails(self, txn, details):
         owner, calendar, resource_id, resource, created, modified, data = details
-        shortname = UserNameFromUID(txn, owner)
+        shortname = yield UserNameFromUID(txn, owner)
         table = tables.Table()
         table.addRow(("Owner UID:", owner,))
         table.addRow(("User Name:", shortname,))
@@ -534,7 +542,7 @@ class Event(EventDetails):
             returnValue(None)
         result = yield self.getData(txn, rid)
         if result:
-            self.printEventDetails(txn, result[0])
+            yield self.printEventDetails(txn, result[0])
         else:
             print("Could not find resource")
 
@@ -556,7 +564,7 @@ class EventsByUID(EventDetails):
         rows = yield self.getData(txn, uid)
         if rows:
             for result in rows:
-                self.printEventDetails(txn, result)
+                yield self.printEventDetails(txn, result)
         else:
             print("Could not find icalendar data")
 
@@ -578,7 +586,7 @@ class EventsByName(EventDetails):
         rows = yield self.getData(txn, name)
         if rows:
             for result in rows:
-                self.printEventDetails(txn, result)
+                yield self.printEventDetails(txn, result)
         else:
             print("Could not find icalendar data")
 
@@ -597,11 +605,11 @@ class EventsByOwner(EventDetails):
     def doIt(self, txn):
 
         uid = raw_input("Owner UID/Name: ")
-        uid = UIDFromInput(txn, uid)
+        uid = yield UIDFromInput(txn, uid)
         rows = yield self.getData(txn, uid)
         if rows:
             for result in rows:
-                self.printEventDetails(txn, result)
+                yield self.printEventDetails(txn, result)
         else:
             print("Could not find icalendar data")
 
@@ -620,12 +628,12 @@ class EventsByOwnerCalendar(EventDetails):
     def doIt(self, txn):
 
         uid = raw_input("Owner UID/Name: ")
-        uid = UIDFromInput(txn, uid)
+        uid = yield UIDFromInput(txn, uid)
         name = raw_input("Calendar resource name: ")
         rows = yield self.getData(txn, uid, name)
         if rows:
             for result in rows:
-                self.printEventDetails(txn, result)
+                yield self.printEventDetails(txn, result)
         else:
             print("Could not find icalendar data")
 
@@ -655,7 +663,7 @@ class EventsByPath(EventDetails):
         rows = yield self.getData(txn, homeName, calendarName, resourceName)
         if rows:
             for result in rows:
-                self.printEventDetails(txn, result)
+                yield self.printEventDetails(txn, result)
         else:
             print("Could not find icalendar data")
 
@@ -687,7 +695,7 @@ class EventsByContent(EventDetails):
         rows = yield self.getData(txn, uid)
         if rows:
             for result in rows:
-                self.printEventDetails(txn, result)
+                yield self.printEventDetails(txn, result)
         else:
             print("Could not find icalendar data")
 
@@ -714,12 +722,12 @@ class EventsInTimerange(Cmd):
             end += "T000000Z"
 
         try:
-            start = PyCalendarDateTime.parseText(start)
+            start = DateTime.parseText(start)
         except ValueError:
             print("Invalid start value")
             returnValue(None)
         try:
-            end = PyCalendarDateTime.parseText(end)
+            end = DateTime.parseText(end)
         except ValueError:
             print("Invalid end value")
             returnValue(None)
@@ -748,25 +756,24 @@ class EventsInTimerange(Cmd):
 
         # Create fake filter element to match time-range
         filter = caldavxml.Filter(
-                      caldavxml.ComponentFilter(
-                          caldavxml.ComponentFilter(
-                              timerange,
-                              name=("VEVENT",),
-                          ),
-                          name="VCALENDAR",
-                       )
-                  )
-        filter = calendarqueryfilter.Filter(filter)
+            caldavxml.ComponentFilter(
+                caldavxml.ComponentFilter(
+                    timerange,
+                    name=("VEVENT",),
+                ),
+                name="VCALENDAR",
+            )
+        )
+        filter = Filter(filter)
         filter.settimezone(None)
 
-        matches = yield calendar._index.indexedSearch(filter, useruid=uid, fbtype=False)
+        matches = yield calendar.search(filter, useruid=uid, fbtype=False)
         if matches is None:
             returnValue(None)
         for name, _ignore_uid, _ignore_type in matches:
             event = yield calendar.calendarObjectWithName(name)
-            ical_data = yield event.component()
-            ical_data = PerUserDataFilter(uid).filter(ical_data)
-            ical_data.stripKnownTimezones()
+            ical_data = yield event.componentForUser()
+            ical_data.stripStandardTimezones()
 
             table = tables.Table()
             table.addRow(("Calendar:", calendar.name(),))
@@ -776,7 +783,7 @@ class EventsInTimerange(Cmd):
             table.addRow(("Modified", event.modified()))
             print("\n")
             table.printTable()
-            print(ical_data.getTextWithTimezones(includeTimezones=False))
+            print(ical_data.getTextWithoutTimezones())
 
 
 
@@ -800,26 +807,26 @@ class Purge(Cmd):
             schema.CALENDAR_OBJECT_REVISIONS,
 
             schema.CALENDAR,
-            #schema.CALENDAR_BIND, - cascades
-            #schema.CALENDAR_OBJECT, - cascades
-            #schema.TIME_RANGE, - cascades
-            #schema.TRANSPARENCY, - cascades
+            # schema.CALENDAR_BIND, - cascades
+            # schema.CALENDAR_OBJECT, - cascades
+            # schema.TIME_RANGE, - cascades
+            # schema.TRANSPARENCY, - cascades
 
 
             schema.CALENDAR_HOME,
-            #schema.CALENDAR_HOME_METADATA - cascades
+            # schema.CALENDAR_HOME_METADATA - cascades
             schema.ATTACHMENT,
 
             schema.ADDRESSBOOK_OBJECT_REVISIONS,
 
             schema.ADDRESSBOOK_HOME,
-            #schema.ADDRESSBOOK_HOME_METADATA, - cascades
-            #schema.ADDRESSBOOK_BIND, - cascades
-            #schema.ADDRESSBOOK_OBJECT, - cascades
+            # schema.ADDRESSBOOK_HOME_METADATA, - cascades
+            # schema.ADDRESSBOOK_BIND, - cascades
+            # schema.ADDRESSBOOK_OBJECT, - cascades
 
             schema.NOTIFICATION_HOME,
             schema.NOTIFICATION,
-            #schema.NOTIFICATION_OBJECT_REVISIONS - cascades,
+            # schema.NOTIFICATION_OBJECT_REVISIONS - cascades,
         )
 
         for tableschema in wipeout:
