@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -63,6 +63,7 @@ def splitICalendarFile(inputFileName, outputDirectory):
             subcalendar_file.close()
 
 
+
 def usage(e=None):
     if e:
         print(e)
@@ -85,6 +86,7 @@ def usage(e=None):
         sys.exit(0)
 
 
+
 def main():
     try:
         (optargs, args) = getopt(
@@ -95,7 +97,7 @@ def main():
     except GetoptError, e:
         usage(e)
 
-    for opt, arg in optargs:
+    for opt, _ignore_arg in optargs:
         if opt in ("-h", "--help"):
             usage()
 
