@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,8 +16,8 @@
 ##
 from __future__ import print_function
 
-from pycalendar.calendar import PyCalendar
-from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.icalendar.calendar import Calendar
+from pycalendar.datetime import DateTime
 
 
 from twext.python.log import StandardIOObserver
@@ -95,17 +95,26 @@ def _doRefresh(tzpath, xmlfile, tzdb, tzvers):
         pass
 
     if not tzvers:
-        tzvers = PyCalendarDateTime.getToday().getText()
+        tzvers = DateTime.getToday().getText()
     print("Converting data (version: %s) at: %s" % (tzvers, zonedir,))
     startYear = 1800
-    endYear = PyCalendarDateTime.getToday().getYear() + 10
-    PyCalendar.sProdID = "-//calendarserver.org//Zonal//EN"
+    endYear = DateTime.getToday().getYear() + 10
+    Calendar.sProdID = "-//calendarserver.org//Zonal//EN"
     zonefiles = "northamerica", "southamerica", "europe", "africa", "asia", "australasia", "antarctica", "etcetera", "backward"
     parser = tzconvert()
     for file in zonefiles:
         parser.parse(os.path.join(zonedir, file))
 
-    parser.generateZoneinfoFiles(os.path.join(rootdir, "zoneinfo"), startYear, endYear, filterzones=())
+    # Check for windows aliases
+    print("Downloading latest data from unicode.org")
+    path = "http://unicode.org/repos/cldr/tags/latest/common/supplemental/windowsZones.xml"
+    data = urllib.urlretrieve(path)
+    wpath = data[0]
+
+    # Generate the iCalendar data
+    print("Generating iCalendar data")
+    parser.generateZoneinfoFiles(os.path.join(rootdir, "zoneinfo"), startYear, endYear, windowsAliases=wpath, filterzones=())
+
     print("Copy new zoneinfo to destination: %s" % (tzpath,))
     z = FilePath(os.path.join(rootdir, "zoneinfo"))
     tz = FilePath(tzpath)
@@ -251,7 +260,8 @@ Options:
 Description:
     This utility will create, update, or list an XML timezone database
     summary file, or refresh iCalendar timezone from IANA (Olson). It can
-    also be used to update the server's own zoneinfo database from IANA.
+    also be used to update the server's own zoneinfo database from IANA. It
+    also creates aliases for the unicode.org windowsZones.
 
 """)
 
