@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 ##
-# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,7 +33,8 @@ from getopt import getopt, GetoptError
 from twistedcaldav.config import ConfigurationError
 from twistedcaldav.upgrade import upgradeData
 
-from calendarserver.tools.util import loadConfig, getDirectory
+from calendarserver.tools.util import loadConfig
+
 
 def usage(e=None):
     if e:
@@ -54,6 +55,8 @@ def usage(e=None):
     else:
         sys.exit(0)
 
+
+
 def main():
     try:
         (optargs, args) = getopt(
@@ -67,7 +70,6 @@ def main():
 
     configFileName = None
 
-
     for opt, arg in optargs:
         if opt in ("-h", "--help"):
             usage()
@@ -80,7 +82,6 @@ def main():
 
     try:
         config = loadConfig(configFileName)
-        config.directory = getDirectory()
     except ConfigurationError, e:
         sys.stdout.write("%s\n" % (e,))
         sys.exit(1)
@@ -89,7 +90,7 @@ def main():
 
     if profiling:
         import cProfile
-        cProfile.runctx("upgradeData(c)", globals(), {"c" : config}, "/tmp/upgrade.prof")
+        cProfile.runctx("upgradeData(c)", globals(), {"c": config}, "/tmp/upgrade.prof")
     else:
         upgradeData(config)
 
