@@ -1,6 +1,6 @@
 # -*- test-case-name: calendarserver.tools.shell.test.test_vfs -*-
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -206,15 +206,15 @@ class Folder(File):
     @inlineCallbacks
     def child(self, name):
         # FIXME: Move this logic to locate()
-        #if not name:
-        #    return succeed(self)
-        #if name == ".":
-        #    return succeed(self)
-        #if name == "..":
-        #    path = self.path[:-1]
-        #    if not path:
-        #        path = "/"
-        #    return RootFolder(self.service).locate(path)
+        # if not name:
+        #     return succeed(self)
+        # if name == ".":
+        #     return succeed(self)
+        # if name == "..":
+        #     path = self.path[:-1]
+        #     if not path:
+        #         path = "/"
+        #     return RootFolder(self.service).locate(path)
 
         if name in self._children:
             returnValue(self._children[name])
@@ -290,10 +290,11 @@ class UIDsFolder(Folder):
         # FIXME: Merge in directory UIDs also?
         # FIXME: Add directory info (eg. name) to list entry
 
+        @inlineCallbacks
         def addResult(ignoredTxn, home):
             uid = home.uid()
 
-            record = self.service.directory.recordWithUID(uid)
+            record = yield self.service.directory.recordWithUID(uid)
             if record:
                 info = {
                     "Record Type": record.recordType,
@@ -331,10 +332,13 @@ class RecordFolder(Folder):
         )
 
 
+    @inlineCallbacks
     def list(self):
         names = set()
 
-        for record in self.service.directory.listRecords(self.recordType):
+        for record in (yield self.service.directory.recordsWithRecordType(
+            self.recordType
+        )):
             for shortName in record.shortNames:
                 if shortName in names:
                     continue
@@ -393,6 +397,7 @@ class PrincipalHomeFolder(Folder):
         Folder.__init__(self, service, path)
 
         if record is None:
+            # FIXME: recordWithUID returns a Deferred but we cannot return or yield it in an __init__ method
             record = self.service.directory.recordWithUID(uid)
 
         if record is not None:
@@ -411,7 +416,7 @@ class PrincipalHomeFolder(Folder):
                 if (
                     self.record is not None and
                     self.service.config.EnableCalDAV and
-                    self.record.enabledForCalendaring
+                    self.record.hasCalendars
                 ):
                     create = True
                 else:
@@ -481,7 +486,7 @@ class PrincipalHomeFolder(Folder):
         self._didInitChildren = True
 
 
-    def _needsChildren(m):
+    def _needsChildren(m): #@NoSelf
         def decorate(self, *args, **kwargs):
             d = self._initChildren()
             d.addCallback(lambda _: m(self, *args, **kwargs))
@@ -731,7 +736,7 @@ class CalendarObject(File):
             organizerName = organizer.parameterValue("CN")
             organizerEmail = organizer.parameterValue("EMAIL")
 
-            name = " (%s)" % (organizerName ,) if organizerName  else ""
+            name = " (%s)" % (organizerName ,) if organizerName else ""
             email = " <%s>" % (organizerEmail,) if organizerEmail else ""
 
             rows.append(("Organizer", "%s%s%s" % (organizer.value(), name, email)))
