@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -48,8 +48,8 @@ from calendarserver.tools.export import usage, exportToFile
 def holiday(uid):
     return (
         getModule("twistedcaldav.test").filePath
-            .sibling("data").child("Holidays").child(uid + ".ics")
-            .getContent()
+        .sibling("data").child("Holidays").child(uid + ".ics")
+        .getContent()
     )
 
 
@@ -277,7 +277,7 @@ class IntegrationTests(StoreTestCase):
         """
         yield populateCalendarsFrom(
             {
-                "home1": {
+                "user01": {
                     "calendar1": {
                         "valentines-day.ics": (valentines, {})
                     }
@@ -291,8 +291,8 @@ class IntegrationTests(StoreTestCase):
 
         io = StringIO()
         yield exportToFile(
-            [(yield self.txn().calendarHomeWithUID("home1"))
-              .calendarWithName("calendar1")], io
+            [(yield self.txn().calendarHomeWithUID("user01"))
+                .calendarWithName("calendar1")], io
         )
         self.assertEquals(Component.fromString(io.getvalue()),
                           expected)
@@ -306,7 +306,7 @@ class IntegrationTests(StoreTestCase):
         """
         yield populateCalendarsFrom(
             {
-                "home1": {
+                "user01": {
                     "calendar1": {
                         "valentines-day.ics": (valentines, {}),
                         "new-years-day.ics": (newYears, {})
@@ -324,8 +324,8 @@ class IntegrationTests(StoreTestCase):
 
         io = StringIO()
         yield exportToFile(
-            [(yield self.txn().calendarHomeWithUID("home1"))
-              .calendarWithName("calendar1")], io
+            [(yield self.txn().calendarHomeWithUID("user01"))
+                .calendarWithName("calendar1")], io
         )
         self.assertEquals(Component.fromString(io.getvalue()),
                           expected)
@@ -342,7 +342,7 @@ class IntegrationTests(StoreTestCase):
         """
         yield populateCalendarsFrom(
             {
-                "home1": {
+                "user01": {
                     "calendar1": {
                         "1.ics": (one, {}), # EST
                         "2.ics": (another, {}), # EST
@@ -354,8 +354,8 @@ class IntegrationTests(StoreTestCase):
 
         io = StringIO()
         yield exportToFile(
-            [(yield self.txn().calendarHomeWithUID("home1"))
-              .calendarWithName("calendar1")], io
+            [(yield self.txn().calendarHomeWithUID("user01"))
+                .calendarWithName("calendar1")], io
         )
         result = Component.fromString(io.getvalue())
 
@@ -396,7 +396,7 @@ class IntegrationTests(StoreTestCase):
         io = StringIO()
         yield exportToFile(
             [(yield self.txn().calendarHomeWithUID("user02"))
-              .calendarWithName("calendar1")], io
+                .calendarWithName("calendar1")], io
         )
         self.assertEquals(
             Component.fromString(resultForUser2),
