@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,24 +16,56 @@
 from __future__ import print_function
 
 import os
-import sys
 from plistlib import readPlistFromString
+import plistlib
 import xml
 
+from twistedcaldav.stdconfig import config
 from twext.python.filepath import CachingFilePath as FilePath
 from twisted.internet import reactor
 from twisted.internet.defer import inlineCallbacks, Deferred, returnValue
+from twisted.trial.unittest import TestCase
+from twistedcaldav import memcacher
+from twistedcaldav.memcacheclient import ClientFactory
+from twistedcaldav.test.util import CapturingProcessProtocol
+from txdav.common.datastore.test.util import (
+    theStoreBuilder, StubNotifierFactory
+)
+from txdav.who.idirectory import AutoScheduleMode
+from txdav.who.util import directoryFromConfig
 
-from twistedcaldav.config import config
-from twistedcaldav.test.util import TestCase, CapturingProcessProtocol
-from calendarserver.tools.util import getDirectory
-import plistlib
 
 
 class RunCommandTestCase(TestCase):
 
+    @inlineCallbacks
     def setUp(self):
-        super(RunCommandTestCase, self).setUp()
+
+        self.serverRoot = self.mktemp()
+        os.mkdir(self.serverRoot)
+        self.absoluteServerRoot = os.path.abspath(self.serverRoot)
+
+        configRoot = os.path.join(self.absoluteServerRoot, "Config")
+        if not os.path.exists(configRoot):
+            os.makedirs(configRoot)
+
+        dataRoot = os.path.join(self.absoluteServerRoot, "Data")
+        if not os.path.exists(dataRoot):
+            os.makedirs(dataRoot)
+
+        documentRoot = os.path.join(self.absoluteServerRoot, "Documents")
+        if not os.path.exists(documentRoot):
+            os.makedirs(documentRoot)
+
+        logRoot = os.path.join(self.absoluteServerRoot, "Logs")
+        if not os.path.exists(logRoot):
+            os.makedirs(logRoot)
+
+        runRoot = os.path.join(self.absoluteServerRoot, "Run")
+        if not os.path.exists(runRoot):
+            os.makedirs(runRoot)
+
+        config.reset()
 
         testRoot = os.path.join(os.path.dirname(__file__), "gateway")
         templateName = os.path.join(testRoot, "caldavd.plist")
@@ -43,41 +75,78 @@ class RunCommandTestCase(TestCase):
 
         databaseRoot = os.path.abspath("_spawned_scripts_db" + str(os.getpid()))
         newConfig = template % {
-            "ServerRoot" : os.path.abspath(config.ServerRoot),
-            "DatabaseRoot" : databaseRoot,
-            "WritablePlist" : os.path.join(os.path.abspath(config.ConfigRoot), "caldavd-writable.plist"),
+            "ServerRoot": self.absoluteServerRoot,
+            "DataRoot": dataRoot,
+            "DatabaseRoot": databaseRoot,
+            "DocumentRoot": documentRoot,
+            "ConfigRoot": configRoot,
+            "LogRoot": logRoot,
+            "RunRoot": runRoot,
+            "WritablePlist": os.path.join(
+                os.path.abspath(configRoot), "caldavd-writable.plist"
+            ),
         }
-        configFilePath = FilePath(os.path.join(config.ConfigRoot, "caldavd.plist"))
+        configFilePath = FilePath(
+            os.path.join(configRoot, "caldavd.plist")
+        )
+
         configFilePath.setContent(newConfig)
 
         self.configFileName = configFilePath.path
         config.load(self.configFileName)
 
-        origUsersFile = FilePath(os.path.join(os.path.dirname(__file__),
-            "gateway", "users-groups.xml"))
-        copyUsersFile = FilePath(os.path.join(config.DataRoot, "accounts.xml"))
+        config.Memcached.Pools.Default.ClientEnabled = False
+        config.Memcached.Pools.Default.ServerEnabled = False
+        ClientFactory.allowTestCache = True
+        memcacher.Memcacher.allowTestCache = True
+        memcacher.Memcacher.reset()
+        config.DirectoryAddressBook.Enabled = False
+        config.UsePackageTimezones = True
+
+        origUsersFile = FilePath(
+            os.path.join(
+                os.path.dirname(__file__),
+                "gateway",
+                "users-groups.xml"
+            )
+        )
+        copyUsersFile = FilePath(
+            os.path.join(config.DataRoot, "accounts.xml")
+        )
         origUsersFile.copyTo(copyUsersFile)
 
-        origResourcesFile = FilePath(os.path.join(os.path.dirname(__file__),
-            "gateway", "resources-locations.xml"))
-        copyResourcesFile = FilePath(os.path.join(config.DataRoot, "resources.xml"))
+        origResourcesFile = FilePath(
+            os.path.join(
+                os.path.dirname(__file__),
+                "gateway",
+                "resources-locations.xml"
+            )
+        )
+        copyResourcesFile = FilePath(
+            os.path.join(config.DataRoot, "resources.xml")
+        )
         origResourcesFile.copyTo(copyResourcesFile)
 
-        origAugmentFile = FilePath(os.path.join(os.path.dirname(__file__),
-            "gateway", "augments.xml"))
+        origAugmentFile = FilePath(
+            os.path.join(
+                os.path.dirname(__file__),
+                "gateway",
+                "augments.xml"
+            )
+        )
         copyAugmentFile = FilePath(os.path.join(config.DataRoot, "augments.xml"))
         origAugmentFile.copyTo(copyAugmentFile)
 
-        # Make sure trial puts the reactor in the right state, by letting it
-        # run one reactor iteration.  (Ignore me, please.)
-        d = Deferred()
-        reactor.callLater(0, d.callback, True)
-        return d
+        self.notifierFactory = StubNotifierFactory()
+        self.store = yield theStoreBuilder.buildStore(self, self.notifierFactory)
+        self.directory = directoryFromConfig(config, self.store)
 
 
     @inlineCallbacks
-    def runCommand(self, command, error=False,
-        script="calendarserver_command_gateway"):
+    def runCommand(
+        self, command, error=False, script="calendarserver_command_gateway",
+        additionalArgs=None, parseOutput=True
+    ):
         """
         Run the given command by feeding it as standard input to
         calendarserver_command_gateway in a subprocess.
@@ -86,31 +155,44 @@ class RunCommandTestCase(TestCase):
         if isinstance(command, unicode):
             command = command.encode("utf-8")
 
-        sourceRoot = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
-        python = sys.executable
-        script = os.path.join(sourceRoot, "bin", script)
+        sourceRoot = os.path.dirname(
+            os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
+        )
+        cmd = script  # assumes it's on PATH
 
-        args = [python, script, "-f", self.configFileName]
+        args = [cmd, "-f", self.configFileName]
         if error:
             args.append("--error")
 
+        if additionalArgs:
+            args.extend(additionalArgs)
+
         cwd = sourceRoot
 
         deferred = Deferred()
-        reactor.spawnProcess(CapturingProcessProtocol(deferred, command), python, args, env=os.environ, path=cwd)
+        reactor.spawnProcess(CapturingProcessProtocol(deferred, command), cmd, args, env=os.environ, path=cwd)
         output = yield deferred
-        try:
-            plist = readPlistFromString(output)
-        except xml.parsers.expat.ExpatError, e:
-            print("Error (%s) parsing (%s)" % (e, output))
-            raise
+        if parseOutput:
+            try:
+                plist = readPlistFromString(output)
+                returnValue(plist)
+            except xml.parsers.expat.ExpatError, e:
+                print("Error (%s) parsing (%s)" % (e, output))
+                raise
+        else:
+            returnValue(output)
 
-        returnValue(plist)
 
 
 
 class GatewayTestCase(RunCommandTestCase):
 
+    def _flush(self):
+        # Flush both XML directories
+        self.directory._directory.services[1].flush()
+        self.directory._directory.services[2].flush()
+
+
     @inlineCallbacks
     def test_getLocationAndResourceList(self):
         results = yield self.runCommand(command_getLocationAndResourceList)
@@ -126,14 +208,19 @@ class GatewayTestCase(RunCommandTestCase):
     @inlineCallbacks
     def test_getLocationAttributes(self):
         yield self.runCommand(command_createLocation)
+
+        # Tell the resources services to flush its cache and re-read XML
+        self._flush()
+
         results = yield self.runCommand(command_getLocationAttributes)
-        self.assertEquals(results["result"]["Capacity"], "40")
-        self.assertEquals(results["result"]["Description"], "Test Description")
+        # self.assertEquals(results["result"]["Capacity"], "40")
+        # self.assertEquals(results["result"]["Description"], "Test Description")
         self.assertEquals(results["result"]["RecordName"], ["createdlocation01"])
-        self.assertEquals(results["result"]["RealName"],
+        self.assertEquals(
+            results["result"]["RealName"],
             "Created Location 01 %s %s" % (unichr(208), u"\ud83d\udca3"))
-        self.assertEquals(results["result"]["Comment"], "Test Comment")
-        self.assertEquals(results["result"]["AutoSchedule"], True)
+        # self.assertEquals(results["result"]["Comment"], "Test Comment")
+        self.assertEquals(results["result"]["AutoScheduleMode"], u"acceptIfFree")
         self.assertEquals(results["result"]["AutoAcceptGroup"], "E5A6142C-4189-4E9E-90B0-9CD0268B314B")
         self.assertEquals(set(results["result"]["ReadProxies"]), set(['user03', 'user04']))
         self.assertEquals(set(results["result"]["WriteProxies"]), set(['user05', 'user06']))
@@ -148,32 +235,36 @@ class GatewayTestCase(RunCommandTestCase):
     @inlineCallbacks
     def test_getResourceAttributes(self):
         yield self.runCommand(command_createResource)
+
+        # Tell the resources services to flush its cache and re-read XML
+        self._flush()
+
         results = yield self.runCommand(command_getResourceAttributes)
-        self.assertEquals(results["result"]["Comment"], "Test Comment")
-        self.assertEquals(results["result"]["Type"], "Computer")
+        # self.assertEquals(results["result"]["Comment"], "Test Comment")
+        # self.assertEquals(results["result"]["Type"], "Computer")
         self.assertEquals(set(results["result"]["ReadProxies"]), set(['user03', 'user04']))
         self.assertEquals(set(results["result"]["WriteProxies"]), set(['user05', 'user06']))
 
 
     @inlineCallbacks
     def test_createAddress(self):
-        directory = getDirectory()
 
-        record = directory.recordWithUID("C701069D-9CA1-4925-A1A9-5CD94767B74B")
+        record = yield self.directory.recordWithUID("C701069D-9CA1-4925-A1A9-5CD94767B74B")
         self.assertEquals(record, None)
         yield self.runCommand(command_createAddress)
 
-        directory.flushCaches()
+        # Tell the resources services to flush its cache and re-read XML
+        self._flush()
 
-        record = directory.recordWithUID("C701069D-9CA1-4925-A1A9-5CD94767B74B")
-        self.assertEquals(record.fullName.decode("utf-8"),
-            "Created Address 01 %s %s" % (unichr(208), u"\ud83d\udca3"))
+        record = yield self.directory.recordWithUID("C701069D-9CA1-4925-A1A9-5CD94767B74B")
+        self.assertEquals(
+            record.displayName,
+            "Created Address 01 %s %s" % (unichr(208), u"\ud83d\udca3")
+        )
 
-        self.assertNotEquals(record, None)
-
-        self.assertEquals(record.extras["abbreviatedName"], "Addr1")
-        self.assertEquals(record.extras["streetAddress"], "1 Infinite Loop\nCupertino, 95014\nCA")
-        self.assertEquals(record.extras["geo"], "geo:37.331,-122.030")
+        self.assertEquals(record.abbreviatedName, "Addr1")
+        self.assertEquals(record.streetAddress, "1 Infinite Loop\nCupertino, 95014\nCA")
+        self.assertEquals(record.geographicLocation, "geo:37.331,-122.030")
 
         results = yield self.runCommand(command_getAddressList)
         self.assertEquals(len(results["result"]), 1)
@@ -186,8 +277,8 @@ class GatewayTestCase(RunCommandTestCase):
         results = yield self.runCommand(command_getAddressAttributes)
         self.assertEquals(results["result"]["RealName"], u'Updated Address')
         self.assertEquals(results["result"]["StreetAddress"], u'Updated Street Address')
-        self.assertEquals(results["result"]["Geo"], u'Updated Geo')
-        
+        self.assertEquals(results["result"]["GeographicLocation"], u'Updated Geo')
+
         results = yield self.runCommand(command_deleteAddress)
 
         results = yield self.runCommand(command_getAddressList)
@@ -196,120 +287,150 @@ class GatewayTestCase(RunCommandTestCase):
 
     @inlineCallbacks
     def test_createLocation(self):
-        directory = getDirectory()
 
-        record = directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
+        record = yield self.directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
         self.assertEquals(record, None)
         yield self.runCommand(command_createLocation)
 
-        directory.flushCaches()
+        # Tell the resources services to flush its cache and re-read XML
+        self._flush()
 
-        # This appears to be necessary in order for record.autoSchedule to
-        # reflect the change prior to the directory record expiration
-        augmentService = directory.serviceForRecordType(directory.recordType_locations).augmentService
-        augmentService.refresh()
-
-        record = directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
-        self.assertEquals(record.fullName.decode("utf-8"),
-            "Created Location 01 %s %s" % (unichr(208), u"\ud83d\udca3"))
+        record = yield self.directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
+        self.assertEquals(
+            record.fullNames[0],
+            u"Created Location 01 %s %s" % (unichr(208), u"\ud83d\udca3"))
 
         self.assertNotEquals(record, None)
-        self.assertEquals(record.autoSchedule, True)
+        # self.assertEquals(record.autoScheduleMode, "")
 
-        self.assertEquals(record.extras["comment"], "Test Comment")
-        self.assertEquals(record.extras["floor"], "First")
-        self.assertEquals(record.extras["capacity"], "40")
-        self.assertEquals(record.extras["geo"], "geo:37.331,-122.030")
+        self.assertEquals(record.floor, u"First")
+        # self.assertEquals(record.extras["capacity"], "40")
 
         results = yield self.runCommand(command_getLocationAttributes)
         self.assertEquals(set(results["result"]["ReadProxies"]), set(['user03', 'user04']))
         self.assertEquals(set(results["result"]["WriteProxies"]), set(['user05', 'user06']))
 
 
+    @inlineCallbacks
+    def test_createLocationMinimal(self):
+        """
+        Ensure we can create a location with just the bare minimum of values,
+        i.e. RealName
+        """
+
+        yield self.runCommand(command_createLocationMinimal)
+
+        # Tell the resources services to flush its cache and re-read XML
+        self._flush()
+
+        records = yield self.directory.recordsWithRecordType(self.directory.recordType.location)
+        for record in records:
+            if record.displayName == u"Minimal":
+                break
+        else:
+            self.fail("We did not find the Minimal record")
+
+
     @inlineCallbacks
     def test_setLocationAttributes(self):
-        directory = getDirectory()
 
         yield self.runCommand(command_createLocation)
         yield self.runCommand(command_setLocationAttributes)
-        directory.flushCaches()
 
-        # This appears to be necessary in order for record.autoSchedule to
-        # reflect the change
-        augmentService = directory.serviceForRecordType(directory.recordType_locations).augmentService
-        augmentService.refresh()
+        # Tell the resources services to flush its cache and re-read XML
+        self._flush()
 
-        record = directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
+        record = yield self.directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
 
-        self.assertEquals(record.extras["comment"], "Updated Test Comment")
-        self.assertEquals(record.extras["floor"], "Second")
-        self.assertEquals(record.extras["capacity"], "41")
-        self.assertEquals(record.extras["streetAddress"], "2 Infinite Loop\nCupertino, 95014\nCA")
-        self.assertEquals(record.autoSchedule, True)
+        # self.assertEquals(record.extras["comment"], "Updated Test Comment")
+        self.assertEquals(record.floor, "Second")
+        # self.assertEquals(record.extras["capacity"], "41")
+        self.assertEquals(record.autoScheduleMode, AutoScheduleMode.acceptIfFree)
         self.assertEquals(record.autoAcceptGroup, "F5A6142C-4189-4E9E-90B0-9CD0268B314B")
 
         results = yield self.runCommand(command_getLocationAttributes)
-        self.assertEquals(results["result"]["AutoSchedule"], True)
+        self.assertEquals(results["result"]["AutoScheduleMode"], "acceptIfFree")
         self.assertEquals(results["result"]["AutoAcceptGroup"], "F5A6142C-4189-4E9E-90B0-9CD0268B314B")
         self.assertEquals(set(results["result"]["ReadProxies"]), set(['user03']))
         self.assertEquals(set(results["result"]["WriteProxies"]), set(['user05', 'user06', 'user07']))
 
 
     @inlineCallbacks
-    def test_destroyLocation(self):
-        directory = getDirectory()
+    def test_setAddressOnLocation(self):
+        yield self.runCommand(command_createLocation)
+        yield self.runCommand(command_createAddress)
+        yield self.runCommand(command_setAddressOnLocation)
+        results = yield self.runCommand(command_getLocationAttributes)
+        self.assertEquals(results["result"]["AssociatedAddress"], "C701069D-9CA1-4925-A1A9-5CD94767B74B")
+        self._flush()
+        record = yield self.directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
+        self.assertEquals(record.associatedAddress, "C701069D-9CA1-4925-A1A9-5CD94767B74B")
+        yield self.runCommand(command_removeAddressFromLocation)
+        results = yield self.runCommand(command_getLocationAttributes)
+        self.assertEquals(results["result"]["AssociatedAddress"], "")
+        self._flush()
+        record = yield self.directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
+        self.assertEquals(record.associatedAddress, u"")
 
-        record = directory.recordWithUID("location01")
+
+    @inlineCallbacks
+    def test_destroyLocation(self):
+
+        record = yield self.directory.recordWithUID("location01")
         self.assertNotEquals(record, None)
 
         yield self.runCommand(command_deleteLocation)
 
-        directory.flushCaches()
-        record = directory.recordWithUID("location01")
+        # Tell the resources services to flush its cache and re-read XML
+        self._flush()
+
+        record = yield self.directory.recordWithUID("location01")
         self.assertEquals(record, None)
 
 
     @inlineCallbacks
     def test_createResource(self):
-        directory = getDirectory()
 
-        record = directory.recordWithUID("AF575A61-CFA6-49E1-A0F6-B5662C9D9801")
+        record = yield self.directory.recordWithUID("AF575A61-CFA6-49E1-A0F6-B5662C9D9801")
         self.assertEquals(record, None)
 
         yield self.runCommand(command_createResource)
 
-        directory.flushCaches()
-        record = directory.recordWithUID("AF575A61-CFA6-49E1-A0F6-B5662C9D9801")
+        # Tell the resources services to flush its cache and re-read XML
+        self._flush()
+
+        record = yield self.directory.recordWithUID("AF575A61-CFA6-49E1-A0F6-B5662C9D9801")
         self.assertNotEquals(record, None)
 
 
     @inlineCallbacks
     def test_setResourceAttributes(self):
-        directory = getDirectory()
 
         yield self.runCommand(command_createResource)
-        directory.flushCaches()
-        record = directory.recordWithUID("AF575A61-CFA6-49E1-A0F6-B5662C9D9801")
-        self.assertEquals(record.fullName, "Laptop 1")
+        record = yield self.directory.recordWithUID("AF575A61-CFA6-49E1-A0F6-B5662C9D9801")
+        self.assertEquals(record.displayName, "Laptop 1")
 
         yield self.runCommand(command_setResourceAttributes)
 
-        directory.flushCaches()
-        record = directory.recordWithUID("AF575A61-CFA6-49E1-A0F6-B5662C9D9801")
-        self.assertEquals(record.fullName, "Updated Laptop 1")
+        # Tell the resources services to flush its cache and re-read XML
+        self._flush()
+
+        record = yield self.directory.recordWithUID("AF575A61-CFA6-49E1-A0F6-B5662C9D9801")
+        self.assertEquals(record.displayName, "Updated Laptop 1")
 
 
     @inlineCallbacks
     def test_destroyResource(self):
-        directory = getDirectory()
 
-        record = directory.recordWithUID("resource01")
+        record = yield self.directory.recordWithUID("resource01")
         self.assertNotEquals(record, None)
 
         yield self.runCommand(command_deleteResource)
 
-        directory.flushCaches()
-        record = directory.recordWithUID("resource01")
+        # Tell the resources services to flush its cache and re-read XML
+        self._flush()
+
+        record = yield self.directory.recordWithUID("resource01")
         self.assertEquals(record, None)
 
 
@@ -334,14 +455,16 @@ class GatewayTestCase(RunCommandTestCase):
         results = yield self.runCommand(command_purgeOldEventsNoDays)
         self.assertEquals(results["result"]["RetainDays"], 365)
 
+
     @inlineCallbacks
     def test_readConfig(self):
         """
         Verify readConfig returns with only the writable keys
         """
-        results = yield self.runCommand(command_readConfig,
-            script="calendarserver_config")
-
+        results = yield self.runCommand(
+            command_readConfig,
+            script="calendarserver_config"
+        )
         self.assertEquals(results["result"]["RedirectHTTPToHTTPS"], False)
         self.assertEquals(results["result"]["EnableSearchAddressBook"], False)
         self.assertEquals(results["result"]["EnableCalDAV"], True)
@@ -350,10 +473,12 @@ class GatewayTestCase(RunCommandTestCase):
         self.assertEquals(results["result"]["DefaultLogLevel"], "warn")
 
         self.assertEquals(results["result"]["Notifications"]["Services"]["APNS"]["Enabled"], False)
-        self.assertEquals(results["result"]["Notifications"]["Services"]["APNS"]["CalDAV"]["CertificatePath"], "/example/calendar.cer")
 
-        # Verify not all keys are present, such as ServerRoot which is not writable
-        self.assertFalse("ServerRoot" in results["result"])
+        # This is a read only key that is returned
+        self.assertEquals(results["result"]["ServerRoot"], self.absoluteServerRoot)
+
+        # Verify other non-writeable keys are not present, such as DataRoot which is not writable
+        self.assertFalse("DataRoot" in results["result"])
 
 
     @inlineCallbacks
@@ -361,16 +486,17 @@ class GatewayTestCase(RunCommandTestCase):
         """
         Verify writeConfig updates the writable plist file only
         """
-        results = yield self.runCommand(command_writeConfig,
-            script="calendarserver_config")
+        results = yield self.runCommand(
+            command_writeConfig,
+            script="calendarserver_config"
+        )
 
         self.assertEquals(results["result"]["EnableCalDAV"], False)
         self.assertEquals(results["result"]["EnableCardDAV"], False)
         self.assertEquals(results["result"]["EnableSSL"], True)
         self.assertEquals(results["result"]["Notifications"]["Services"]["APNS"]["Enabled"], True)
-        self.assertEquals(results["result"]["Notifications"]["Services"]["APNS"]["CalDAV"]["CertificatePath"], "/example/changed.cer")
-        dataRoot = "Data/%s/%s" % (unichr(208), u"\ud83d\udca3")
-        self.assertTrue(results["result"]["DataRoot"].endswith(dataRoot))
+        hostName = "hostname_%s_%s" % (unichr(208), u"\ud83d\udca3")
+        self.assertTrue(results["result"]["ServerHostName"].endswith(hostName))
 
         # The static plist should still have EnableCalDAV = True
         staticPlist = plistlib.readPlist(self.configFileName)
@@ -384,9 +510,9 @@ command_addReadProxy = """<?xml version="1.0" encoding="UTF-8"?>
         <key>command</key>
         <string>addReadProxy</string>
         <key>Principal</key>
-        <string>locations:location01</string>
+        <string>location01</string>
         <key>Proxy</key>
-        <string>users:user03</string>
+        <string>user03</string>
 </dict>
 </plist>
 """
@@ -398,9 +524,9 @@ command_addWriteProxy = """<?xml version="1.0" encoding="UTF-8"?>
         <key>command</key>
         <string>addWriteProxy</string>
         <key>Principal</key>
-        <string>locations:location01</string>
+        <string>location01</string>
         <key>Proxy</key>
-        <string>users:user01</string>
+        <string>user01</string>
 </dict>
 </plist>
 """
@@ -423,7 +549,7 @@ command_createAddress = """<?xml version="1.0" encoding="UTF-8"?>
         </array>
         <key>StreetAddress</key>
         <string>1 Infinite Loop\nCupertino, 95014\nCA</string>
-        <key>Geo</key>
+        <key>GeographicLocation</key>
         <string>geo:37.331,-122.030</string>
 </dict>
 </plist>
@@ -436,8 +562,8 @@ command_createLocation = """<?xml version="1.0" encoding="UTF-8"?>
 <dict>
         <key>command</key>
         <string>createLocation</string>
-        <key>AutoSchedule</key>
-        <true/>
+        <key>AutoScheduleMode</key>
+        <string>acceptIfFree</string>
         <key>AutoAcceptGroup</key>
         <string>E5A6142C-4189-4E9E-90B0-9CD0268B314B</string>
         <key>GeneratedUID</key>
@@ -454,58 +580,73 @@ command_createLocation = """<?xml version="1.0" encoding="UTF-8"?>
         <string>Test Description</string>
         <key>Floor</key>
         <string>First</string>
+        <!--
         <key>Capacity</key>
         <string>40</string>
+        -->
         <key>AssociatedAddress</key>
         <string>C701069D-9CA1-4925-A1A9-5CD94767B74B</string>
-        <key>Geo</key>
-        <string>geo:37.331,-122.030</string>
         <key>ReadProxies</key>
         <array>
-            <string>users:user03</string>
-            <string>users:user04</string>
+            <string>user03</string>
+            <string>user04</string>
         </array>
         <key>WriteProxies</key>
         <array>
-            <string>users:user05</string>
-            <string>users:user06</string>
+            <string>user05</string>
+            <string>user06</string>
         </array>
 </dict>
 </plist>
 """ % (unichr(208), u"\ud83d\udca3")
 
 
+command_createLocationMinimal = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>createLocation</string>
+        <key>RealName</key>
+        <string>Minimal</string>
+</dict>
+</plist>
+"""
+
+
 command_createResource = """<?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
 <plist version="1.0">
 <dict>
         <key>command</key>
         <string>createResource</string>
-        <key>AutoSchedule</key>
-        <true/>
+        <key>AutoScheduleMode</key>
+        <string>declineIfBusy</string>
         <key>GeneratedUID</key>
         <string>AF575A61-CFA6-49E1-A0F6-B5662C9D9801</string>
         <key>RealName</key>
         <string>Laptop 1</string>
+        <!--
         <key>Comment</key>
         <string>Test Comment</string>
         <key>Description</key>
         <string>Test Description</string>
         <key>Type</key>
         <string>Computer</string>
+        -->
         <key>RecordName</key>
         <array>
                 <string>laptop1</string>
         </array>
         <key>ReadProxies</key>
         <array>
-            <string>users:user03</string>
-            <string>users:user04</string>
+            <string>user03</string>
+            <string>user04</string>
         </array>
         <key>WriteProxies</key>
         <array>
-            <string>users:user05</string>
-            <string>users:user06</string>
+            <string>user05</string>
+            <string>user06</string>
         </array>
 </dict>
 </plist>
@@ -620,9 +761,9 @@ command_removeReadProxy = """<?xml version="1.0" encoding="UTF-8"?>
         <key>command</key>
         <string>removeReadProxy</string>
         <key>Principal</key>
-        <string>locations:location01</string>
+        <string>location01</string>
         <key>Proxy</key>
-        <string>users:user03</string>
+        <string>user03</string>
 </dict>
 </plist>
 """
@@ -634,9 +775,9 @@ command_removeWriteProxy = """<?xml version="1.0" encoding="UTF-8"?>
         <key>command</key>
         <string>removeWriteProxy</string>
         <key>Principal</key>
-        <string>locations:location01</string>
+        <string>location01</string>
         <key>Proxy</key>
-        <string>users:user01</string>
+        <string>user01</string>
 </dict>
 </plist>
 """
@@ -665,24 +806,53 @@ command_setLocationAttributes = """<?xml version="1.0" encoding="UTF-8"?>
         <string>Updated Test Description</string>
         <key>Floor</key>
         <string>Second</string>
+        <!--
         <key>Capacity</key>
         <string>41</string>
-        <key>StreetAddress</key>
-        <string>2 Infinite Loop\nCupertino, 95014\nCA</string>
+        -->
         <key>ReadProxies</key>
         <array>
-            <string>users:user03</string>
+            <string>user03</string>
         </array>
         <key>WriteProxies</key>
         <array>
-            <string>users:user05</string>
-            <string>users:user06</string>
-            <string>users:user07</string>
+            <string>user05</string>
+            <string>user06</string>
+            <string>user07</string>
         </array>
 </dict>
 </plist>
 """
 
+command_setAddressOnLocation = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>setLocationAttributes</string>
+        <key>GeneratedUID</key>
+        <string>836B1B66-2E9A-4F46-8B1C-3DD6772C20B2</string>
+        <key>AssociatedAddress</key>
+        <string>C701069D-9CA1-4925-A1A9-5CD94767B74B</string>
+</dict>
+</plist>
+"""
+
+command_removeAddressFromLocation = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>setLocationAttributes</string>
+        <key>GeneratedUID</key>
+        <string>836B1B66-2E9A-4F46-8B1C-3DD6772C20B2</string>
+        <key>AssociatedAddress</key>
+        <string></string>
+</dict>
+</plist>
+"""
+
+
 command_getLocationAttributes = """<?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
 <plist version="1.0">
@@ -719,7 +889,7 @@ command_setAddressAttributes = """<?xml version="1.0" encoding="UTF-8"?>
         <string>Updated Address</string>
         <key>StreetAddress</key>
         <string>Updated Street Address</string>
-        <key>Geo</key>
+        <key>GeographicLocation</key>
         <string>Updated Geo</string>
 
 </dict>
@@ -733,8 +903,8 @@ command_setResourceAttributes = """<?xml version="1.0" encoding="UTF-8"?>
 <dict>
         <key>command</key>
         <string>setResourceAttributes</string>
-        <key>AutoSchedule</key>
-        <false/>
+        <key>AutoScheduleMode</key>
+        <string>acceptIfFree</string>
         <key>GeneratedUID</key>
         <string>AF575A61-CFA6-49E1-A0F6-B5662C9D9801</string>
         <key>RealName</key>
@@ -807,10 +977,8 @@ command_writeConfig = """<?xml version="1.0" encoding="UTF-8"?>
             <true/>
             <key>Notifications.Services.APNS.Enabled</key>
             <true/>
-            <key>Notifications.Services.APNS.CalDAV.CertificatePath</key>
-            <string>/example/changed.cer</string>
-            <key>DataRoot</key>
-            <string>Data/%s/%s</string>
+            <key>ServerHostName</key>
+            <string>hostname_%s_%s</string>
         </dict>
 </dict>
 </plist>
