@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,28 +18,26 @@
 Tests for calendarserver.tools.purge
 """
 
-from calendarserver.tools.purge import PurgeOldEventsService, PurgeAttachmentsService, \
-    PurgePrincipalService
-
-from pycalendar.datetime import PyCalendarDateTime
-from pycalendar.timezone import PyCalendarTimezone
+import os
 
+from calendarserver.tools.purge import (
+    PurgeOldEventsService, PurgeAttachmentsService, PurgePrincipalService, PrincipalPurgeHomeWork
+)
+from pycalendar.datetime import DateTime
 from twext.enterprise.dal.syntax import Update, Delete
-from twext.web2.http_headers import MimeType
-
-from twisted.internet.defer import inlineCallbacks, returnValue
-
+from twext.enterprise.util import parseSQLTimestamp
+from twisted.internet import reactor
+from twisted.internet.defer import inlineCallbacks, returnValue, Deferred
 from twistedcaldav.config import config
 from twistedcaldav.test.util import StoreTestCase
 from twistedcaldav.vcard import Component as VCardComponent
-
 from txdav.common.datastore.sql_tables import schema
 from txdav.common.datastore.test.util import populateCalendarsFrom
-
-import os
+from txweb2.http_headers import MimeType
 
 
-now = PyCalendarDateTime.getToday().getYear()
+
+now = DateTime.getToday().getYear()
 
 OLD_ICS = """BEGIN:VCALENDAR
 VERSION:2.0
@@ -388,7 +386,8 @@ class PurgeOldEventsTests(StoreTestCase):
                 "oldmattachment1.ics" : (MATTACHMENT_ICS % {"year": now - 5, "uid": "1.1m"}, metadata,),
                 "oldmattachment2.ics" : (MATTACHMENT_ICS % {"year": now - 5, "uid": "1.2m"}, metadata,),
                 "currentmattachment3.ics" : (MATTACHMENT_ICS % {"year": now + 1, "uid": "1.3m"}, metadata,),
-            }
+            },
+            "inbox": {},
         },
         "home2" : {
             "calendar2" : {
@@ -405,26 +404,35 @@ class PurgeOldEventsTests(StoreTestCase):
             },
             "calendar3" : {
                 "repeating_awhile.ics" : (REPEATING_AWHILE_ICS, metadata,),
-            }
+            },
+            "inbox": {},
         }
     }
 
+
     def configure(self):
         super(PurgeOldEventsTests, self).configure()
 
         # Turn off delayed indexing option so we can have some useful tests
         self.patch(config, "FreeBusyIndexDelayedExpand", False)
 
-        self.patch(config.DirectoryService.params, "xmlFile",
-            os.path.join(
-                os.path.dirname(__file__), "purge", "accounts.xml"
-            )
-        )
-        self.patch(config.ResourceService.params, "xmlFile",
-            os.path.join(
-                os.path.dirname(__file__), "purge", "resources.xml"
-            )
-        )
+        # Tweak queue timing to speed things up
+        self.patch(config.Scheduling.Options.WorkQueues, "Enabled", False)
+        self.patch(config.AutomaticPurging, "PollingIntervalSeconds", -1)
+        self.patch(config.AutomaticPurging, "CheckStaggerSeconds", 1)
+        self.patch(config.AutomaticPurging, "PurgeIntervalSeconds", 3)
+        self.patch(config.AutomaticPurging, "HomePurgeDelaySeconds", 1)
+
+        # self.patch(config.DirectoryService.params, "xmlFile",
+        #     os.path.join(
+        #         os.path.dirname(__file__), "purge", "accounts.xml"
+        #     )
+        # )
+        # self.patch(config.ResourceService.params, "xmlFile",
+        #     os.path.join(
+        #         os.path.dirname(__file__), "purge", "resources.xml"
+        #     )
+        # )
 
 
     @inlineCallbacks
@@ -443,26 +451,27 @@ class PurgeOldEventsTests(StoreTestCase):
 
     @inlineCallbacks
     def test_eventsOlderThan(self):
-        cutoff = PyCalendarDateTime(now, 4, 1, 0, 0, 0)
+        cutoff = DateTime(now, 4, 1, 0, 0, 0)
         txn = self._sqlCalendarStore.newTransaction()
 
         # Query for all old events
         results = (yield txn.eventsOlderThan(cutoff))
-        self.assertEquals(sorted(results),
+        self.assertEquals(
+            sorted(results),
             sorted([
-                ['home1', 'calendar1', 'old.ics', '1901-01-01 01:00:00'],
-                ['home1', 'calendar1', 'oldattachment1.ics', '1901-01-01 01:00:00'],
-                ['home1', 'calendar1', 'oldattachment2.ics', '1901-01-01 01:00:00'],
-                ['home1', 'calendar1', 'oldmattachment1.ics', '1901-01-01 01:00:00'],
-                ['home1', 'calendar1', 'oldmattachment2.ics', '1901-01-01 01:00:00'],
-                ['home2', 'calendar3', 'repeating_awhile.ics', '1901-01-01 01:00:00'],
-                ['home2', 'calendar2', 'recent.ics', '%s-03-04 22:15:00' % (now,)],
-                ['home2', 'calendar2', 'oldattachment1.ics', '1901-01-01 01:00:00'],
-                ['home2', 'calendar2', 'oldattachment3.ics', '1901-01-01 01:00:00'],
-                ['home2', 'calendar2', 'oldattachment4.ics', '1901-01-01 01:00:00'],
-                ['home2', 'calendar2', 'oldmattachment1.ics', '1901-01-01 01:00:00'],
-                ['home2', 'calendar2', 'oldmattachment3.ics', '1901-01-01 01:00:00'],
-                ['home2', 'calendar2', 'oldmattachment4.ics', '1901-01-01 01:00:00'],
+                ['home1', 'calendar1', 'old.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home1', 'calendar1', 'oldattachment1.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home1', 'calendar1', 'oldattachment2.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home1', 'calendar1', 'oldmattachment1.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home1', 'calendar1', 'oldmattachment2.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home2', 'calendar3', 'repeating_awhile.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home2', 'calendar2', 'recent.ics', parseSQLTimestamp('%s-03-04 22:15:00' % (now,))],
+                ['home2', 'calendar2', 'oldattachment1.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home2', 'calendar2', 'oldattachment3.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home2', 'calendar2', 'oldattachment4.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home2', 'calendar2', 'oldmattachment1.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home2', 'calendar2', 'oldmattachment3.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
+                ['home2', 'calendar2', 'oldmattachment4.ics', parseSQLTimestamp('1901-01-01 01:00:00')],
             ])
         )
 
@@ -475,7 +484,7 @@ class PurgeOldEventsTests(StoreTestCase):
 
     @inlineCallbacks
     def test_removeOldEvents(self):
-        cutoff = PyCalendarDateTime(now, 4, 1, 0, 0, 0)
+        cutoff = DateTime(now, 4, 1, 0, 0, 0)
         txn = self._sqlCalendarStore.newTransaction()
 
         # Remove oldest event - except we don't know what that is because of the dummy timestamps
@@ -489,7 +498,7 @@ class PurgeOldEventsTests(StoreTestCase):
         count = (yield txn.removeOldEvents(cutoff))
         self.assertEquals(count, 12)
         results = (yield txn.eventsOlderThan(cutoff))
-        self.assertEquals(results, [])
+        self.assertEquals(list(results), [])
 
         # Remove oldest events (none left)
         count = (yield txn.removeOldEvents(cutoff))
@@ -598,7 +607,7 @@ class PurgeOldEventsTests(StoreTestCase):
         self.assertTrue(os.path.exists(mattachmentPath2))
 
         # Delete all old events (including the event containing the attachment)
-        cutoff = PyCalendarDateTime(now, 4, 1, 0, 0, 0)
+        cutoff = DateTime(now, 4, 1, 0, 0, 0)
         count = (yield self.transactionUnderTest().removeOldEvents(cutoff))
 
         # See which events have gone and which exist
@@ -633,28 +642,118 @@ class PurgeOldEventsTests(StoreTestCase):
         # Dry run
         total = (yield PurgeOldEventsService.purgeOldEvents(
             self._sqlCalendarStore,
-            PyCalendarDateTime(now, 4, 1, 0, 0, 0),
+            None,
+            DateTime(now, 4, 1, 0, 0, 0),
             2,
             dryrun=True,
-            verbose=False
+            debug=True
         ))
         self.assertEquals(total, 13)
 
         # Actually remove
         total = (yield PurgeOldEventsService.purgeOldEvents(
             self._sqlCalendarStore,
-            PyCalendarDateTime(now, 4, 1, 0, 0, 0),
+            None,
+            DateTime(now, 4, 1, 0, 0, 0),
             2,
-            verbose=False
+            debug=True
         ))
         self.assertEquals(total, 13)
 
         # There should be no more left
         total = (yield PurgeOldEventsService.purgeOldEvents(
             self._sqlCalendarStore,
-            PyCalendarDateTime(now, 4, 1, 0, 0, 0),
+            None,
+            DateTime(now, 4, 1, 0, 0, 0),
             2,
-            verbose=False
+            debug=True
+        ))
+        self.assertEquals(total, 0)
+
+
+    @inlineCallbacks
+    def test_purgeOldEvents_home_filtering(self):
+
+        # Dry run
+        total = (yield PurgeOldEventsService.purgeOldEvents(
+            self._sqlCalendarStore,
+            "ho",
+            DateTime(now, 4, 1, 0, 0, 0),
+            2,
+            dryrun=True,
+            debug=True
+        ))
+        self.assertEquals(total, 13)
+
+        # Dry run
+        total = (yield PurgeOldEventsService.purgeOldEvents(
+            self._sqlCalendarStore,
+            "home",
+            DateTime(now, 4, 1, 0, 0, 0),
+            2,
+            dryrun=True,
+            debug=True
+        ))
+        self.assertEquals(total, 13)
+
+        # Dry run
+        total = (yield PurgeOldEventsService.purgeOldEvents(
+            self._sqlCalendarStore,
+            "home1",
+            DateTime(now, 4, 1, 0, 0, 0),
+            2,
+            dryrun=True,
+            debug=True
+        ))
+        self.assertEquals(total, 5)
+
+        # Dry run
+        total = (yield PurgeOldEventsService.purgeOldEvents(
+            self._sqlCalendarStore,
+            "home2",
+            DateTime(now, 4, 1, 0, 0, 0),
+            2,
+            dryrun=True,
+            debug=True
+        ))
+        self.assertEquals(total, 8)
+
+
+    @inlineCallbacks
+    def test_purgeOldEvents_old_cutoff(self):
+
+        # Dry run
+        cutoff = DateTime.getToday()
+        cutoff.setDateOnly(False)
+        cutoff.offsetDay(-400)
+
+        total = (yield PurgeOldEventsService.purgeOldEvents(
+            self._sqlCalendarStore,
+            "ho",
+            cutoff,
+            2,
+            dryrun=True,
+            debug=True
+        ))
+        self.assertEquals(total, 12)
+
+        # Actually remove
+        total = (yield PurgeOldEventsService.purgeOldEvents(
+            self._sqlCalendarStore,
+            None,
+            cutoff,
+            2,
+            debug=True
+        ))
+        self.assertEquals(total, 12)
+
+        total = (yield PurgeOldEventsService.purgeOldEvents(
+            self._sqlCalendarStore,
+            "ho",
+            cutoff,
+            2,
+            dryrun=True,
+            debug=True
         ))
         self.assertEquals(total, 0)
 
@@ -666,44 +765,8 @@ class PurgeOldEventsTests(StoreTestCase):
         # Create an addressbook and one CardDAV resource
         abHome = (yield txn.addressbookHomeWithUID("home1", create=True))
         abColl = (yield abHome.addressbookWithName("addressbook"))
-        (yield abColl.createAddressBookObjectWithName("card1",
-            VCardComponent.fromString(VCARD_1)))
-        self.assertEquals(len((yield abColl.addressbookObjects())), 1)
-
-        # Verify there are 8 events in calendar1
-        calHome = (yield txn.calendarHomeWithUID("home1"))
-        calColl = (yield calHome.calendarWithName("calendar1"))
-        self.assertEquals(len((yield calColl.calendarObjects())), 8)
-
-        # Make the newly created objects available to the purgeUID transaction
-        (yield txn.commit())
-
-        # Purge home1
-        total, ignored = (yield PurgePrincipalService.purgeUIDs(self._sqlCalendarStore, self.directory,
-            self.rootResource, ("home1",), verbose=False, proxies=False,
-            when=PyCalendarDateTime(now, 4, 1, 12, 0, 0, 0, PyCalendarTimezone(utc=True))))
-
-        # 4 items deleted: 3 events and 1 vcard
-        self.assertEquals(total, 4)
-
-        txn = self._sqlCalendarStore.newTransaction()
-        # adressbook home is deleted since it's now empty
-        abHome = (yield txn.addressbookHomeWithUID("home1"))
-        self.assertEquals(abHome, None)
-
-        calHome = (yield txn.calendarHomeWithUID("home1"))
-        calColl = (yield calHome.calendarWithName("calendar1"))
-        self.assertEquals(len((yield calColl.calendarObjects())), 5)
-
-
-    @inlineCallbacks
-    def test_purgeUIDCompletely(self):
-        txn = self._sqlCalendarStore.newTransaction()
-
-        # Create an addressbook and one CardDAV resource
-        abHome = (yield txn.addressbookHomeWithUID("home1", create=True))
-        abColl = (yield abHome.addressbookWithName("addressbook"))
-        (yield abColl.createAddressBookObjectWithName("card1",
+        (yield abColl.createAddressBookObjectWithName(
+            "card1",
             VCardComponent.fromString(VCARD_1)))
         self.assertEquals(len((yield abColl.addressbookObjects())), 1)
 
@@ -716,8 +779,20 @@ class PurgeOldEventsTests(StoreTestCase):
         (yield txn.commit())
 
         # Purge home1 completely
-        total, ignored = (yield PurgePrincipalService.purgeUIDs(self._sqlCalendarStore, self.directory,
-            self.rootResource, ("home1",), verbose=False, proxies=False, completely=True))
+        total = yield PurgePrincipalService.purgeUIDs(
+            self._sqlCalendarStore, self.directory,
+            ("home1",), verbose=False, proxies=False)
+
+        # Wait for queue to process
+        while(True):
+            txn = self.transactionUnderTest()
+            work = yield PrincipalPurgeHomeWork.all(txn)
+            yield self.commit()
+            if len(work) == 0:
+                break
+            d = Deferred()
+            reactor.callLater(1, lambda : d.callback(None))
+            yield d
 
         # 9 items deleted: 8 events and 1 vcard
         self.assertEquals(total, 9)
@@ -768,9 +843,10 @@ class PurgeOldEventsTests(StoreTestCase):
         # Remove old events first
         total = (yield PurgeOldEventsService.purgeOldEvents(
             self._sqlCalendarStore,
-            PyCalendarDateTime(now, 4, 1, 0, 0, 0),
+            None,
+            DateTime(now, 4, 1, 0, 0, 0),
             2,
-            verbose=False
+            debug=False
         ))
         self.assertEquals(total, 13)
 
