@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -80,13 +80,13 @@ END:VEVENT
 
 attendee = """\
 ATTENDEE;CN=User %(SEQUENCE)02d;CUTYPE=INDIVIDUAL;EMAIL=user%(SEQUENCE)02d@example.com;PARTSTAT=NE
- EDS-ACTION;ROLE=REQ-PARTICIPANT;RSVP=TRUE:urn:uuid:user%(SEQUENCE)02d
+ EDS-ACTION;ROLE=REQ-PARTICIPANT;RSVP=TRUE:urn:x-uid:user%(SEQUENCE)02d
 """
 
 organizer = """\
-ORGANIZER;CN=User %(SEQUENCE)02d;EMAIL=user%(SEQUENCE)02d@example.com:urn:uuid:user%(SEQUENCE)02d
+ORGANIZER;CN=User %(SEQUENCE)02d;EMAIL=user%(SEQUENCE)02d@example.com:urn:x-uid:user%(SEQUENCE)02d
 ATTENDEE;CN=User %(SEQUENCE)02d;EMAIL=user%(SEQUENCE)02d@example.com;PARTSTAT=ACCEPTE
- D:urn:uuid:user%(SEQUENCE)02d
+ D:urn:x-uid:user%(SEQUENCE)02d
 """
 
 def formatDate(d):
@@ -119,8 +119,8 @@ def makeVCalendar(uid, start, end, recurrence, organizerSequence, attendees):
             'ORGANIZER': makeOrganizer(organizerSequence),
             'ATTENDEES': ''.join(attendees),
             'RRULE': rrule,
-            },
-        }
+        },
+    }
     return cal.replace("\n", "\r\n")
 
 
