@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,7 +36,7 @@ def makeEvent(i, organizerSequence, attendeeCount):
     attendees = makeAttendees(attendeeCount)
     attendees.append(
         'ATTENDEE;CN="Resource 01";CUTYPE=INDIVIDUAL;PARTSTAT=NEEDS-ACTION;RSVP=T\n'
-        ' RUE;SCHEDULE-STATUS="1.2":urn:uuid:resource01\n')
+        ' RUE;SCHEDULE-STATUS="1.2":urn:x-uid:40000000-0000-0000-0000-000000000001\n')
     return makeVCalendar(
         uuid4(), start, end, "RRULE:FREQ=WEEKLY", organizerSequence, attendees)
 
