@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -34,6 +34,7 @@ from contrib.performance.benchlib import CalDAVAccount, sample
 
 from contrib.performance.benchmarks.vfreebusy import VFREEBUSY, formatDate, makeEventNear
 
+
 @inlineCallbacks
 def measure(host, port, dtrace, attendees, samples):
     userNumber = 1
@@ -51,7 +52,7 @@ def measure(host, port, dtrace, attendees, samples):
     for i in [userNumber] + targets:
         targetUser = "user%02d" % (i,)
         for path in ["calendars/users/%s/" % (targetUser,),
-                     "calendars/__uids__/%s/" % (targetUser,)]:
+                     "calendars/__uids__/10000000-0000-0000-0000-000000000%03d/" % (i,)]:
             authinfo.add_password(
                 realm="Test Realm",
                 uri="http://%s:%d/%s" % (host, port, path),
@@ -75,18 +76,18 @@ def measure(host, port, dtrace, attendees, samples):
 
     # And now issue the actual VFREEBUSY request
     method = 'POST'
-    uri = 'http://%s:%d/calendars/__uids__/%s/outbox/' % (host, port, user)
+    uri = 'http://%s:%d/calendars/__uids__/10000000-0000-0000-0000-000000000001/outbox/' % (host, port)
     headers = Headers({
-            "content-type": ["text/calendar"],
-            "originator": ["mailto:%s@example.com" % (user,)],
-            "recipient": [", ".join(["urn:uuid:user%02d" % (i,) for i in [userNumber] + targets])]})
+        "content-type": ["text/calendar"],
+        "originator": ["mailto:%s@example.com" % (user,)],
+        "recipient": [", ".join(["urn:x-uid:10000000-0000-0000-0000-000000000%03d" % (i,) for i in [userNumber] + targets])]})
     body = StringProducer(VFREEBUSY % {
-            "attendees": "".join([
-                    "ATTENDEE:urn:uuid:user%02d\n" % (i,)
-                    for i in [userNumber] + targets]),
-            "start": formatDate(baseTime.replace(hour=0, minute=0)) + 'Z',
-            "end": formatDate(
-                baseTime.replace(hour=0, minute=0) + timedelta(days=1)) + 'Z'})
+        "attendees": "".join([
+            "ATTENDEE:urn:x-uid:10000000-0000-0000-0000-000000000%03d\n" % (i,)
+            for i in [userNumber] + targets]),
+        "start": formatDate(baseTime.replace(hour=0, minute=0)) + 'Z',
+        "end": formatDate(
+            baseTime.replace(hour=0, minute=0) + timedelta(days=1)) + 'Z'})
 
     samples = yield sample(
         dtrace, samples,
