@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -34,7 +34,7 @@ if __name__ == '__main__':
             from twisted.internet import reactor
             from twisted.internet.stdio import StandardIO
 
-            from contrib.performance.loadtest.ampsim import Worker
+            from contrib.performance.loadtest.ampsim import Worker # @UnresolvedImport
             from contrib.performance.loadtest.sim import LagTrackingReactor
 
             StandardIO(Worker(LagTrackingReactor(reactor)))
@@ -87,6 +87,7 @@ class Account(Command):
         ("password", Unicode()),
         ("commonName", Unicode()),
         ("email", Unicode()),
+        ("guid", Unicode()),
     ]
 
 
@@ -156,7 +157,8 @@ class Manager(AMP):
                             uid=record.uid,
                             password=record.password,
                             commonName=record.commonName,
-                            email=record.email)
+                            email=record.email,
+                            guid=record.guid)
 
         workerConfig = deepcopy(self.loadsim.configTemplate)
         # The list of workers is for the manager only; the workers themselves
@@ -177,13 +179,12 @@ class Manager(AMP):
         self.output.write("Initiating worker configuration\n")
         def completed(x):
             self.output.write("Worker configuration complete.\n")
-        (self.callRemote(Configure, plist=plist)
-         .addCallback(completed))
+        self.callRemote(Configure, plist=plist).addCallback(completed)
 
 
     @LogMessage.responder
     def observed(self, event):
-        #from pprint import pformat
-        #self.output.write(pformat(event)+"\n")
+        # from pprint import pformat
+        # self.output.write(pformat(event)+"\n")
         msg(**event)
         return {}
