@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -29,12 +29,13 @@ from contrib.performance.httpauth import AuthHandlerAgent
 from contrib.performance.httpclient import StringProducer, readBody
 from contrib.performance.loadtest.subscribe import Periodical
 
-from pycalendar.datetime import PyCalendarDateTime
-from pycalendar.duration import PyCalendarDuration
-from pycalendar.timezone import PyCalendarTimezone
+from pycalendar.datetime import DateTime
+from pycalendar.duration import Duration
+from pycalendar.timezone import Timezone
 
 from twext.internet.adaptendpoint import connect
 from twext.internet.gaiendpoint import GAIEndpoint
+from twisted.internet.ssl import ClientContextFactory
 
 from twisted.internet.defer import Deferred, inlineCallbacks, returnValue, \
     succeed
@@ -42,7 +43,8 @@ from twisted.internet.task import LoopingCall
 from twisted.python.filepath import FilePath
 from twisted.python.log import addObserver, err, msg
 from twisted.python.util import FancyEqMixin
-from twisted.web.client import Agent, ContentDecoderAgent, GzipDecoder
+from twisted.web.client import Agent, ContentDecoderAgent, GzipDecoder, \
+    _DeprecatedToCurrentPolicyForHTTPS
 from twisted.web.http import OK, MULTI_STATUS, CREATED, NO_CONTENT, PRECONDITION_FAILED, MOVED_PERMANENTLY, \
     FORBIDDEN, FOUND
 from twisted.web.http_headers import Headers
@@ -291,6 +293,13 @@ class BaseClient(object):
         raise NotImplementedError("%r does not implement addInvite" % (self.__class__,))
 
 
+    def changeEvent(self, href, calendar):
+        """
+        Called when a profile needs to change an event (no scheduling).
+        """
+        raise NotImplementedError("%r does not implement changeEvent" % (self.__class__,))
+
+
     def deleteEvent(self, href):
         """
         Called when a profile needs to delete an event.
@@ -342,6 +351,16 @@ class _PubSubClientFactory(PubSubClientFactory):
 
 
 
+class WebClientContextFactory(ClientContextFactory):
+    """
+    A web context factory which ignores the hostname and port and does no
+    certificate verification.
+    """
+    def getContext(self, hostname, port):
+        return ClientContextFactory.getContext(self)
+
+
+
 class BaseAppleClient(BaseClient):
     """
     Implementation of common OS X/iOS client behavior.
@@ -414,7 +433,10 @@ class BaseAppleClient(BaseClient):
         self.reactor = reactor
 
         # The server might use gzip encoding
-        agent = Agent(self.reactor)
+        agent = Agent(
+            self.reactor,
+            contextFactory=_DeprecatedToCurrentPolicyForHTTPS(WebClientContextFactory()),
+        )
         agent = ContentDecoderAgent(agent, [("gzip", GzipDecoder)])
         self.agent = AuthHandlerAgent(agent, auth)
 
@@ -472,7 +494,7 @@ class BaseAppleClient(BaseClient):
         # Allow events to go out into the world.
         self.catalog = {
             "eventChanged": Periodical(),
-            }
+        }
 
 
     def _addDefaultHeaders(self, headers):
@@ -550,9 +572,9 @@ class BaseAppleClient(BaseClient):
             return parser.getResults()
 
     _CALENDAR_TYPES = set([
-            caldavxml.calendar,
-            caldavxml.schedule_inbox,
-            ])
+        caldavxml.calendar,
+        caldavxml.schedule_inbox,
+    ])
 
     @inlineCallbacks
     def _propfind(self, url, body, depth='0', allowedStatus=(MULTI_STATUS,), method_label=None):
@@ -729,7 +751,9 @@ class BaseAppleClient(BaseClient):
         for cuaddr in cuaddrs:
             if cuaddr.toString().startswith(u"mailto:"):
                 self.email = cuaddr.toString()
-            elif cuaddr.toString().startswith(u"urn:"):
+            elif cuaddr.toString().startswith(u"urn:x-uid"):
+                self.uuid = cuaddr.toString()
+            elif cuaddr.toString().startswith(u"urn:uuid") and self.uuid is None:
                 self.uuid = cuaddr.toString()
         if self.email is None:
             raise ValueError("Cannot operate without a mail-style principal URL")
@@ -785,12 +809,12 @@ class BaseAppleClient(BaseClient):
 
                     changeTag = davxml.sync_token if self.supportSync else csxml.getctag
                     calendars.append(Calendar(
-                            nodeType.tag,
-                            componentTypes,
-                            textProps.get(davxml.displayname, None),
-                            href,
-                            textProps.get(changeTag, None),
-                            ))
+                        nodeType.tag,
+                        componentTypes,
+                        textProps.get(davxml.displayname, None),
+                        href,
+                        textProps.get(changeTag, None),
+                    ))
                     break
         return calendars
 
@@ -949,6 +973,7 @@ class BaseAppleClient(BaseClient):
         local cache.
         """
 
+        changed.sort()
         while changed:
             batchedHrefs = changed[:self.MULTIGET_BATCH_SIZE]
             changed = changed[self.MULTIGET_BATCH_SIZE:]
@@ -1199,7 +1224,7 @@ class BaseAppleClient(BaseClient):
         connect(GAIEndpoint(self.reactor, host, port), factory)
 
 
-    def _receivedPush(self, inboundID, dataChangedTimestamp):
+    def _receivedPush(self, inboundID, dataChangedTimestamp, priority=5):
         for href, id in self.ampPushKeys.iteritems():
             if inboundID == id:
                 self._checkCalendarsForEvents(href, push=True)
@@ -1213,8 +1238,10 @@ class BaseAppleClient(BaseClient):
         """
         Start monitoring for AMP-based push notifications
         """
-        subscribeToIDs(self.ampPushHost, self.ampPushPort, pushKeys,
-            self._receivedPush, self.reactor)
+        subscribeToIDs(
+            self.ampPushHost, self.ampPushPort, pushKeys,
+            self._receivedPush, self.reactor
+        )
 
 
     @inlineCallbacks
@@ -1411,8 +1438,10 @@ class BaseAppleClient(BaseClient):
             # test account names are all too similar
             # name = attendee.parameterValue('CN').encode("utf-8")
             # prefix = name[:4].lower()
-            prefix = random.choice(["chris", "cyru", "dre", "eric", "morg",
-                "well", "wilfr", "witz"])
+            prefix = random.choice([
+                "chris", "cyru", "dre", "eric", "morg",
+                "well", "wilfr", "witz"
+            ])
 
             email = attendee.value()
             if email.startswith("mailto:"):
@@ -1425,14 +1454,14 @@ class BaseAppleClient(BaseClient):
             yield self._report(
                 self.principalCollection,
                 self._USER_LIST_PRINCIPAL_PROPERTY_SEARCH % {
-                'displayname': prefix,
-                'email': prefix,
-                'firstname': prefix,
-                'lastname': prefix,
+                    'displayname': prefix,
+                    'email': prefix,
+                    'firstname': prefix,
+                    'lastname': prefix,
                 },
                 depth=None,
                 method_label="REPORT{psearch}",
-           )
+            )
 
             # Now learn about the attendee's availability
             yield self.requestAvailability(
@@ -1453,8 +1482,8 @@ class BaseAppleClient(BaseClient):
         component.mainComponent().addProperty(newAttendee)
         okCodes = NO_CONTENT
         headers = Headers({
-                'content-type': ['text/calendar'],
-                })
+            'content-type': ['text/calendar'],
+        })
         if event.scheduleTag is not None:
             headers.addRawHeader('if-schedule-tag-match', event.scheduleTag)
             okCodes = (NO_CONTENT, PRECONDITION_FAILED,)
@@ -1477,7 +1506,7 @@ class BaseAppleClient(BaseClient):
         )
 
         # Finally, re-retrieve the event to update the etag
-        self._updateEvent(response, href)
+        yield self._updateEvent(response, href)
 
 
     @inlineCallbacks
@@ -1501,8 +1530,8 @@ class BaseAppleClient(BaseClient):
     @inlineCallbacks
     def addEvent(self, href, component, invite=False):
         headers = Headers({
-                'content-type': ['text/calendar'],
-                })
+            'content-type': ['text/calendar'],
+        })
 
         attendees = list(component.mainComponent().properties('ATTENDEE'))
         label_suffix = "small"
@@ -1542,6 +1571,29 @@ class BaseAppleClient(BaseClient):
         yield self.addEvent(href, component, invite=True)
 
 
+    @inlineCallbacks
+    def changeEvent(self, href):
+
+        event = self._events[href]
+        component = event.component
+
+        # At last, upload the new event definition
+        response = yield self._request(
+            (NO_CONTENT, PRECONDITION_FAILED,),
+            'PUT',
+            self.root + href.encode('utf-8'),
+            Headers({
+                'content-type': ['text/calendar'],
+                'if-match': [event.etag]
+            }),
+            StringProducer(component.getTextWithTimezones(includeTimezones=True)),
+            method_label="PUT{update}"
+        )
+
+        # Finally, re-retrieve the event to update the etag
+        yield self._updateEvent(response, href)
+
+
     def _localUpdateEvent(self, response, href, component):
         headers = response.headers
         etag = headers.getRawHeaders("etag", [None])[0]
@@ -1609,13 +1661,13 @@ class BaseAppleClient(BaseClient):
             msg("Availability request spanning multiple days (%r to %r), "
                 "dropping the end date." % (start, end))
 
-        start.setTimezone(PyCalendarTimezone(utc=True))
+        start.setTimezone(Timezone.UTCTimezone)
         start.setHHMMSS(0, 0, 0)
-        end = start + PyCalendarDuration(hours=24)
+        end = start + Duration(hours=24)
 
         start = start.getText()
         end = end.getText()
-        now = PyCalendarDateTime.getNowUTC().getText()
+        now = DateTime.getNowUTC().getText()
 
         label_suffix = "small"
         if len(users) > 5:
@@ -1632,14 +1684,15 @@ class BaseAppleClient(BaseClient):
                     'originator': [self.email],
                     'recipient': [u', '.join(users).encode('utf-8')]}),
             StringProducer(self._POST_AVAILABILITY % {
-                    'attendees': attendeeStr,
-                    'summary': (u'Availability for %s' % (', '.join(users),)).encode('utf-8'),
-                    'organizer': self.email.encode('utf-8'),
-                    'vfreebusy-uid': str(uuid4()).upper(),
-                    'event-mask': maskStr,
-                    'start': start,
-                    'end': end,
-                    'now': now}),
+                'attendees': attendeeStr,
+                'summary': (u'Availability for %s' % (', '.join(users),)).encode('utf-8'),
+                'organizer': self.email.encode('utf-8'),
+                'vfreebusy-uid': str(uuid4()).upper(),
+                'event-mask': maskStr,
+                'start': start,
+                'end': end,
+                'now': now,
+            }),
             method_label="POST{fb-%s}" % (label_suffix,),
         )
         body = yield readBody(response)
@@ -1919,7 +1972,7 @@ class iOS_5(BaseAppleClient):
         # the sim can fire a PUT between the PROPFIND and when process the removals.
         old_hrefs = set([calendar.url + child for child in calendar.events.keys()])
 
-        now = PyCalendarDateTime.getNowUTC()
+        now = DateTime.getNowUTC()
         now.setDateOnly(True)
         now.offsetMonth(-1) # 1 month back default
         result = yield self._report(
@@ -1997,7 +2050,7 @@ class RequestLogger(object):
                 url=urlunparse(('', '') + urlparse(event['url'])[2:]),
                 code=event['code'],
                 duration=event['duration'],
-                )
+            )
 
             if event['success']:
                 formatArgs['success'] = self.success
