@@ -1,6 +1,6 @@
 # -*- test-case-name: contrib.performance.loadtest.test_population -*-
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,19 +16,19 @@
 #
 ##
 from __future__ import print_function
+from __future__ import division
 
 """
 Tools for generating a population of CalendarServer users based on
 certain usage parameters.
 """
 
-from __future__ import division
-
 from tempfile import mkdtemp
 from itertools import izip
 from datetime import datetime
 from urllib2 import HTTPBasicAuthHandler
 from urllib2 import HTTPDigestAuthHandler
+from urllib2 import HTTPPasswordMgrWithDefaultRealm
 import collections
 import json
 import os
@@ -86,7 +86,10 @@ class ClientType(object, FancyEqMixin):
         Create a new instance of this client type.
         """
         return self.clientType(
-            reactor, serverAddress, principalPathTemplate, serializationPath, userRecord, authInfo, **self.clientParams)
+            reactor, serverAddress, principalPathTemplate,
+            serializationPath, userRecord, authInfo,
+            **self.clientParams
+        )
 
 
 
@@ -190,15 +193,15 @@ class CalendarClientSimulator(object):
     def _createUser(self, number):
         record = self._records[number]
         user = record.uid
-        authBasic = HTTPBasicAuthHandler()
+        authBasic = HTTPBasicAuthHandler(password_mgr=HTTPPasswordMgrWithDefaultRealm())
         authBasic.add_password(
-            realm="Test Realm",
+            realm=None,
             uri=self.server,
             user=user.encode('utf-8'),
             passwd=record.password.encode('utf-8'))
-        authDigest = HTTPDigestAuthHandler()
+        authDigest = HTTPDigestAuthHandler(passwd=HTTPPasswordMgrWithDefaultRealm())
         authDigest.add_password(
-            realm="Test Realm",
+            realm=None,
             uri=self.server,
             user=user.encode('utf-8'),
             passwd=record.password.encode('utf-8'))
@@ -251,8 +254,12 @@ class CalendarClientSimulator(object):
                 for profileType in clientType.profileTypes:
                     profile = profileType(reactor, self, client, number)
                     if profile.enabled:
-                        d = profile.run()
-                        d.addErrback(self._profileFailure, profileType, reactor)
+                        d = profile.initialize()
+                        def _run(result):
+                            d2 = profile.run()
+                            d2.addErrback(self._profileFailure, profileType, reactor)
+                            return d2
+                        d.addCallback(_run)
 
         # XXX this status message is prone to be slightly inaccurate, but isn't
         # really used by much anyway.
@@ -275,7 +282,7 @@ class CalendarClientSimulator(object):
         if not self._stopped:
             where = self._dumpLogs(reactor, reason)
             err(reason, "Client stopped with error; recent traffic in %r" % (
-                    where.path,))
+                where.path,))
             if not isinstance(reason, Failure):
                 reason = Failure(reason)
             msg(type="client-failure", reason="%s: %s" % (reason.type, reason.value,))
@@ -285,7 +292,7 @@ class CalendarClientSimulator(object):
         if not self._stopped:
             where = self._dumpLogs(reactor, reason)
             err(reason, "Profile stopped with error; recent traffic in %r" % (
-                    where.path,))
+                where.path,))
 
 
     def _simFailure(self, reason, reactor):
@@ -556,12 +563,15 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
         output.write("* Details\n")
 
         self.printHeader(output, [
-                (label, width)
-                for (label, width, _ignore_fmt)
-                in self._fields])
-        self.printData(output,
+            (label, width)
+            for (label, width, _ignore_fmt)
+            in self._fields
+        ])
+        self.printData(
+            output,
             [fmt for (label, width, fmt) in self._fields],
-            sorted(self._perMethodTimes.items()))
+            sorted(self._perMethodTimes.items())
+        )
 
     _FAILED_REASON = "Greater than %(cutoff)g%% %(method)s failed"
 
@@ -586,7 +596,7 @@ class ReportStatistics(StatisticsBase, SummarizingMixin):
 
             checks = [
                 (failures, self._fail_cut_off, self._FAILED_REASON),
-                ]
+            ]
 
             for ctr, item in enumerate(self._thresholds):
                 threshold, fail_at = item
