@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -42,8 +42,8 @@ from contrib.performance.stats import LogNormalDistribution, RecurrenceDistribut
 from contrib.performance.loadtest.logger import SummarizingMixin
 from contrib.performance.loadtest.ical import IncorrectResponseCode
 
-from pycalendar.datetime import PyCalendarDateTime
-from pycalendar.duration import PyCalendarDuration
+from pycalendar.datetime import DateTime
+from pycalendar.duration import Duration
 
 class ProfileBase(object):
     """
@@ -64,6 +64,15 @@ class ProfileBase(object):
         pass
 
 
+    def initialize(self):
+        """
+        Called before the profile runs for real. Can be used to initialize client state.
+
+        @return: a L{Deferred} that fires when initialization is done
+        """
+        return succeed(None)
+
+
     def _calendarsOfType(self, calendarType, componentType):
         return [
             cal
@@ -205,8 +214,9 @@ class Inviter(ProfileBase):
             except IndexError:
                 continue
             cuaddr = u'mailto:%s' % (record.email,)
-            uuid = u'urn:uuid:%s' % (record.uid,)
-            if cuaddr not in invitees and uuid not in invitees:
+            uuidx = u'urn:x-uid:%s' % (record.guid,)
+            uuid = u'urn:uuid:%s' % (record.guid,)
+            if cuaddr not in invitees and uuidx not in invitees and uuid not in invitees:
                 break
         else:
             return fail(CannotAddAttendee("Can't find uninvited user to invite."))
@@ -215,11 +225,11 @@ class Inviter(ProfileBase):
             name=u'ATTENDEE',
             value=cuaddr.encode("utf-8"),
             params={
-            'CN': record.commonName,
-            'CUTYPE': 'INDIVIDUAL',
-            'PARTSTAT': 'NEEDS-ACTION',
-            'ROLE': 'REQ-PARTICIPANT',
-            'RSVP': 'TRUE',
+                'CN': record.commonName,
+                'CUTYPE': 'INDIVIDUAL',
+                'PARTSTAT': 'NEEDS-ACTION',
+                'ROLE': 'REQ-PARTICIPANT',
+                'RSVP': 'TRUE',
             },
         )
 
@@ -367,11 +377,11 @@ END:VCALENDAR
             name=u'ATTENDEE',
             value=cuaddr.encode("utf-8"),
             params={
-            'CN': record.commonName,
-            'CUTYPE': 'INDIVIDUAL',
-            'PARTSTAT': 'NEEDS-ACTION',
-            'ROLE': 'REQ-PARTICIPANT',
-            'RSVP': 'TRUE',
+                'CN': record.commonName,
+                'CUTYPE': 'INDIVIDUAL',
+                'PARTSTAT': 'NEEDS-ACTION',
+                'ROLE': 'REQ-PARTICIPANT',
+                'RSVP': 'TRUE',
             },
         )
 
@@ -407,9 +417,9 @@ END:VCALENDAR
             vevent = vcalendar.mainComponent()
             uid = str(uuid4())
             dtstart = self._eventStartDistribution.sample()
-            dtend = dtstart + PyCalendarDuration(seconds=self._eventDurationDistribution.sample())
-            vevent.replaceProperty(Property("CREATED", PyCalendarDateTime.getNowUTC()))
-            vevent.replaceProperty(Property("DTSTAMP", PyCalendarDateTime.getNowUTC()))
+            dtend = dtstart + Duration(seconds=self._eventDurationDistribution.sample())
+            vevent.replaceProperty(Property("CREATED", DateTime.getNowUTC()))
+            vevent.replaceProperty(Property("DTSTAMP", DateTime.getNowUTC()))
             vevent.replaceProperty(Property("DTSTART", dtstart))
             vevent.replaceProperty(Property("DTEND", dtend))
             vevent.replaceProperty(Property("UID", uid))
@@ -650,9 +660,9 @@ END:VCALENDAR
             vevent = vcalendar.mainComponent()
             uid = str(uuid4())
             dtstart = self._eventStartDistribution.sample()
-            dtend = dtstart + PyCalendarDuration(seconds=self._eventDurationDistribution.sample())
-            vevent.replaceProperty(Property("CREATED", PyCalendarDateTime.getNowUTC()))
-            vevent.replaceProperty(Property("DTSTAMP", PyCalendarDateTime.getNowUTC()))
+            dtend = dtstart + Duration(seconds=self._eventDurationDistribution.sample())
+            vevent.replaceProperty(Property("CREATED", DateTime.getNowUTC()))
+            vevent.replaceProperty(Property("DTSTAMP", DateTime.getNowUTC()))
             vevent.replaceProperty(Property("DTSTART", dtstart))
             vevent.replaceProperty(Property("DTEND", dtend))
             vevent.replaceProperty(Property("UID", uid))
@@ -667,6 +677,132 @@ END:VCALENDAR
 
 
 
+class EventUpdater(ProfileBase):
+    """
+    A Calendar user who creates a new event, and then updates its alarm.
+    """
+    _eventTemplate = Component.fromString("""\
+BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//Apple Inc.//iCal 4.0.3//EN
+CALSCALE:GREGORIAN
+BEGIN:VEVENT
+CREATED:20101018T155431Z
+UID:C98AD237-55AD-4F7D-9009-0D355D835822
+DTEND;TZID=America/New_York:20101021T130000
+TRANSP:OPAQUE
+SUMMARY:Simple event
+DTSTART;TZID=America/New_York:20101021T120000
+DTSTAMP:20101018T155438Z
+SEQUENCE:2
+BEGIN:VALARM
+X-WR-ALARMUID:D9D1AC84-F629-4B9D-9B6B-4A6CA9A11FEF
+UID:D9D1AC84-F629-4B9D-9B6B-4A6CA9A11FEF
+DESCRIPTION:Event reminder
+TRIGGER:-PT8M
+ACTION:DISPLAY
+END:VALARM
+END:VEVENT
+END:VCALENDAR
+""".replace("\n", "\r\n"))
+
+    def setParameters(
+        self,
+        enabled=True,
+        interval=25,
+        eventStartDistribution=NearFutureDistribution(),
+        eventDurationDistribution=UniformDiscreteDistribution([
+            15 * 60, 30 * 60,
+            45 * 60, 60 * 60,
+            120 * 60
+        ]),
+        recurrenceDistribution=RecurrenceDistribution(False),
+    ):
+        self.enabled = enabled
+        self._interval = interval
+        self._eventStartDistribution = eventStartDistribution
+        self._eventDurationDistribution = eventDurationDistribution
+        self._recurrenceDistribution = recurrenceDistribution
+
+
+    def initialize(self):
+        """
+        Called before the profile runs for real. Can be used to initialize client state.
+
+        @return: a L{Deferred} that fires when initialization is done
+        """
+        return self._initEvent()
+
+
+    def run(self):
+        self._call = LoopingCall(self._updateEvent)
+        self._call.clock = self._reactor
+        return self._call.start(self._interval)
+
+
+    def _initEvent(self):
+        if not self._client.started:
+            return succeed(None)
+
+        # If it already exists, don't re-create
+        calendar = self._calendarsOfType(caldavxml.calendar, "VEVENT")[0]
+        if calendar.events:
+            events = [event for event in calendar.events.values() if event.url.endswith("event_to_update.ics")]
+            if events:
+                return succeed(None)
+
+        # Copy the template event and fill in some of its fields
+        # to make a new event to create on the calendar.
+        vcalendar = self._eventTemplate.duplicate()
+        vevent = vcalendar.mainComponent()
+        uid = str(uuid4())
+        dtstart = self._eventStartDistribution.sample()
+        dtend = dtstart + Duration(seconds=self._eventDurationDistribution.sample())
+        vevent.replaceProperty(Property("CREATED", DateTime.getNowUTC()))
+        vevent.replaceProperty(Property("DTSTAMP", DateTime.getNowUTC()))
+        vevent.replaceProperty(Property("DTSTART", dtstart))
+        vevent.replaceProperty(Property("DTEND", dtend))
+        vevent.replaceProperty(Property("UID", uid))
+
+        rrule = self._recurrenceDistribution.sample()
+        if rrule is not None:
+            vevent.addProperty(Property(None, None, None, pycalendar=rrule))
+
+        href = '%s%s' % (calendar.url, "event_to_update.ics")
+        d = self._client.addEvent(href, vcalendar)
+        return self._newOperation("create", d)
+
+
+    def _updateEvent(self):
+        """
+        Try to add a new attendee to an event, or perhaps remove an
+        existing attendee from an event.
+
+        @return: C{None} if there are no events to play with,
+            otherwise a L{Deferred} which fires when the attendee
+            change has been made.
+        """
+
+        if not self._client.started:
+            return succeed(None)
+
+        # If it does not exist, try to create it
+        calendar = self._calendarsOfType(caldavxml.calendar, "VEVENT")[0]
+        if not calendar.events:
+            return self._initEvent()
+        events = [event for event in calendar.events.values() if event.url.endswith("event_to_update.ics")]
+        if not events:
+            return self._initEvent()
+        event = events[0]
+
+        # Add/update the ACKNOWLEDGED property
+        component = event.component.mainComponent()
+        component.replaceProperty(Property("ACKNOWLEDGED", DateTime.getNowUTC()))
+        d = self._client.changeEvent(event.url)
+        return self._newOperation("update", d)
+
+
+
 class Tasker(ProfileBase):
     """
     A Calendar user who creates new tasks.
@@ -719,8 +855,8 @@ END:VCALENDAR
             vtodo = vcalendar.mainComponent()
             uid = str(uuid4())
             due = self._taskStartDistribution.sample()
-            vtodo.replaceProperty(Property("CREATED", PyCalendarDateTime.getNowUTC()))
-            vtodo.replaceProperty(Property("DTSTAMP", PyCalendarDateTime.getNowUTC()))
+            vtodo.replaceProperty(Property("CREATED", DateTime.getNowUTC()))
+            vtodo.replaceProperty(Property("DTSTAMP", DateTime.getNowUTC()))
             vtodo.replaceProperty(Property("DUE", due))
             vtodo.replaceProperty(Property("UID", uid))
 
@@ -740,7 +876,7 @@ class OperationLogger(SummarizingMixin):
         u"start" : u"%(user)s - - - - - - - - - - - %(label)8s BEGIN %(lag)s",
         u"end"   : u"%(user)s - - - - - - - - - - - %(label)8s END [%(duration)5.2f s]",
         u"failed": u"%(user)s x x x x x x x x x x x %(label)8s FAILED %(reason)s",
-        }
+    }
 
     lagFormat = u'{lag %5.2f ms}'
 
@@ -830,12 +966,14 @@ class OperationLogger(SummarizingMixin):
     def report(self, output):
         output.write("\n")
         self.printHeader(output, [
-                (label, width)
-                for (label, width, _ignore_fmt)
-                in self._fields])
-        self.printData(output,
+            (label, width)
+            for (label, width, _ignore_fmt) in self._fields
+        ])
+        self.printData(
+            output,
             [fmt for (label, width, fmt) in self._fields],
-            sorted(self._perOperationTimes.items()))
+            sorted(self._perOperationTimes.items())
+        )
 
     _LATENCY_REASON = "Median %(operation)s scheduling lag greater than %(cutoff)sms"
     _FAILED_REASON = "Greater than %(cutoff).0f%% %(operation)s failed"
@@ -846,12 +984,12 @@ class OperationLogger(SummarizingMixin):
         for operation, lags in self._perOperationLags.iteritems():
             if median(lags) > self._lag_cut_off:
                 reasons.append(self._LATENCY_REASON % dict(
-                        operation=operation.upper(), cutoff=self._lag_cut_off * 1000))
+                    operation=operation.upper(), cutoff=self._lag_cut_off * 1000))
 
         for operation, times in self._perOperationTimes.iteritems():
             failures = len([success for (success, _ignore_duration) in times if not success])
             if failures * 100.0 / len(times) > self._fail_cut_off:
                 reasons.append(self._FAILED_REASON % dict(
-                        operation=operation.upper(), cutoff=self._fail_cut_off))
+                    operation=operation.upper(), cutoff=self._fail_cut_off))
 
         return reasons
