@@ -1,6 +1,6 @@
 # -*- test-case-name: contrib.performance.loadtest.test_sim -*-
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -43,6 +43,8 @@ from twisted.internet.defer import gatherResults
 from twisted.internet.defer import inlineCallbacks
 from twisted.internet.protocol import ProcessProtocol
 
+from twisted.web.server import Site
+
 from contrib.performance.loadtest.ical import OS_X_10_6
 from contrib.performance.loadtest.profiles import Eventer, Inviter, Accepter
 from contrib.performance.loadtest.population import (
@@ -51,12 +53,14 @@ from contrib.performance.loadtest.population import (
 from contrib.performance.loadtest.webadmin import LoadSimAdminResource
 
 
+
 class _DirectoryRecord(object):
-    def __init__(self, uid, password, commonName, email):
+    def __init__(self, uid, password, commonName, email, guid):
         self.uid = uid
         self.password = password
         self.commonName = commonName
         self.email = email
+        self.guid = guid
 
 
 
@@ -65,15 +69,19 @@ def safeDivision(value, total, factor=1):
 
 
 
-def generateRecords(count, uidPattern="user%d", passwordPattern="user%d",
-    namePattern="User %d", emailPattern="user%d@example.com"):
+def generateRecords(
+    count, uidPattern="user%d", passwordPattern="user%d",
+    namePattern="User %d", emailPattern="user%d@example.com",
+    guidPattern="user%d"
+):
     for i in xrange(count):
         i += 1
         uid = uidPattern % (i,)
         password = passwordPattern % (i,)
         name = namePattern % (i,)
         email = emailPattern % (i,)
-        yield _DirectoryRecord(uid, password, name, email)
+        guid = guidPattern % (i,)
+        yield _DirectoryRecord(uid, password, name, email, guid)
 
 
 
@@ -89,14 +97,6 @@ def recordsFromCSVFile(path):
 
 
 
-def recordsFromCount(count, uid=u"user%02d", password=u"user%02d",
-                     commonName=u"User %02d", email=u"user%02d@example.com"):
-    for i in range(1, count + 1):
-        yield _DirectoryRecord(uid % i, password % i,
-                               commonName % i, email % i)
-
-
-
 class LagTrackingReactor(object):
     """
     This reactor wraps another reactor and proxies all attribute
@@ -142,7 +142,7 @@ class SimOptions(Options):
         ("clients", None, _defaultClients,
          "Configuration plist file name from which to read client parameters.",
          FilePath),
-        ]
+    ]
 
 
     def opt_logfile(self, filename):
@@ -185,7 +185,7 @@ class SimOptions(Options):
             configFile = self['config'].open()
         except IOError, e:
             raise UsageError("--config %s: %s" % (
-                    self['config'].path, e.strerror))
+                self['config'].path, e.strerror))
         try:
             try:
                 self.config = readPlist(configFile)
@@ -198,7 +198,7 @@ class SimOptions(Options):
             clientFile = self['clients'].open()
         except IOError, e:
             raise UsageError("--clients %s: %s" % (
-                    self['clients'].path, e.strerror))
+                self['clients'].path, e.strerror))
         try:
             try:
                 client_config = readPlist(clientFile)
@@ -214,7 +214,6 @@ class SimOptions(Options):
 Arrival = namedtuple('Arrival', 'factory parameters')
 
 
-from twisted.web import server
 
 class LoadSimulator(object):
     """
@@ -279,16 +278,10 @@ class LoadSimulator(object):
             workerID = config.get("workerID", 0)
             workerCount = config.get("workerCount", 1)
             configTemplate = None
-            server = 'http://127.0.0.1:8008'
-            principalPathTemplate = "/principals/users/%s/"
+            server = config.get('server', 'http://127.0.0.1:8008')
+            principalPathTemplate = config.get('principalPathTemplate', '/principals/users/%s/')
             serializationPath = None
 
-            if 'server' in config:
-                server = config['server']
-
-            if 'principalPathTemplate' in config:
-                principalPathTemplate = config['principalPathTemplate']
-
             if 'clientDataSerialization' in config:
                 serializationPath = config['clientDataSerialization']['Path']
                 if not config['clientDataSerialization']['UseOldData']:
@@ -318,10 +311,12 @@ class LoadSimulator(object):
                         ClientType(
                             namedAny(clientConfig["software"]),
                             cls._convertParams(clientConfig["params"]),
-                            [ProfileType(
+                            [
+                                ProfileType(
                                     namedAny(profile["class"]),
-                                    cls._convertParams(profile["params"]))
-                             for profile in clientConfig["profiles"]]))
+                                    cls._convertParams(profile["params"])
+                                ) for profile in clientConfig["profiles"]
+                            ]))
             if not parameters.clients:
                 parameters.addClient(1,
                                      ClientType(OS_X_10_6, {},
@@ -461,7 +456,7 @@ class LoadSimulator(object):
         if self.runtime is not None:
             self.reactor.callLater(self.runtime, self.stopAndReport)
         if self.webadminPort:
-            self.reactor.listenTCP(self.webadminPort, server.Site(LoadSimAdminResource(self)))
+            self.reactor.listenTCP(self.webadminPort, Site(LoadSimAdminResource(self)))
         self.reactor.run()
 
 
@@ -487,7 +482,7 @@ class LoadSimulator(object):
             _ignore_scheme, hostname, _ignore_path, _ignore_query, _ignore_fragment = urlsplit(self.serverStats["server"])
             data = self.readStatsSock((hostname.split(":")[0], self.serverStats["Port"],), True)
             if "Failed" not in data:
-                data = data["5 Minutes"]
+                data = data["stats"]["5m"] if "stats" in data else data["5 Minutes"]
                 result = (
                     safeDivision(float(data["requests"]), 5 * 60),
                     safeDivision(data["t"], data["requests"]),
@@ -510,8 +505,9 @@ class LoadSimulator(object):
         try:
             s = socket.socket(socket.AF_INET if useTCP else socket.AF_UNIX, socket.SOCK_STREAM)
             s.connect(sockname)
+            s.sendall('["stats"]' + "\r\n")
             data = ""
-            while True:
+            while not data.endswith("\n"):
                 d = s.recv(1024)
                 if d:
                     data += d
