@@ -6,13 +6,13 @@ class Subscription(object):
 
 
     def cancel(self):
-        self.peridical.subscriptions.remove(self)
+        self.periodical.subscriptions.remove(self)
 
 
     def issue(self, issue):
         self.subscriber(issue)
 
-        
+
 
 class Periodical(object):
     def __init__(self):
