@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -51,11 +51,11 @@ DTEND;TZID=America/Los_Angeles:20100730T111508
 
 calendarTail = """\
 ATTENDEE;CN=User 02;CUTYPE=INDIVIDUAL;EMAIL=user02@example.com;PARTSTAT=NE
- EDS-ACTION;ROLE=REQ-PARTICIPANT;RSVP=TRUE;SCHEDULE-STATUS=1.2:urn:uuid:use
+ EDS-ACTION;ROLE=REQ-PARTICIPANT;RSVP=TRUE;SCHEDULE-STATUS=1.2:urn:x-uid:use
  r02
 CREATED:20100729T193912Z
 DTSTAMP:20100729T195557Z
-ORGANIZER;CN=User 03;EMAIL=user03@example.com:urn:uuid:user03
+ORGANIZER;CN=User 03;EMAIL=user03@example.com:urn:x-uid:user03
 SEQUENCE:1
 SUMMARY:STUFF IS THINGS
 TRANSP:OPAQUE
