@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -78,7 +78,7 @@ def analyze(fpath, noweekends, startDate=None, endDate=None, title=None):
 
                     if reqs <= 80:
                         data.append((dtstamp, reqs, resp, lqnon, cpu))
-                    #print("%s %d %d %d %d" % (dtstamp, reqs, resp, lqnon, cpu))
+                    # print("%s %d %d %d %d" % (dtstamp, reqs, resp, lqnon, cpu))
             except StopIteration:
                 break
 
@@ -106,7 +106,7 @@ def plotListenQBands(data, first, last, xlim, ylim):
     y2 = []
     x3 = []
     y3 = []
-    for datetime, reqs, resp, lq, _ignore_cpu in data:
+    for _ignore_dt, reqs, resp, lq, _ignore_cpu in data:
         if lq == 0:
             x1.append(reqs)
             y1.append(resp)
@@ -120,8 +120,10 @@ def plotListenQBands(data, first, last, xlim, ylim):
     plt.plot(x1, y1, "b+", x2, y2, "g+", x3, y3, "y+")
 
     if first:
-        plt.legend(('ListenQ at zero', 'ListenQ < 50', 'ListenQ >= 50'),
-               'upper left', shadow=True, fancybox=True)
+        plt.legend(
+            ('ListenQ at zero', 'ListenQ < 50', 'ListenQ >= 50'),
+            'upper left', shadow=True, fancybox=True
+        )
     if last:
         plt.xlabel("Requests/second")
     plt.ylabel("Av. Response Time (ms)")
@@ -134,7 +136,7 @@ def plotCPUBands(data, first, last, xlim, ylim):
 
     x = [[], [], [], []]
     y = [[], [], [], []]
-    for datetime, reqs, resp, _ignore_lq, cpu in data:
+    for _ignore_dt, reqs, resp, _ignore_lq, cpu in data:
         if cpu > 75:
             x[0].append(reqs)
             y[0].append(resp)
@@ -156,8 +158,10 @@ def plotCPUBands(data, first, last, xlim, ylim):
     )
 
     if first:
-        plt.legend(('CPU < 1/4', 'CPU < 1/2', 'CPU < 3/4', "CPU High"),
-               'upper left', shadow=True, fancybox=True)
+        plt.legend(
+            ('CPU < 1/4', 'CPU < 1/2', 'CPU < 3/4', "CPU High"),
+            'upper left', shadow=True, fancybox=True
+        )
     if last:
         plt.xlabel("Requests/second")
     plt.ylabel("Av. Response Time (ms)")
