@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 ##
-# Copyright (c) 2012-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,6 +25,7 @@ import socket
 import sys
 import tables
 import time
+import errno
 
 """
 This tool reads data from the server's statistics socket and prints a summary.
@@ -39,23 +40,30 @@ def readSock(sockname, useTCP):
     try:
         s = socket.socket(socket.AF_INET if useTCP else socket.AF_UNIX, socket.SOCK_STREAM)
         s.connect(sockname)
+        s.sendall(json.dumps(["stats"]) + "\r\n")
+        s.setblocking(0)
         data = ""
-        while True:
-            d = s.recv(1024)
+        while not data.endswith("\n"):
+            try:
+                d = s.recv(1024)
+            except socket.error as se:
+                if se.args[0] != errno.EWOULDBLOCK:
+                    raise
+                continue
             if d:
                 data += d
             else:
                 break
         s.close()
-        data = json.loads(data)
-    except socket.error:
-        data = {"Failed": "Unable to read statistics from server: %s" % (sockname,)}
-    data["Server"] = sockname
+        data = json.loads(data)["stats"]
+    except socket.error as e:
+        data = {"Failed": "Unable to read statistics from server: %s %s" % (sockname, e)}
+    data["server"] = sockname
     return data
 
 
 
-def printStats(stats, multimode, showMethods, topUsers, showAgents):
+def printStats(stats, multimode, showMethods, topUsers, showAgents, dumpFile):
     if len(stats) == 1:
         if "Failed" in stats[0]:
             printFailedStats(stats[0]["Failed"])
@@ -74,18 +82,18 @@ def printStats(stats, multimode, showMethods, topUsers, showAgents):
 def printStat(stats, index, showMethods, topUsers, showAgents):
 
     print("- " * 40)
-    print("Server: %s" % (stats["Server"],))
+    print("Server: %s" % (stats["server"],))
     print(datetime.datetime.now().strftime("%Y/%m/%d %H:%M:%S"))
-    print("Service Uptime: %s" % (datetime.timedelta(seconds=(int(time.time() - stats["System"]["start time"]))),))
-    if stats["System"]["cpu count"] > 0:
+    print("Service Uptime: %s" % (datetime.timedelta(seconds=(int(time.time() - stats["system"]["start time"]))),))
+    if stats["system"]["cpu count"] > 0:
         print("Current CPU: %.1f%% (%d CPUs)" % (
-            stats["System"]["cpu use"],
-            stats["System"]["cpu count"],
+            stats["system"]["cpu use"],
+            stats["system"]["cpu count"],
         ))
         print("Current Memory Used: %d bytes (%.1f GB) (%.1f%% of total)" % (
-            stats["System"]["memory used"],
-            stats["System"]["memory used"] / (1024.0 * 1024 * 1024),
-            stats["System"]["memory percent"],
+            stats["system"]["memory used"],
+            stats["system"]["memory used"] / (1024.0 * 1024 * 1024),
+            stats["system"]["memory percent"],
         ))
     else:
         print("Current CPU: Unavailable")
@@ -112,7 +120,7 @@ def printMultipleStats(stats, multimode, showMethods, topUsers, showAgents):
     times = []
     for stat in stats:
         try:
-            t = str(datetime.timedelta(seconds=int(time.time() - stat["System"]["start time"])))
+            t = str(datetime.timedelta(seconds=int(time.time() - stat["system"]["start time"])))
         except KeyError:
             t = "-"
         times.append(t)
@@ -120,9 +128,9 @@ def printMultipleStats(stats, multimode, showMethods, topUsers, showAgents):
     cpus = []
     memories = []
     for stat in stats:
-        if stat["System"]["cpu count"] > 0:
-            cpus.append(stat["System"]["cpu use"])
-            memories.append(stat["System"]["memory percent"])
+        if stat["system"]["cpu count"] > 0:
+            cpus.append(stat["system"]["cpu use"])
+            memories.append(stat["system"]["memory percent"])
         else:
             cpus.append(-1)
             memories.append(-1)
@@ -139,7 +147,7 @@ def printMultipleStats(stats, multimode, showMethods, topUsers, showAgents):
 
 
 def serverLabels(stats):
-    servers = [stat["Server"] for stat in stats]
+    servers = [stat["server"] for stat in stats]
     if isinstance(servers[0], tuple):
         hosts = set([item[0] for item in servers])
         ports = set([item[1] for item in servers])
@@ -180,7 +188,7 @@ def printRequestSummary(stats):
         ("", "", "per second", "(ms)", "no write(ms)", "(ms)", "Average", "Average", ""),
     )
     table.setDefaultColumnFormats(
-       (
+        (
             tables.Table.ColumnFormat("%s", tables.Table.ColumnFormat.LEFT_JUSTIFY),
             tables.Table.ColumnFormat("%d", tables.Table.ColumnFormat.RIGHT_JUSTIFY),
             tables.Table.ColumnFormat("%.1f", tables.Table.ColumnFormat.RIGHT_JUSTIFY),
@@ -193,7 +201,7 @@ def printRequestSummary(stats):
         )
     )
 
-    for key, seconds in (("Current", 60,), ("1 Minute", 60,), ("5 Minutes", 5 * 60,), ("1 Hour", 60 * 60,),):
+    for key, seconds in (("current", 60,), ("1m", 60,), ("5m", 5 * 60,), ("1h", 60 * 60,),):
 
         stat = stats[key]
         table.addRow((
@@ -227,7 +235,7 @@ def printMultiRequestSummary(stats, cpus, memories, times, labels, index):
     )
     max_column = 5
     table.setDefaultColumnFormats(
-       (
+        (
             tables.Table.ColumnFormat("%s", tables.Table.ColumnFormat.LEFT_JUSTIFY),
             tables.Table.ColumnFormat("%d", tables.Table.ColumnFormat.RIGHT_JUSTIFY),
             tables.Table.ColumnFormat("%.1f", tables.Table.ColumnFormat.RIGHT_JUSTIFY),
@@ -287,7 +295,7 @@ def printHistogramSummary(stat, index):
         ("", "<10ms", "10ms<->100ms", "100ms<->1s", "1s<->10s", "10s<->30s", "30s<->60s", ">60s", "Over 1s", "Over 10s"),
     )
     table.setDefaultColumnFormats(
-       (
+        (
             tables.Table.ColumnFormat("%s", tables.Table.ColumnFormat.LEFT_JUSTIFY),
             tables.Table.ColumnFormat("%d (%.1f%%)", tables.Table.ColumnFormat.RIGHT_JUSTIFY),
             tables.Table.ColumnFormat("%d (%.1f%%)", tables.Table.ColumnFormat.RIGHT_JUSTIFY),
@@ -350,7 +358,7 @@ def printMethodCounts(stat):
         ("", "", "", "(ms)", "", ""),
     )
     table.setDefaultColumnFormats(
-       (
+        (
             tables.Table.ColumnFormat("%s", tables.Table.ColumnFormat.LEFT_JUSTIFY),
             tables.Table.ColumnFormat("%d", tables.Table.ColumnFormat.RIGHT_JUSTIFY),
             tables.Table.ColumnFormat("%.1f%%", tables.Table.ColumnFormat.RIGHT_JUSTIFY),
@@ -445,7 +453,7 @@ def printAgentCounts(stat):
         ("User-Agent", "Total", "Percentage"),
     )
     table.setDefaultColumnFormats(
-       (
+        (
             tables.Table.ColumnFormat("%s", tables.Table.ColumnFormat.LEFT_JUSTIFY),
             tables.Table.ColumnFormat("%d", tables.Table.ColumnFormat.RIGHT_JUSTIFY),
             tables.Table.ColumnFormat("%.1f%%", tables.Table.ColumnFormat.RIGHT_JUSTIFY),
@@ -493,6 +501,7 @@ Options:
     --methods     Include details about HTTP method usage
     --users N     Include details about top N users
     --agents      Include details about HTTP User-Agent usage
+    --dump FILE   File to dump JSON data to
 
 Description:
     This utility will print a summary of statistics read from a
@@ -514,11 +523,12 @@ if __name__ == '__main__':
     showMethods = False
     topUsers = 0
     showAgents = False
+    dumpFile = None
 
-    multimodes = (("Current", 60,), ("1 Minute", 60,), ("5 Minutes", 5 * 60,), ("1 Hour", 60 * 60,),)
+    multimodes = (("current", 60,), ("1m", 60,), ("5m", 5 * 60,), ("1h", 60 * 60,),)
     multimode = multimodes[2]
 
-    options, args = getopt.getopt(sys.argv[1:], "hs:t:", ["tcp=", "0", "1", "5", "60", "methods", "users=", "agents"])
+    options, args = getopt.getopt(sys.argv[1:], "hs:t:", ["tcp=", "0", "1", "5", "60", "methods", "users=", "agents", "dump="])
 
     for option, value in options:
         if option == "-h":
@@ -544,7 +554,16 @@ if __name__ == '__main__':
             topUsers = int(value)
         elif option == "--agents":
             showAgents = True
+        elif option == "--dump":
+            dumpFile = value
 
     while True:
-        printStats([readSock(server, useTCP) for server in servers], multimode, showMethods, topUsers, showAgents)
+        stats = [readSock(server, useTCP) for server in servers]
+        printStats(stats, multimode, showMethods, topUsers, showAgents, dumpFile)
+        if dumpFile is not None:
+            stats = dict([("{}:{}".format(*servers[ctr]), stat) for ctr, stat in enumerate(stats)])
+            stats["timestamp"] = datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
+            with open(dumpFile, "a") as f:
+                f.write(json.dumps(stats))
+                f.write("\n")
         time.sleep(delay)
