@@ -1,6 +1,6 @@
 ##
 # Copyright (c) 2001-2004 Twisted Matrix Laboratories.
-# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,7 +22,7 @@ This implements two authentication modes:
 
     1. An alternative to password based BASIC authentication in which the BASIC credentials are
         verified against Kerberos.
-   
+
     2. The NEGOTIATE mechanism (as defined in http://www.ietf.org/rfc/rfc4559.txt)
         that implements full GSSAPI authentication.
 """
@@ -42,12 +42,13 @@ import kerberos
 
 from twisted.cred import checkers, credentials, error
 from twisted.internet.defer import succeed
-from twext.web2 import responsecode
-from twext.web2.auth.interfaces import ICredentialFactory
-from twext.web2.dav.auth import IPrincipalCredentials
+from txweb2 import responsecode
+from txweb2.auth.interfaces import ICredentialFactory
+from txweb2.dav.auth import IPrincipalCredentials
 
 from twext.python.log import Logger
 
+
 class KerberosCredentialFactoryBase(object):
     """
     Code common to Kerberos-based credential factories.
@@ -56,32 +57,33 @@ class KerberosCredentialFactoryBase(object):
 
     implements(ICredentialFactory)
 
-    def __init__(self, principal=None, type=None, hostname=None):
+    def __init__(self, principal=None, serviceType=None, hostname=None):
         """
-        
+
         @param principal:  full Kerberos principal (e.g., 'HTTP/server.example.com@EXAMPLE.COM'). If C{None}
             then the type and hostname arguments are used instead.
-        @type service:     str
-        @param type:       service type for Kerberos (e.g., 'HTTP'). Must be C{None} if principal used.
-        @type type:        str
+        @type principal:     str
+        @param serviceType:       service type for Kerberos (e.g., 'HTTP'). Must be C{None} if principal used.
+        @type serviceType:        str
         @param hostname:   hostname for this server. Must be C{None} if principal used.
         @type hostname:    str
         """
 
         # Only certain combinations of arguments allowed
-        assert (principal and not type and not hostname) or (not principal and type and hostname)
+        assert (principal and not serviceType and not hostname) or (not principal and serviceType and hostname)
 
         if not principal:
             # Look up the Kerberos principal given the service type and hostname, and extract
             # the realm and a service principal value for later use.
             try:
-                principal = kerberos.getServerPrincipalDetails(type, hostname)
+                principal = kerberos.getServerPrincipalDetails(serviceType, hostname)
             except kerberos.KrbError, ex:
                 self.log.error("getServerPrincipalDetails: %s" % (ex[0],))
                 raise ValueError('Authentication System Failure: %s' % (ex[0],))
 
         self.service, self.realm = self._splitPrincipal(principal)
 
+
     def _splitPrincipal(self, principal):
 
         try:
@@ -93,12 +95,14 @@ class KerberosCredentialFactoryBase(object):
         except IndexError:
             self.log.error("Invalid Kerberos principal: %s" % (principal,))
             raise ValueError('Authentication System Failure: Invalid Kerberos principal: %s' % (principal,))
-                
+
         service = "%s@%s" % (servicetype, service,)
         realm = realm
-        
+
         return (service, realm,)
-        
+
+
+
 class BasicKerberosCredentials(credentials.UsernamePassword):
     """
     A set of user/password credentials that checks itself against Kerberos.
@@ -106,7 +110,7 @@ class BasicKerberosCredentials(credentials.UsernamePassword):
 
     def __init__(self, username, password, service, realm):
         """
-        
+
         @param username:   user name of user to authenticate
         @type username:    str
         @param password:   password for user being authenticated
@@ -117,11 +121,13 @@ class BasicKerberosCredentials(credentials.UsernamePassword):
         @type hostname:    str
         """
         credentials.UsernamePassword.__init__(self, username, password)
-        
+
         # Convert Kerberos principal spec into service and realm
         self.service = service
         self.default_realm = realm
-        
+
+
+
 class BasicKerberosCredentialFactory(KerberosCredentialFactoryBase):
     """
     Authorizer for insecure Basic (base64-encoded plaintext) authentication.
@@ -132,24 +138,26 @@ class BasicKerberosCredentialFactory(KerberosCredentialFactoryBase):
 
     scheme = 'basic'
 
-    def __init__(self, principal=None, type=None, hostname=None):
+    def __init__(self, principal=None, serviceType=None, hostname=None):
         """
-        
+
         @param principal:  full Kerberos principal (e.g., 'HTTP/server.example.com@EXAMPLE.COM'). If C{None}
             then the type and hostname arguments are used instead.
-        @type service:     str
-        @param type:       service type for Kerberos (e.g., 'HTTP'). Must be C{None} if principal used.
-        @type type:        str
+        @type principal:     str
+        @param serviceType:       service type for Kerberos (e.g., 'HTTP'). Must be C{None} if principal used.
+        @type serviceType:        str
         @param hostname:   hostname for this server. Must be C{None} if principal used.
         @type hostname:    str
         """
 
-        super(BasicKerberosCredentialFactory, self).__init__(principal, type, hostname)
+        super(BasicKerberosCredentialFactory, self).__init__(principal, serviceType, hostname)
+
 
     def getChallenge(self, _ignore_peer):
         return succeed({'realm': self.realm})
 
-    def decode(self, response, request): #@UnusedVariable
+
+    def decode(self, response, request):  # @UnusedVariable
         try:
             creds = (response + '===').decode('base64')
         except:
@@ -161,6 +169,8 @@ class BasicKerberosCredentialFactory(KerberosCredentialFactoryBase):
             return succeed(c)
         raise error.LoginFailed('Invalid credentials')
 
+
+
 class BasicKerberosCredentialsChecker(object):
     log = Logger()
 
@@ -182,14 +192,14 @@ class BasicKerberosCredentialsChecker(object):
                 raise error.UnauthorizedLogin("Bad credentials for: %s (%s: %s)" % (pcreds.authnURI, ex[0], ex[1],))
             else:
                 return succeed((
-                    pcreds.authnPrincipal.principalURL(),
-                    pcreds.authzPrincipal.principalURL(),
                     pcreds.authnPrincipal,
                     pcreds.authzPrincipal,
                 ))
-        
+
         raise error.UnauthorizedLogin("Bad credentials for: %s" % (pcreds.authnURI,))
 
+
+
 class NegotiateCredentials(object):
     """
     A set of user/password credentials that checks itself against Kerberos.
@@ -198,10 +208,12 @@ class NegotiateCredentials(object):
     implements(credentials.ICredentials)
 
     def __init__(self, principal, username):
-        
+
         self.principal = principal
         self.username = username
-        
+
+
+
 class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
     """
     Authorizer for Negotiate authentication (http://www.ietf.org/rfc/rfc4559.txt).
@@ -209,36 +221,38 @@ class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
 
     scheme = 'negotiate'
 
-    def __init__(self, principal=None, type=None, hostname=None):
+    def __init__(self, principal=None, serviceType=None, hostname=None):
         """
-        
+
         @param principal:  full Kerberos principal (e.g., 'HTTP/server.example.com@EXAMPLE.COM'). If C{None}
             then the type and hostname arguments are used instead.
-        @type service:     str
-        @param type:       service type for Kerberos (e.g., 'HTTP'). Must be C{None} if principal used.
-        @type type:        str
+        @type principal:     str
+        @param serviceType:       service type for Kerberos (e.g., 'HTTP'). Must be C{None} if principal used.
+        @type serviceType:        str
         @param hostname:   hostname for this server. Must be C{None} if principal used.
         @type hostname:    str
         """
 
-        super(NegotiateCredentialFactory, self).__init__(principal, type, hostname)
+        super(NegotiateCredentialFactory, self).__init__(principal, serviceType, hostname)
+
 
     def getChallenge(self, _ignore_peer):
         return succeed({})
 
+
     def decode(self, base64data, request):
-        
+
         # Init GSSAPI first - we won't specify the service now as we need to accept a target
         # name that is case-insenstive as some clients will use "http" instead of "HTTP"
         try:
-            _ignore_result, context = kerberos.authGSSServerInit("");
+            _ignore_result, context = kerberos.authGSSServerInit("")
         except kerberos.GSSError, ex:
             self.log.error("authGSSServerInit: %s(%s)" % (ex[0][0], ex[1][0],))
             raise error.LoginFailed('Authentication System Failure: %s(%s)' % (ex[0][0], ex[1][0],))
 
         # Do the GSSAPI step and get response and username
         try:
-            kerberos.authGSSServerStep(context, base64data);
+            kerberos.authGSSServerStep(context, base64data)
         except kerberos.GSSError, ex:
             self.log.error("authGSSServerStep: %s(%s)" % (ex[0][0], ex[1][0],))
             kerberos.authGSSServerClean(context)
@@ -264,13 +278,13 @@ class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
         principal = kerberos.authGSSServerUserName(context)
         username = principal
         realmname = ""
-        
+
         # Username may include realm suffix which we want to strip
         if username.find("@") != -1:
             splits = username.split("@", 1)
             username = splits[0]
             realmname = splits[1]
-        
+
         # We currently do not support cross-realm authentication, so we
         # must verify that the realm we got exactly matches the one we expect.
         if realmname != self.realm:
@@ -278,17 +292,17 @@ class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
 
         # Close the context
         try:
-            kerberos.authGSSServerClean(context);
+            kerberos.authGSSServerClean(context)
         except kerberos.GSSError, ex:
             self.log.error("authGSSServerClean: %s" % (ex[0][0], ex[1][0],))
             raise error.LoginFailed('Authentication System Failure %s(%s)' % (ex[0][0], ex[1][0],))
-        
+
         # If we successfully decoded and verified the Kerberos credentials we need to add the Kerberos
         # response data to the outgoing request
 
         wwwauth = '%s %s' % (self.scheme, response)
 
-        def responseFilterAddWWWAuthenticate(request, response): #@UnusedVariable
+        def responseFilterAddWWWAuthenticate(request, response):  # @UnusedVariable
             if response.code != responsecode.UNAUTHORIZED:
                 response.headers.addRawHeader('www-authenticate', wwwauth)
             return response
@@ -299,6 +313,8 @@ class NegotiateCredentialFactory(KerberosCredentialFactoryBase):
 
         return succeed(NegotiateCredentials(principal, username))
 
+
+
 class NegotiateCredentialsChecker(object):
 
     implements(checkers.ICredentialsChecker)
@@ -315,11 +331,8 @@ class NegotiateCredentialsChecker(object):
         creds = pcreds.credentials
         if isinstance(creds, NegotiateCredentials):
             return succeed((
-                pcreds.authnPrincipal.principalURL(),
-                pcreds.authzPrincipal.principalURL(),
                 pcreds.authnPrincipal,
                 pcreds.authzPrincipal,
             ))
-        
-        raise error.UnauthorizedLogin("Bad credentials for: %s" % (pcreds.authnURI,))
 
+        raise error.UnauthorizedLogin("Bad credentials for: %s" % (pcreds.authnURI,))
