@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,10 +15,10 @@
 ##
 
 from twext.python.log import Logger
-from twext.web2.dav.util import allDataFromStream
-from twext.web2.http import Response
-from twext.web2.iweb import IResource
-from twext.web2.stream import MemoryStream
+from txweb2.dav.util import allDataFromStream
+from txweb2.http import Response
+from txweb2.iweb import IResource
+from txweb2.stream import MemoryStream
 
 from twisted.internet.defer import succeed, inlineCallbacks, returnValue
 
@@ -69,6 +69,7 @@ is making the request (including proxy state changes etc).
 
 """
 
+
 class DisabledCacheNotifier(object):
     def __init__(self, *args, **kwargs):
         pass
@@ -134,6 +135,40 @@ class MemcacheChangeNotifier(CachePoolUserMixIn):
 
 
 
+class MemcacheURLPatternChangeNotifier(CachePoolUserMixIn):
+    """
+    A change notifier used to target arbitrary tokens.
+    """
+    log = Logger()
+
+    def __init__(self, urlPattern, cachePool=None, cacheHandle="Default"):
+        self._urlPattern = urlPattern
+        self._cachePool = cachePool
+        self._cachePoolHandle = cacheHandle
+
+
+    def _newCacheToken(self):
+        return str(uuid.uuid4())
+
+
+    def changed(self, token):
+        """
+        Change the cache token for a resource
+
+        return: A L{Deferred} that fires when the token has been changed.
+        """
+
+        url = self._urlPattern.format(token=token)
+
+        self.log.debug("Changing Cache Token for {url}", url=url)
+        return self.getCachePool().set(
+            'cacheToken:{url}'.format(url=url),
+            self._newCacheToken(),
+            expireTime=config.ResponseCacheTimeout * 60,
+        )
+
+
+
 class BaseResponseCache(object):
     """
     A base class which provides some common operations
@@ -141,7 +176,7 @@ class BaseResponseCache(object):
     log = Logger()
 
     def _principalURI(self, principal):
-        return str(principal.children[0])
+        return principal.principalURL() if principal is not None else "unauthenticated"
 
 
     def _uriNotFound(self, f, uri):
@@ -164,6 +199,7 @@ class BaseResponseCache(object):
             raise URINotFoundException(uri)
 
 
+    @inlineCallbacks
     def _canonicalizeURIForRequest(self, uri, request):
         """
         Always use canonicalized forms of the URIs for caching (i.e. __uids__ paths).
@@ -174,21 +210,24 @@ class BaseResponseCache(object):
         uribits = uri.split("/")
         if len(uribits) > 1 and uribits[1] in ("principals", "calendars", "addressbooks"):
             if uribits[2] == "__uids__":
-                return succeed(uri)
+                returnValue(uri)
             else:
                 recordType = uribits[2]
                 recordName = uribits[3]
                 directory = request.site.resource.getDirectory()
-                record = directory.recordWithShortName(recordType, recordName)
+                record = yield directory.recordWithShortName(
+                    directory.oldNameToRecordType(recordType),
+                    recordName
+                )
                 if record is not None:
                     uribits[2] = "__uids__"
-                    uribits[3] = record.uid
-                    return succeed("/".join(uribits))
+                    uribits[3] = record.uid.encode("utf-8")
+                    returnValue("/".join(uribits))
 
         # Fall back to the locateResource approach
         try:
-            return request.locateResource(uri).addCallback(
-                lambda resrc: resrc.url()).addErrback(self._uriNotFound, uri)
+            resrc = yield request.locateResource(uri)
+            returnValue(resrc.url())
         except AssertionError:
             raise URINotFoundException(uri)
 
@@ -252,7 +291,8 @@ class MemcacheResponseCache(BaseResponseCache, CachePoolUserMixIn):
         """
         Get the current token for a particular URI.
         """
-
+        if isinstance(uri, unicode):
+            uri = uri.encode("utf-8")
         if cachePoolHandle:
             result = (yield defaultCachePool(cachePoolHandle).get('cacheToken:%s' % (uri,)))
         else:
@@ -436,8 +476,9 @@ class MemcacheResponseCache(BaseResponseCache, CachePoolUserMixIn):
                     cTokens,
                 )
             )
-            yield self.getCachePool().set(key, cacheEntry,
-                expireTime=config.ResponseCacheTimeout * 60)
+            yield self.getCachePool().set(
+                key, cacheEntry, expireTime=config.ResponseCacheTimeout * 60
+            )
 
         except URINotFoundException, e:
             self.log.debug("Could not locate URI: {e!r}", e=e)
