@@ -1,5 +1,5 @@
 # #
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,8 +25,8 @@ change.
 See draft spec: http://ietf.webdav.org/caldav/draft-dusseault-caldav.txt
 """
 
-from pycalendar.datetime import PyCalendarDateTime
-from pycalendar.timezone import PyCalendarTimezone
+from pycalendar.datetime import DateTime
+from pycalendar.timezone import Timezone
 
 from txdav.xml.element import registerElement, dav_namespace
 from txdav.xml.element import WebDAVElement, PCDATAElement
@@ -110,8 +110,8 @@ class CalDAVTimeRangeElement (CalDAVEmptyElement):
         if "start" not in attributes and "end" not in attributes:
             raise ValueError("One of 'start' or 'end' must be present in CALDAV:time-range")
 
-        self.start = PyCalendarDateTime.parseText(attributes["start"]) if "start" in attributes else None
-        self.end = PyCalendarDateTime.parseText(attributes["end"]) if "end" in attributes else None
+        self.start = DateTime.parseText(attributes["start"]) if "start" in attributes else None
+        self.end = DateTime.parseText(attributes["end"]) if "end" in attributes else None
 
 
     def valid(self, level=0):
@@ -139,23 +139,103 @@ class CalDAVTimeRangeElement (CalDAVEmptyElement):
 
 
 
-class CalDAVTimeZoneElement (CalDAVTextElement):
+class CalDAVDataMixin(object):
+    """
+    A mixin to support accept/returning data in various formats.
+    """
+
+    def __init__(self, *children, **attributes):
+
+        if "content-type" in attributes:
+            self.content_type = attributes["content-type"]
+        else:
+            self.content_type = "text/calendar"
+
+        if "version" in attributes:
+            self.version = attributes["version"]
+        else:
+            self.version = "2.0"
+
+        super(CalDAVDataMixin, self).__init__(*children, **attributes)
+
+
+    def verifyTypeVersion(self):
+        """
+        Make sure any content-type and version matches at least one supported set.
+
+        @return: C{True} if there is at least one match, C{False} otherwise.
+        """
+        allowedTypes = set()
+        allowedTypes.add(("text/calendar", "2.0",))
+        if config.EnableJSONData:
+            allowedTypes.add(("application/calendar+json", "2.0",))
+        for format, version in allowedTypes:
+            if (format == self.content_type) and (version == self.version):
+                return True
+
+        return False
+
+
+    @classmethod
+    def fromCalendar(clazz, calendar, format=None):
+        attrs = {}
+        if format is not None and format != "text/calendar":
+            attrs["content-type"] = format
+
+        if isinstance(calendar, str):
+            if not calendar:
+                raise ValueError("Missing calendar data")
+            return clazz(PCDATAElement(calendar), **attrs)
+        elif isinstance(calendar, iComponent):
+            assert calendar.name() == "VCALENDAR", "Not a calendar: %r" % (calendar,)
+            return clazz(PCDATAElement(calendar.getTextWithTimezones(includeTimezones=not config.EnableTimezonesByReference, format=format)), **attrs)
+        else:
+            raise ValueError("Not a calendar: %s" % (calendar,))
+
+    fromTextData = fromCalendar
+    fromComponent = fromCalendar
+
+    def calendar(self):
+        """
+        Returns a calendar component derived from this element.
+        """
+        data = self.calendarData()
+        if data:
+            return iComponent.fromString(data, format=self.content_type)
+        else:
+            return None
+
+    generateComponent = calendar
+
+
+    def calendarData(self):
+        """
+        Returns the calendar data derived from this element.
+        """
+        data = None
+        for data in self.children:
+            if not isinstance(data, PCDATAElement):
+                return None
+            else:
+                # We guaranteed in __init__() that there is only one child...
+                break
+
+        return str(data)
+
+    textData = calendarData
+
+
+
+class CalDAVTimeZoneElement (CalDAVDataMixin, CalDAVTextElement):
     """
     CalDAV element containing iCalendar data with a single VTIMEZONE component.
     """
-    def calendar(self):
-        """
-        Returns a calendar component derived from this element, which contains
-        exactly one VTIMEZONE component.
-        """
-        return iComponent.fromString(str(self))
-
 
     def gettimezone(self):
         """
         Get the timezone to use. If none, return UTC timezone.
 
-        @return: the L{PyCalendarTimezone} derived from the VTIMEZONE or utc.
+        @return: the L{Timezone} derived from the VTIMEZONE or utc.
         """
         calendar = self.calendar()
         if calendar is not None:
@@ -164,7 +244,7 @@ class CalDAVTimeZoneElement (CalDAVTextElement):
                 return tz
 
         # Default to using utc tzinfo
-        return PyCalendarTimezone(utc=True)
+        return Timezone.UTCTimezone
 
 
     def valid(self):
@@ -231,6 +311,22 @@ class CalendarTimeZone (CalDAVTimeZoneElement):
     name = "calendar-timezone"
     hidden = True
 
+    allowed_attributes = {
+        "content-type": False,
+        "version"     : False,
+    }
+
+
+
+@registerElement
+class CalendarTimeZoneID (CalDAVTextElement):
+    """
+    Specifies a time zone id on a calendar collection.
+    (draft-ietf-tzdist-caldav-timezone-ref-01, Section-5.2)
+    """
+    name = "calendar-timezone-id"
+    hidden = True
+
 
 
 @registerElement
@@ -385,6 +481,7 @@ class CalendarQuery (CalDAVElement):
         (dav_namespace, "propname"): (0, None),
         (dav_namespace, "prop"): (0, None),
         (caldav_namespace, "timezone"): (0, 1),
+        (caldav_namespace, "timezone-id"): (0, 1),
         (caldav_namespace, "filter"): (0, 1), # Actually (1, 1) unless element is empty
     }
 
@@ -395,6 +492,7 @@ class CalendarQuery (CalDAVElement):
         props = None
         filter = None
         timezone = None
+        timezone_id = None
 
         for child in self.children:
             qname = child.qname()
@@ -412,8 +510,15 @@ class CalendarQuery (CalDAVElement):
                 filter = child
 
             elif qname == (caldav_namespace, "timezone"):
+                if timezone_id is not None:
+                    raise ValueError("Only one of CalDAV:timezone and CalDAV:timezone-id allowed")
                 timezone = child
 
+            elif qname == (caldav_namespace, "timezone-id"):
+                if timezone is not None:
+                    raise ValueError("Only one of CalDAV:timezone and CalDAV:timezone-id allowed")
+                timezone_id = child
+
             else:
                 raise AssertionError("We shouldn't be here")
 
@@ -424,11 +529,12 @@ class CalendarQuery (CalDAVElement):
         self.props = props
         self.filter = filter
         self.timezone = timezone
+        self.timezone_id = timezone_id
 
 
 
 @registerElement
-class CalendarData (CalDAVElement):
+class CalendarData (CalDAVDataMixin, CalDAVElement):
     """
     Defines which parts of a calendar component object should be returned by a
     report.
@@ -448,21 +554,6 @@ class CalendarData (CalDAVElement):
         "version"     : False,
     }
 
-    @classmethod
-    def fromCalendar(clazz, calendar):
-        if isinstance(calendar, str):
-            if not calendar:
-                raise ValueError("Missing calendar data")
-            return clazz(PCDATAElement(calendar))
-        elif isinstance(calendar, iComponent):
-            assert calendar.name() == "VCALENDAR", "Not a calendar: %r" % (calendar,)
-            return clazz(PCDATAElement(calendar.getTextWithTimezones(includeTimezones=not config.EnableTimezonesByReference)))
-        else:
-            raise ValueError("Not a calendar: %s" % (calendar,))
-
-    fromTextData = fromCalendar
-
-
     def __init__(self, *children, **attributes):
         super(CalendarData, self).__init__(*children, **attributes)
 
@@ -515,59 +606,6 @@ class CalendarData (CalDAVElement):
                 # optimize them originals away
                 self.children = (data,)
 
-        if "content-type" in attributes:
-            self.content_type = attributes["content-type"]
-        else:
-            self.content_type = "text/calendar"
-
-        if "version" in attributes:
-            self.version = attributes["version"]
-        else:
-            self.version = "2.0"
-
-
-    def verifyTypeVersion(self, types_and_versions):
-        """
-        Make sure any content-type and version matches at least one of the supplied set.
-
-        @param types_and_versions: a list of (content-type, version) tuples to test against.
-        @return:                   True if there is at least one match, False otherwise.
-        """
-        for item in types_and_versions:
-            if (item[0] == self.content_type) and (item[1] == self.version):
-                return True
-
-        return False
-
-
-    def calendar(self):
-        """
-        Returns a calendar component derived from this element.
-        """
-        data = self.calendarData()
-        if data:
-            return iComponent.fromString(data)
-        else:
-            return None
-
-    generateComponent = calendar
-
-
-    def calendarData(self):
-        """
-        Returns the calendar data derived from this element.
-        """
-        for data in self.children:
-            if not isinstance(data, PCDATAElement):
-                return None
-            else:
-                # We guaranteed in __init__() that there is only one child...
-                break
-
-        return str(data)
-
-    textData = calendarData
-
 
 
 @registerElement
@@ -882,6 +920,21 @@ class TimeZone (CalDAVTimeZoneElement):
     """
     name = "timezone"
 
+    allowed_attributes = {
+        "content-type": False,
+        "version"     : False,
+    }
+
+
+
+@registerElement
+class TimeZoneID (CalDAVTextElement):
+    """
+    Specifies a time zone id.
+    (draft-ietf-tzdist-caldav-timezone-ref-01, Section-5.2)
+    """
+    name = "timezone-id"
+
 
 
 @registerElement
@@ -1145,6 +1198,15 @@ class ScheduleOutbox (CalDAVEmptyElement):
 
 
 
+@registerElement
+class Trash (CalDAVEmptyElement):
+    """
+    Denotes the resource type of a trash collection.
+    """
+    name = "trash"
+
+
+
 @registerElement
 class ScheduleResponse (CalDAVElement):
     """
@@ -1383,6 +1445,26 @@ class ManagedAttachmentsServerURL (CalDAVElement):
     allowed_children = {(dav_namespace, "href"): (0, 1)}
 
 
+#
+# draft-ietf-tzdist-caldav-timezone-ref
+#
+
+caldav_timezone_service_set_compliance = (
+    "timezone-service-set",
+)
+
+
+
+@registerElement
+class TimezoneServiceSet (CalDAVElement):
+    """
+    Zero or one href elements defining the base scheme/host for attachments.
+    """
+    name = "timezone-service-set"
+
+    allowed_children = {(dav_namespace, "href"): (0, 1)}
+
+
 
 #
 # Extensions to ResourceType
@@ -1396,3 +1478,4 @@ ResourceType.isCalendar = _isCalendar
 ResourceType.calendar = ResourceType(Collection(), Calendar())
 ResourceType.scheduleInbox = ResourceType(Collection(), ScheduleInbox())
 ResourceType.scheduleOutbox = ResourceType(Collection(), ScheduleOutbox())
+ResourceType.trash = ResourceType(Collection(), Trash())
