@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # limitations under the License.
 ##
 
-from twext.web2.http_headers import DefaultHTTPHandler, tokenize, generateList, singleHeader
+from txweb2.http_headers import DefaultHTTPHandler, tokenize, generateList, singleHeader
 
 DefaultHTTPHandler.updateParsers({
     "x-forwarded-for": (tokenize, list),
