@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -56,8 +56,10 @@ class AccumulatingProtocol(protocol.Protocol):
 
     def connectionMade(self):
         self.made = 1
-        if (self.factory is not None and
-            self.factory.protocolConnectionMade is not None):
+        if (
+            self.factory is not None and
+            self.factory.protocolConnectionMade is not None
+        ):
             d = self.factory.protocolConnectionMade
             self.factory.protocolConnectionMade = None
             d.callback(self)
