@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,11 +33,11 @@ from twisted.internet.defer import Deferred, inlineCallbacks, returnValue
 from twisted.internet.error import ConnectionLost, ConnectionDone, ConnectError
 from twisted.internet.protocol import ClientFactory
 
-from twext.web2 import responsecode
-from twext.web2.client.http import HTTPClientProtocol
-from twext.web2.http import StatusResponse, HTTPError
-from twext.web2.dav.util import allDataFromStream
-from twext.web2.stream import MemoryStream
+from txweb2 import responsecode
+from txweb2.client.http import HTTPClientProtocol
+from txweb2.http import StatusResponse, HTTPError
+from txweb2.dav.util import allDataFromStream
+from txweb2.stream import MemoryStream
 
 class PooledHTTPClientFactory(ClientFactory):
     """
@@ -178,7 +178,7 @@ class HTTPClientPool(object):
         self._pendingConnects += 1
 
         self.log.debug("Initating new client connection to: %r" % (
-                self._endpoint,))
+            self._endpoint,))
         self._logClientStats()
 
         factory = self.clientFactory(self._reactor)
@@ -318,12 +318,14 @@ class HTTPClientPool(object):
 
 
     def _logClientStats(self):
-        self.log.debug("Clients #free: %d, #busy: %d, "
-                       "#pending: %d, #queued: %d" % (
+        self.log.debug(
+            "Clients #free: %d, #busy: %d, #pending: %d, #queued: %d" % (
                 len(self._freeClients),
                 len(self._busyClients),
                 self._pendingConnects,
-                len(self._pendingRequests)))
+                len(self._pendingRequests)
+            )
+        )
 
 
     def clientGone(self, client):
@@ -385,7 +387,7 @@ class HTTPClientPool(object):
             d, request, args, kwargs = self._pendingRequests.pop(0)
 
             self.log.debug("Performing Queued Request: %s, %r, %r" % (
-                    request, args, kwargs))
+                request, args, kwargs))
             self._logClientStats()
 
             _ign_d = self._submitRequest(request, *args, **kwargs)
