@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -34,7 +34,7 @@ from twistedcaldav import caldavxml, carddavxml
 from twistedcaldav.caldavxml import caldav_namespace
 from twistedcaldav.ical import Component as iComponent
 
-from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.datetime import DateTime
 
 
 calendarserver_namespace = "http://calendarserver.org/ns/"
@@ -68,10 +68,18 @@ calendarserver_sharing_no_scheduling_compliance = (
     "calendarserver-sharing-no-scheduling",
 )
 
+calendarserver_group_sharee_compliance = (
+    "calendarserver-group-sharee",
+)
+
 calendarserver_partstat_changes_compliance = (
     "calendarserver-partstat-changes",
 )
 
+calendarserver_group_attendee_compliance = (
+    "calendarserver-group-attendee",
+)
+
 calendarserver_home_sync_compliance = (
     "calendarserver-home-sync",
 )
@@ -643,8 +651,9 @@ class DTStamp (WebDAVTextElement):
     name = "dtstamp"
 
     def __init__(self, *children):
-        super(DTStamp, self).__init__(children)
-        self.children = (PCDATAElement(PyCalendarDateTime.getNowUTC().getText()),)
+        super(DTStamp, self).__init__(*children)
+        if not self.children:
+            self.children = (PCDATAElement(DateTime.getNowUTC().getText()),)
 
 
 
@@ -816,16 +825,6 @@ class RecordType (WebDAVTextElement):
 
 
 
-@registerElement
-class AutoSchedule (WebDAVTextElement):
-    """
-    Whether the principal automatically accepts invitations
-    """
-    namespace = calendarserver_namespace
-    name = "auto-schedule"
-
-
-
 @registerElement
 class AutoScheduleMode (WebDAVTextElement):
     """
@@ -1455,6 +1454,8 @@ ResourceType.dropbox = ResourceType(Collection(), DropBox())
 
 ResourceType.calendarproxyread = ResourceType(Principal(), Collection(), CalendarProxyRead())
 ResourceType.calendarproxywrite = ResourceType(Principal(), Collection(), CalendarProxyWrite())
+ResourceType.calendarproxyreadfor = ResourceType(Principal(), Collection(), CalendarProxyReadFor())
+ResourceType.calendarproxywritefor = ResourceType(Principal(), Collection(), CalendarProxyWriteFor())
 
 ResourceType.timezones = ResourceType(Timezones())
 
