@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # limitations under the License.
 ##
 
-from twext.web2.http import HTTPError
+from txweb2.http import HTTPError
 import twistedcaldav.test.util
 from twistedcaldav.datafilters.privateevents import PrivateEventFilter
 from twistedcaldav.ical import Component
