@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.directory.test.test_proxyprincipalmembers -*-
 ##
-# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,38 +28,41 @@ __all__ = [
 ]
 
 import itertools
-import time
-
-from twisted.internet.defer import succeed, inlineCallbacks, returnValue
-from twext.web2 import responsecode
-from twext.web2.http import HTTPError, StatusResponse
-from txdav.xml import element as davxml
-from txdav.xml.base import dav_namespace
-from twext.web2.dav.util import joinURL
-from twext.web2.dav.noneprops import NonePropertyStore
+import uuid
 
 from twext.python.log import Logger
-
-from twisted.web.template import XMLFile, Element, renderer
+from twext.who.idirectory import RecordType as BaseRecordType
+from twisted.internet.defer import succeed, inlineCallbacks, returnValue
 from twisted.python.modules import getModule
-from twistedcaldav.extensions import DirectoryElement
-from twistedcaldav.directory.principal import formatLink
-from twistedcaldav.directory.principal import formatLinks
-from twistedcaldav.directory.principal import formatPrincipals
-
-from twistedcaldav.directory.util import normalizeUUID
+from twisted.web.template import XMLFile, Element, renderer
 from twistedcaldav.config import config, fullServerPath
-from twistedcaldav.database import AbstractADBAPIDatabase, ADBAPISqliteMixin,\
-    ADBAPIPostgreSQLMixin
-from twistedcaldav.extensions import DAVPrincipalResource,\
-    DAVResourceWithChildrenMixin
+from twistedcaldav.database import (
+    AbstractADBAPIDatabase, ADBAPISqliteMixin, ADBAPIPostgreSQLMixin
+)
+from twistedcaldav.directory.util import normalizeUUID
+from twistedcaldav.directory.util import (
+    formatLink, formatLinks, formatPrincipals
+)
+
+from twistedcaldav.extensions import (
+    DAVPrincipalResource, DAVResourceWithChildrenMixin
+)
+from twistedcaldav.extensions import DirectoryElement
 from twistedcaldav.extensions import ReadOnlyWritePropertiesResourceMixIn
 from twistedcaldav.memcacher import Memcacher
 from twistedcaldav.resource import CalDAVComplianceMixIn
+from txdav.who.delegates import RecordType as DelegateRecordType
+from txdav.xml import element as davxml
+from txdav.xml.base import dav_namespace
+from txweb2 import responsecode
+from txweb2.dav.noneprops import NonePropertyStore
+from txweb2.dav.util import joinURL
+from txweb2.http import HTTPError, StatusResponse
 
 thisModule = getModule(__name__)
 log = Logger()
 
+
 class PermissionsMixIn (ReadOnlyWritePropertiesResourceMixIn):
     def defaultAccessControlList(self):
         aces = (
@@ -86,13 +89,13 @@ class PermissionsMixIn (ReadOnlyWritePropertiesResourceMixIn):
             for principal in config.AdminPrincipals
         ))
 
-        return davxml.ACL(*aces)
+        return succeed(davxml.ACL(*aces))
 
 
     def accessControlList(self, request, inheritance=True, expanding=False,
                           inherited_aces=None):
         # Permissions here are fixed, and are not subject to inheritance rules, etc.
-        return succeed(self.defaultAccessControlList())
+        return self.defaultAccessControlList()
 
 
 
@@ -119,13 +122,24 @@ class ProxyPrincipalDetailElement(Element):
         record = self.resource.parent.record
         resource = self.resource
         parent = self.resource.parent
+        try:
+            if isinstance(record.guid, uuid.UUID):
+                guid = str(record.guid).upper()
+            else:
+                guid = record.guid
+        except AttributeError:
+            guid = ""
+        try:
+            shortNames = record.shortNames
+        except AttributeError:
+            shortNames = []
         return tag.fillSlots(
             directoryGUID=record.service.guid,
             realm=record.service.realmName,
-            guid=record.guid,
-            recordType=record.recordType,
-            shortNames=record.shortNames,
-            fullName=record.fullName,
+            guid=guid,
+            recordType=record.service.recordTypeToOldName(record.recordType),
+            shortNames=shortNames,
+            fullName=record.displayName,
             principalUID=parent.principalUID(),
             principalURL=formatLink(parent.principalURL()),
             proxyPrincipalUID=resource.principalUID(),
@@ -177,16 +191,16 @@ class CalendarUserProxyPrincipalResource (
         super(CalendarUserProxyPrincipalResource, self).__init__()
         DAVResourceWithChildrenMixin.__init__(self)
 
-        self.parent          = parent
-        self.proxyType       = proxyType
-        self._url            = url
+        self.parent = parent
+        self.proxyType = proxyType
+        self._url = url
 
         # FIXME: if this is supposed to be public, it needs a better name:
-        self.pcollection     = self.parent.parent.parent
+        self.pcollection = self.parent.parent.parent
 
         # Principal UID is parent's GUID plus the proxy type; this we can easily
         # map back to a principal.
-        self.uid             = "%s#%s" % (self.parent.principalUID(), proxyType)
+        self.uid = "%s#%s" % (self.parent.principalUID(), proxyType)
         self._alternate_urls = tuple(
             joinURL(url, proxyType) + slash
             for url in parent.alternateURIs()
@@ -194,8 +208,12 @@ class CalendarUserProxyPrincipalResource (
         )
 
 
+    def __repr__(self):
+        return "<{}: {}>".format(self.__class__.__name__, str(self))
+
+
     def __str__(self):
-        return "%s [%s]" % (self.parent, self.proxyType)
+        return "{} [{}]".format(self.parent, self.proxyType)
 
 
     def _index(self):
@@ -206,14 +224,20 @@ class CalendarUserProxyPrincipalResource (
         """
         return ProxyDBService
 
+
     def resourceType(self):
         if self.proxyType == "calendar-proxy-read":
-            return davxml.ResourceType.calendarproxyread #@UndefinedVariable
+            return davxml.ResourceType.calendarproxyread  # @UndefinedVariable
         elif self.proxyType == "calendar-proxy-write":
-            return davxml.ResourceType.calendarproxywrite #@UndefinedVariable
+            return davxml.ResourceType.calendarproxywrite  # @UndefinedVariable
+        elif self.proxyType == "calendar-proxy-read-for":
+            return davxml.ResourceType.calendarproxyreadfor  # @UndefinedVariable
+        elif self.proxyType == "calendar-proxy-write-for":
+            return davxml.ResourceType.calendarproxywritefor  # @UndefinedVariable
         else:
             return super(CalendarUserProxyPrincipalResource, self).resourceType()
 
+
     def isProxyType(self, read_write):
         if (
             read_write and self.proxyType == "calendar-proxy-write" or
@@ -223,12 +247,15 @@ class CalendarUserProxyPrincipalResource (
         else:
             return False
 
+
     def isCollection(self):
         return True
 
+
     def etag(self):
         return succeed(None)
 
+
     def deadProperties(self):
         if not hasattr(self, "_dead_properties"):
             self._dead_properties = NonePropertyStore(self)
@@ -265,7 +292,7 @@ class CalendarUserProxyPrincipalResource (
         principals = []
         newUIDs = set()
         for uri in members:
-            principal = self.pcollection._principalForURI(uri)
+            principal = yield self.pcollection._principalForURI(uri)
             # Invalid principals MUST result in an error.
             if principal is None or principal.principalURL() != uri:
                 raise HTTPError(StatusResponse(
@@ -276,31 +303,48 @@ class CalendarUserProxyPrincipalResource (
             principals.append(principal)
             newUIDs.add(principal.principalUID())
 
-        # Get the old set of UIDs
-        oldUIDs = (yield self._index().getMembers(self.uid))
+        # Get the old set of expanded UIDs
+        oldUIDs = set()
+        oldPrincipals = yield self.groupMembers()
+        oldUIDs.update([p.principalUID() for p in oldPrincipals])
+        oldPrincipals = yield self.expandedGroupMembers()
+        oldUIDs.update([p.principalUID() for p in oldPrincipals])
 
         # Change membership
         yield self.setGroupMemberSetPrincipals(principals)
 
+        # Get the new set of UIDs
+        newUIDs = set()
+        newPrincipals = yield self.groupMembers()
+        newUIDs.update([p.principalUID() for p in newPrincipals])
+        newPrincipals = yield self.expandedGroupMembers()
+        newUIDs.update([p.principalUID() for p in newPrincipals])
+
         # Invalidate the primary principal's cache, and any principal's whose
         # membership status changed
         yield self.parent.cacheNotifier.changed()
 
         changedUIDs = newUIDs.symmetric_difference(oldUIDs)
         for uid in changedUIDs:
-            principal = self.pcollection.principalForUID(uid)
+            principal = yield self.pcollection.principalForUID(uid)
             if principal:
                 yield principal.cacheNotifier.changed()
 
         returnValue(True)
 
 
+    @inlineCallbacks
     def setGroupMemberSetPrincipals(self, principals):
-        # Map the principals to UIDs.
-        return self._index().setGroupMembers(
-            self.uid,
-            [p.principalUID() for p in principals],
+
+        # Find our pseudo-record
+        record = yield self.parent.record.service.recordWithShortName(
+            self._recordTypeFromProxyType(),
+            self.parent.principalUID()
         )
+        # Set the members
+        memberRecords = [p.record for p in principals]
+        yield record.setMembers(memberRecords)
+
 
     ##
     # HTTP
@@ -329,103 +373,87 @@ class CalendarUserProxyPrincipalResource (
         # FIXME: Add API to IDirectoryRecord for getting a record URI?
         return self._alternate_urls
 
+
     def principalURL(self):
         return self._url
 
+
     def principalUID(self):
         return self.uid
 
+
     def principalCollections(self):
         return self.parent.principalCollections()
 
-    @inlineCallbacks
-    def _expandMemberUIDs(self, uid=None, relatives=None, uids=None, infinity=False):
-        if uid is None:
-            uid = self.principalUID()
-        if relatives is None:
-            relatives = set()
-        if uids is None:
-            uids = set()
 
-        if uid not in uids:
-            from twistedcaldav.directory.principal import DirectoryPrincipalResource
-            uids.add(uid)
-            principal = self.pcollection.principalForUID(uid)
-            if isinstance(principal, CalendarUserProxyPrincipalResource):
-                members = yield self._directGroupMembers()
-                for member in members:
-                    if member.principalUID() not in uids:
-                        relatives.add(member)
-                        if infinity:
-                            yield self._expandMemberUIDs(member.principalUID(), relatives, uids, infinity=infinity)
-            elif isinstance(principal, DirectoryPrincipalResource):
-                if infinity:
-                    members = yield principal.expandedGroupMembers()
-                else:
-                    members = yield principal.groupMembers()
-                relatives.update(members)
+    def _recordTypeFromProxyType(self):
+        return {
+            "calendar-proxy-read": DelegateRecordType.readDelegateGroup,
+            "calendar-proxy-write": DelegateRecordType.writeDelegateGroup,
+            "calendar-proxy-read-for": DelegateRecordType.readDelegatorGroup,
+            "calendar-proxy-write-for": DelegateRecordType.writeDelegatorGroup,
+        }.get(self.proxyType)
 
-        returnValue(relatives)
 
     @inlineCallbacks
-    def _directGroupMembers(self):
-        # Get member UIDs from database and map to principal resources
-        members = yield self._index().getMembers(self.uid)
-        found = []
-        for uid in members:
-            p = self.pcollection.principalForUID(uid)
-            if p:
-                # Only principals enabledForLogin can be a delegate
-                # (and groups as well)
-                if (p.record.enabledForLogin or 
-                    p.record.recordType == p.record.service.recordType_groups):
-                    found.append(p)
-                # Make sure any outstanding deletion timer entries for
-                # existing principals are removed
-                yield self._index().refreshPrincipal(uid)
+    def _directGroupMembers(self, expanded=False):
+        """
+        Fault in the record representing the sub principal for this proxy type
+        (either read-only or read-write), then fault in the direct members of
+        that record.
+        """
+        memberPrincipals = []
+        record = yield self.parent.record.service.recordWithShortName(
+            self._recordTypeFromProxyType(),
+            self.parent.principalUID()
+        )
+        if record is not None:
+            if expanded:
+                memberRecords = yield record.expandedMembers()
             else:
-                self.log.warn("Delegate is missing from directory: %s" % (uid,))
+                memberRecords = yield record.members()
+            for record in memberRecords:
+                if record is not None and (record.loginAllowed or record.recordType is BaseRecordType.group):
+                    principal = yield self.pcollection.principalForRecord(record)
+                    if principal is not None:
+                        memberPrincipals.append(principal)
+        returnValue(memberPrincipals)
 
-        returnValue(found)
 
     def groupMembers(self):
-        return self._expandMemberUIDs()
+        return self._directGroupMembers(expanded=False)
+
 
-    @inlineCallbacks
     def expandedGroupMembers(self):
         """
         Return the complete, flattened set of principals belonging to this
         group.
         """
-        returnValue((yield self._expandMemberUIDs(infinity=True)))
+        return self._directGroupMembers(expanded=True)
+
 
     def groupMemberships(self):
-        # Get membership UIDs and map to principal resources
-        d = self._index().getMemberships(self.uid)
-        d.addCallback(lambda memberships: [
-            p for p
-            in [self.pcollection.principalForUID(uid) for uid in memberships]
-            if p
-        ])
-        return d
+        # Unlikely to ever want to put a subprincipal into a group
+        return succeed([])
+
 
-    @inlineCallbacks
     def containsPrincipal(self, principal):
         """
-        Uses proxyFor information to turn the "contains principal" question around;
-        rather than expanding this principal's groups to see if the other principal
-        is a member, ask the other principal if they are a proxy for this principal's
-        parent resource, since this principal is a proxy principal.
+        Optimize this to ignore the owner principal when doing this test. This is a common occurrence as the owner
+        is the one most likely to be looking at their own resources, each of which will have the owner's two proxy
+        principals listed in the ACLs (and this method is called during each ACL L{checkPrivileges} call.
 
         @param principal: The principal to check
         @type principal: L{DirectoryCalendarPrincipalResource}
-        @return: True if principal is a proxy (of the correct type) of our parent
+        @return: L{True} if principal is a proxy (of the correct type) of our parent
         @rtype: C{boolean}
         """
-        readWrite = self.isProxyType(True) # is read-write
-        if principal and self.parent in (yield principal.proxyFor(readWrite)):
-            returnValue(True)
-        returnValue(False)
+
+        if principal == self.parent:
+            return succeed(False)
+        return super(CalendarUserProxyPrincipalResource, self).containsPrincipal(principal)
+
+
 
 class ProxyDB(AbstractADBAPIDatabase):
     """
@@ -440,7 +468,7 @@ class ProxyDB(AbstractADBAPIDatabase):
     log = Logger()
 
     schema_version = "4"
-    schema_type    = "CALENDARUSERPROXY"
+    schema_type = "CALENDARUSERPROXY"
 
     class ProxyDBMemcacher(Memcacher):
 
@@ -483,38 +511,13 @@ class ProxyDB(AbstractADBAPIDatabase):
         def deleteMembership(self, guid):
             return self.delete("memberships:%s" % (str(guid),))
 
-        def setDeletionTimer(self, guid, delay):
-            return self.set("del:%s" % (str(guid),), str(self.getTime()+delay))
-
-        def checkDeletionTimer(self, guid):
-            # True means it's overdue, False means it's not, None means no timer
-            def _value(value):
-                if value:
-                    if int(value) <= self.getTime():
-                        return True
-                    else:
-                        return False
-                else:
-                    return None
-            d = self.get("del:%s" % (str(guid),))
-            d.addCallback(_value)
-            return d
-
-        def clearDeletionTimer(self, guid):
-            return self.delete("del:%s" % (str(guid),))
-
-        def getTime(self):
-            if hasattr(self, 'theTime'):
-                theTime = self.theTime
-            else:
-                theTime = int(time.time())
-            return theTime
 
     def __init__(self, dbID, dbapiName, dbapiArgs, **kwargs):
         AbstractADBAPIDatabase.__init__(self, dbID, dbapiName, dbapiArgs, True, **kwargs)
-        
+
         self._memcacher = ProxyDB.ProxyDBMemcacher("ProxyDB")
 
+
     @inlineCallbacks
     def setGroupMembers(self, principalUID, members):
         """
@@ -542,6 +545,7 @@ class ProxyDB(AbstractADBAPIDatabase):
             yield self._memcacher.deleteMembership(member)
         yield self._memcacher.deleteMember(principalUID)
 
+
     @inlineCallbacks
     def setGroupMembersInDatabase(self, principalUID, members):
         """
@@ -553,7 +557,8 @@ class ProxyDB(AbstractADBAPIDatabase):
         # Remove what is there, then add it back.
         yield self._delete_from_db(principalUID)
         yield self._add_to_db(principalUID, members)
-        
+
+
     @inlineCallbacks
     def changeGroupMembersInDatabase(self, principalUID, addMembers, removeMembers):
         """
@@ -568,7 +573,8 @@ class ProxyDB(AbstractADBAPIDatabase):
             yield self._delete_from_db_one(principalUID, member)
         for member in addMembers:
             yield self._add_to_db_one(principalUID, member)
-        
+
+
     @inlineCallbacks
     def removeGroup(self, principalUID):
         """
@@ -581,74 +587,13 @@ class ProxyDB(AbstractADBAPIDatabase):
         members = yield self.getMembers(principalUID)
 
         yield self._delete_from_db(principalUID)
-        
+
         # Update cache
         if members:
             for member in members:
                 yield self._memcacher.deleteMembership(member)
             yield self._memcacher.deleteMember(principalUID)
 
-    @inlineCallbacks
-    def removePrincipal(self, principalUID, delay=None):
-        """
-        Remove a group membership record.
-
-        @param principalUID: the UID of the principal to remove.
-        """
-        # FIXME: This method doesn't appear to be used anywhere.  Still needed?
-        
-        if delay:
-            # We are going to remove the principal only after <delay> seconds
-            # has passed since we first chose to remove it, to protect against
-            # transient directory problems.
-            # If <delay> is specified, first see if there was a timer set
-            # previously.  If the timer is more than delay seconds old, we
-            # go ahead and remove the principal.  Otherwise, do nothing.
-
-            overdue = yield self._memcacher.checkDeletionTimer(principalUID)
-
-            if overdue == False:
-                # Do nothing
-                returnValue(None)
-
-            elif overdue is None:
-                # No timer was previously set
-                self.log.debug("Delaying removal of missing proxy principal '%s'"
-                               % (principalUID,))
-                yield self._memcacher.setDeletionTimer(principalUID, delay=delay)
-                returnValue(None)
-
-        self.log.warn("Removing missing proxy principal for '%s'"
-                      % (principalUID,))
-
-        for suffix in ("calendar-proxy-read", "calendar-proxy-write",):
-            groupUID = "%s#%s" % (principalUID, suffix,)
-            yield self._delete_from_db(groupUID)
-
-            # Update cache
-            members = yield self.getMembers(groupUID)
-            if members:
-                for member in members:
-                    yield self._memcacher.deleteMembership(member)
-                yield self._memcacher.deleteMember(groupUID)
-
-        memberships = (yield self.getMemberships(principalUID))
-        for groupUID in memberships:
-            yield self._memcacher.deleteMember(groupUID)
-
-        yield self._delete_from_db_member(principalUID)
-        yield self._memcacher.deleteMembership(principalUID)
-        yield self._memcacher.clearDeletionTimer(principalUID)
-
-    def refreshPrincipal(self, principalUID):
-        """
-        Bring back to life a principal that was previously deleted.
-
-        @param principalUID:
-        @type principalUID:
-        """
-        
-        return self._memcacher.clearDeletionTimer(principalUID)
 
     def getMembers(self, principalUID):
         """
@@ -667,7 +612,7 @@ class ProxyDB(AbstractADBAPIDatabase):
                 d.addCallback(lambda _: members)
                 return d
 
-            d =  self.query("select MEMBER from GROUPS where GROUPNAME = :1", (principalUID.decode("utf-8"),))
+            d = self.query("select MEMBER from GROUPS where GROUPNAME = :1", (principalUID.decode("utf-8"),))
             d.addCallback(gotMembersFromDB)
             return d
 
@@ -675,10 +620,11 @@ class ProxyDB(AbstractADBAPIDatabase):
         d.addCallback(gotCachedMembers)
         return d
 
+
     def getMemberships(self, principalUID):
         """
         Return the list of group principal UIDs the specified principal is a member of.
-        
+
         @return: a deferred returning a C{set} of memberships.
         """
         def gotCachedMemberships(memberships):
@@ -692,7 +638,7 @@ class ProxyDB(AbstractADBAPIDatabase):
                 d.addCallback(lambda _: memberships)
                 return d
 
-            d =  self.query("select GROUPNAME from GROUPS where MEMBER = :1", (principalUID.decode("utf-8"),))
+            d = self.query("select GROUPNAME from GROUPS where MEMBER = :1", (principalUID.decode("utf-8"),))
             d.addCallback(gotMembershipsFromDB)
             return d
 
@@ -700,6 +646,7 @@ class ProxyDB(AbstractADBAPIDatabase):
         d.addCallback(gotCachedMemberships)
         return d
 
+
     @inlineCallbacks
     def _add_to_db(self, principalUID, members):
         """
@@ -716,6 +663,7 @@ class ProxyDB(AbstractADBAPIDatabase):
                 """, (principalUID.decode("utf-8"), member,)
             )
 
+
     def _add_to_db_one(self, principalUID, memberUID):
         """
         Insert the specified entry into the database.
@@ -730,6 +678,7 @@ class ProxyDB(AbstractADBAPIDatabase):
             """, (principalUID.decode("utf-8"), memberUID.decode("utf-8"),)
         )
 
+
     def _delete_from_db(self, principalUID):
         """
         Deletes the specified entry from the database.
@@ -738,6 +687,7 @@ class ProxyDB(AbstractADBAPIDatabase):
         """
         return self.execute("delete from GROUPS where GROUPNAME = :1", (principalUID.decode("utf-8"),))
 
+
     def _delete_from_db_one(self, principalUID, memberUID):
         """
         Deletes the specified entry from the database.
@@ -747,6 +697,7 @@ class ProxyDB(AbstractADBAPIDatabase):
         """
         return self.execute("delete from GROUPS where GROUPNAME = :1 and MEMBER = :2", (principalUID.decode("utf-8"), memberUID.decode("utf-8"),))
 
+
     def _delete_from_db_member(self, principalUID):
         """
         Deletes the specified member entry from the database.
@@ -755,18 +706,21 @@ class ProxyDB(AbstractADBAPIDatabase):
         """
         return self.execute("delete from GROUPS where MEMBER = :1", (principalUID.decode("utf-8"),))
 
+
     def _db_version(self):
         """
         @return: the schema version assigned to this index.
         """
         return ProxyDB.schema_version
 
+
     def _db_type(self):
         """
         @return: the collection type assigned to this index.
         """
         return ProxyDB.schema_type
 
+
     @inlineCallbacks
     def _db_init_data_tables(self):
         """
@@ -781,7 +735,7 @@ class ProxyDB(AbstractADBAPIDatabase):
             "GROUPS",
             (
                 ("GROUPNAME", "text"),
-                ("MEMBER",    "text"),
+                ("MEMBER", "text"),
             ),
             ifnotexists=True,
         )
@@ -821,9 +775,9 @@ class ProxyDB(AbstractADBAPIDatabase):
         )
         if alreadyDone is None:
             for (groupname, member) in (
-                    (yield self._db_all_values_for_sql(
-                        "select GROUPNAME, MEMBER from GROUPS"))
-                ):
+                (yield self._db_all_values_for_sql(
+                    "select GROUPNAME, MEMBER from GROUPS"))
+            ):
                 grouplist = groupname.split("#")
                 grouplist[0] = normalizeUUID(grouplist[0])
                 newGroupName = "#".join(grouplist)
@@ -877,17 +831,19 @@ class ProxyDB(AbstractADBAPIDatabase):
         #
         return self._db_execute("delete from GROUPS")
 
+
     @inlineCallbacks
     def clean(self):
-        
+
         if not self.initialized:
             yield self.open()
 
         for group in [row[0] for row in (yield self.query("select GROUPNAME from GROUPS"))]:
-            self.removeGroup(group)
-        
+            yield self.removeGroup(group)
+
         yield super(ProxyDB, self).clean()
 
+
     @inlineCallbacks
     def getAllMembers(self):
         """
@@ -905,8 +861,11 @@ class ProxySqliteDB(ADBAPISqliteMixin, ProxyDB):
 
     def __init__(self, dbpath):
 
+        self.dbpath = fullServerPath(config.DataRoot, dbpath)
         ADBAPISqliteMixin.__init__(self)
-        ProxyDB.__init__(self, "Proxies", "sqlite3", (fullServerPath(config.DataRoot, dbpath),))
+        ProxyDB.__init__(self, "Proxies", "sqlite3", (self.dbpath,))
+
+
 
 class ProxyPostgreSQLDB(ADBAPIPostgreSQLMixin, ProxyDB):
     """
@@ -914,9 +873,13 @@ class ProxyPostgreSQLDB(ADBAPIPostgreSQLMixin, ProxyDB):
     """
 
     def __init__(self, host, database, user=None, password=None, dbtype=None):
+        from txdav.base.datastore.subpostgres import postgres
 
-        ADBAPIPostgreSQLMixin.__init__(self, )
-        ProxyDB.__init__(self, "Proxies", "pgdb", (), host=host, database=database, user=user, password=password,)
+        ADBAPIPostgreSQLMixin.__init__(self,)
+        ProxyDB.__init__(
+            self, "Proxies", postgres.__name__, (),
+            host=host, database=database, user=user, password=password,
+        )
         if dbtype:
             ProxyDB.schema_type = dbtype
 
