@@ -1,5 +1,6 @@
+# -*- test-case-name: twistedcaldav.directory.test.test_digest -*-
 ##
-# Copyright (c) 2006-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,14 +17,15 @@
 
 from twisted.cred import error
 from twisted.internet.defer import inlineCallbacks, returnValue
-from twisted.web2.auth.digest import DigestCredentialFactory
-from twisted.web2.auth.digest import DigestedCredentials
-from twisted.web2.http_headers import Token
-from twisted.web2.http_headers import parseKeyValue
-from twisted.web2.http_headers import split
-from twisted.web2.http_headers import tokenize
+from txweb2.auth.digest import DigestCredentialFactory
+from txweb2.auth.digest import DigestedCredentials
+from txweb2.http_headers import Token
+from txweb2.http_headers import parseKeyValue
+from txweb2.http_headers import split
+from txweb2.http_headers import tokenize
+
+from twext.python.log import Logger
 
-from twistedcaldav.log import Logger
 from twistedcaldav.memcacher import Memcacher
 
 from zope.interface import implements, Interface
@@ -33,7 +35,7 @@ import time
 log = Logger()
 
 """
-Overrides twisted.web2.auth.digest to allow specifying a qop value as a configuration parameter.
+Overrides txweb2.auth.digest to allow specifying a qop value as a configuration parameter.
 Also adds an sqlite-based credentials cache that is multi-process safe.
 
 """
@@ -43,18 +45,19 @@ class IDigestCredentialsDatabase(Interface):
     An interface to a digest credentials database that is used to hold per-client digest credentials so that fast
     re-authentication can be done with replay attacks etc prevented.
     """
-    
+
     def has_key(self, key):
         """
         See whether the matching key exists.
 
         @param key:    the key to check.
         @type key:     C{str}.
-        
+
         @return:       C{True} if the key exists, C{False} otherwise.
         """
         pass
 
+
     def set(self, key, value):
         """
         Store per-client credential information the first time a nonce is generated and used.
@@ -65,7 +68,8 @@ class IDigestCredentialsDatabase(Interface):
         @type value:       any.
         """
         pass
-    
+
+
     def get(self, key):
         """
         Validate client supplied credentials by comparing with the cached values. If valid, store the new
@@ -77,7 +81,8 @@ class IDigestCredentialsDatabase(Interface):
         @return:       the value for the corresponding key, or C{None} if the key is not found.
         """
         pass
-    
+
+
     def delete(self, key):
         """
         Remove the record associated with the supplied key.
@@ -87,6 +92,8 @@ class IDigestCredentialsDatabase(Interface):
         """
         pass
 
+
+
 class DigestCredentialsMemcache(Memcacher):
 
     implements(IDigestCredentialsDatabase)
@@ -98,15 +105,17 @@ class DigestCredentialsMemcache(Memcacher):
             namespace=namespace,
             pickle=True,
         )
-    
+
+
     def has_key(self, key):
         """
         See IDigestCredentialsDatabase.
         """
         d = self.get(key)
-        d.addCallback(lambda value:value is not None)
+        d.addCallback(lambda value: value is not None)
         return d
 
+
     def set(self, key, value):
         """
         See IDigestCredentialsDatabase.
@@ -114,12 +123,14 @@ class DigestCredentialsMemcache(Memcacher):
         super(DigestCredentialsMemcache, self).set(
             key,
             value,
-            expire_time=self.CHALLENGE_MAXTIME_SECS
+            expireTime=self.CHALLENGE_MAXTIME_SECS
         )
 
+
+
 class QopDigestCredentialFactory(DigestCredentialFactory):
     """
-    See twisted.web2.auth.digest.DigestCredentialFactory
+    See txweb2.auth.digest.DigestCredentialFactory
     """
 
     def __init__(self, algorithm, qop, realm, namespace="DIGESTCREDENTIALS"):
@@ -142,11 +153,12 @@ class QopDigestCredentialFactory(DigestCredentialFactory):
         self.qop = qop
         self.db = DigestCredentialsMemcache(namespace)
 
+
     @inlineCallbacks
     def getChallenge(self, peer):
         """
-        Generate the challenge for use in the WWW-Authenticate header
-        Do the default behavior but then strip out any 'qop' from the challenge fields
+        Generate the challenge for use in the WWW-Authenticate header Do the
+        default behavior but then strip out any 'qop' from the challenge fields
         if no qop was specified.
 
         @param peer: The L{IAddress} of the requesting client.
@@ -155,34 +167,29 @@ class QopDigestCredentialFactory(DigestCredentialFactory):
             header.
         """
 
-        c = self.generateNonce()
-        
+        challenge = yield (super(QopDigestCredentialFactory, self).getChallenge(peer))
+        c = challenge['nonce']
+
         # Make sure it is not a duplicate
-        result = (yield self.db.has_key(c))
+        result = (yield self.db.has_key(c)) #@IgnorePep8
         if result:
             raise AssertionError("nonce value already cached in credentials database: %s" % (c,))
 
-        # The database record is a tuple of (client ip, nonce-count, timestamp)
-        yield self.db.set(c, (peer.host, 0, time.time()))
-
-        challenge = {
-            'nonce': c,
-            'qop': 'auth',
-            'algorithm': self.algorithm,
-            'realm': self.realm,
-        }
+        # The database record is a tuple of (nonce-count, timestamp)
+        yield self.db.set(c, (0, time.time()))
 
         if self.qop:
             challenge['qop'] = self.qop
         else:
             del challenge['qop']
-        
+
         # If stale was marked when decoding this request's Authorization header, add that to the challenge
         if hasattr(peer, 'stale') and peer.stale:
             challenge['stale'] = 'true'
 
         returnValue(challenge)
 
+
     @inlineCallbacks
     def decode(self, response, request):
         """
@@ -197,7 +204,7 @@ class QopDigestCredentialFactory(DigestCredentialFactory):
         @type response: C{str}
         @param response: A string of comma seperated key=value pairs
 
-        @type request: L{twisted.web2.server.Request}
+        @type request: L{txweb2.server.Request}
         @param request: the request being processed
 
         @return: L{DigestedCredentials}
@@ -207,17 +214,17 @@ class QopDigestCredentialFactory(DigestCredentialFactory):
         """
 
         response = ' '.join(response.splitlines())
-        
+
         try:
             parts = split(tokenize((response,), foldCase=False), Token(","))
-    
+
             auth = {}
-    
+
             for (k, v) in [parseKeyValue(p) for p in parts]:
                 auth[k.strip()] = v.strip()
         except ValueError:
             raise error.LoginFailed('Invalid response.')
-            
+
         username = auth.get('username')
         if not username:
             raise error.LoginFailed('Invalid response, no username given.')
@@ -234,18 +241,18 @@ class QopDigestCredentialFactory(DigestCredentialFactory):
                 originalMethod = None
 
             credentials = DigestedCredentials(username,
-                                              request.method,
-                                              self.realm,
-                                              auth,
-                                              originalMethod)
+                                              originalMethod or request.method,
+                                              self._real.authenticationRealm,
+                                              auth)
 
-            if not self.qop and credentials.fields.has_key('qop'):
+            if not self.qop and 'qop' in credentials.fields:
                 del credentials.fields['qop']
 
             returnValue(credentials)
         else:
             raise error.LoginFailed('Invalid nonce/cnonce values')
 
+
     @inlineCallbacks
     def _validate(self, auth, request):
         """
@@ -255,27 +262,21 @@ class QopDigestCredentialFactory(DigestCredentialFactory):
         @param auth:        the response parameters.
         @type auth:         C{dict}
         @param request:     the request being processed.
-        @type request:      L{twisted.web2.server.Request}
-        
+        @type request:      L{txweb2.server.Request}
+
         @return:            C{True} if validated.
         @raise LoginFailed: if validation fails.
         """
 
         nonce = auth.get('nonce')
-        clientip = request.remoteAddr.host
         nonce_count = auth.get('nc')
 
         # First check we have this nonce
         result = (yield self.db.get(nonce))
         if result is None:
             raise error.LoginFailed('Invalid nonce value: %s' % (nonce,))
-        db_clientip, db_nonce_count, db_timestamp = result
+        db_nonce_count, db_timestamp = result
 
-        # Next check client ip
-        if db_clientip != clientip:
-            yield self._invalidate(nonce)
-            raise error.LoginFailed('Client IPs do not match: %s and %s' % (clientip, db_clientip,))
-        
         # cnonce and nonce-count MUST be present if qop is present
         if auth.get('qop') is not None:
             if auth.get('cnonce') is None:
@@ -284,24 +285,24 @@ class QopDigestCredentialFactory(DigestCredentialFactory):
             if nonce_count is None:
                 yield self._invalidate(nonce)
                 raise error.LoginFailed('nonce-count is required when qop is specified')
-                
+
             # Next check the nonce-count is one greater than the previous one and update it in the DB
             try:
                 nonce_count = int(nonce_count, 16)
             except ValueError:
                 yield self._invalidate(nonce)
-                raise error.LoginFailed('nonce-count is not a valid hex string: %s' % (auth.get('nonce-count'),))            
+                raise error.LoginFailed('nonce-count is not a valid hex string: %s' % (auth.get('nonce-count'),))
             if nonce_count != db_nonce_count + 1:
                 yield self._invalidate(nonce)
                 raise error.LoginFailed('nonce-count value out of sequence: %s should be one more than %s' % (nonce_count, db_nonce_count,))
-            yield self.db.set(nonce, (db_clientip, nonce_count, db_timestamp))
+            yield self.db.set(nonce, (nonce_count, db_timestamp))
         else:
             # When not using qop the stored nonce-count must always be zero.
             # i.e. we can't allow a qop auth then a non-qop auth with the same nonce
             if db_nonce_count != 0:
                 yield self._invalidate(nonce)
-                raise error.LoginFailed('nonce-count was sent with this nonce: %s' % (nonce,))                
-        
+                raise error.LoginFailed('nonce-count was sent with this nonce: %s' % (nonce,))
+
         # Now check timestamp
         if db_timestamp + DigestCredentialFactory.CHALLENGE_LIFETIME_SECS <= time.time():
             yield self._invalidate(nonce)
@@ -310,7 +311,8 @@ class QopDigestCredentialFactory(DigestCredentialFactory):
             raise error.LoginFailed('Digest credentials expired')
 
         returnValue(True)
-    
+
+
     def _invalidate(self, nonce):
         """
         Invalidate cached credentials for the specified nonce value.
