@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,16 +21,16 @@ from twisted.cred import error
 from twisted.internet import address
 from twisted.internet.defer import inlineCallbacks, returnValue
 from twisted.python import failure
-from twext.web2.auth import digest
-from twext.web2.auth.wrapper import UnauthorizedResponse
-from twext.web2.test.test_server import SimpleRequest
+from txweb2.auth import digest
+from txweb2.auth.wrapper import UnauthorizedResponse
+from txweb2.test.test_server import SimpleRequest
 
 from twistedcaldav.directory.digest import QopDigestCredentialFactory
 from twistedcaldav.test.util import TestCase
 from twistedcaldav.config import config
-from twext.web2.auth.digest import DigestCredentialFactory
-from twext.web2.test.test_httpauth import makeDigestDeterministic
-from twext.web2.test.test_httpauth import FAKE_STATIC_NONCE
+from txweb2.auth.digest import DigestCredentialFactory
+from txweb2.test.test_httpauth import makeDigestDeterministic
+from txweb2.test.test_httpauth import FAKE_STATIC_NONCE
 
 class FakeDigestCredentialFactory(QopDigestCredentialFactory):
     """
@@ -52,36 +52,61 @@ challengeOpaque = ('75c4bd95b96b7b7341c646c6502f0833-MTc4Mjg4NzU'
 
 challengeNonce = '178288758716122392881254770685'
 
-challengeResponse = ('digest',
-                     {'nonce': challengeNonce,
-                      'qop': 'auth', 'realm': 'test realm',
-                      'algorithm': 'md5',})
+challengeResponse = (
+    'digest',
+    {
+        'nonce': challengeNonce,
+        'qop': 'auth',
+        'realm': 'test realm',
+        'algorithm': 'md5',
+    }
+)
 
 cnonce = "29fc54aa1641c6fa0e151419361c8f23"
 
-authRequest1 = (('username="username", realm="test realm", nonce="%s", '
-                 'uri="/write/", response="%s", algorithm="md5", '
-                 'cnonce="29fc54aa1641c6fa0e151419361c8f23", nc=00000001, '
-                 'qop="auth"'),
-                ('username="username", realm="test realm", nonce="%s", '
-                 'uri="/write/", response="%s", algorithm="md5"'))
+authRequest1 = (
+    (
+        'username="username", realm="test realm", nonce="%s", '
+        'uri="/write/", response="%s", algorithm="md5", '
+        'cnonce="29fc54aa1641c6fa0e151419361c8f23", nc=00000001, '
+        'qop="auth"'
+    ),
+    (
+        'username="username", realm="test realm", nonce="%s", '
+        'uri="/write/", response="%s", algorithm="md5"'
+    )
+)
 
-authRequest2 = (('username="username", realm="test realm", nonce="%s", '
-                 'uri="/write/", response="%s", algorithm="md5", '
-                 'cnonce="29fc54aa1641c6fa0e151419361c8f23", nc=00000002, '
-                 'qop="auth"'),
-                ('username="username", realm="test realm", nonce="%s", '
-                 'uri="/write/", response="%s", algorithm="md5"'))
+authRequest2 = (
+    (
+        'username="username", realm="test realm", nonce="%s", '
+        'uri="/write/", response="%s", algorithm="md5", '
+        'cnonce="29fc54aa1641c6fa0e151419361c8f23", nc=00000002, '
+        'qop="auth"'
+    ),
+    (
+        'username="username", realm="test realm", nonce="%s", '
+        'uri="/write/", response="%s", algorithm="md5"'
+    )
+)
 
-authRequest3 = ('username="username", realm="test realm", nonce="%s", '
-                'uri="/write/", response="%s", algorithm="md5"')
+authRequest3 = (
+    'username="username", realm="test realm", nonce="%s", '
+    'uri="/write/", response="%s", algorithm="md5"'
+)
 
-authRequestComma = (('username="user,name", realm="test realm", nonce="%s", '
-                 'uri="/write/1,2.txt", response="%s", algorithm="md5", '
-                 'cnonce="29fc54aa1641c6fa0e151419361c8f23", nc=00000001, '
-                 'qop="auth"'),
-                ('username="user,name", realm="test realm", nonce="%s", '
-                 'uri="/write/1,2.txt", response="%s", algorithm="md5"'))
+authRequestComma = (
+    (
+        'username="user,name", realm="test realm", nonce="%s", '
+        'uri="/write/1,2.txt", response="%s", algorithm="md5", '
+        'cnonce="29fc54aa1641c6fa0e151419361c8f23", nc=00000001, '
+        'qop="auth"'
+    ),
+    (
+        'username="user,name", realm="test realm", nonce="%s", '
+        'uri="/write/1,2.txt", response="%s", algorithm="md5"'
+    )
+)
 
 namelessAuthRequest = 'realm="test realm",nonce="doesn\'t matter"'
 
@@ -103,18 +128,21 @@ class DigestAuthTestCase(TestCase):
         self.namespace1 = "DIGEST1"
         self.namespace2 = "DIGEST2"
 
-        self.credentialFactories = (QopDigestCredentialFactory(
-                                          'md5',
-                                          'auth',
-                                          'test realm',
-                                          self.namespace1
-                                      ),
-                                      QopDigestCredentialFactory(
-                                          'md5',
-                                          '',
-                                          'test realm',
-                                          self.namespace2
-                                      ))
+        self.credentialFactories = (
+            QopDigestCredentialFactory(
+                'md5',
+                'auth',
+                'test realm',
+                self.namespace1
+            ),
+            QopDigestCredentialFactory(
+                'md5',
+                '',
+                'test realm',
+                self.namespace2
+            )
+        )
+
 
     def getDigestResponse(self, challenge, ncount):
         """
@@ -126,26 +154,31 @@ class DigestAuthTestCase(TestCase):
 
         if qop:
             expected = digest.calcResponse(
-                digest.calcHA1(algo,
-                               "username",
-                               "test realm",
-                               "password",
-                               nonce,
-                               cnonce),
+                digest.calcHA1(
+                    algo,
+                    "username",
+                    "test realm",
+                    "password",
+                    nonce,
+                    cnonce
+                ),
                 algo, nonce, ncount, cnonce, qop, "GET", "/write/", None
-                )
+            )
         else:
             expected = digest.calcResponse(
-                digest.calcHA1(algo,
-                               "username",
-                               "test realm",
-                               "password",
-                               nonce,
-                               cnonce),
+                digest.calcHA1(
+                    algo,
+                    "username",
+                    "test realm",
+                    "password",
+                    nonce,
+                    cnonce
+                ),
                 algo, nonce, None, None, None, "GET", "/write/", None
-                )
+            )
         return expected
 
+
     def getDigestResponseComma(self, challenge, ncount):
         """
         Calculate the response for the given challenge
@@ -156,26 +189,31 @@ class DigestAuthTestCase(TestCase):
 
         if qop:
             expected = digest.calcResponse(
-                digest.calcHA1(algo,
-                               "user,name",
-                               "test realm",
-                               "password",
-                               nonce,
-                               cnonce),
+                digest.calcHA1(
+                    algo,
+                    "user,name",
+                    "test realm",
+                    "password",
+                    nonce,
+                    cnonce
+                ),
                 algo, nonce, ncount, cnonce, qop, "GET", "/write/1,2.txt", None
-                )
+            )
         else:
             expected = digest.calcResponse(
-                digest.calcHA1(algo,
-                               "user,name",
-                               "test realm",
-                               "password",
-                               nonce,
-                               cnonce),
+                digest.calcHA1(
+                    algo,
+                    "user,name",
+                    "test realm",
+                    "password",
+                    nonce,
+                    cnonce
+                ),
                 algo, nonce, None, None, None, "GET", "/write/1,2.txt", None
-                )
+            )
         return expected
 
+
     @inlineCallbacks
     def assertRaisesDeferred(self, exception, f, *args, **kwargs):
         try:
@@ -191,6 +229,7 @@ class DigestAuthTestCase(TestCase):
             raise self.failureException('%s not raised (%r returned)'
                                         % (exception.__name__, result))
 
+
     @inlineCallbacks
     def test_getChallenge(self):
         """
@@ -203,13 +242,14 @@ class DigestAuthTestCase(TestCase):
         self.assertEquals(challenge['qop'], 'auth')
         self.assertEquals(challenge['realm'], 'test realm')
         self.assertEquals(challenge['algorithm'], 'md5')
-        self.assertTrue(challenge.has_key("nonce"))
+        self.assertTrue("nonce" in challenge)
 
         challenge = (yield self.credentialFactories[1].getChallenge(clientAddress))
-        self.assertFalse(challenge.has_key('qop'))
+        self.assertFalse('qop' in challenge)
         self.assertEquals(challenge['realm'], 'test realm')
         self.assertEquals(challenge['algorithm'], 'md5')
-        self.assertTrue(challenge.has_key("nonce"))
+        self.assertTrue("nonce" in challenge)
+
 
     @inlineCallbacks
     def test_response(self):
@@ -219,15 +259,16 @@ class DigestAuthTestCase(TestCase):
 
         for ctr, factory in enumerate(self.credentialFactories):
             challenge = (yield factory.getChallenge(clientAddress))
-    
+
             clientResponse = authRequest1[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponse(challenge, "00000001"),
             )
-    
+
             creds = (yield factory.decode(clientResponse, _trivial_GET()))
             self.failUnless(creds.checkPassword('password'))
 
+
     @inlineCallbacks
     def test_multiResponse(self):
         """
@@ -237,23 +278,24 @@ class DigestAuthTestCase(TestCase):
 
         for ctr, factory in enumerate(self.credentialFactories):
             challenge = (yield factory.getChallenge(clientAddress))
-    
+
             clientResponse = authRequest1[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponse(challenge, "00000001"),
             )
-    
+
             creds = (yield factory.decode(clientResponse, _trivial_GET()))
             self.failUnless(creds.checkPassword('password'))
-    
+
             clientResponse = authRequest2[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponse(challenge, "00000002"),
             )
-    
+
             creds = (yield factory.decode(clientResponse, _trivial_GET()))
             self.failUnless(creds.checkPassword('password'))
 
+
     @inlineCallbacks
     def test_failsWithDifferentMethod(self):
         """
@@ -263,16 +305,19 @@ class DigestAuthTestCase(TestCase):
 
         for ctr, factory in enumerate(self.credentialFactories):
             challenge = (yield factory.getChallenge(clientAddress))
-    
+
             clientResponse = authRequest1[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponse(challenge, "00000001"),
             )
-    
-            creds = (yield factory.decode(clientResponse,
-                                                  SimpleRequest(None, 'POST', '/')))
+
+            creds = (yield factory.decode(
+                clientResponse,
+                SimpleRequest(None, 'POST', '/')
+            ))
             self.failIf(creds.checkPassword('password'))
 
+
     @inlineCallbacks
     def test_noUsername(self):
         """
@@ -282,19 +327,24 @@ class DigestAuthTestCase(TestCase):
 
         # Check for no username
         for factory in self.credentialFactories:
-            e = (yield self.assertRaisesDeferred(error.LoginFailed,
-                                  factory.decode,
-                                  namelessAuthRequest,
-                                  _trivial_GET()))
+            e = (yield self.assertRaisesDeferred(
+                error.LoginFailed,
+                factory.decode,
+                namelessAuthRequest,
+                _trivial_GET()
+            ))
             self.assertEquals(str(e), "Invalid response, no username given.")
-    
+
             # Check for an empty username
-            e = (yield self.assertRaisesDeferred(error.LoginFailed,
-                                  factory.decode,
-                                  namelessAuthRequest + ',username=""',
-                                  _trivial_GET()))
+            e = (yield self.assertRaisesDeferred(
+                error.LoginFailed,
+                factory.decode,
+                namelessAuthRequest + ',username=""',
+                _trivial_GET()
+            ))
             self.assertEquals(str(e), "Invalid response, no username given.")
 
+
     @inlineCallbacks
     def test_noNonce(self):
         """
@@ -302,12 +352,15 @@ class DigestAuthTestCase(TestCase):
         """
 
         for factory in self.credentialFactories:
-            e = (yield self.assertRaisesDeferred(error.LoginFailed,
-                                  factory.decode,
-                                  'realm="Test",username="Foo",opaque="bar"',
-                                  _trivial_GET()))
+            e = (yield self.assertRaisesDeferred(
+                error.LoginFailed,
+                factory.decode,
+                'realm="Test",username="Foo",opaque="bar"',
+                _trivial_GET()
+            ))
             self.assertEquals(str(e), "Invalid response, no nonce given.")
 
+
     @inlineCallbacks
     def test_emptyAttribute(self):
         """
@@ -317,12 +370,15 @@ class DigestAuthTestCase(TestCase):
 
         # Check for no username
         for factory in self.credentialFactories:
-            e = (yield self.assertRaisesDeferred(error.LoginFailed,
-                                  factory.decode,
-                                  emtpyAttributeAuthRequest,
-                                  _trivial_GET()))
+            e = (yield self.assertRaisesDeferred(
+                error.LoginFailed,
+                factory.decode,
+                emtpyAttributeAuthRequest,
+                _trivial_GET()
+            ))
             self.assertEquals(str(e), "Invalid response, no username given.")
 
+
     @inlineCallbacks
     def test_checkHash(self):
         """
@@ -332,19 +388,20 @@ class DigestAuthTestCase(TestCase):
 
         for ctr, factory in enumerate(self.credentialFactories):
             challenge = (yield factory.getChallenge(clientAddress))
-    
+
             clientResponse = authRequest1[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponse(challenge, "00000001"),
             )
-    
+
             creds = (yield factory.decode(clientResponse, _trivial_GET()))
-    
+
             self.failUnless(creds.checkHash(
-                    md5('username:test realm:password').hexdigest()))
-    
+                md5('username:test realm:password').hexdigest()))
+
             self.failIf(creds.checkHash(
-                    md5('username:test realm:bogus').hexdigest()))
+                md5('username:test realm:bogus').hexdigest()))
+
 
     @inlineCallbacks
     def test_invalidNonceCount(self):
@@ -359,20 +416,20 @@ class DigestAuthTestCase(TestCase):
 
         for ctr, factory in enumerate(credentialFactories):
             challenge = (yield factory.getChallenge(clientAddress))
-    
+
             clientResponse1 = authRequest1[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponse(challenge, "00000001"),
             )
-    
+
             clientResponse2 = authRequest2[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponse(challenge, "00000002"),
             )
-    
+
             yield factory.decode(clientResponse1, _trivial_GET())
             yield factory.decode(clientResponse2, _trivial_GET())
-    
+
             if challenge.get('qop') is not None:
                 yield self.assertRaisesDeferred(
                     error.LoginFailed,
@@ -380,7 +437,7 @@ class DigestAuthTestCase(TestCase):
                     clientResponse2,
                     _trivial_GET()
                 )
-                
+
                 challenge = (yield factory.getChallenge(clientAddress))
 
                 clientResponse1 = authRequest1[ctr] % (
@@ -400,6 +457,7 @@ class DigestAuthTestCase(TestCase):
                     _trivial_GET()
                 )
 
+
     @inlineCallbacks
     def test_invalidNonce(self):
         """
@@ -415,12 +473,12 @@ class DigestAuthTestCase(TestCase):
         for ctr, factory in enumerate(credentialFactories):
             challenge = (yield factory.getChallenge(clientAddress))
             challenge['nonce'] = "noNoncense"
-    
+
             clientResponse = authRequest1[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponse(challenge, "00000001"),
             )
-    
+
             request = _trivial_GET()
             yield self.assertRaisesDeferred(
                 error.LoginFailed,
@@ -431,11 +489,12 @@ class DigestAuthTestCase(TestCase):
 
             factory._invalidate(FAKE_STATIC_NONCE)
             response = (yield UnauthorizedResponse.makeResponse(
-                {"Digest":factory},
+                {"Digest": factory},
                 request.remoteAddr
             ))
             response.headers.getHeader("www-authenticate")[0][1]
 
+
     @inlineCallbacks
     def test_oldNonce(self):
         """
@@ -452,12 +511,12 @@ class DigestAuthTestCase(TestCase):
             challenge = (yield factory.getChallenge(clientAddress))
             nonce_count, timestamp = (yield factory.db.get(challenge['nonce']))
             factory.db.set(challenge['nonce'], (nonce_count, timestamp - 2 * digest.DigestCredentialFactory.CHALLENGE_LIFETIME_SECS))
-    
+
             clientResponse = authRequest1[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponse(challenge, "00000001"),
             )
-    
+
             request = _trivial_GET()
             yield self.assertRaisesDeferred(
                 error.LoginFailed,
@@ -465,15 +524,16 @@ class DigestAuthTestCase(TestCase):
                 clientResponse,
                 request
             )
-            
+
             response = (yield UnauthorizedResponse.makeResponse(
-                {"Digest":factory},
+                {"Digest": factory},
                 request.remoteAddr,
             ))
             wwwhdrs = response.headers.getHeader("www-authenticate")[0][1]
             self.assertTrue('stale' in wwwhdrs, msg="No stale parameter in Digest WWW-Authenticate headers: %s" % (wwwhdrs,))
             self.assertEquals(wwwhdrs['stale'], 'true', msg="stale parameter not set to true in Digest WWW-Authenticate headers: %s" % (wwwhdrs,))
 
+
     def test_incompatibleCalcHA1Options(self):
         """
         Test that the appropriate error is raised when any of the
@@ -485,7 +545,7 @@ class DigestAuthTestCase(TestCase):
             ("user", "realm", "password", "preHA1"),
             (None, "realm", None, "preHA1"),
             (None, None, "password", "preHA1"),
-            )
+        )
 
         for pszUsername, pszRealm, pszPassword, preHA1 in arguments:
             self.assertRaises(
@@ -498,7 +558,8 @@ class DigestAuthTestCase(TestCase):
                 "nonce",
                 "cnonce",
                 preHA1=preHA1
-                )
+            )
+
 
     @inlineCallbacks
     def test_commaURI(self):
@@ -508,15 +569,16 @@ class DigestAuthTestCase(TestCase):
 
         for ctr, factory in enumerate(self.credentialFactories):
             challenge = (yield factory.getChallenge(clientAddress))
-    
+
             clientResponse = authRequestComma[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponseComma(challenge, "00000001"),
             )
-    
+
             creds = (yield factory.decode(clientResponse, _trivial_GET()))
             self.failUnless(creds.checkPassword('password'))
 
+
     @inlineCallbacks
     def test_stale_response(self):
         """
@@ -526,20 +588,20 @@ class DigestAuthTestCase(TestCase):
         class newtime(object):
             def time(self):
                 return theTime
-        from twistedcaldav.directory import digest
-        self.patch(digest, "time", newtime())
+        from twistedcaldav.directory import digest as ddigest
+        self.patch(ddigest, "time", newtime())
 
         for ctr, factory in enumerate(self.credentialFactories):
             challenge = (yield factory.getChallenge(clientAddress))
-    
+
             clientResponse = authRequest1[ctr] % (
                 challenge['nonce'],
                 self.getDigestResponse(challenge, "00000001"),
             )
-    
+
             creds = (yield factory.decode(clientResponse, _trivial_GET()))
             self.failUnless(creds.checkPassword('password'))
-            
+
             theTime += DigestCredentialFactory.CHALLENGE_LIFETIME_SECS + 1
             request = _trivial_GET()
             try:
@@ -555,8 +617,8 @@ class DigestAuthTestCase(TestCase):
                 self.fail("Invalid exception from nonce timeout: %s" % e)
             challenge = (yield factory.getChallenge(request.remoteAddr))
             self.assertTrue(challenge.get("stale") == "true")
-            
+
+
 
 def _trivial_GET():
     return SimpleRequest(None, 'GET', '/')
-
