@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.directory.test.test_util -*-
 ##
-# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,16 +27,19 @@ __all__ = [
 
 from twext.enterprise.ienterprise import AlreadyFinishedError
 from twext.python.log import Logger
-from twext.web2 import responsecode
-from twext.web2.auth.wrapper import UnauthorizedResponse
-from twext.web2.dav.resource import DAVResource
-from twext.web2.http import StatusResponse
+from txweb2 import responsecode
+from txweb2.auth.wrapper import UnauthorizedResponse
+from txweb2.dav.resource import DAVResource
+from txweb2.http import StatusResponse
 from twisted.internet.defer import inlineCallbacks, returnValue
-from txdav.xml import element as davxml
 from uuid import UUID, uuid5
+from twisted.python.failure import Failure
+from twisted.web.template import tags
+
 
 log = Logger()
 
+
 def uuidFromName(namespace, name):
     """
     Generate a version 5 (SHA-1) UUID from a namespace UUID and a name.
@@ -51,6 +54,7 @@ def uuidFromName(namespace, name):
     return normalizeUUID(str(uuid5(UUID(namespace), name)))
 
 
+
 def normalizeUUID(value):
     """
     Convert strings which the uuid.UUID( ) method can parse into normalized
@@ -66,6 +70,7 @@ def normalizeUUID(value):
 
 TRANSACTION_KEY = '_newStoreTransaction'
 
+
 def transactionFromRequest(request, newStore):
     """
     Return the associated transaction from the given HTTP request, creating a
@@ -89,7 +94,12 @@ def transactionFromRequest(request, newStore):
     """
     transaction = getattr(request, TRANSACTION_KEY, None)
     if transaction is None:
-        transaction = newStore.newTransaction(repr(request))
+        if hasattr(request, "authzUser") and request.authzUser is not None:
+            authz_uid = request.authzUser.record.uid
+        else:
+            authz_uid = None
+        transaction = newStore.newTransaction(repr(request), authz_uid=authz_uid)
+
         def abortIfUncommitted(request, response):
             try:
                 # TODO: missing 'yield' here.  For formal correctness as per
@@ -106,6 +116,7 @@ def transactionFromRequest(request, newStore):
     return transaction
 
 
+
 def splitIntoBatches(data, size):
     """
     Return a generator of sets consisting of the contents of the data set
@@ -119,6 +130,7 @@ def splitIntoBatches(data, size):
         del data[:size]
 
 
+
 class NotFoundResource(DAVResource):
     """
     In order to prevent unauthenticated discovery of existing users via 401/404
@@ -131,12 +143,12 @@ class NotFoundResource(DAVResource):
     def renderHTTP(self, request):
 
         try:
-            authnUser, authzUser = yield self.authenticate(request)
+            _ignore_authnUser, authzUser = yield self.authenticate(request)
         except Exception:
-            authzUser = davxml.Principal(davxml.Unauthenticated())
+            authzUser = None
 
         # Turn 404 into 401
-        if authzUser == davxml.Principal(davxml.Unauthenticated()):
+        if authzUser is None:
             response = (yield UnauthorizedResponse.makeResponse(
                 request.credentialFactories,
                 request.remoteAddr
@@ -146,3 +158,77 @@ class NotFoundResource(DAVResource):
             response = StatusResponse(responsecode.NOT_FOUND, "Resource not found")
             returnValue(response)
 
+
+
+def formatLink(url):
+    """
+    Convert a URL string into some twisted.web.template DOM objects for
+    rendering as a link to itself.
+    """
+    return tags.a(href=url)(url)
+
+
+
+def formatLinks(urls):
+    """
+    Format a list of URL strings as a list of twisted.web.template DOM links.
+    """
+    return formatList(formatLink(link) for link in urls)
+
+
+
+def formatPrincipals(principals):
+    """
+    Format a list of principals into some twisted.web.template DOM objects.
+    """
+    def recordKey(principal):
+        try:
+            record = principal.record
+        except AttributeError:
+            try:
+                record = principal.parent.record
+            except:
+                return None
+        try:
+            shortName = record.shortNames[0]
+        except AttributeError:
+            shortName = u""
+        return (record.recordType, shortName)
+
+
+    def describe(principal):
+        if hasattr(principal, "record"):
+            return " - %s" % (principal.record.displayName,)
+        else:
+            return ""
+
+    return formatList(
+        tags.a(href=principal.principalURL())(
+            str(principal), describe(principal)
+        )
+        for principal in sorted(principals, key=recordKey)
+    )
+
+
+
+def formatList(iterable):
+    """
+    Format a list of stuff as an interable.
+    """
+    thereAreAny = False
+    try:
+        item = None
+        for item in iterable:
+            thereAreAny = True
+            yield " -> "
+            if item is None:
+                yield "None"
+            else:
+                yield item
+            yield "\n"
+    except Exception, e:
+        log.error("Exception while rendering: %s" % (e,))
+        Failure().printTraceback()
+        yield "  ** %s **: %s\n" % (e.__class__.__name__, e)
+    if not thereAreAny:
+        yield " '()\n"
