@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.test.test_extensions -*-
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -40,23 +40,23 @@ from twisted.internet.defer import inlineCallbacks, returnValue
 from twisted.web.template import Element, XMLFile, renderer, tags, flattenString
 from twisted.python.modules import getModule
 
-from twext.web2 import responsecode, server
-from twext.web2.http import HTTPError, Response, RedirectResponse
-from twext.web2.http import StatusResponse
-from twext.web2.http_headers import MimeType
-from twext.web2.stream import FileStream
-from twext.web2.static import MetaDataMixin, StaticRenderMixin
+from txweb2 import responsecode, server
+from txweb2.http import HTTPError, Response, RedirectResponse
+from txweb2.http import StatusResponse
+from txweb2.http_headers import MimeType
+from txweb2.stream import FileStream
+from txweb2.static import MetaDataMixin, StaticRenderMixin
 from txdav.xml import element
 from txdav.xml.base import encodeXMLName
 from txdav.xml.element import dav_namespace
-from twext.web2.dav.http import MultiStatusResponse
-from twext.web2.dav.static import DAVFile as SuperDAVFile
-from twext.web2.dav.resource import DAVResource as SuperDAVResource
-from twext.web2.dav.resource import (
+from txweb2.dav.http import MultiStatusResponse
+from txweb2.dav.static import DAVFile as SuperDAVFile
+from txweb2.dav.resource import DAVResource as SuperDAVResource
+from txweb2.dav.resource import (
     DAVPrincipalResource as SuperDAVPrincipalResource
 )
 from twisted.internet.defer import gatherResults
-from twext.web2.dav.method import prop_common
+from txweb2.dav.method import prop_common
 
 from twext.python.log import Logger
 
@@ -66,6 +66,9 @@ from twistedcaldav.customxml import calendarserver_namespace
 from twistedcaldav.method.report import http_REPORT
 
 from twistedcaldav.config import config
+from txdav.who.directory import CalendarDirectoryRecordMixin
+from twext.who.expression import Operand, MatchType, MatchFlags
+
 
 
 thisModule = getModule(__name__)
@@ -76,8 +79,10 @@ log = Logger()
 class DirectoryPrincipalPropertySearchMixIn(object):
 
     @inlineCallbacks
-    def report_DAV__principal_property_search(self, request,
-        principal_property_search):
+    def report_DAV__principal_property_search(
+        self, request,
+        principal_property_search
+    ):
         """
         Generate a principal-property-search REPORT. (RFC 3744, section 9.4)
         Overrides twisted implementation, targeting only directory-enabled
@@ -95,7 +100,7 @@ class DirectoryPrincipalPropertySearchMixIn(object):
             msg = "Bad XML: unknown value for test attribute: %s" % (testMode,)
             log.warn(msg)
             raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, msg))
-        operand = "and" if testMode == "allof" else "or"
+        operand = Operand.AND if testMode == "allof" else Operand.OR
 
         # Are we narrowing results down to a single CUTYPE?
         cuType = principal_property_search.attributes.get("type", None)
@@ -123,8 +128,10 @@ class DirectoryPrincipalPropertySearchMixIn(object):
                 propertiesForResource = prop_common.propertyListForResource
                 propElement = child
 
-            elif child.qname() == (dav_namespace,
-                "apply-to-principal-collection-set"):
+            elif child.qname() == (
+                dav_namespace,
+                "apply-to-principal-collection-set"
+            ):
                 applyTo = True
 
             elif child.qname() == (dav_namespace, "property-search"):
@@ -144,10 +151,18 @@ class DirectoryPrincipalPropertySearchMixIn(object):
                     log.warn(msg)
                     raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, msg))
 
+                # Convert to twext.who.expression form
+                matchType = {
+                    "starts-with": MatchType.startsWith,
+                    "contains": MatchType.contains,
+                    "equals": MatchType.equals
+                }.get(matchType)
+                matchFlags = MatchFlags.caseInsensitive if caseless else MatchFlags.none
+
                 # Ignore any query strings under three letters
-                matchText = str(match)
+                matchText = match.toString()  # gives us unicode
                 if len(matchText) >= 3:
-                    propertySearches.append((props.children, matchText, caseless, matchType))
+                    propertySearches.append((props.children, matchText, matchFlags, matchType))
 
             elif child.qname() == (calendarserver_namespace, "limit"):
                 try:
@@ -182,7 +197,7 @@ class DirectoryPrincipalPropertySearchMixIn(object):
         # See if we can take advantage of the directory
         fields = []
         nonDirectorySearches = []
-        for props, match, caseless, matchType in propertySearches:
+        for props, match, matchFlags, matchType in propertySearches:
             nonDirectoryProps = []
             for prop in props:
                 try:
@@ -191,12 +206,14 @@ class DirectoryPrincipalPropertySearchMixIn(object):
                 except ValueError, e:
                     raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, str(e)))
                 if fieldName:
-                    fields.append((fieldName, match, caseless, matchType))
+                    fields.append((fieldName, match, matchFlags, matchType))
                 else:
                     nonDirectoryProps.append(prop)
             if nonDirectoryProps:
-                nonDirectorySearches.append((nonDirectoryProps, match,
-                    caseless, matchType))
+                nonDirectorySearches.append((
+                    nonDirectoryProps, match,
+                    matchFlags, matchType
+                ))
 
         matchingResources = []
         matchcount = 0
@@ -204,11 +221,23 @@ class DirectoryPrincipalPropertySearchMixIn(object):
         # nonDirectorySearches are ignored
         if fields:
 
-            records = (yield dir.recordsMatchingFieldsWithCUType(fields,
-                operand=operand, cuType=cuType))
+            if cuType:
+                recordType = CalendarDirectoryRecordMixin.fromCUType(cuType)
+            else:
+                recordType = None
+
+            records = (
+                yield dir.recordsMatchingFields(
+                    fields,
+                    operand=operand,
+                    recordType=recordType,
+                    limitResults=clientLimit,
+                    timeoutSeconds=10
+                )
+            )
 
             for record in records:
-                resource = principalCollection.principalForRecord(record)
+                resource = yield principalCollection.principalForRecord(record)
                 if resource:
                     matchingResources.append(resource)
 
@@ -243,15 +272,19 @@ class DirectoryPrincipalPropertySearchMixIn(object):
                     responsecode.INSUFFICIENT_STORAGE_SPACE
                 ),
                 element.Error(element.NumberOfMatchesWithinLimits()),
-                element.ResponseDescription("Results limited by %s at %d"
-                                           % resultsWereLimited),
+                element.ResponseDescription(
+                    "Results limited by %s at %d"
+                    % resultsWereLimited
+                ),
             ))
         returnValue(MultiStatusResponse(responses))
 
 
     @inlineCallbacks
-    def report_http___calendarserver_org_ns__calendarserver_principal_search(self, request,
-        calendarserver_principal_search):
+    def report_http___calendarserver_org_ns__calendarserver_principal_search(
+        self, request,
+        calendarserver_principal_search
+    ):
         """
         Generate a calendarserver-principal-search REPORT.
 
@@ -274,8 +307,10 @@ class DirectoryPrincipalPropertySearchMixIn(object):
         tokens, context, applyTo, clientLimit, propElement = extractCalendarServerPrincipalSearchData(calendarserver_principal_search)
 
         if not validateTokens(tokens):
-            raise HTTPError(StatusResponse(responsecode.FORBIDDEN,
-                "Insufficient search token length"))
+            raise HTTPError(StatusResponse(
+                responsecode.FORBIDDEN,
+                "Insufficient search token length"
+            ))
 
         # Run report
         resultsWereLimited = None
@@ -296,10 +331,19 @@ class DirectoryPrincipalPropertySearchMixIn(object):
         matchingResources = []
         matchcount = 0
 
-        records = (yield dir.recordsMatchingTokens(tokens, context=context))
+        limitResults = config.MaxPrincipalSearchReportResults
+        if clientLimit is not None:
+            limitResults = min(clientLimit, limitResults)
+
+        records = (yield dir.recordsMatchingTokens(
+            tokens,
+            context=context,
+            limitResults=limitResults,
+            timeoutSeconds=10
+        ))
 
         for record in records:
-            resource = principalCollection.principalForRecord(record)
+            resource = yield principalCollection.principalForRecord(record)
             if resource:
                 matchingResources.append(resource)
 
@@ -334,8 +378,10 @@ class DirectoryPrincipalPropertySearchMixIn(object):
                     responsecode.INSUFFICIENT_STORAGE_SPACE
                 ),
                 element.Error(element.NumberOfMatchesWithinLimits()),
-                element.ResponseDescription("Results limited by %s at %d"
-                                           % resultsWereLimited),
+                element.ResponseDescription(
+                    "Results limited by %s at %d"
+                    % resultsWereLimited
+                ),
             ))
         returnValue(MultiStatusResponse(responses))
 
@@ -420,9 +466,9 @@ class DirectoryElement(Element):
                 f.trap(HTTPError)
                 code = f.value.response.code
                 if code == responsecode.NOT_FOUND:
-                    log.error("Property %s was returned by listProperties() "
-                              "but does not exist for resource %s."
-                              % (name, self.resource))
+                    log.error("Property {p} was returned by listProperties() "
+                              "but does not exist for resource {r}.",
+                              p=name, r=self.resource)
                     return (name, None)
                 if code == responsecode.UNAUTHORIZED:
                     return (name, accessDeniedValue)
@@ -541,7 +587,7 @@ class DAVResource (DirectoryPrincipalPropertySearchMixIn,
                    SuperDAVResource,
                    DirectoryRenderingMixIn, StaticRenderMixin):
     """
-    Extended L{twext.web2.dav.resource.DAVResource} implementation.
+    Extended L{txweb2.dav.resource.DAVResource} implementation.
 
     Note we add StaticRenderMixin as a base class because we need all the etag etc behavior
     that is currently in static.py but is actually applicable to any type of resource.
@@ -577,7 +623,7 @@ class DAVResource (DirectoryPrincipalPropertySearchMixIn,
 
 class DAVResourceWithChildrenMixin (object):
     """
-    Bits needed from twext.web2.static
+    Bits needed from txweb2.static
     """
 
     def __init__(self, principalCollections=None):
@@ -646,7 +692,7 @@ class DAVResourceWithChildrenMixin (object):
 
 class DAVResourceWithoutChildrenMixin (object):
     """
-    Bits needed from twext.web2.static
+    Bits needed from txweb2.static
     """
 
     def __init__(self, principalCollections=None):
@@ -670,7 +716,7 @@ class DAVPrincipalResource (DirectoryPrincipalPropertySearchMixIn,
                             SuperDAVPrincipalResource,
                             DirectoryRenderingMixIn):
     """
-    Extended L{twext.web2.dav.static.DAVFile} implementation.
+    Extended L{txweb2.dav.static.DAVFile} implementation.
     """
     log = Logger()
 
@@ -721,7 +767,13 @@ class DAVPrincipalResource (DirectoryPrincipalPropertySearchMixIn,
 
             elif name == "record-type":
                 if hasattr(self, "record"):
-                    returnValue(customxml.RecordType(self.record.recordType))
+                    returnValue(
+                        customxml.RecordType(
+                            self.record.service.recordTypeToOldName(
+                                self.record.recordType
+                            )
+                        )
+                    )
                 else:
                     raise HTTPError(StatusResponse(
                         responsecode.NOT_FOUND,
@@ -761,7 +813,7 @@ class DAVPrincipalResource (DirectoryPrincipalPropertySearchMixIn,
 
 class DAVFile (SuperDAVFile, DirectoryRenderingMixIn):
     """
-    Extended L{twext.web2.dav.static.DAVFile} implementation.
+    Extended L{txweb2.dav.static.DAVFile} implementation.
     """
     log = Logger()
 
@@ -848,13 +900,14 @@ class ReadOnlyResourceMixIn (ReadOnlyWritePropertiesResourceMixIn):
     ):
         # Permissions here are fixed, and are not subject to
         # inheritance rules, etc.
-        return succeed(self.defaultAccessControlList())
+        return self.defaultAccessControlList()
 
 
 
 class PropertyNotFoundError (HTTPError):
     def __init__(self, qname):
-        HTTPError.__init__(self,
+        HTTPError.__init__(
+            self,
             StatusResponse(
                 responsecode.NOT_FOUND,
                 "No such property: %s" % encodeXMLName(*qname)
@@ -876,7 +929,7 @@ class CachingPropertyStore (object):
 
 
     def get(self, qname, uid=None):
-        #self.log.debug("Get: %r, %r" % (self.resource.fp.path, qname))
+        # self.log.debug("Get: %r, %r" % (self.resource.fp.path, qname))
 
         cache = self._cache()
 
@@ -899,7 +952,7 @@ class CachingPropertyStore (object):
 
 
     def set(self, property, uid=None):
-        #self.log.debug("Set: %r, %r" % (self.resource.fp.path, property))
+        # self.log.debug("Set: %r, %r" % (self.resource.fp.path, property))
 
         cache = self._cache()
 
@@ -911,7 +964,7 @@ class CachingPropertyStore (object):
 
 
     def contains(self, qname, uid=None):
-        #self.log.debug("Contains: %r, %r" % (self.resource.fp.path, qname))
+        # self.log.debug("Contains: %r, %r" % (self.resource.fp.path, qname))
 
         cachedQname = qname + (uid,)
 
@@ -924,14 +977,14 @@ class CachingPropertyStore (object):
                 raise
 
         if cachedQname in cache:
-            #self.log.debug("Contains cache hit: %r, %r, %r" % (self, self.resource.fp.path, qname))
+            # self.log.debug("Contains cache hit: %r, %r, %r" % (self, self.resource.fp.path, qname))
             return True
         else:
             return False
 
 
     def delete(self, qname, uid=None):
-        #self.log.debug("Delete: %r, %r" % (self.resource.fp.path, qname))
+        # self.log.debug("Delete: %r, %r" % (self.resource.fp.path, qname))
 
         cachedQname = qname + (uid,)
 
@@ -942,7 +995,7 @@ class CachingPropertyStore (object):
 
 
     def list(self, uid=None, filterByUID=True):
-        #self.log.debug("List: %r" % (self.resource.fp.path,))
+        # self.log.debug("List: %r" % (self.resource.fp.path,))
         keys = self._cache().iterkeys()
         if filterByUID:
             return [
@@ -956,7 +1009,7 @@ class CachingPropertyStore (object):
 
     def _cache(self):
         if not hasattr(self, "_data"):
-            #self.log.debug("Cache init: %r" % (self.resource.fp.path,))
+            # self.log.debug("Cache init: %r" % (self.resource.fp.path,))
             self._data = dict(
                 (name, None)
                 for name in self.propertyStore.list(filterByUID=False)
@@ -985,12 +1038,16 @@ def extractCalendarServerPrincipalSearchData(doc):
         if child.qname() == (dav_namespace, "prop"):
             propElement = child
 
-        elif child.qname() == (dav_namespace,
-            "apply-to-principal-collection-set"):
+        elif child.qname() == (
+            dav_namespace,
+            "apply-to-principal-collection-set"
+        ):
             applyTo = True
 
         elif child.qname() == (calendarserver_namespace, "search-token"):
-            tokens.append(str(child))
+            tokenValue = child.toString().strip()
+            if tokenValue:
+                tokens.append(tokenValue)
 
         elif child.qname() == (calendarserver_namespace, "limit"):
             try:
