@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,20 +23,20 @@ __all__ = [
     "ICalendarPrincipalResource",
 ]
 
-from twext.web2.dav.idav import IDAVResource
+from txweb2.dav.idav import IDAVResource
 
 class ICalDAVResource(IDAVResource):
     """
     CalDAV resource.
     """
-    def isCalendarCollection():
+    def isCalendarCollection(): #@NoSelf
         """
         (CalDAV-access-10, Section 4.2)
         @return: True if this resource is a calendar collection, False
             otherwise.
         """
 
-    def isSpecialCollection(collectiontype):
+    def isSpecialCollection(collectiontype): #@NoSelf
         """
         (CalDAV-access-10, Section 4.2)
         @param collectiontype: L{WebDAVElement} for the collection type to test for.
@@ -44,30 +44,30 @@ class ICalDAVResource(IDAVResource):
             False otherwise.
         """
 
-    def isPseudoCalendarCollection():
+    def isPseudoCalendarCollection(): #@NoSelf
         """
         @return: True if this resource is a calendar collection like (e.g.
             a regular calendar collection or schedule inbox/outbox), False
             otherwise.
         """
 
-    def findCalendarCollections(depth):
+    def findCalendarCollections(depth): #@NoSelf
         """
         Returns an iterable of child calendar collection resources for the given
         depth.
-        Because resources do not know their request URIs, chidren are returned
+        Because resources do not know their request URIs, children are returned
         as tuples C{(resource, uri)}, where C{resource} is the child resource
         and C{uri} is a URL path relative to this resource.
         @param depth: the search depth (one of "0", "1", or "infinity")
         @return: an iterable of tuples C{(resource, uri)}.
         """
 
-    def createCalendar(request):
+    def createCalendar(request): #@NoSelf
         """
         Create a calendar collection for this resource.
         """
 
-    def iCalendar():
+    def component(): #@NoSelf
         """
         Instantiate an iCalendar component object representing this resource or
         its child with the given name.
@@ -78,52 +78,42 @@ class ICalDAVResource(IDAVResource):
         @return: a L{twistedcaldav.ical.Component} of type C{"VCALENDAR"}.
         """
 
-    def iCalendarText():
-        """
-        Obtains the iCalendar text representing this resource or its child with
-        the given name.
-        The behavior of this method is not specified if it is called on a
-        resource that is not a calendar collection or a calendar resource within
-        a calendar collection.
 
-        @return: a string containing iCalendar text with a top-level component
-            of type C{"VCALENDAR"}.
-        """
 
 class ICalendarPrincipalResource(IDAVResource):
     """
     CalDAV principle resource.
     """
-    def principalUID():
+    def principalUID(): #@NoSelf
         """
         @return: the user id for this principal.
         """
 
-    def calendarHomeURLs():
+    def calendarHomeURLs(): #@NoSelf
         """
         @return: a list of calendar home URLs for this principal's calendar user.
         """
 
-    def calendarUserAddresses():
+    def calendarUserAddresses(): #@NoSelf
         """
         @return: a list of calendar user addresses for this principal's calendar
             user.
         """
 
-    def calendarFreeBusyURIs(self, request):
+    def calendarFreeBusyURIs(request): #@NoSelf
         """
         @param request: the request being processed.
         @return: a L{Deferred} list of URIs for calendars that contribute to
             free-busy for this principal's calendar user.
         """
 
-    def scheduleInboxURL():
+    def scheduleInboxURL(): #@NoSelf
         """
         Get the schedule INBOX URL for this principal's calendar user.
         @return: a string containing the URL from the schedule-inbox-URL property.
         """
 
-    def scheduleOutboxURL():
+    def scheduleOutboxURL(): #@NoSelf
         """
         Get the schedule OUTBOX URL for this principal's calendar user.
         @return: a string containing the URL from the schedule-outbox-URL property.
