@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,10 +16,10 @@
 
 
 from twext.python.log import Logger
-from twext.web2 import responsecode, server, http
+from txweb2 import responsecode, server, http
 from txdav.xml import element as davxml
-from twext.web2.http import HTTPError, StatusResponse
-from twext.web2.resource import WrapperResource
+from txweb2.http import HTTPError, StatusResponse
+from txweb2.resource import WrapperResource
 
 from twisted.internet.defer import inlineCallbacks, returnValue, maybeDeferred
 
@@ -47,16 +47,17 @@ class LinkResource(CalDAVComplianceMixIn, WrapperResource):
     case of a missing underlying resource (broken link) as indicated by self._linkedResource being None.
     """
     log = Logger()
-    
+
     def __init__(self, parent, link_url):
         self.parent = parent
         self.linkURL = link_url
         self.loopDetect = set()
         super(LinkResource, self).__init__(self.parent.principalCollections())
 
+
     @inlineCallbacks
     def linkedResource(self, request):
-        
+
         if not hasattr(self, "_linkedResource"):
             if self.linkURL in self.loopDetect:
                 raise HTTPError(StatusResponse(responsecode.LOOP_DETECTED, "Recursive link target: %s" % (self.linkURL,)))
@@ -67,17 +68,20 @@ class LinkResource(CalDAVComplianceMixIn, WrapperResource):
 
         if self._linkedResource is None:
             raise HTTPError(StatusResponse(responsecode.NOT_FOUND, "Missing link target: %s" % (self.linkURL,)))
-            
+
         returnValue(self._linkedResource)
 
+
     def isCollection(self):
         return True if hasattr(self, "_linkedResource") else False
 
+
     def resourceType(self):
         return self._linkedResource.resourceType() if hasattr(self, "_linkedResource") else davxml.ResourceType.link
-        
+
+
     def locateChild(self, request, segments):
-        
+
         def _defer(result):
             if result is None:
                 return (self, server.StopTraversal)
@@ -87,6 +91,7 @@ class LinkResource(CalDAVComplianceMixIn, WrapperResource):
         d.addCallback(_defer)
         return d
 
+
     @inlineCallbacks
     def renderHTTP(self, request):
         linked_to = (yield self.linkedResource(request))
@@ -95,27 +100,33 @@ class LinkResource(CalDAVComplianceMixIn, WrapperResource):
         else:
             returnValue(http.StatusResponse(responsecode.OK, "Link resource with missing target: %s" % (self.linkURL,)))
 
+
     def getChild(self, name):
         return self._linkedResource.getChild(name) if hasattr(self, "_linkedResource") else None
 
+
     @inlineCallbacks
     def hasProperty(self, property, request):
         hosted = (yield self.linkedResource(request))
         result = (yield hosted.hasProperty(property, request)) if hosted else False
         returnValue(result)
 
+
     @inlineCallbacks
     def readProperty(self, property, request):
         hosted = (yield self.linkedResource(request))
         result = (yield hosted.readProperty(property, request)) if hosted else None
         returnValue(result)
 
+
     @inlineCallbacks
     def writeProperty(self, property, request):
         hosted = (yield self.linkedResource(request))
         result = (yield hosted.writeProperty(property, request)) if hosted else None
         returnValue(result)
 
+
+
 class LinkFollowerMixIn(object):
 
     @inlineCallbacks
@@ -128,6 +139,5 @@ class LinkFollowerMixIn(object):
             if linked_to is None:
                 break
             resource = linked_to
-        
+
         returnValue((resource, path))
-        
