@@ -1,5 +1,5 @@
 # #
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ CalDAV POST method.
 
 __all__ = ["http_POST"]
 
-from twext.web2 import responsecode
+from txweb2 import responsecode
 
 from twisted.internet.defer import inlineCallbacks, returnValue
 
