@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Computer, Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -29,20 +29,21 @@ __all__ = ["http_REPORT"]
 import string
 
 from twisted.internet.defer import inlineCallbacks, returnValue
-from twext.web2 import responsecode
-from twext.web2.http import HTTPError, StatusResponse
-from twext.web2.dav.util import davXMLFromStream
+from txweb2 import responsecode
+from txweb2.http import HTTPError, StatusResponse
+from txweb2.dav.util import davXMLFromStream
 from txdav.xml import element as davxml
 from txdav.xml.base import encodeXMLName
 from txdav.xml.element import lookupElement
 
 from twext.python.log import Logger
-from twext.web2.dav.http import ErrorResponse
+from txweb2.dav.http import ErrorResponse
 
 from twistedcaldav import caldavxml
 
 log = Logger()
 
+
 @inlineCallbacks
 def http_REPORT(self, request):
     """
@@ -58,7 +59,7 @@ def http_REPORT(self, request):
     try:
         doc = (yield davXMLFromStream(request.stream))
     except ValueError, e:
-        log.error("Error while handling REPORT body: %s" % (e,))
+        log.error("Error while handling REPORT body: {err}", err=str(e))
         raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, str(e)))
 
     if doc is None:
@@ -116,8 +117,8 @@ def http_REPORT(self, request):
         #
         # Requested report is not supported.
         #
-        log.error("Unsupported REPORT %s for resource %s (no method %s)"
-                  % (encodeXMLName(namespace, name), self, method_name))
+        log.error("Unsupported REPORT {name} for resource {resource} (no method {method})",
+                  name=encodeXMLName(namespace, name), resource=self, method=method_name)
 
         raise HTTPError(ErrorResponse(
             responsecode.FORBIDDEN,
