@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,6 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
+from twistedcaldav.timezones import TimezoneException, readVTZ
+from twistedcaldav.ical import Component
 
 """
 CalDAV calendar-query report
@@ -20,26 +22,26 @@ CalDAV calendar-query report
 
 __all__ = ["report_urn_ietf_params_xml_ns_caldav_calendar_query"]
 
-from twisted.internet.defer import inlineCallbacks, returnValue, \
-    maybeDeferred
+from twisted.internet.defer import inlineCallbacks, returnValue
 
 from twext.python.log import Logger
-from twext.web2 import responsecode
-from twext.web2.dav.http import MultiStatusResponse
-from twext.web2.dav.http import ErrorResponse
-from twext.web2.dav.method.report import NumberOfMatchesWithinLimits
-from twext.web2.dav.util import joinURL
-from twext.web2.http import HTTPError, StatusResponse
+from txweb2 import responsecode
+from txweb2.dav.http import MultiStatusResponse
+from txweb2.dav.http import ErrorResponse
+from txweb2.dav.method.report import NumberOfMatchesWithinLimits
+from txweb2.dav.util import joinURL
+from txweb2.http import HTTPError, StatusResponse
 
 from twistedcaldav import caldavxml
-from twistedcaldav.caldavxml import caldav_namespace, MaxInstances
+from twistedcaldav.caldavxml import caldav_namespace, MaxInstances, \
+    CalendarTimeZone
 from twistedcaldav.config import config
 from txdav.common.icommondatastore import IndexedSearchException, \
     ConcurrentModification
 from twistedcaldav.instance import TooManyInstancesError
 from twistedcaldav.method import report_common
-from twistedcaldav.query import calendarqueryfilter
 
+from txdav.caldav.datastore.query.filter import Filter
 from txdav.caldav.icalendarstore import TimeRangeLowerLimit, TimeRangeUpperLimit
 from txdav.xml import element as davxml
 
@@ -65,25 +67,37 @@ def report_urn_ietf_params_xml_ns_caldav_calendar_query(self, request, calendar_
     responses = []
 
     xmlfilter = calendar_query.filter
-    filter = calendarqueryfilter.Filter(xmlfilter)
+    filter = Filter(xmlfilter)
     props = calendar_query.props
 
     assert props is not None
 
     # Get the original timezone provided in the query, if any, and validate it now
     query_timezone = None
-    query_tz = calendar_query.timezone
-    if query_tz is not None and not query_tz.valid():
-        msg = "CalDAV:timezone must contain one VTIMEZONE component only: %s" % (query_tz,)
-        log.error(msg)
-        raise HTTPError(ErrorResponse(
-            responsecode.FORBIDDEN,
-            (caldav_namespace, "valid-calendar-data"),
-            "Invalid calendar-data",
-        ))
-    if query_tz:
+    if calendar_query.timezone:
+        query_tz = calendar_query.timezone
+        if not query_tz.valid():
+            msg = "CalDAV:timezone must contain one VTIMEZONE component only: %s" % (query_tz,)
+            log.error(msg)
+            raise HTTPError(ErrorResponse(
+                responsecode.FORBIDDEN,
+                (caldav_namespace, "valid-calendar-data"),
+                "Invalid calendar-data",
+            ))
         filter.settimezone(query_tz)
-        query_timezone = tuple(calendar_query.timezone.calendar().subcomponents())[0]
+        query_timezone = tuple(query_tz.calendar().subcomponents())[0]
+    elif calendar_query.timezone_id:
+        query_tzid = calendar_query.timezone_id.toString()
+        try:
+            query_tz = Component(None, pycalendar=readVTZ(query_tzid))
+        except TimezoneException:
+            raise HTTPError(ErrorResponse(
+                responsecode.FORBIDDEN,
+                (caldav_namespace, "valid-timezone"),
+                "Invalid timezone-id",
+            ))
+        filter.settimezone(query_tz)
+        query_timezone = tuple(query_tz.subcomponents())[0]
 
     if props.qname() == ("DAV:", "allprop"):
         propertiesForResource = report_common.allPropertiesForResource
@@ -171,12 +185,13 @@ def report_urn_ietf_params_xml_ns_caldav_calendar_query(self, request, calendar_
         if calresource.isPseudoCalendarCollection():
             # Get the timezone property from the collection if one was not set in the query,
             # and store in the query filter for later use
-            has_prop = (yield calresource.hasProperty((caldav_namespace, "calendar-timezone"), request))
             timezone = query_timezone
-            if query_tz is None and has_prop:
-                tz = (yield calresource.readProperty((caldav_namespace, "calendar-timezone"), request))
-                filter.settimezone(tz)
-                timezone = tuple(tz.calendar().subcomponents())[0]
+            if timezone is None:
+                has_prop = (yield calresource.hasProperty(CalendarTimeZone(), request))
+                if has_prop:
+                    tz = (yield calresource.readProperty(CalendarTimeZone(), request))
+                    filter.settimezone(tz)
+                    timezone = tuple(tz.calendar().subcomponents())[0]
 
             # Do some optimization of access control calculation by determining any inherited ACLs outside of
             # the child resource loop and supply those to the checkPrivileges on each child.
@@ -189,13 +204,11 @@ def report_urn_ietf_params_xml_ns_caldav_calendar_query(self, request, calendar_
             if filteredaces is not None:
                 index_query_ok = True
                 try:
-                    # Get list of children that match the search and have read
-                    # access
-                    records = yield maybeDeferred(calresource.index().indexedSearch, filter)
+                    # Get list of children that match the search and have read access
+                    names = [name for name, ignore_uid, ignore_type in (yield calresource.search(filter))]
                 except IndexedSearchException:
-                    records = yield maybeDeferred(calresource.index().bruteForceSearch)
+                    names = yield calresource.listChildren()
                     index_query_ok = False
-                names = [name for name, ignore_uid, ignore_type in records]
 
                 if not names:
                     returnValue(True)
@@ -218,7 +231,7 @@ def report_urn_ietf_params_xml_ns_caldav_calendar_query(self, request, calendar_
                     child_uri_name = child_uri[child_uri.rfind("/") + 1:]
 
                     if generate_calendar_data or not index_query_ok:
-                        calendar = (yield child.iCalendarForUser(request))
+                        calendar = (yield child.componentForUser())
                         assert calendar is not None, "Calendar %s is missing from calendar collection %r" % (child_uri_name, self)
                     else:
                         calendar = None
@@ -228,21 +241,21 @@ def report_urn_ietf_params_xml_ns_caldav_calendar_query(self, request, calendar_
             # Get the timezone property from the collection if one was not set in the query,
             # and store in the query object for later use
             timezone = query_timezone
-            if query_tz is None:
+            if timezone is None:
 
                 parent = (yield calresource.locateParent(request, uri))
                 assert parent is not None and parent.isPseudoCalendarCollection()
 
-                has_prop = (yield parent.hasProperty((caldav_namespace, "calendar-timezone"), request))
+                has_prop = (yield parent.hasProperty(CalendarTimeZone(), request))
                 if has_prop:
-                    tz = (yield parent.readProperty((caldav_namespace, "calendar-timezone"), request))
+                    tz = (yield parent.readProperty(CalendarTimeZone(), request))
                     filter.settimezone(tz)
                     timezone = tuple(tz.calendar().subcomponents())[0]
 
             # Check private events access status
             isowner = (yield calresource.isOwner(request))
 
-            calendar = (yield calresource.iCalendarForUser(request))
+            calendar = (yield calresource.componentForUser())
             yield queryCalendarObjectResource(calresource, uri, None, calendar, timezone)
 
         returnValue(True)
