@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.directory.test.test_calendar -*-
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,11 +33,11 @@ from twistedcaldav.config import config
 # around that for now.
 __import__("twistedcaldav.stdconfig") # FIXME
 
-from twext.web2 import responsecode
-from twext.web2.dav.http import ErrorResponse, MultiStatusResponse
-from twext.web2.dav.resource import davPrivilegeSet
-from twext.web2.dav.util import joinURL, normalizeURL
-from twext.web2.http import HTTPError
+from txweb2 import responsecode
+from txweb2.dav.http import ErrorResponse, MultiStatusResponse
+from txweb2.dav.resource import davPrivilegeSet
+from txweb2.dav.util import joinURL, normalizeURL
+from txweb2.http import HTTPError
 
 from twisted.internet.defer import inlineCallbacks, returnValue, succeed
 from twisted.python.failure import Failure
@@ -45,7 +45,7 @@ from twisted.python.failure import Failure
 from twistedcaldav import caldavxml, customxml
 from twistedcaldav.caldavxml import caldav_namespace, CalendarFreeBusySet
 from twistedcaldav.customxml import calendarserver_namespace
-from twistedcaldav.ical import allowedComponents, Component
+from twistedcaldav.ical import Component, allowedSchedulingComponents
 from twistedcaldav.resource import CalDAVResource
 from twistedcaldav.resource import isCalendarCollectionResource
 
@@ -149,6 +149,12 @@ class ScheduleInboxResource (CalendarSchedulingCollectionResource):
         )
 
 
+    def dynamicProperties(self):
+        return super(ScheduleInboxResource, self).dynamicProperties() + (
+            customxml.CalendarAvailability.qname(),
+        )
+
+
     def resourceType(self):
         return davxml.ResourceType.scheduleInbox
 
@@ -300,8 +306,14 @@ class ScheduleInboxResource (CalendarSchedulingCollectionResource):
         """
         Write either the default VEVENT or VTODO calendar property, validating and canonicalizing the value
         """
-        tasks = property.qname() == customxml.ScheduleDefaultTasksURL
-        error_element = (calendarserver_namespace, "valid-schedule-default-tasks-URL") if tasks else (caldav_namespace, "valid-schedule-default-calendar-URL")
+        if property.qname() == caldavxml.ScheduleDefaultCalendarURL.qname():
+            ctype = "VEVENT"
+            error_element = (caldav_namespace, "valid-schedule-default-calendar-URL")
+        elif property.qname() == customxml.ScheduleDefaultTasksURL.qname():
+            ctype = "VTODO"
+            error_element = (calendarserver_namespace, "valid-schedule-default-tasks-URL")
+        else:
+            returnValue(None)
 
         # Verify that the calendar added in the PROPPATCH is valid.
         property.children = [davxml.HRef(normalizeURL(str(href))) for href in property.children]
@@ -325,7 +337,7 @@ class ScheduleInboxResource (CalendarSchedulingCollectionResource):
 
         try:
             # Now set it on the new store object
-            yield self.parent._newStoreHome.setDefaultCalendar(cal._newStoreObject, tasks)
+            yield self.parent._newStoreHome.setDefaultCalendar(cal._newStoreObject, ctype)
         except InvalidDefaultCalendar as e:
             raise HTTPError(ErrorResponse(
                 responsecode.CONFLICT,
@@ -367,12 +379,14 @@ class ScheduleInboxResource (CalendarSchedulingCollectionResource):
         if config.Scheduling.CalDAV.OldDraftCompatibility:
             privs += (davxml.Privilege(caldavxml.Schedule()),)
 
-        return davxml.ACL(
-            # CalDAV:schedule-deliver for any authenticated user
-            davxml.ACE(
-                davxml.Principal(davxml.Authenticated()),
-                davxml.Grant(*privs),
-            ),
+        return succeed(
+            davxml.ACL(
+                # CalDAV:schedule-deliver for any authenticated user
+                davxml.ACE(
+                    davxml.Principal(davxml.Authenticated()),
+                    davxml.Grant(*privs),
+                ),
+            )
         )
 
 
@@ -390,7 +404,7 @@ class ScheduleOutboxResource (CalendarSchedulingCollectionResource):
 
     def getSupportedComponentSet(self):
         return caldavxml.SupportedCalendarComponentSet(
-            *[caldavxml.CalendarComponent(name=item) for item in allowedComponents]
+            *[caldavxml.CalendarComponent(name=item) for item in allowedSchedulingComponents]
         )
 
 
@@ -407,21 +421,10 @@ class ScheduleOutboxResource (CalendarSchedulingCollectionResource):
         # Check authentication and access controls
         yield self.authorize(request, (caldavxml.ScheduleSend(),))
 
-        calendar = (yield self.loadCalendarFromRequest(request))
+        calendar, format = (yield self.loadCalendarFromRequest(request))
         originator = (yield self.loadOriginatorFromRequestDetails(request))
         recipients = self.loadRecipientsFromCalendarData(calendar)
 
-        # storeComponent needs to know who the auth'd user is for access control
-        # TODO: this needs to be done in a better way - ideally when the txn is created for the request,
-        # we should set a txn.authzid attribute.
-        authz = None
-        authz_principal = self.parent.currentPrincipal(request).children[0]
-        if isinstance(authz_principal, davxml.HRef):
-            principalURL = str(authz_principal)
-            if principalURL:
-                authz = (yield request.locateResource(principalURL))
-                self._associatedTransaction._authz_uid = authz.record.guid
-
         # Log extended item
         if not hasattr(request, "extendedLogItems"):
             request.extendedLogItems = {}
@@ -431,14 +434,25 @@ class ScheduleOutboxResource (CalendarSchedulingCollectionResource):
 
         # Do the POST processing treating
         result = (yield scheduler.doSchedulingViaPOST(originator, recipients, calendar))
-        returnValue(result.response())
+        returnValue(result.response(format=format))
+
+
+    def determineType(self, content_type):
+        """
+        Determine if the supplied content-type is valid for storing and return the matching PyCalendar type.
+        """
+        format = None
+        if content_type is not None:
+            format = "%s/%s" % (content_type.mediaType, content_type.mediaSubtype,)
+        return format if format in Component.allowedTypes() else None
 
 
     @inlineCallbacks
     def loadCalendarFromRequest(self, request):
         # Must be content-type text/calendar
         contentType = request.headers.getHeader("content-type")
-        if contentType is not None and (contentType.mediaType, contentType.mediaSubtype) != ("text", "calendar"):
+        format = self.determineType(contentType)
+        if format is None:
             self.log.error("MIME type %s not allowed in calendar collection" % (contentType,))
             raise HTTPError(ErrorResponse(
                 responsecode.FORBIDDEN,
@@ -448,7 +462,7 @@ class ScheduleOutboxResource (CalendarSchedulingCollectionResource):
 
         # Parse the calendar object from the HTTP request stream
         try:
-            calendar = (yield Component.fromIStream(request.stream))
+            calendar = (yield Component.fromIStream(request.stream, format=format))
         except:
             # FIXME: Bare except
             self.log.error("Error while handling POST: %s" % (Failure(),))
@@ -458,22 +472,18 @@ class ScheduleOutboxResource (CalendarSchedulingCollectionResource):
                 description="Can't parse calendar data"
             ))
 
-        returnValue(calendar)
+        returnValue((calendar, format,))
 
 
     @inlineCallbacks
     def loadOriginatorFromRequestDetails(self, request):
-        # Get the originator who is the authenticated user
-        originatorPrincipal = None
+        # The originator is the owner of the Outbox. We will have checked prior to this
+        # that the authenticated user has privileges to schedule as the owner.
         originator = ""
-        authz_principal = self.currentPrincipal(request).children[0]
-        if isinstance(authz_principal, davxml.HRef):
-            originatorPrincipalURL = str(authz_principal)
-            if originatorPrincipalURL:
-                originatorPrincipal = (yield request.locateResource(originatorPrincipalURL))
-                if originatorPrincipal:
-                    # Pick the canonical CUA:
-                    originator = originatorPrincipal.canonicalCalendarUserAddress()
+        originatorPrincipal = (yield self.ownerPrincipal(request))
+        if originatorPrincipal:
+            # Pick the canonical CUA:
+            originator = originatorPrincipal.canonicalCalendarUserAddress()
 
         if not originator:
             self.log.error("%s request must have Originator" % (self.method,))
@@ -515,9 +525,10 @@ class ScheduleOutboxResource (CalendarSchedulingCollectionResource):
         return succeed(sendSchedulePrivilegeSet)
 
 
+    @inlineCallbacks
     def defaultAccessControlList(self):
         if config.EnableProxyPrincipals:
-            myPrincipal = self.parent.principalForRecord()
+            myPrincipal = yield self.parent.principalForRecord()
 
             privs = (
                 davxml.Privilege(caldavxml.ScheduleSend()),
@@ -525,16 +536,19 @@ class ScheduleOutboxResource (CalendarSchedulingCollectionResource):
             if config.Scheduling.CalDAV.OldDraftCompatibility:
                 privs += (davxml.Privilege(caldavxml.Schedule()),)
 
-            return davxml.ACL(
-                # CalDAV:schedule for associated write proxies
-                davxml.ACE(
-                    davxml.Principal(davxml.HRef(joinURL(myPrincipal.principalURL(), "calendar-proxy-write"))),
-                    davxml.Grant(*privs),
-                    davxml.Protected(),
-                ),
+            returnValue(
+                davxml.ACL(
+                    # CalDAV:schedule for associated write proxies
+                    davxml.ACE(
+                        davxml.Principal(davxml.HRef(joinURL(myPrincipal.principalURL(), "calendar-proxy-write"))),
+                        davxml.Grant(*privs),
+                        davxml.Protected(),
+                    ),
+                )
             )
         else:
-            return super(ScheduleOutboxResource, self).defaultAccessControlList()
+            result = yield super(ScheduleOutboxResource, self).defaultAccessControlList()
+            returnValue(result)
 
 
     def report_urn_ietf_params_xml_ns_caldav_calendar_query(self, request, calendar_query):
