@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,7 @@
 Generic SQL database access object.
 """
 
-__all__ = [ 
+__all__ = [
     "db_prefix",
     "DatabaseError",
     "AbstractSQLDatabase",
@@ -50,7 +50,7 @@ class AbstractSQLDatabase (object):
 
     def __init__(self, dbpath, persistent, autocommit=False):
         """
-        
+
         @param dbpath: the path where the db file is stored.
         @type dbpath: str
         @param persistent: C{True} if the data in the DB must be perserved during upgrades,
@@ -63,21 +63,25 @@ class AbstractSQLDatabase (object):
         self.persistent = persistent
         self.autocommit = autocommit
 
+
     def __repr__(self):
         return "<%s %r>" % (self.__class__.__name__, self.dbpath)
 
+
     def _db_version(self):
         """
         @return: the schema version assigned to this index.
         """
         raise NotImplementedError
-        
+
+
     def _db_type(self):
         """
         @return: the collection type assigned to this index.
         """
         raise NotImplementedError
-        
+
+
     def _db(self):
         """
         Access the underlying database.
@@ -108,7 +112,7 @@ class AbstractSQLDatabase (object):
                 # Create CALDAV table if needed
 
                 if self._test_schema_table(q):
-                    
+
                     version, dbtype = self._get_schema_version(q)
 
                     if (version != self._db_version()) or (dbtype != self._db_type()):
@@ -130,7 +134,7 @@ class AbstractSQLDatabase (object):
                         if version != self._db_version():
                             log.error("Database %s has different schema (v.%s vs. v.%s)"
                                       % (db_filename, version, self._db_version()))
-                            
+
                             # Upgrade the DB
                             return self._db_upgrade(version)
 
@@ -139,9 +143,11 @@ class AbstractSQLDatabase (object):
 
                 self._db_connection.commit()
             finally:
-                if q is not None: q.close()
+                if q is not None:
+                    q.close()
         return self._db_connection
 
+
     def _test_schema_table(self, q):
         q.execute("""
         select (1) from SQLITE_MASTER
@@ -149,6 +155,7 @@ class AbstractSQLDatabase (object):
         """)
         return q.fetchone()
 
+
     def _get_schema_version(self, q):
         q.execute(
             """
@@ -157,7 +164,8 @@ class AbstractSQLDatabase (object):
             """)
         version = q.fetchone()
 
-        if version is not None: version = version[0]
+        if version is not None:
+            version = version[0]
 
         q.execute(
             """
@@ -166,10 +174,12 @@ class AbstractSQLDatabase (object):
             """)
         dbtype = q.fetchone()
 
-        if dbtype is not None: dbtype = dbtype[0]
+        if dbtype is not None:
+            dbtype = dbtype[0]
 
         return version, dbtype
 
+
     def _db_init(self, db_filename, q):
         """
         Initialise the underlying database tables.
@@ -183,7 +193,7 @@ class AbstractSQLDatabase (object):
         old_isolation = self._db_connection.isolation_level
         self._db_connection.isolation_level = None
         q.execute("begin exclusive transaction")
-        
+
         # We re-check whether the schema table is present again AFTER we've got an exclusive
         # lock as some other server process may have snuck in and already created it
         # before we got the lock, or whilst we were waiting for it.
@@ -199,6 +209,7 @@ class AbstractSQLDatabase (object):
 
         self._db_connection.isolation_level = old_isolation
 
+
     def _db_init_schema_table(self, q):
         """
         Initialise the underlying database tables.
@@ -229,6 +240,7 @@ class AbstractSQLDatabase (object):
             """, [self._db_type()]
         )
 
+
     def _db_init_data_tables(self, q):
         """
         Initialise the underlying database tables.
@@ -237,6 +249,7 @@ class AbstractSQLDatabase (object):
         """
         raise NotImplementedError
 
+
     def _db_recreate(self, do_commit=True):
         """
         Recreate the database tables.
@@ -246,15 +259,17 @@ class AbstractSQLDatabase (object):
         if do_commit:
             self._db_commit()
 
+
     def _db_can_upgrade(self, old_version):
-        
+
         return self.persistent
 
+
     def _db_upgrade(self, old_version):
         """
         Upgrade the database tables.
         """
-        
+
         if self._db_can_upgrade(old_version):
             self._db_connection = sqlite.connect(self.dbpath, isolation_level=None)
             q = self._db_connection.cursor()
@@ -278,6 +293,7 @@ class AbstractSQLDatabase (object):
 
         return self._db()
 
+
     def _db_upgrade_data_tables(self, q, old_version):
         """
         Upgrade the data from an older version of the DB.
@@ -286,6 +302,7 @@ class AbstractSQLDatabase (object):
         # cannot be thrown away.
         raise NotImplementedError("Persistent databases MUST support an upgrade method.")
 
+
     def _db_upgrade_schema(self, q):
         """
         Upgrade the stored schema version to the current one.
@@ -297,11 +314,13 @@ class AbstractSQLDatabase (object):
             """, [self._db_version()]
         )
 
+
     def _db_close(self):
         if hasattr(self, "_db_connection"):
             self._db_connection.close()
             del self._db_connection
 
+
     def _db_values_for_sql(self, sql, *query_params):
         """
         Execute an SQL query and obtain the resulting values.
@@ -313,6 +332,7 @@ class AbstractSQLDatabase (object):
         """
         return (row[0] for row in self._db_execute(sql, *query_params))
 
+
     def _db_value_for_sql(self, sql, *query_params):
         """
         Execute an SQL query and obtain a single value.
@@ -328,6 +348,7 @@ class AbstractSQLDatabase (object):
             value = row
         return value
 
+
     def _db_execute(self, sql, *query_params):
         """
         Execute an SQL query and obtain the resulting values.
@@ -347,5 +368,10 @@ class AbstractSQLDatabase (object):
         finally:
             q.close()
 
-    def _db_commit  (self): self._db_connection.commit()
-    def _db_rollback(self): self._db_connection.rollback()
+
+    def _db_commit(self):
+        self._db_connection.commit()
+
+
+    def _db_rollback(self):
+        self._db_connection.rollback()
