@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,20 +16,24 @@
 
 import os
 
-from twext.web2 import responsecode
-from twext.web2.iweb import IResponse
-from twext.web2.stream import MemoryStream
-from txdav.xml import element as davxml
-from twext.web2.dav.util import davXMLFromStream, joinURL
+from txweb2 import responsecode
+from txweb2.iweb import IResponse
+from txweb2.stream import MemoryStream
+from txweb2.dav.util import davXMLFromStream, joinURL
+from txweb2.http_headers import Headers, MimeType
 
 from twistedcaldav import carddavxml
 from twistedcaldav import vcard
-
 from twistedcaldav.config import config
 from twistedcaldav.test.util import StoreTestCase, SimpleStoreRequest
+
 from twisted.python.filepath import FilePath
 from twisted.internet.defer import inlineCallbacks, returnValue
 
+from txdav.xml import element as davxml
+
+
+
 class AddressBookMultiget (StoreTestCase):
     """
     addressbook-multiget REPORT
@@ -37,6 +41,13 @@ class AddressBookMultiget (StoreTestCase):
     data_dir = os.path.join(os.path.dirname(__file__), "data")
     vcards_dir = os.path.join(data_dir, "vCards")
 
+
+    @inlineCallbacks
+    def setUp(self):
+        yield StoreTestCase.setUp(self)
+        self.authPrincipal = yield self.actualRoot.findPrincipalForAuthID("wsanchez")
+
+
     def test_multiget_some_vcards(self):
         """
         All vcards.
@@ -135,8 +146,10 @@ class AddressBookMultiget (StoreTestCase):
                     status = propstat.childOfType(davxml.Status)
 
                     if status.code != responsecode.OK:
-                        self.fail("REPORT failed (status %s) to locate properties: %r"
-                              % (status.code, href))
+                        self.fail(
+                            "REPORT failed (status %s) to locate properties: %r"
+                            % (status.code, href)
+                        )
 
                     properties = propstat.childOfType(davxml.PropertyContainer).children
 
@@ -176,8 +189,10 @@ class AddressBookMultiget (StoreTestCase):
                     status = response.childOfType(davxml.Status)
                 if status.code != responsecode.OK:
                     if href in okhrefs:
-                        self.fail("REPORT failed (status %s) to locate properties: %r"
-                              % (status.code, href))
+                        self.fail(
+                            "REPORT failed (status %s) to locate properties: %r"
+                            % (status.code, href)
+                        )
                     else:
                         if href in badhrefs:
                             badhrefs.remove(href)
@@ -205,7 +220,7 @@ class AddressBookMultiget (StoreTestCase):
 </D:set>
 </D:mkcol>
 """
-            response = yield self.send(SimpleStoreRequest(self, "MKCOL", addressbook_uri, content=mkcol, authid="wsanchez"))
+            response = yield self.send(SimpleStoreRequest(self, "MKCOL", addressbook_uri, content=mkcol, authPrincipal=self.authPrincipal))
 
             response = IResponse(response)
 
@@ -214,7 +229,13 @@ class AddressBookMultiget (StoreTestCase):
             '''
             if data:
                 for filename, icaldata in data.iteritems():
-                    request = SimpleStoreRequest(self, "PUT", joinURL(addressbook_uri, filename + ".vcf"), authid="wsanchez")
+                    request = SimpleStoreRequest(
+                        self,
+                        "PUT",
+                        joinURL(addressbook_uri, filename + ".vcf"),
+                        headers=Headers({"content-type": MimeType.fromString("text/vcard")}),
+                        authPrincipal=self.authPrincipal
+                    )
                     request.stream = MemoryStream(icaldata)
                     yield self.send(request)
             else:
@@ -222,11 +243,17 @@ class AddressBookMultiget (StoreTestCase):
                 for child in FilePath(self.vcards_dir).children():
                     if os.path.splitext(child.basename())[1] != ".vcf":
                         continue
-                    request = SimpleStoreRequest(self, "PUT", joinURL(addressbook_uri, child.basename()), authid="wsanchez")
+                    request = SimpleStoreRequest(
+                        self,
+                        "PUT",
+                        joinURL(addressbook_uri, child.basename()),
+                        headers=Headers({"content-type": MimeType.fromString("text/vcard")}),
+                        authPrincipal=self.authPrincipal
+                    )
                     request.stream = MemoryStream(child.getContent())
                     yield self.send(request)
 
-        request = SimpleStoreRequest(self, "REPORT", addressbook_uri, authid="wsanchez")
+        request = SimpleStoreRequest(self, "REPORT", addressbook_uri, authPrincipal=self.authPrincipal)
         request.stream = MemoryStream(query.toxml())
         response = yield self.send(request)
 
