@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,11 +16,11 @@
 
 import os
 
-from twext.web2 import responsecode
-from twext.web2.iweb import IResponse
-from twext.web2.stream import MemoryStream
+from txweb2 import responsecode
+from txweb2.iweb import IResponse
+from txweb2.stream import MemoryStream
 from txdav.xml import element as davxml
-from twext.web2.dav.util import davXMLFromStream, joinURL
+from txweb2.dav.util import davXMLFromStream, joinURL
 
 from twistedcaldav import carddavxml, vcard
 from twistedcaldav.config import config
@@ -28,6 +28,8 @@ from twistedcaldav.test.util import StoreTestCase, SimpleStoreRequest
 from twisted.internet.defer import inlineCallbacks, returnValue
 from twisted.python.filepath import FilePath
 
+
+
 class AddressBookQuery(StoreTestCase):
     """
     addressbook-query REPORT
@@ -67,6 +69,7 @@ class AddressBookQuery(StoreTestCase):
 
         oldValue = config.MaxQueryWithDataResults
         config.MaxQueryWithDataResults = 1
+
         def _restoreValueOK(f):
             config.MaxQueryWithDataResults = oldValue
             return None
@@ -89,6 +92,7 @@ class AddressBookQuery(StoreTestCase):
 
         oldValue = config.MaxQueryWithDataResults
         config.MaxQueryWithDataResults = 1
+
         def _restoreValueOK(f):
             config.MaxQueryWithDataResults = oldValue
             return None
@@ -191,15 +195,16 @@ class AddressBookQuery(StoreTestCase):
         if response.code != responsecode.CREATED:
             self.fail("MKCOL failed: %s" % (response.code,))
         '''
+        principal = yield self.actualRoot.findPrincipalForAuthID("wsanchez")
         # Add vCards to addressbook
         for child in FilePath(self.vcards_dir).children():
             if os.path.splitext(child.basename())[1] != ".vcf":
                 continue
-            request = SimpleStoreRequest(self, "PUT", joinURL(addressbook_uri, child.basename()), authid="wsanchez")
+            request = SimpleStoreRequest(self, "PUT", joinURL(addressbook_uri, child.basename()), authPrincipal=principal)
             request.stream = MemoryStream(child.getContent())
             yield self.send(request)
 
-        request = SimpleStoreRequest(self, "REPORT", addressbook_uri, authid="wsanchez")
+        request = SimpleStoreRequest(self, "REPORT", addressbook_uri, authPrincipal=principal)
         request.stream = MemoryStream(query.toxml())
         response = yield self.send(request)
 
