@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,11 +20,9 @@ import cPickle
 
 from twisted.internet.defer import succeed, maybeDeferred, inlineCallbacks
 
-from twext.web2.dav.util import allDataFromStream
-from twext.web2.stream import MemoryStream
-from twext.web2.http_headers import Headers
-
-from txdav.xml import element as davxml
+from txweb2.dav.util import allDataFromStream
+from txweb2.stream import MemoryStream
+from txweb2.http_headers import Headers
 
 from twistedcaldav.cache import MemcacheResponseCache, CacheStoreNotifier
 from twistedcaldav.cache import MemcacheChangeNotifier
@@ -32,6 +30,7 @@ from twistedcaldav.cache import PropfindCacheMixin
 
 from twistedcaldav.test.util import InMemoryMemcacheProtocol
 from twistedcaldav.test.util import TestCase
+from txdav.xml import element
 
 
 def _newCacheToken(self):
@@ -63,6 +62,10 @@ class StubDirectoryRecord(object):
 
 class StubDirectory(object):
 
+    def oldNameToRecordType(self, oldName):
+        return oldName
+
+
     def recordWithShortName(self, recordType, recordName):
         return StubDirectoryRecord(recordName)
 
@@ -86,13 +89,27 @@ class StubSite(object):
 
 
 
+class StubPrincipal(object):
+    def __init__(self, user):
+        self.user = user
+
+
+    def principalURL(self):
+        return self.user
+
+
+    def principalElement(self):
+        return element.Principal(element.HRef.fromString(self.user))
+
+
+
 class StubRequest(object):
     resources = {}
 
     def __init__(self, method, uri, authnUser, depth='1', body=None):
         self.method = method
         self.uri = uri
-        self.authnUser = davxml.Principal(davxml.HRef.fromString(authnUser))
+        self.authnUser = StubPrincipal(authnUser)
         self.headers = Headers({'depth': depth})
 
         if body is None:
@@ -237,9 +254,10 @@ class BaseCacheTestMixin(object):
 
     def test_getResponseForRequestNotInCache(self):
         d = self.rc.getResponseForRequest(StubRequest(
-                'PROPFIND',
-                '/calendars/__uids__/dreid/',
-                '/principals/__uids__/dreid/'))
+            'PROPFIND',
+            '/calendars/__uids__/dreid/',
+            '/principals/__uids__/dreid/'
+        ))
 
         d.addCallback(self.assertEquals, None)
         return d
@@ -247,9 +265,10 @@ class BaseCacheTestMixin(object):
 
     def test_getResponseForRequestInCache(self):
         d = self.rc.getResponseForRequest(StubRequest(
-                'PROPFIND',
-                '/calendars/__uids__/cdaboo/',
-                '/principals/__uids__/cdaboo/'))
+            'PROPFIND',
+            '/calendars/__uids__/cdaboo/',
+            '/principals/__uids__/cdaboo/'
+        ))
 
         d.addCallback(self.assertResponse, self.expected_response)
         return d
@@ -259,9 +278,10 @@ class BaseCacheTestMixin(object):
         self.tokens['/principals/__uids__/cdaboo/'] = 'principalToken1'
 
         d = self.rc.getResponseForRequest(StubRequest(
-                'PROPFIND',
-                '/calendars/__uids__/cdaboo/',
-                '/principals/__uids__/cdaboo/'))
+            'PROPFIND',
+            '/calendars/__uids__/cdaboo/',
+            '/principals/__uids__/cdaboo/'
+        ))
 
         d.addCallback(self.assertEquals, None)
         return d
@@ -271,9 +291,10 @@ class BaseCacheTestMixin(object):
         self.tokens['/calendars/__uids__/cdaboo/'] = 'uriToken1'
 
         d = self.rc.getResponseForRequest(StubRequest(
-                'PROPFIND',
-                '/calendars/__uids__/cdaboo/',
-                '/principals/__uids__/cdaboo/'))
+            'PROPFIND',
+            '/calendars/__uids__/cdaboo/',
+            '/principals/__uids__/cdaboo/'
+        ))
 
         d.addCallback(self.assertEquals, None)
         return d
@@ -283,9 +304,10 @@ class BaseCacheTestMixin(object):
         self.tokens['/calendars/__uids__/cdaboo/calendars/'] = 'childToken1'
 
         d = self.rc.getResponseForRequest(StubRequest(
-                'PROPFIND',
-                '/calendars/__uids__/cdaboo/',
-                '/principals/__uids__/cdaboo/'))
+            'PROPFIND',
+            '/calendars/__uids__/cdaboo/',
+            '/principals/__uids__/cdaboo/'
+        ))
 
         d.addCallback(self.assertEquals, None)
         return d
@@ -293,10 +315,11 @@ class BaseCacheTestMixin(object):
 
     def test_getResponseForDepthZero(self):
         d = self.rc.getResponseForRequest(StubRequest(
-                'PROPFIND',
-                '/calendars/__uids__/cdaboo/',
-                '/principals/__uids__/cdaboo/',
-                depth='0'))
+            'PROPFIND',
+            '/calendars/__uids__/cdaboo/',
+            '/principals/__uids__/cdaboo/',
+            depth='0'
+        ))
 
         d.addCallback(self.assertEquals, None)
         return d
@@ -304,10 +327,11 @@ class BaseCacheTestMixin(object):
 
     def test_getResponseForBody(self):
         d = self.rc.getResponseForRequest(StubRequest(
-                'PROPFIND',
-                '/calendars/__uids__/cdaboo/',
-                '/principals/__uids__/cdaboo/',
-                body='bazbax'))
+            'PROPFIND',
+            '/calendars/__uids__/cdaboo/',
+            '/principals/__uids__/cdaboo/',
+            body='bazbax'
+        ))
 
         d.addCallback(self.assertEquals, None)
         return d
@@ -315,10 +339,11 @@ class BaseCacheTestMixin(object):
 
     def test_getResponseForUnauthenticatedRequest(self):
         d = self.rc.getResponseForRequest(StubRequest(
-                'PROPFIND',
-                '/calendars/__uids__/cdaboo/',
-                '{DAV:}unauthenticated',
-                body='bazbax'))
+            'PROPFIND',
+            '/calendars/__uids__/cdaboo/',
+            '{DAV:}unauthenticated',
+            body='bazbax'
+        ))
 
         d.addCallback(self.assertEquals, None)
         return d
@@ -346,9 +371,10 @@ class BaseCacheTestMixin(object):
 
         def _assertResponse(ign):
             d1 = self.rc.getResponseForRequest(StubRequest(
-                    'PROPFIND',
-                    '/principals/__uids__/dreid/',
-                    '/principals/__uids__/dreid/'))
+                'PROPFIND',
+                '/principals/__uids__/dreid/',
+                '/principals/__uids__/dreid/'
+            ))
 
             d1.addCallback(self.assertResponse,
                            (expected_response.code,
@@ -357,9 +383,11 @@ class BaseCacheTestMixin(object):
             return d1
 
         d = self.rc.cacheResponseForRequest(
-            StubRequest('PROPFIND',
-                        '/principals/__uids__/dreid/',
-                        '/principals/__uids__/dreid/'),
+            StubRequest(
+                'PROPFIND',
+                '/principals/__uids__/dreid/',
+                '/principals/__uids__/dreid/'
+            ),
             expected_response)
 
         d.addCallback(_assertResponse)
@@ -402,23 +430,27 @@ class MemcacheResponseCacheTests(BaseCacheTestMixin, TestCase):
         self.expected_response = (200, Headers({}), "Foo")
 
         expected_key = hashlib.md5(':'.join([str(t) for t in (
-                'PROPFIND',
-                '/principals/__uids__/cdaboo/',
-                '/calendars/__uids__/cdaboo/',
-                '1',
-                hash('foobar'),
-                )])).hexdigest()
+            'PROPFIND',
+            '/principals/__uids__/cdaboo/',
+            '/calendars/__uids__/cdaboo/',
+            '1',
+            hash('foobar'),
+        )])).hexdigest()
 
         memcacheStub._cache[expected_key] = (
             0, #flags
             cPickle.dumps((
-            'principalToken0',
-            StubDirectoryRecord('cdaboo').cacheToken(),
-            'uriToken0',
-            {'/calendars/__uids__/cdaboo/calendars/': 'childToken0'},
-            (self.expected_response[0],
-             dict(list(self.expected_response[1].getAllRawHeaders())),
-             self.expected_response[2]))))
+                'principalToken0',
+                StubDirectoryRecord('cdaboo').cacheToken(),
+                'uriToken0',
+                {'/calendars/__uids__/cdaboo/calendars/': 'childToken0'},
+                (
+                    self.expected_response[0],
+                    dict(list(self.expected_response[1].getAllRawHeaders())),
+                    self.expected_response[2]
+                )
+            ))
+        )
 
         self.memcacheStub = memcacheStub
 
@@ -432,25 +464,29 @@ class MemcacheResponseCacheTests(BaseCacheTestMixin, TestCase):
         expected_response = (200, Headers({}), "Foobarbaz")
 
         _key = (
-                'PROPFIND',
-                '/principals/__uids__/cdaboo/',
-                '/calendars/users/cdaboo/',
-                '1',
-                hash('foobar'),
-                )
+            'PROPFIND',
+            '/principals/__uids__/cdaboo/',
+            '/calendars/users/cdaboo/',
+            '1',
+            hash('foobar'),
+        )
 
         expected_key = hashlib.md5(':'.join([str(t) for t in _key])).hexdigest()
 
         self.memcacheStub._cache[expected_key] = (
             0, #flags
             cPickle.dumps((
-                    'principalToken0',
-                    StubDirectoryRecord('cdaboo').cacheToken(),
-                    'uriToken0',
-                    {'/calendars/__uids__/cdaboo/calendars/': 'childToken0'},
-                    (expected_response[0],
-                     dict(list(expected_response[1].getAllRawHeaders())),
-                     expected_response[2]))))
+                'principalToken0',
+                StubDirectoryRecord('cdaboo').cacheToken(),
+                'uriToken0',
+                {'/calendars/__uids__/cdaboo/calendars/': 'childToken0'},
+                (
+                    expected_response[0],
+                    dict(list(expected_response[1].getAllRawHeaders())),
+                    expected_response[2]
+                )
+            ))
+        )
 
         d = self.rc.getResponseForRequest(
             StubRequest('PROPFIND',
