@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,33 +18,34 @@ import os
 
 from twisted.trial.unittest import SkipTest
 
-from twext.web2 import responsecode
-from twext.web2.iweb import IResponse
-from twext.web2.stream import MemoryStream
+from txweb2 import responsecode
+from txweb2.iweb import IResponse
+from txweb2.stream import MemoryStream
 from txdav.xml import element as davxml
-from twext.web2.dav.util import davXMLFromStream
+from txweb2.dav.util import davXMLFromStream, allDataFromStream
 
 from twistedcaldav import caldavxml
 from twistedcaldav import ical
 
-from twistedcaldav.query import calendarqueryfilter
 from twistedcaldav.config import config
 from twistedcaldav.test.util import StoreTestCase, SimpleStoreRequest
 from twisted.internet.defer import inlineCallbacks, returnValue
 
-from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.datetime import DateTime
 from twistedcaldav.ical import Component
 from txdav.caldav.icalendarstore import ComponentUpdateState
-from twistedcaldav.directory.directory import DirectoryService
+from txdav.caldav.datastore.query.filter import TimeRange
+from twext.who.idirectory import RecordType
+from twistedcaldav.timezones import readVTZ, TimezoneCache
 
 
 @inlineCallbacks
 def addEventsDir(testCase, eventsDir, uri):
     """
-    Add events to a L{HomeTestCase} from a directory.
+    Add events to a L{twistedcaldav.test.util.TestCase} from a directory.
 
     @param testCase: The test case to add events to.
-    @type testCase: L{HomeTestCase}
+    @type testCase: L{twistedcaldav.test.util.TestCase}
 
     @param eventsDir: A directory full of events.
     @type eventsDir: L{FilePath}
@@ -79,7 +80,7 @@ class CalendarQuery (StoreTestCase):
         """
         Put the contents of the Holidays directory into the store.
         """
-        record = self.directory.recordWithShortName(DirectoryService.recordType_users, "wsanchez")
+        record = yield self.directory.recordWithShortName(RecordType.user, u"wsanchez")
         yield self.transactionUnderTest().calendarHomeWithUID(record.uid, create=True)
         calendar = yield self.calendarUnderTest(name="calendar", home=record.uid)
         for f in os.listdir(self.holidays_dir):
@@ -124,8 +125,8 @@ class CalendarQuery (StoreTestCase):
         )
 
         query_timerange = caldavxml.TimeRange(
-            start="%04d1001T000000Z" % (PyCalendarDateTime.getToday().getYear(),),
-            end="%04d1101T000000Z" % (PyCalendarDateTime.getToday().getYear(),),
+            start="%04d1001T000000Z" % (DateTime.getToday().getYear(),),
+            end="%04d1101T000000Z" % (DateTime.getToday().getYear(),),
         )
 
         query = caldavxml.CalendarQuery(
@@ -167,13 +168,174 @@ class CalendarQuery (StoreTestCase):
                             cal = property.calendar()
                             instances = cal.expandTimeRanges(query_timerange.end)
                             vevents = [x for x in cal.subcomponents() if x.name() == "VEVENT"]
-                            if not calendarqueryfilter.TimeRange(query_timerange).matchinstance(vevents[0], instances):
+                            if not TimeRange(query_timerange).matchinstance(vevents[0], instances):
                                 self.fail("REPORT property %r returned calendar %s outside of request time range %r"
                                           % (property, property.calendar, query_timerange))
 
         return self.calendar_query(query, got_xml)
 
 
+    def test_calendar_query_timezone(self):
+        """
+        Partial retrieval of events by time range.
+        (CalDAV-access-09, section 7.6.1)
+        """
+        TimezoneCache.create()
+        self.addCleanup(TimezoneCache.clear)
+
+        tzid1 = "Etc/GMT+1"
+        tz1 = Component(None, pycalendar=readVTZ(tzid1))
+
+        calendar_properties = (
+            davxml.GETETag(),
+            caldavxml.CalendarData(),
+        )
+
+        query_timerange = caldavxml.TimeRange(
+            start="%04d1001T000000Z" % (DateTime.getToday().getYear(),),
+            end="%04d1101T000000Z" % (DateTime.getToday().getYear(),),
+        )
+
+        query = caldavxml.CalendarQuery(
+            davxml.PropertyContainer(*calendar_properties),
+            caldavxml.Filter(
+                caldavxml.ComponentFilter(
+                    caldavxml.ComponentFilter(
+                        query_timerange,
+                        name="VEVENT",
+                    ),
+                    name="VCALENDAR",
+                ),
+            ),
+            caldavxml.TimeZone.fromCalendar(tz1),
+        )
+
+        def got_xml(doc):
+            if not isinstance(doc.root_element, davxml.MultiStatus):
+                self.fail("REPORT response XML root element is not multistatus: %r" % (doc.root_element,))
+
+        return self.calendar_query(query, got_xml)
+
+
+    def test_calendar_query_timezone_id(self):
+        """
+        Partial retrieval of events by time range.
+        (CalDAV-access-09, section 7.6.1)
+        """
+        TimezoneCache.create()
+        self.addCleanup(TimezoneCache.clear)
+
+        tzid1 = "Etc/GMT+1"
+
+        calendar_properties = (
+            davxml.GETETag(),
+            caldavxml.CalendarData(),
+        )
+
+        query_timerange = caldavxml.TimeRange(
+            start="%04d1001T000000Z" % (DateTime.getToday().getYear(),),
+            end="%04d1101T000000Z" % (DateTime.getToday().getYear(),),
+        )
+
+        query = caldavxml.CalendarQuery(
+            davxml.PropertyContainer(*calendar_properties),
+            caldavxml.Filter(
+                caldavxml.ComponentFilter(
+                    caldavxml.ComponentFilter(
+                        query_timerange,
+                        name="VEVENT",
+                    ),
+                    name="VCALENDAR",
+                ),
+            ),
+            caldavxml.TimeZoneID.fromString(tzid1),
+        )
+
+        def got_xml(doc):
+            if not isinstance(doc.root_element, davxml.MultiStatus):
+                self.fail("REPORT response XML root element is not multistatus: %r" % (doc.root_element,))
+
+        return self.calendar_query(query, got_xml)
+
+
+    @inlineCallbacks
+    def test_calendar_query_bogus_timezone_id(self):
+        """
+        Partial retrieval of events by time range.
+        (CalDAV-access-09, section 7.6.1)
+        """
+        TimezoneCache.create()
+        self.addCleanup(TimezoneCache.clear)
+
+        calendar_properties = (
+            davxml.GETETag(),
+            caldavxml.CalendarData(),
+        )
+
+        query_timerange = caldavxml.TimeRange(
+            start="%04d1001T000000Z" % (DateTime.getToday().getYear(),),
+            end="%04d1101T000000Z" % (DateTime.getToday().getYear(),),
+        )
+
+        query = caldavxml.CalendarQuery(
+            davxml.PropertyContainer(*calendar_properties),
+            caldavxml.Filter(
+                caldavxml.ComponentFilter(
+                    caldavxml.ComponentFilter(
+                        query_timerange,
+                        name="VEVENT",
+                    ),
+                    name="VCALENDAR",
+                ),
+            ),
+            caldavxml.TimeZoneID.fromString("bogus"),
+        )
+
+        result = yield self.calendar_query(query, got_xml=None, expected_code=responsecode.FORBIDDEN)
+        self.assertTrue("valid-timezone" in result)
+
+
+    @inlineCallbacks
+    def test_calendar_query_wrong_timezone_elements(self):
+        """
+        Partial retrieval of events by time range.
+        (CalDAV-access-09, section 7.6.1)
+        """
+        TimezoneCache.create()
+        self.addCleanup(TimezoneCache.clear)
+
+        tzid1 = "Etc/GMT+1"
+        tz1 = Component(None, pycalendar=readVTZ(tzid1))
+
+        calendar_properties = (
+            davxml.GETETag(),
+            caldavxml.CalendarData(),
+        )
+
+        query_timerange = caldavxml.TimeRange(
+            start="%04d1001T000000Z" % (DateTime.getToday().getYear(),),
+            end="%04d1101T000000Z" % (DateTime.getToday().getYear(),),
+        )
+
+        query = caldavxml.CalendarQuery(
+            davxml.PropertyContainer(*calendar_properties),
+            caldavxml.Filter(
+                caldavxml.ComponentFilter(
+                    caldavxml.ComponentFilter(
+                        query_timerange,
+                        name="VEVENT",
+                    ),
+                    name="VCALENDAR",
+                ),
+            ),
+            caldavxml.TimeZone.fromCalendar(tz1),
+        )
+        query.children += (caldavxml.TimeZoneID.fromString(tzid1),)
+
+        result = yield self.calendar_query(query, got_xml=None, expected_code=responsecode.BAD_REQUEST)
+        self.assertTrue("Only one of" in result)
+
+
     def test_calendar_query_partial_recurring(self):
         """
         Partial retrieval of recurring events.
@@ -248,6 +410,7 @@ class CalendarQuery (StoreTestCase):
         """
 
         self.patch(config, "MaxQueryWithDataResults", 1)
+
         def _restoreValueOK(f):
             self.fail("REPORT must fail with 403")
 
@@ -268,6 +431,7 @@ class CalendarQuery (StoreTestCase):
         """
 
         self.patch(config, "MaxQueryWithDataResults", 1)
+
         def _restoreValueError(f):
             self.fail("REPORT must not fail with 403")
 
@@ -341,17 +505,23 @@ class CalendarQuery (StoreTestCase):
 
 
     @inlineCallbacks
-    def calendar_query(self, query, got_xml):
+    def calendar_query(self, query, got_xml, expected_code=responsecode.MULTI_STATUS):
 
-        request = SimpleStoreRequest(self, "REPORT", "/calendars/users/wsanchez/calendar/", authid="wsanchez")
+        principal = yield self.actualRoot.findPrincipalForAuthID("wsanchez")
+        request = SimpleStoreRequest(self, "REPORT", "/calendars/users/wsanchez/calendar/", authPrincipal=principal)
         request.stream = MemoryStream(query.toxml())
         response = yield self.send(request)
 
         response = IResponse(response)
 
-        if response.code != responsecode.MULTI_STATUS:
+        if response.code != expected_code:
             self.fail("REPORT failed: %s" % (response.code,))
 
-        returnValue(
-            (yield davXMLFromStream(response.stream).addCallback(got_xml))
-        )
+        if got_xml is not None:
+            returnValue(
+                (yield davXMLFromStream(response.stream).addCallback(got_xml))
+            )
+        else:
+            returnValue(
+                (yield allDataFromStream(response.stream))
+            )
