@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,9 +14,9 @@
 # limitations under the License.
 ##
 
-from twext.web2 import responsecode
-from twext.web2.http import HTTPError
-from twext.web2.test.test_server import SimpleRequest
+from txweb2 import responsecode
+from txweb2.http import HTTPError
+from txweb2.test.test_server import SimpleRequest
 
 from twisted.internet.defer import inlineCallbacks, succeed
 
