@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,15 +18,17 @@ import os
 
 from twisted.internet.defer import inlineCallbacks
 
-from twext.web2 import responsecode
-from twext.web2.iweb import IResponse
-from twext.web2.stream import MemoryStream
+from txweb2 import responsecode
+from txweb2.iweb import IResponse
+from txweb2.stream import MemoryStream
 from txdav.xml import element as davxml
-from twext.web2.dav.fileop import rmdir
+from txweb2.dav.fileop import rmdir
 
 from twistedcaldav import caldavxml
 from twistedcaldav.test.util import StoreTestCase, SimpleStoreRequest
 
+
+
 class MKCALENDAR (StoreTestCase):
     """
     MKCALENDAR request
@@ -35,6 +37,12 @@ class MKCALENDAR (StoreTestCase):
     # Try nesting calendars (should fail)
     # HEAD request on calendar: resourcetype = (collection, calendar)
 
+    @inlineCallbacks
+    def setUp(self):
+        yield StoreTestCase.setUp(self)
+        self.authPrincipal = yield self.actualRoot.findPrincipalForAuthID("user01")
+
+
     def test_make_calendar(self):
         """
         Make calendar
@@ -45,7 +53,7 @@ class MKCALENDAR (StoreTestCase):
         if os.path.exists(path):
             rmdir(path)
 
-        request = SimpleStoreRequest(self, "MKCALENDAR", uri, authid="user01")
+        request = SimpleStoreRequest(self, "MKCALENDAR", uri, authPrincipal=self.authPrincipal)
 
         @inlineCallbacks
         def do_test(response):
@@ -118,7 +126,7 @@ class MKCALENDAR (StoreTestCase):
                     caldavxml.CalendarDescription("Calendar restricted to events."), # FIXME: lang=en
                     caldavxml.SupportedCalendarComponentSet(caldavxml.CalendarComponent(name="VEVENT")),
                     caldavxml.CalendarTimeZone(
-"""BEGIN:VCALENDAR
+                        """BEGIN:VCALENDAR
 PRODID:-//Example Corp.//CalDAV Client//EN
 VERSION:2.0
 BEGIN:VTIMEZONE
@@ -146,7 +154,7 @@ END:VCALENDAR
             )
         )
 
-        request = SimpleStoreRequest(self, "MKCALENDAR", uri, authid="user01")
+        request = SimpleStoreRequest(self, "MKCALENDAR", uri, authPrincipal=self.authPrincipal)
         request.stream = MemoryStream(mk.toxml())
         return self.send(request, do_test)
 
@@ -165,7 +173,7 @@ END:VCALENDAR
 
             # FIXME: Check for DAV:resource-must-be-null element
 
-        request = SimpleStoreRequest(self, "MKCALENDAR", uri, authid="user01")
+        request = SimpleStoreRequest(self, "MKCALENDAR", uri, authPrincipal=self.authPrincipal)
         return self.send(request, do_test)
 
 
@@ -190,8 +198,8 @@ END:VCALENDAR
 
             nested_uri = os.path.join(first_uri, "nested")
 
-            request = SimpleStoreRequest(self, "MKCALENDAR", nested_uri, authid="user01")
+            request = SimpleStoreRequest(self, "MKCALENDAR", nested_uri, authPrincipal=self.authPrincipal)
             yield self.send(request, do_test)
 
-        request = SimpleStoreRequest(self, "MKCALENDAR", first_uri, authid="user01")
+        request = SimpleStoreRequest(self, "MKCALENDAR", first_uri, authPrincipal=self.authPrincipal)
         return self.send(request, next)
