@@ -1,4 +1,4 @@
-# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,10 +14,11 @@
 ##
 
 from twext.python.filepath import CachingFilePath as FilePath
-from twext.web2 import responsecode
-from twext.web2.dav.util import davXMLFromStream, joinURL
-from twext.web2.iweb import IResponse
-from twext.web2.stream import MemoryStream
+from txweb2 import responsecode
+from txweb2.dav.util import davXMLFromStream, joinURL
+from txweb2.http_headers import Headers, MimeType
+from txweb2.iweb import IResponse
+from txweb2.stream import MemoryStream
 
 from twisted.internet.defer import inlineCallbacks, returnValue
 
@@ -37,6 +38,12 @@ class CalendarMultiget (StoreTestCase):
     data_dir = os.path.join(os.path.dirname(__file__), "data")
     holidays_dir = os.path.join(data_dir, "Holidays")
 
+    @inlineCallbacks
+    def setUp(self):
+        yield StoreTestCase.setUp(self)
+        self.authPrincipal = yield self.actualRoot.findPrincipalForAuthID("wsanchez")
+
+
     def test_multiget_some_events(self):
         """
         All events.
@@ -129,6 +136,7 @@ DTEND;VALUE=DATE:20020102
 DTSTAMP:20020101T121212Z
 RRULE:FREQ=YEARLY;INTERVAL=1;UNTIL=20031231;BYMONTH=1
 SUMMARY:New Year's Day
+TRANSP:TRANSPARENT
 END:VEVENT
 END:VCALENDAR
 """.replace("\n", "\r\n"),
@@ -143,6 +151,7 @@ DTEND;VALUE=DATE:20020215
 DTSTAMP:20020101T121212Z
 RRULE:FREQ=YEARLY;INTERVAL=1;BYMONTH=2
 SUMMARY:Valentine's Day
+TRANSP:TRANSPARENT
 END:VEVENT
 END:VCALENDAR
 """.replace("\n", "\r\n")
@@ -201,8 +210,9 @@ END:VCALENDAR
                     status = propstat.childOfType(davxml.Status)
 
                     if status.code != responsecode.OK:
-                        self.fail("REPORT failed (status %s) to locate properties: %r"
-                              % (status.code, href))
+                        self.fail(
+                            "REPORT failed (status %s) to locate properties: %r"
+                            % (status.code, href))
 
                     properties = propstat.childOfType(davxml.PropertyContainer).children
 
@@ -242,8 +252,9 @@ END:VCALENDAR
                     status = response.childOfType(davxml.Status)
                 if status.code != responsecode.OK:
                     if href in okhrefs:
-                        self.fail("REPORT failed (status %s) to locate properties: %r"
-                              % (status.code, href))
+                        self.fail(
+                            "REPORT failed (status %s) to locate properties: %r"
+                            % (status.code, href))
                     else:
                         if href in badhrefs:
                             badhrefs.remove(href)
@@ -261,14 +272,20 @@ END:VCALENDAR
     def calendar_query(self, calendar_uri, query, got_xml, data, no_init):
 
         if not no_init:
-            response = yield self.send(SimpleStoreRequest(self, "MKCALENDAR", calendar_uri, authid="wsanchez"))
+            response = yield self.send(SimpleStoreRequest(self, "MKCALENDAR", calendar_uri, authPrincipal=self.authPrincipal))
             response = IResponse(response)
             if response.code != responsecode.CREATED:
                 self.fail("MKCALENDAR failed: %s" % (response.code,))
 
             if data:
                 for filename, icaldata in data.iteritems():
-                    request = SimpleStoreRequest(self, "PUT", joinURL(calendar_uri, filename + ".ics"), authid="wsanchez")
+                    request = SimpleStoreRequest(
+                        self,
+                        "PUT",
+                        joinURL(calendar_uri, filename + ".ics"),
+                        headers=Headers({"content-type": MimeType.fromString("text/calendar")}),
+                        authPrincipal=self.authPrincipal
+                    )
                     request.stream = MemoryStream(icaldata)
                     yield self.send(request)
             else:
@@ -276,11 +293,17 @@ END:VCALENDAR
                 for child in FilePath(self.holidays_dir).children():
                     if os.path.splitext(child.basename())[1] != ".ics":
                         continue
-                    request = SimpleStoreRequest(self, "PUT", joinURL(calendar_uri, child.basename()), authid="wsanchez")
+                    request = SimpleStoreRequest(
+                        self,
+                        "PUT",
+                        joinURL(calendar_uri, child.basename()),
+                        headers=Headers({"content-type": MimeType.fromString("text/calendar")}),
+                        authPrincipal=self.authPrincipal
+                    )
                     request.stream = MemoryStream(child.getContent())
                     yield self.send(request)
 
-        request = SimpleStoreRequest(self, "REPORT", calendar_uri, authid="wsanchez")
+        request = SimpleStoreRequest(self, "REPORT", calendar_uri, authPrincipal=self.authPrincipal)
         request.stream = MemoryStream(query.toxml())
         response = yield self.send(request)
 
