@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,8 +14,8 @@
 # limitations under the License.
 ##
 
-from twext.web2.iweb import IResponse
-from twext.web2.test.test_server import SimpleRequest
+from txweb2.iweb import IResponse
+from txweb2.test.test_server import SimpleRequest
 
 import twistedcaldav.test.util
 from twistedcaldav.config import config
