@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,20 +14,27 @@
 # limitations under the License.
 ##
 
-from txdav.xml.element import HRef, Principal, Unauthenticated
-from twext.web2.http import HTTPError
-from twext.web2.test.test_server import SimpleRequest
-
-from twisted.internet.defer import inlineCallbacks
+from twisted.internet import reactor
+from twisted.internet.defer import inlineCallbacks, returnValue, Deferred
 
 from twistedcaldav import carddavxml
 from twistedcaldav.config import config
-from twistedcaldav.resource import CalDAVResource, CommonHomeResource, \
- CalendarHomeResource, AddressBookHomeResource
-from twistedcaldav.test.util import InMemoryPropertyStore, StoreTestCase, \
-    SimpleStoreRequest
-from twistedcaldav.test.util import TestCase
 from twistedcaldav.notifications import NotificationCollectionResource
+from twistedcaldav.resource import \
+    CalDAVResource, CommonHomeResource, \
+    CalendarHomeResource, AddressBookHomeResource
+from twistedcaldav.storebridge import CalendarCollectionResource
+from twistedcaldav.test.util import TestCase
+from twistedcaldav.test.util import \
+    InMemoryPropertyStore, StoreTestCase, SimpleStoreRequest
+
+from txdav.xml import element
+from txdav.xml.element import HRef
+
+from txweb2 import responsecode
+from txweb2.http import HTTPError
+from txweb2.test.test_server import SimpleRequest
+
 
 
 class StubProperty(object):
@@ -58,6 +65,16 @@ class StubHome(object):
 
 
 
+class StubPrincipal(object):
+    def __init__(self, user):
+        self.user = user
+
+
+    def principalElement(self):
+        return element.Principal(element.HRef.fromString(self.user))
+
+
+
 class CalDAVResourceTests(TestCase):
     def setUp(self):
         TestCase.setUp(self)
@@ -73,6 +90,47 @@ class CalDAVResourceTests(TestCase):
 
 
 
+class TransactionTimeoutTests(StoreTestCase):
+
+    @inlineCallbacks
+    def setUp(self):
+        yield super(TransactionTimeoutTests, self).setUp()
+
+
+    @inlineCallbacks
+    def test_timeoutRetry(self):
+        """
+        Test that a timed out transaction during an HTTP request results in a 503 error
+        with a Retry-After header.
+        """
+
+        # Patch request handling to add a delay to trigger the txn time out
+        original = CalendarCollectionResource.iCalendarRolledup
+        @inlineCallbacks
+        def _iCalendarRolledup(self, request):
+            d = Deferred()
+            reactor.callLater(2, d.callback, None)
+            yield d
+            result = yield original(self, request)
+            returnValue(result)
+        self.patch(CalendarCollectionResource, "iCalendarRolledup", _iCalendarRolledup)
+
+        self.patch(self.store, "timeoutTransactions", 1)
+
+        # Run delayed request
+        authPrincipal = yield self.actualRoot.findPrincipalForAuthID("user01")
+        request = SimpleStoreRequest(self, "GET", "/calendars/__uids__/user01/calendar/", authPrincipal=authPrincipal)
+        try:
+            yield self.send(request)
+        except HTTPError as e:
+            self.assertEqual(e.response.code, responsecode.SERVICE_UNAVAILABLE)
+            self.assertTrue(e.response.headers.hasHeader("Retry-After"))
+            self.assertApproximates(int(e.response.headers.getRawHeaders("Retry-After")[0]), config.TransactionHTTPRetrySeconds, 1)
+        else:
+            self.fail("HTTPError not raised")
+
+
+
 class CommonHomeResourceTests(TestCase):
 
     def test_commonHomeliveProperties(self):
@@ -98,6 +156,13 @@ class CommonHomeResourceTests(TestCase):
         self.assertTrue(('http://calendarserver.org/ns/', 'getctag') in resource.liveProperties())
 
 
+    def test_commonHomeResourceMergeSyncToken(self):
+        resource = CommonHomeResource(None, None, None, StubHome())
+        self.assertEquals(resource._mergeSyncTokens("1_2/A", "1_3/A"), "1_3/A")
+        self.assertEquals(resource._mergeSyncTokens("1_2", "1_3"), "1_3")
+        self.assertEquals(resource._mergeSyncTokens("1_4", "1_3"), "1_4")
+
+
 
 class OwnershipTests(TestCase):
     """
@@ -112,7 +177,7 @@ class OwnershipTests(TestCase):
         """
         site = None
         request = SimpleRequest(site, "GET", "/not/a/real/url/")
-        request.authzUser = request.authnUser = Principal(Unauthenticated())
+        request.authzUser = request.authnUser = None
         rsrc = CalDAVResource()
         rsrc.owner = lambda igreq: HRef("/somebody/")
         self.assertEquals((yield rsrc.isOwner(request)), False)
@@ -126,8 +191,7 @@ class OwnershipTests(TestCase):
         """
         site = None
         request = SimpleRequest(site, "GET", "/not/a/real/url/")
-        theOwner = Principal(HRef("/yes-i-am-the-owner/"))
-        request.authzUser = request.authnUser = theOwner
+        request.authzUser = request.authnUser = StubPrincipal("/yes-i-am-the-owner/")
         rsrc = CalDAVResource()
         rsrc.owner = lambda igreq: HRef("/no-i-am-not-the-owner/")
         self.assertEquals((yield rsrc.isOwner(request)), False)
@@ -141,13 +205,32 @@ class OwnershipTests(TestCase):
         """
         site = None
         request = SimpleRequest(site, "GET", "/not/a/real/url/")
-        theOwner = Principal(HRef("/yes-i-am-the-owner/"))
-        request.authzUser = request.authnUser = theOwner
+        request.authzUser = request.authnUser = StubPrincipal("/yes-i-am-the-owner/")
         rsrc = CalDAVResource()
         rsrc.owner = lambda igreq: HRef("/yes-i-am-the-owner/")
         self.assertEquals((yield rsrc.isOwner(request)), True)
 
 
+    @inlineCallbacks
+    def test_isOwnerYes_noStoreObject(self):
+        """
+        L{CalDAVResource.isOwner} returns C{True} for authenticated requests
+        with a principal that matches the resource's owner.
+        """
+        site = None
+        request = SimpleRequest(site, "GET", "/not/a/real/url/")
+        request.authzUser = request.authnUser = StubPrincipal("/yes-i-am-the-owner/")
+        parent = CalDAVResource()
+        parent.owner = lambda igreq: HRef("/yes-i-am-the-owner/")
+        rsrc = CalDAVResource()
+        rsrc._newStoreObject = None
+
+        request._rememberResource(parent, "/not/a/real/")
+        request._rememberResource(rsrc, "/not/a/real/url/")
+
+        self.assertEquals((yield rsrc.isOwner(request)), True)
+
+
     @inlineCallbacks
     def test_isOwnerAdmin(self):
         """
@@ -159,7 +242,7 @@ class OwnershipTests(TestCase):
         self.patch(config, "AdminPrincipals", [theAdmin])
         site = None
         request = SimpleRequest(site, "GET", "/not/a/real/url/")
-        request.authzUser = request.authnUser = Principal(HRef(theAdmin))
+        request.authzUser = request.authnUser = StubPrincipal(theAdmin)
         rsrc = CalDAVResource()
         rsrc.owner = lambda igreq: HRef("/some-other-user/")
         self.assertEquals((yield rsrc.isOwner(request)), True)
@@ -176,7 +259,7 @@ class OwnershipTests(TestCase):
         self.patch(config, "ReadPrincipals", [theAdmin])
         site = None
         request = SimpleRequest(site, "GET", "/not/a/real/url/")
-        request.authzUser = request.authnUser = Principal(HRef(theAdmin))
+        request.authzUser = request.authnUser = StubPrincipal(theAdmin)
         rsrc = CalDAVResource()
         rsrc.owner = lambda igreq: HRef("/some-other-user/")
         self.assertEquals((yield rsrc.isOwner(request)), True)
@@ -185,13 +268,20 @@ class OwnershipTests(TestCase):
 
 class DefaultAddressBook (StoreTestCase):
 
+
+    @inlineCallbacks
+    def setUp(self):
+        yield StoreTestCase.setUp(self)
+        self.authPrincipal = yield self.actualRoot.findPrincipalForAuthID("wsanchez")
+
+
     @inlineCallbacks
     def test_pick_default_addressbook(self):
         """
         Get adbk
         """
 
-        request = SimpleStoreRequest(self, "GET", "/addressbooks/users/wsanchez/", authid="wsanchez")
+        request = SimpleStoreRequest(self, "GET", "/addressbooks/users/wsanchez/", authPrincipal=self.authPrincipal)
         home = yield request.locateResource("/addressbooks/users/wsanchez")
 
         # default property initially not present
