@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,8 +19,8 @@ from twistedcaldav.config import config
 from twistedcaldav.ical import Component
 from twistedcaldav.timezones import TimezoneCache, TimezoneException
 from twistedcaldav.timezones import readTZ, listTZs
-from pycalendar.datetime import PyCalendarDateTime
-from pycalendar.timezone import PyCalendarTimezone
+from pycalendar.datetime import DateTime
+from pycalendar.timezone import Timezone
 
 import os
 import threading
@@ -49,7 +49,7 @@ class TimezoneProblemTest (twistedcaldav.test.util.TestCase):
         if calendar.name() != "VCALENDAR":
             self.fail("Calendar is not a VCALENDAR")
 
-        instances = calendar.expandTimeRanges(PyCalendarDateTime(2100, 1, 1))
+        instances = calendar.expandTimeRanges(DateTime(2100, 1, 1))
         for key in instances:
             instance = instances[key]
             start = instance.start
@@ -69,8 +69,8 @@ class TimezoneProblemTest (twistedcaldav.test.util.TestCase):
 
         self.doTest(
             "TruncatedApr01.ics",
-            PyCalendarDateTime(2007, 04, 01, 16, 0, 0, PyCalendarTimezone(utc=True)),
-            PyCalendarDateTime(2007, 04, 01, 17, 0, 0, PyCalendarTimezone(utc=True))
+            DateTime(2007, 04, 01, 16, 0, 0, Timezone.UTCTimezone),
+            DateTime(2007, 04, 01, 17, 0, 0, Timezone.UTCTimezone)
         )
 
 
@@ -84,8 +84,8 @@ class TimezoneProblemTest (twistedcaldav.test.util.TestCase):
 
         self.doTest(
             "TruncatedDec10.ics",
-            PyCalendarDateTime(2007, 12, 10, 17, 0, 0, PyCalendarTimezone(utc=True)),
-            PyCalendarDateTime(2007, 12, 10, 18, 0, 0, PyCalendarTimezone(utc=True))
+            DateTime(2007, 12, 10, 17, 0, 0, Timezone.UTCTimezone),
+            DateTime(2007, 12, 10, 18, 0, 0, Timezone.UTCTimezone)
         )
 
 
@@ -99,13 +99,13 @@ class TimezoneProblemTest (twistedcaldav.test.util.TestCase):
 
         self.doTest(
             "TruncatedApr01.ics",
-            PyCalendarDateTime(2007, 04, 01, 16, 0, 0, PyCalendarTimezone(utc=True)),
-            PyCalendarDateTime(2007, 04, 01, 17, 0, 0, PyCalendarTimezone(utc=True)),
+            DateTime(2007, 04, 01, 16, 0, 0, Timezone.UTCTimezone),
+            DateTime(2007, 04, 01, 17, 0, 0, Timezone.UTCTimezone),
         )
         self.doTest(
             "TruncatedDec10.ics",
-            PyCalendarDateTime(2007, 12, 10, 17, 0, 0, PyCalendarTimezone(utc=True)),
-            PyCalendarDateTime(2007, 12, 10, 18, 0, 0, PyCalendarTimezone(utc=True)),
+            DateTime(2007, 12, 10, 17, 0, 0, Timezone.UTCTimezone),
+            DateTime(2007, 12, 10, 18, 0, 0, Timezone.UTCTimezone),
             testEqual=False
         )
 
@@ -119,13 +119,13 @@ class TimezoneProblemTest (twistedcaldav.test.util.TestCase):
 
         self.doTest(
             "TruncatedApr01.ics",
-            PyCalendarDateTime(2007, 04, 01, 16, 0, 0, PyCalendarTimezone(utc=True)),
-            PyCalendarDateTime(2007, 04, 01, 17, 0, 0, PyCalendarTimezone(utc=True)),
+            DateTime(2007, 04, 01, 16, 0, 0, Timezone.UTCTimezone),
+            DateTime(2007, 04, 01, 17, 0, 0, Timezone.UTCTimezone),
         )
         self.doTest(
             "TruncatedDec10.ics",
-            PyCalendarDateTime(2007, 12, 10, 17, 0, 0, PyCalendarTimezone(utc=True)),
-            PyCalendarDateTime(2007, 12, 10, 18, 0, 0, PyCalendarTimezone(utc=True)),
+            DateTime(2007, 12, 10, 17, 0, 0, Timezone.UTCTimezone),
+            DateTime(2007, 12, 10, 18, 0, 0, Timezone.UTCTimezone),
         )
 
 
@@ -139,13 +139,13 @@ class TimezoneProblemTest (twistedcaldav.test.util.TestCase):
 
         self.doTest(
             "TruncatedDec10.ics",
-            PyCalendarDateTime(2007, 12, 10, 17, 0, 0, PyCalendarTimezone(utc=True)),
-            PyCalendarDateTime(2007, 12, 10, 18, 0, 0, PyCalendarTimezone(utc=True))
+            DateTime(2007, 12, 10, 17, 0, 0, Timezone.UTCTimezone),
+            DateTime(2007, 12, 10, 18, 0, 0, Timezone.UTCTimezone)
         )
         self.doTest(
             "TruncatedApr01.ics",
-            PyCalendarDateTime(2007, 04, 01, 16, 0, 0, PyCalendarTimezone(utc=True)),
-            PyCalendarDateTime(2007, 04, 01, 17, 0, 0, PyCalendarTimezone(utc=True))
+            DateTime(2007, 04, 01, 16, 0, 0, Timezone.UTCTimezone),
+            DateTime(2007, 04, 01, 17, 0, 0, Timezone.UTCTimezone)
         )
 
 
@@ -201,13 +201,13 @@ END:VCALENDAR
         calendar = Component.fromString(data)
         if calendar.name() != "VCALENDAR":
             self.fail("Calendar is not a VCALENDAR")
-        instances = calendar.expandTimeRanges(PyCalendarDateTime(2100, 1, 1))
+        instances = calendar.expandTimeRanges(DateTime(2100, 1, 1))
         for key in instances:
             instance = instances[key]
             start = instance.start
             end = instance.end
-            self.assertEqual(start, PyCalendarDateTime(2007, 12, 25, 05, 0, 0, PyCalendarTimezone(utc=True)))
-            self.assertEqual(end, PyCalendarDateTime(2007, 12, 25, 06, 0, 0, PyCalendarTimezone(utc=True)))
+            self.assertEqual(start, DateTime(2007, 12, 25, 05, 0, 0, Timezone.UTCTimezone))
+            self.assertEqual(end, DateTime(2007, 12, 25, 06, 0, 0, Timezone.UTCTimezone))
             break
 
 
@@ -279,6 +279,24 @@ class TimezonePackageTest (twistedcaldav.test.util.TestCase):
         self.assertEqual(str(pkg_tz), str(copy_tz))
 
 
+    def test_copyPackage_missingVersion(self):
+        """
+        Test that tz data is updated if the version is missing.
+        """
+
+        self.patch(config, "UsePackageTimezones", False)
+        TimezoneCache.clear()
+        TimezoneCache.create()
+
+        self.assertTrue(os.path.exists(os.path.join(config.DataRoot, "zoneinfo")))
+        self.assertTrue(os.path.exists(os.path.join(config.DataRoot, "zoneinfo", "version.txt")))
+        os.remove(os.path.join(config.DataRoot, "zoneinfo", "version.txt"))
+        self.assertFalse(os.path.exists(os.path.join(config.DataRoot, "zoneinfo", "version.txt")))
+
+        TimezoneCache.create()
+        self.assertTrue(os.path.exists(os.path.join(config.DataRoot, "zoneinfo", "version.txt")))
+
+
     def test_copyPackage_Concurrency(self):
         """
         Test that concurrent copying of the tz package works.
