@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,27 +14,26 @@
 # limitations under the License.
 ##
 
+import cPickle
 import hashlib
 import os
 import zlib
-import cPickle
-
-from twisted.python.reflect import namedClass
-from twisted.internet.defer import inlineCallbacks
-
-from txdav.xml.parser import WebDAVDocument
-from txdav.caldav.datastore.index_file import db_basename
 
+from twisted.internet.defer import inlineCallbacks, succeed
+from twisted.python.filepath import FilePath
 from twistedcaldav.config import config
-from twistedcaldav.directory.xmlfile import XMLDirectoryService
-from twistedcaldav.directory.resourceinfo import ResourceInfoDatabase
-from txdav.caldav.datastore.scheduling.imip.mailgateway import MailGatewayTokensDatabase
+from twistedcaldav.directory.calendaruserproxy import ProxySqliteDB
+from twistedcaldav.test.util import StoreTestCase
 from twistedcaldav.upgrade import (
     xattrname, upgradeData, updateFreeBusySet,
-    removeIllegalCharacters, normalizeCUAddrs
+    removeIllegalCharacters, normalizeCUAddrs,
+    loadDelegatesFromXMLintoProxyDB, migrateDelegatesToStore,
+    upgradeResourcesXML, upgradeAugmentsXML
 )
-from twistedcaldav.test.util import TestCase
-from calendarserver.tools.util import getDirectory
+from txdav.caldav.datastore.index_file import db_basename
+from txdav.caldav.datastore.scheduling.imip.mailgateway import MailGatewayTokensDatabase
+from txdav.who.delegates import Delegates
+from txdav.xml.parser import WebDAVDocument
 
 
 
@@ -51,33 +50,18 @@ md5Attr = xattrname(
 OLDPROXYFILE = ".db.calendaruserproxy"
 NEWPROXYFILE = "proxies.sqlite"
 
-class UpgradeTests(TestCase):
 
-
-    def setUpXMLDirectory(self):
-        xmlFile = os.path.join(os.path.dirname(os.path.dirname(__file__)),
-            "directory", "test", "accounts.xml")
-        config.DirectoryService.params.xmlFile = xmlFile
-
-        xmlAugmentsFile = os.path.join(os.path.dirname(os.path.dirname(__file__)),
-            "directory", "test", "augments.xml")
-        config.AugmentService.type = "twistedcaldav.directory.augment.AugmentXMLDB"
-        config.AugmentService.params.xmlFiles = (xmlAugmentsFile,)
-
-        resourceFile = os.path.join(os.path.dirname(os.path.dirname(__file__)),
-            "directory", "test", "resources.xml")
-        config.ResourceService.params.xmlFile = resourceFile
+class UpgradeTests(StoreTestCase):
 
 
     def doUpgrade(self, config):
         """
         Perform the actual upgrade.  (Hook for parallel tests.)
         """
-        return upgradeData(config)
+        return upgradeData(config, self.directory)
 
 
     def setUpInitialStates(self):
-        self.setUpXMLDirectory()
 
         self.setUpOldDocRoot()
         self.setUpOldDocRootWithoutDB()
@@ -196,13 +180,14 @@ class UpgradeTests(TestCase):
         self.assertTrue(os.path.exists(os.path.join(config.DataRoot, NEWPROXYFILE)))
 
 
+    @inlineCallbacks
     def test_freeBusyUpgrade(self):
         """
         Test the updating of calendar-free-busy-set xattrs on inboxes
         """
 
         self.setUpInitialStates()
-        directory = getDirectory()
+        directory = self.directory
 
         #
         # Verify these values require no updating:
@@ -210,18 +195,18 @@ class UpgradeTests(TestCase):
 
         # Uncompressed XML
         value = "<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/__uids__/BB05932F-DCE7-4195-9ED4-0896EAFF3B0B/calendar</href>\r\n</calendar-free-busy-set>\r\n"
-        self.assertEquals(updateFreeBusySet(value, directory), None)
+        self.assertEquals((yield updateFreeBusySet(value, directory)), None)
 
         # Zlib compressed XML
         value = "<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/__uids__/BB05932F-DCE7-4195-9ED4-0896EAFF3B0B/calendar</href>\r\n</calendar-free-busy-set>\r\n"
         value = zlib.compress(value)
-        self.assertEquals(updateFreeBusySet(value, directory), None)
+        self.assertEquals((yield updateFreeBusySet(value, directory)), None)
 
         # Pickled XML
         value = "<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/__uids__/BB05932F-DCE7-4195-9ED4-0896EAFF3B0B/calendar</href>\r\n</calendar-free-busy-set>\r\n"
         doc = WebDAVDocument.fromString(value)
         value = cPickle.dumps(doc.root_element)
-        self.assertEquals(updateFreeBusySet(value, directory), None)
+        self.assertEquals((yield updateFreeBusySet(value, directory)), None)
 
         #
         # Verify these values do require updating:
@@ -230,14 +215,14 @@ class UpgradeTests(TestCase):
 
         # Uncompressed XML
         value = "<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/users/wsanchez/calendar</href>\r\n</calendar-free-busy-set>\r\n"
-        newValue = updateFreeBusySet(value, directory)
+        newValue = yield updateFreeBusySet(value, directory)
         newValue = zlib.decompress(newValue)
         self.assertEquals(newValue, expected)
 
         # Zlib compressed XML
         value = "<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/users/wsanchez/calendar</href>\r\n</calendar-free-busy-set>\r\n"
         value = zlib.compress(value)
-        newValue = updateFreeBusySet(value, directory)
+        newValue = yield updateFreeBusySet(value, directory)
         newValue = zlib.decompress(newValue)
         self.assertEquals(newValue, expected)
 
@@ -245,7 +230,7 @@ class UpgradeTests(TestCase):
         value = "<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/users/wsanchez/calendar</href>\r\n</calendar-free-busy-set>\r\n"
         doc = WebDAVDocument.fromString(value)
         value = cPickle.dumps(doc.root_element)
-        newValue = updateFreeBusySet(value, directory)
+        newValue = yield updateFreeBusySet(value, directory)
         newValue = zlib.decompress(newValue)
         self.assertEquals(newValue, expected)
 
@@ -254,7 +239,7 @@ class UpgradeTests(TestCase):
         #
         expected = "<?xml version='1.0' encoding='UTF-8'?>\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'/>"
         value = "<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/users/nonexistent/calendar</href>\r\n</calendar-free-busy-set>\r\n"
-        newValue = updateFreeBusySet(value, directory)
+        newValue = yield updateFreeBusySet(value, directory)
         newValue = zlib.decompress(newValue)
         self.assertEquals(newValue, expected)
 
@@ -296,7 +281,6 @@ class UpgradeTests(TestCase):
         The upgrade process should remove unused notification directories in
         users' calendar homes, as well as the XML files found therein.
         """
-        self.setUpXMLDirectory()
 
         before = {
             "calendars": {
@@ -306,7 +290,7 @@ class UpgradeTests(TestCase):
                             db_basename : {
                                 "@contents": "",
                             },
-                         },
+                        },
                         "notifications": {
                             "sample-notification.xml": {
                                 "@contents": "<?xml version='1.0'>\n<should-be-ignored />"
@@ -336,7 +320,7 @@ class UpgradeTests(TestCase):
             ".calendarserver_version" : {
                 "@contents" : "2",
             },
-            MailGatewayTokensDatabase.dbFilename : { "@contents" : None },
+            MailGatewayTokensDatabase.dbFilename : {"@contents" : None},
             "%s-journal" % (MailGatewayTokensDatabase.dbFilename,) : {
                 "@contents" : None,
                 "@optional" : True,
@@ -353,8 +337,6 @@ class UpgradeTests(TestCase):
         are upgraded to /calendars/__uids__/XX/YY/<guid> form
         """
 
-        self.setUpXMLDirectory()
-
         before = {
             "calendars" :
             {
@@ -503,8 +485,6 @@ class UpgradeTests(TestCase):
         whose records don't exist are moved into dataroot/archived/
         """
 
-        self.setUpXMLDirectory()
-
         before = {
             "calendars" :
             {
@@ -575,8 +555,6 @@ class UpgradeTests(TestCase):
         whose records don't exist are moved into dataroot/archived/
         """
 
-        self.setUpXMLDirectory()
-
         before = {
             "archived" :
             {
@@ -663,8 +641,6 @@ class UpgradeTests(TestCase):
         interrupt an upgrade.
         """
 
-        self.setUpXMLDirectory()
-
         ignoredUIDContents = {
             "64" : {
                 "23" : {
@@ -757,8 +733,6 @@ class UpgradeTests(TestCase):
         interrupt an upgrade.
         """
 
-        self.setUpXMLDirectory()
-
         beforeUIDContents = {
             "64" : {
                 "23" : {
@@ -867,8 +841,6 @@ class UpgradeTests(TestCase):
         are upgraded to /calendars/__uids__/XX/YY/<guid>/ form
         """
 
-        self.setUpXMLDirectory()
-
         before = {
             "calendars" :
             {
@@ -978,8 +950,6 @@ class UpgradeTests(TestCase):
         form are upgraded correctly in place
         """
 
-        self.setUpXMLDirectory()
-
         before = {
             "calendars" :
             {
@@ -1106,8 +1076,6 @@ class UpgradeTests(TestCase):
         form which require no changes are untouched
         """
 
-        self.setUpXMLDirectory()
-
         before = {
             "calendars" :
             {
@@ -1233,8 +1201,6 @@ class UpgradeTests(TestCase):
         Verify that inbox items older than 60 days are deleted
         """
 
-        self.setUpXMLDirectory()
-
         before = {
             "calendars" :
             {
@@ -1337,8 +1303,6 @@ class UpgradeTests(TestCase):
         also doesn't write the new version file
         """
 
-        self.setUpXMLDirectory()
-
         before = {
             "calendars" :
             {
@@ -1434,93 +1398,6 @@ class UpgradeTests(TestCase):
         self.assertTrue(self.verifyHierarchy(root, after))
 
 
-    @inlineCallbacks
-    def test_migrateResourceInfo(self):
-        # Fake getResourceInfo( )
-
-        assignments = {
-            'guid1' : (False, None, None),
-            'guid2' : (True, 'guid1', None),
-            'guid3' : (False, 'guid1', 'guid2'),
-            'guid4' : (True, None, 'guid3'),
-        }
-
-        def _getResourceInfo(ignored):
-            results = []
-            for guid, info in assignments.iteritems():
-                results.append((guid, info[0], info[1], info[2]))
-            return results
-
-        self.setUpInitialStates()
-        # Override the normal getResourceInfo method with our own:
-        # XMLDirectoryService.getResourceInfo = _getResourceInfo
-        self.patch(XMLDirectoryService, "getResourceInfo", _getResourceInfo)
-
-        before = {
-            "trigger_resource_migration" : {
-                "@contents" : "x",
-            }
-        }
-        after = {
-            ".calendarserver_version" :
-            {
-                "@contents" : "2",
-            },
-            NEWPROXYFILE :
-            {
-                "@contents" : None,
-            },
-            MailGatewayTokensDatabase.dbFilename :
-            {
-                "@contents" : None,
-            },
-            "%s-journal" % (MailGatewayTokensDatabase.dbFilename,) :
-            {
-                "@contents" : None,
-                "@optional" : True,
-            },
-            ResourceInfoDatabase.dbFilename :
-            {
-                "@contents" : None,
-            },
-            "%s-journal" % (ResourceInfoDatabase.dbFilename,) :
-            {
-                "@contents" : None,
-                "@optional" : True,
-            }
-        }
-        root = self.createHierarchy(before)
-        config.DocumentRoot = root
-        config.DataRoot = root
-        config.ServerRoot = root
-
-        (yield self.doUpgrade(config))
-        self.assertTrue(self.verifyHierarchy(root, after))
-
-        proxydbClass = namedClass(config.ProxyDBService.type)
-        calendarUserProxyDatabase = proxydbClass(**config.ProxyDBService.params)
-        resourceInfoDatabase = ResourceInfoDatabase(root)
-
-        for guid, info in assignments.iteritems():
-            proxyGroup = "%s#calendar-proxy-write" % (guid,)
-            result = (yield calendarUserProxyDatabase.getMembers(proxyGroup))
-            if info[1]:
-                self.assertTrue(info[1] in result)
-            else:
-                self.assertTrue(not result)
-
-            readOnlyProxyGroup = "%s#calendar-proxy-read" % (guid,)
-            result = (yield calendarUserProxyDatabase.getMembers(readOnlyProxyGroup))
-            if info[2]:
-                self.assertTrue(info[2] in result)
-            else:
-                self.assertTrue(not result)
-
-            autoSchedule = resourceInfoDatabase._db_value_for_sql("select AUTOSCHEDULE from RESOURCEINFO where GUID = :1", guid)
-            autoSchedule = autoSchedule == 1
-            self.assertEquals(info[0], autoSchedule)
-
-
     def test_removeIllegalCharacters(self):
         """
         Control characters aside from NL and CR are removed.
@@ -1536,55 +1413,247 @@ class UpgradeTests(TestCase):
         self.assertFalse(changed)
 
 
+    @inlineCallbacks
     def test_normalizeCUAddrs(self):
         """
         Ensure that calendar user addresses (CUAs) are cached so we can
         reduce the number of principal lookup calls during upgrade.
         """
 
-        class StubPrincipal(object):
-            def __init__(self, record):
-                self.record = record
-
         class StubRecord(object):
-            def __init__(self, fullName, guid, cuas):
-                self.fullName = fullName
-                self.guid = guid
+            def __init__(self, fullNames, uid, cuas):
+                self.fullNames = fullNames
+                self.uid = uid
                 self.calendarUserAddresses = cuas
 
+            def getCUType(self):
+                return "INDIVIDUAL"
+
+            @property
+            def displayName(self):
+                return self.fullNames[0]
+
         class StubDirectory(object):
             def __init__(self):
                 self.count = 0
 
-            def principalForCalendarUserAddress(self, cuaddr):
+            def recordWithCalendarUserAddress(self, cuaddr):
                 self.count += 1
                 record = records.get(cuaddr, None)
                 if record is not None:
-                    return StubPrincipal(record)
+                    return succeed(record)
                 else:
                     raise Exception
 
         records = {
-            "mailto:a@example.com" :
-                StubRecord("User A", 123, ("mailto:a@example.com", "urn:uuid:123")),
-            "mailto:b@example.com" :
-                StubRecord("User B", 234, ("mailto:b@example.com", "urn:uuid:234")),
-            "/principals/users/a" :
-                StubRecord("User A", 123, ("mailto:a@example.com", "urn:uuid:123")),
-            "/principals/users/b" :
-                StubRecord("User B", 234, ("mailto:b@example.com", "urn:uuid:234")),
+            "mailto:a@example.com":
+                StubRecord(("User A",), u"123", ("mailto:a@example.com", "urn:x-uid:123")),
+            "mailto:b@example.com":
+                StubRecord(("User B",), u"234", ("mailto:b@example.com", "urn:x-uid:234")),
+            "/principals/users/a":
+                StubRecord(("User A",), u"123", ("mailto:a@example.com", "urn:x-uid:123")),
+            "/principals/users/b":
+                StubRecord(("User B",), u"234", ("mailto:b@example.com", "urn:x-uid:234")),
         }
 
         directory = StubDirectory()
         cuaCache = {}
-        normalizeCUAddrs(normalizeEvent, directory, cuaCache)
-        normalizeCUAddrs(normalizeEvent, directory, cuaCache)
+        yield normalizeCUAddrs(normalizeEvent, directory, cuaCache)
+        yield normalizeCUAddrs(normalizeEvent, directory, cuaCache)
 
         # Ensure we only called principalForCalendarUserAddress 3 times.  It
         # would have been 8 times without the cuaCache.
         self.assertEquals(directory.count, 3)
 
 
+    @inlineCallbacks
+    def test_migrateDelegates(self):
+        store = self.storeUnderTest()
+        record = yield self.directory.recordWithUID(u"mercury")
+        txn = store.newTransaction()
+        writeDelegates = yield Delegates.delegatesOf(txn, record, True)
+        self.assertEquals(len(writeDelegates), 0)
+        yield txn.commit()
+
+        # Load delegates from xml into sqlite
+        sqliteProxyService = ProxySqliteDB("proxies.sqlite")
+        proxyFile = os.path.join(config.DataRoot, "proxies.xml")
+        yield loadDelegatesFromXMLintoProxyDB(proxyFile, sqliteProxyService)
+
+        # Load delegates from sqlite into store
+        yield migrateDelegatesToStore(store)
+
+        # Check delegates in store
+        txn = store.newTransaction()
+        writeDelegates = yield Delegates.delegatesOf(txn, record, True)
+        self.assertEquals(len(writeDelegates), 1)
+        self.assertEquals(
+            set([d.uid for d in writeDelegates]),
+            set([u"left_coast"])
+        )
+
+        record = yield self.directory.recordWithUID(u"non_calendar_proxy")
+
+        readDelegates = yield Delegates.delegatesOf(txn, record, False)
+        self.assertEquals(len(readDelegates), 1)
+        self.assertEquals(
+            set([d.uid for d in readDelegates]),
+            set([u"recursive2_coasts"])
+        )
+
+        yield txn.commit()
+
+
+    def test_resourcesXML(self):
+        """
+        Verify conversion of old resources.xml format to twext.who.xml format
+        """
+        fileName = self.mktemp()
+        fp = FilePath(fileName)
+        fp.setContent(oldResourcesFormat)
+        upgradeResourcesXML(fp)
+        self.assertEquals(fp.getContent(), newResourcesFormat)
+
+
+    def test_augmentsXML(self):
+        """
+        Verify conversion of old augments.xml auto-schedule related elements to twext.who format
+        """
+        fileName = self.mktemp()
+        fp = FilePath(fileName)
+        fp.setContent(oldAugmentsFormat)
+        upgradeAugmentsXML(fp)
+        self.assertEquals(fp.getContent(), newAugmentsFormat)
+
+
+oldResourcesFormat = """<accounts realm="/Search">
+  <location>
+    <uid>location1</uid>
+    <guid>C4F46062-9094-4D34-8591-61A42D993FAA</guid>
+    <name>location name</name>
+  </location>
+  <location>
+    <uid>5456580a-08ee-4288-8a87-2b4204a62a12</uid>
+    <guid>5456580a-08ee-4288-8a87-2b4204a62a12</guid>
+    <name>Fake Room</name>
+  </location>
+  <resource>
+    <uid>resource1</uid>
+    <guid>60B771CC-D727-4453-ACE0-0FE13CD7445A</guid>
+    <name>resource name</name>
+  </resource>
+</accounts>
+"""
+
+newResourcesFormat = """<directory realm="/Search"><record type="location"><short-name>location1</short-name><guid>C4F46062-9094-4D34-8591-61A42D993FAA</guid><uid>C4F46062-9094-4D34-8591-61A42D993FAA</uid><full-name>location name</full-name></record><record type="location"><short-name>5456580A-08EE-4288-8A87-2B4204A62A12</short-name><guid>5456580A-08EE-4288-8A87-2B4204A62A12</guid><uid>5456580A-08EE-4288-8A87-2B4204A62A12</uid><full-name>Fake Room</full-name></record><record type="resource"><short-name>resource1</short-name><guid>60B771CC-D727-4453-ACE0-0FE13CD7445A</guid><uid>60B771CC-D727-4453-ACE0-0FE13CD7445A</uid><full-name>resource name</full-name></record></directory>"""
+
+
+
+oldAugmentsFormat = """<?xml version="1.0" encoding="utf-8"?>
+<!DOCTYPE augments SYSTEM "augments.dtd">
+<augments>
+  <record>
+    <uid>9F3603DD-65D0-480D-A1D1-5D33CAC41A13</uid>
+    <enable>true</enable>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <enable-login>true</enable-login>
+    <auto-schedule>false</auto-schedule>
+    <auto-schedule-mode>default</auto-schedule-mode>
+  </record>
+  <record>
+    <uid>6A49C436-4CDB-4184-AD87-6F945040E37A</uid>
+    <enable>true</enable>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <enable-login>true</enable-login>
+    <auto-schedule>true</auto-schedule>
+  </record>
+  <record>
+    <uid>60B771CC-D727-4453-ACE0-0FE13CD7445A</uid>
+    <enable>true</enable>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <enable-login>true</enable-login>
+    <auto-schedule-mode>none</auto-schedule-mode>
+  </record>
+  <record>
+    <uid>E173AADC-4642-43CB-9745-8CE436A6FE4A</uid>
+    <enable>false</enable>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <enable-login>true</enable-login>
+    <auto-schedule>false</auto-schedule>
+    <auto-schedule-mode>automatic</auto-schedule-mode>
+  </record>
+  <record>
+    <uid>FC9A7F56-CCCA-4401-9160-903902880A37</uid>
+    <enable>false</enable>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <enable-login>true</enable-login>
+    <auto-schedule>true</auto-schedule>
+    <auto-schedule-mode>accept-if-free</auto-schedule-mode>
+  </record>
+  <record>
+    <uid>B35DDFCF-C5E2-475E-A57E-8AB7422E9BB8</uid>
+    <enable>true</enable>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <auto-schedule>false</auto-schedule>
+  </record>
+</augments>
+"""
+
+newAugmentsFormat = """<?xml version="1.0" encoding="utf-8"?>
+<!DOCTYPE augments SYSTEM "augments.dtd">
+
+<augments>
+  <record>
+    <uid>9F3603DD-65D0-480D-A1D1-5D33CAC41A13</uid>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <enable-login>true</enable-login>
+    <auto-schedule-mode>none</auto-schedule-mode>
+  </record>
+  <record>
+    <uid>6A49C436-4CDB-4184-AD87-6F945040E37A</uid>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <enable-login>true</enable-login>
+    <auto-schedule-mode>automatic</auto-schedule-mode>
+  </record>
+  <record>
+    <uid>60B771CC-D727-4453-ACE0-0FE13CD7445A</uid>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <enable-login>true</enable-login>
+    <auto-schedule-mode>none</auto-schedule-mode>
+  </record>
+  <record>
+    <uid>E173AADC-4642-43CB-9745-8CE436A6FE4A</uid>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <enable-login>true</enable-login>
+    <auto-schedule-mode>none</auto-schedule-mode>
+  </record>
+  <record>
+    <uid>FC9A7F56-CCCA-4401-9160-903902880A37</uid>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <enable-login>true</enable-login>
+    <auto-schedule-mode>accept-if-free</auto-schedule-mode>
+  </record>
+  <record>
+    <uid>B35DDFCF-C5E2-475E-A57E-8AB7422E9BB8</uid>
+    <enable-calendar>true</enable-calendar>
+    <enable-addressbook>true</enable-addressbook>
+    <auto-schedule-mode>none</auto-schedule-mode>
+  </record>
+</augments>
+"""
+
+
 normalizeEvent = """BEGIN:VCALENDAR
 VERSION:2.0
 BEGIN:VEVENT
@@ -1670,18 +1739,19 @@ BEGIN:VEVENT
 UID:1E238CA1-3C95-4468-B8CD-C8A399F78C71
 DTSTART;TZID=US/Pacific:20090203T120000
 DTEND;TZID=US/Pacific:20090203T130000
-ATTENDEE;CN=Wilfredo Sanchez;CUTYPE=INDIVIDUAL;EMAIL=wsanchez@example.com;
- PARTSTAT=ACCEPTED:urn:uuid:6423F94A-6B76-4A3A-815B-D52CFD77935D
+ATTENDEE;CN=Wilfredo Sanchez-Vega;CUTYPE=INDIVIDUAL;EMAIL=wsanchez@example
+ .com;PARTSTAT=ACCEPTED:urn:x-uid:6423F94A-6B76-4A3A-815B-D52CFD77935D
 ATTENDEE;CN=Double 'quotey' Quotes;CUTYPE=INDIVIDUAL;EMAIL=doublequotes@ex
- ample.com;PARTSTAT=ACCEPTED:urn:uuid:8E04787E-336D-41ED-A70B-D233AD0DCE6F
+ ample.com;PARTSTAT=ACCEPTED:urn:x-uid:8E04787E-336D-41ED-A70B-D233AD0DCE6
+ F
 ATTENDEE;CN=Cyrus Daboo;CUTYPE=INDIVIDUAL;EMAIL=cdaboo@example.com;PARTSTA
- T=ACCEPTED;ROLE=REQ-PARTICIPANT:urn:uuid:5A985493-EE2C-4665-94CF-4DFEA3A89
- 500
+ T=ACCEPTED;ROLE=REQ-PARTICIPANT:urn:x-uid:5A985493-EE2C-4665-94CF-4DFEA3A
+ 89500
 CREATED:20090203T181910Z
 DESCRIPTION:This has " Bad Quotes " in it
 DTSTAMP:20090203T181924Z
-ORGANIZER;CN=Cyrus Daboo;EMAIL=cdaboo@example.com:urn:uuid:5A985493-EE2C-4
- 665-94CF-4DFEA3A89500
+ORGANIZER;CN=Cyrus Daboo;EMAIL=cdaboo@example.com:urn:x-uid:5A985493-EE2C-
+ 4665-94CF-4DFEA3A89500
 SEQUENCE:2
 SUMMARY:New Event
 TRANSP:OPAQUE
