@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,18 +21,19 @@ Tests for the interaction between model-level and protocol-level logic.
 
 from twext.enterprise.ienterprise import AlreadyFinishedError
 from twext.enterprise.locking import NamedLock
-from twext.web2 import responsecode
-from twext.web2.http import HTTPError
-from twext.web2.http_headers import Headers, MimeType
-from twext.web2.responsecode import INSUFFICIENT_STORAGE_SPACE
-from twext.web2.responsecode import UNAUTHORIZED
-from twext.web2.stream import MemoryStream
+from txweb2 import responsecode
+from txweb2.http import HTTPError
+from txweb2.http_headers import Headers, MimeType
+from txweb2.responsecode import INSUFFICIENT_STORAGE_SPACE
+from txweb2.responsecode import UNAUTHORIZED
+from txweb2.stream import MemoryStream
+
+from twext.who.idirectory import RecordType
 
 from twisted.internet.defer import inlineCallbacks, returnValue
 from twisted.internet.defer import maybeDeferred
 
 from twistedcaldav.config import config
-from twistedcaldav.directory.test.test_xmlfile import XMLFileBase
 from twistedcaldav.ical import Component as VComponent
 from twistedcaldav.storebridge import DropboxCollection, \
     CalendarCollectionResource
@@ -51,11 +52,14 @@ from txdav.xml import element as davxml
 
 import hashlib
 
+
 def _todo(f, why):
     f.todo = why
     return f
 rewriteOrRemove = lambda f: _todo(f, "Rewrite or remove")
 
+
+
 class FakeChanRequest(object):
     code = 'request-not-finished'
 
@@ -113,10 +117,10 @@ class WrappingTests(StoreTestCase):
         @param objectText: Some iCalendar text to populate it with.
         @type objectText: str
         """
-        record = self.directory.recordWithShortName("users", "wsanchez")
+        record = yield self.directory.recordWithShortName(RecordType.user, u"wsanchez")
         uid = record.uid
         txn = self.transactionUnderTest()
-        home = yield txn.calendarHomeWithUID(uid, True)
+        home = yield txn.calendarHomeWithUID(uid, create=True)
         cal = yield home.calendarWithName("calendar")
         yield cal.createCalendarObjectWithName(objectName, VComponent.fromString(objectText))
         yield self.commit()
@@ -132,10 +136,10 @@ class WrappingTests(StoreTestCase):
         @param objectText: Some iVcard text to populate it with.
         @type objectText: str
         """
-        record = self.directory.recordWithShortName("users", "wsanchez")
+        record = yield self.directory.recordWithShortName(RecordType.user, u"wsanchez")
         uid = record.uid
         txn = self.transactionUnderTest()
-        home = yield txn.addressbookHomeWithUID(uid, True)
+        home = yield txn.addressbookHomeWithUID(uid, create=True)
         adbk = yield home.addressbookWithName("addressbook")
         yield adbk.createAddressBookObjectWithName(objectName, VCComponent.fromString(objectText))
         yield self.commit()
@@ -171,10 +175,8 @@ class WrappingTests(StoreTestCase):
             "http://localhost:8008/" + path
         )
         if user is not None:
-            guid = XMLFileBase.users[user]["guid"]
-            req.authnUser = req.authzUser = (
-                davxml.Principal(davxml.HRef('/principals/__uids__/' + guid + '/'))
-            )
+            principal = yield self.actualRoot.findPrincipalForAuthID(user)
+            req.authnUser = req.authzUser = principal
         returnValue(aResource)
 
 
@@ -201,8 +203,10 @@ class WrappingTests(StoreTestCase):
         Verify that the C{_principalCollections} attribute of the given
         L{Resource} is accurately set.
         """
-        self.assertEquals(resource._principalCollections,
-                          frozenset([self.principalsResource]))
+        self.assertEquals(
+            resource._principalCollections,
+            frozenset([self.actualRoot.getChild("principals")])
+        )
 
 
     @inlineCallbacks
@@ -271,7 +275,7 @@ class WrappingTests(StoreTestCase):
         )
         yield self.commit()
         self.assertIsInstance(dropBoxResource, DropboxCollection)
-        dropboxHomeType = davxml.ResourceType.dropboxhome #@UndefinedVariable
+        dropboxHomeType = davxml.ResourceType.dropboxhome  # @UndefinedVariable
         self.assertEquals(dropBoxResource.resourceType(),
                           dropboxHomeType)
 
@@ -285,7 +289,7 @@ class WrappingTests(StoreTestCase):
         C{CalendarHome.calendarWithName}.
         """
         calDavFile = yield self.getResource("calendars/users/wsanchez/calendar")
-        regularCalendarType = davxml.ResourceType.calendar #@UndefinedVariable
+        regularCalendarType = davxml.ResourceType.calendar  # @UndefinedVariable
         self.assertEquals(calDavFile.resourceType(),
                           regularCalendarType)
         yield self.commit()
@@ -344,8 +348,11 @@ class WrappingTests(StoreTestCase):
             self.assertIdentical(
                 homeChild._associatedTransaction,
                 homeTransaction,
-                "transaction mismatch on %s; %r is not %r " %
-                    (name, homeChild._associatedTransaction, homeTransaction))
+                "transaction mismatch on {n}; {at} is not {ht} ".format(
+                    n=name, at=homeChild._associatedTransaction,
+                    ht=homeTransaction
+                )
+            )
 
 
     @inlineCallbacks
@@ -415,7 +422,7 @@ class WrappingTests(StoreTestCase):
         Creating a AddressBookHomeProvisioningFile will create a paired
         AddressBookStore.
         """
-        assertProvides(self, IDataStore, self.addressbookCollection._newStore)
+        assertProvides(self, IDataStore, self.actualRoot.getChild("addressbooks")._newStore)
 
 
     @inlineCallbacks
@@ -537,7 +544,7 @@ END:VCALENDAR
 """
         CR = "\n"
         CRLF = "\r\n"
-        #validEvent = eventTemplate.format("", wsanchez=wsanchez, cdaboo=cdaboo).replace(CR, CRLF)
+        # validEvent = eventTemplate.format("", wsanchez=wsanchez, cdaboo=cdaboo).replace(CR, CRLF)
         invalidInstance = """
 BEGIN:VEVENT
 UID:20060110T231240Z-4011c71-187-6f73
@@ -575,12 +582,13 @@ class TimeoutTests(StoreTestCase):
         yield NamedLock.acquire(txn, "ImplicitUIDLock:%s" % (hashlib.md5("uid1").hexdigest(),))
 
         # PUT fails
+        principal = yield self.actualRoot.findPrincipalForAuthID("wsanchez")
         request = SimpleStoreRequest(
             self,
             "PUT",
             "/calendars/users/wsanchez/calendar/1.ics",
             headers=Headers({"content-type": MimeType.fromString("text/calendar")}),
-            authid="wsanchez"
+            authPrincipal=principal
         )
         request.stream = MemoryStream("""BEGIN:VCALENDAR
 CALSCALE:GREGORIAN
@@ -606,12 +614,13 @@ END:VCALENDAR
         """
 
         # PUT works
+        principal = yield self.actualRoot.findPrincipalForAuthID("wsanchez")
         request = SimpleStoreRequest(
             self,
             "PUT",
             "/calendars/users/wsanchez/calendar/1.ics",
             headers=Headers({"content-type": MimeType.fromString("text/calendar")}),
-            authid="wsanchez"
+            authPrincipal=principal
         )
         request.stream = MemoryStream("""BEGIN:VCALENDAR
 CALSCALE:GREGORIAN
@@ -639,7 +648,7 @@ END:VCALENDAR
             self,
             "DELETE",
             "/calendars/users/wsanchez/calendar/1.ics",
-            authid="wsanchez"
+            authPrincipal=principal
         )
         response = yield self.send(request)
         self.assertEqual(response.code, responsecode.SERVICE_UNAVAILABLE)
