@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,44 +17,30 @@ from __future__ import print_function
 from __future__ import with_statement
 
 import os
-import xattr
 
-from twistedcaldav.stdconfig import config
-
-from twisted.python.failure import Failure
+from calendarserver.provision.root import RootResource
+from calendarserver.tap.util import getRootResource
+from twext.python.filepath import CachingFilePath as FilePath
+from twext.python.log import Logger
 from twisted.internet.base import DelayedCall
 from twisted.internet.defer import succeed, fail, inlineCallbacks, returnValue
 from twisted.internet.protocol import ProcessProtocol
-
-from twext.python.memcacheclient import ClientFactory
-from twext.python.filepath import CachingFilePath as FilePath
-import twext.web2.dav.test.util
-from txdav.xml import element as davxml, element
-from twext.web2.http import HTTPError, StatusResponse
-
+from twisted.python.failure import Failure
 from twistedcaldav import memcacher
 from twistedcaldav.bind import doBind
-from twistedcaldav.directory import augment
 from twistedcaldav.directory.addressbook import DirectoryAddressBookHomeProvisioningResource
-from twistedcaldav.directory.calendar import (
-    DirectoryCalendarHomeProvisioningResource
-)
-from twistedcaldav.directory.principal import (
-    DirectoryPrincipalProvisioningResource)
-from twistedcaldav.directory.aggregate import AggregateDirectoryService
-from twistedcaldav.directory.xmlfile import XMLDirectoryService
-
-from txdav.common.datastore.test.util import deriveQuota, CommonCommonTests
-from txdav.common.datastore.file import CommonDataStore
-
-from calendarserver.provision.root import RootResource
-
-from twext.python.log import Logger
-from txdav.caldav.datastore.test.util import buildCalendarStore
-from calendarserver.tap.util import getRootResource, directoryFromConfig
-from twext.web2.dav.test.util import SimpleRequest
+from twistedcaldav.directory.calendar import DirectoryCalendarHomeProvisioningResource
 from twistedcaldav.directory.util import transactionFromRequest
-from twistedcaldav.directory.directory import DirectoryService
+from twistedcaldav.memcacheclient import ClientFactory
+from twistedcaldav.stdconfig import config
+from txdav.common.datastore.file import CommonDataStore
+from txdav.common.datastore.test.util import deriveQuota, CommonCommonTests
+from txweb2.dav.test.util import SimpleRequest
+from txweb2.http import HTTPError, StatusResponse
+from txweb2.server import Site
+import txweb2.dav.test.util
+import xattr
+
 
 log = Logger()
 
@@ -67,10 +53,12 @@ __all__ = [
 ]
 DelayedCall.debug = True
 
+
 def _todo(f, why):
     f.todo = why
     return f
 
+
 featureUnimplemented = lambda f: _todo(f, "Feature unimplemented")
 testUnimplemented = lambda f: _todo(f, "Test unimplemented")
 todo = lambda why: lambda f: _todo(f, why)
@@ -78,97 +66,25 @@ todo = lambda why: lambda f: _todo(f, why)
 dirTest = FilePath(__file__).parent().sibling("directory").child("test")
 
 xmlFile = dirTest.child("accounts.xml")
+resourcesFile = dirTest.child("resources.xml")
 augmentsFile = dirTest.child("augments.xml")
 proxiesFile = dirTest.child("proxies.xml")
 
 
 
-class DirectoryFixture(object):
-    """
-    Test fixture for creating various parts of the resource hierarchy related
-    to directories.
-    """
-
-    def __init__(self):
-        def _setUpPrincipals(ds):
-            # FIXME: see FIXME in
-            # DirectoryPrincipalProvisioningResource.__init__; this performs a
-            # necessary modification to any directory service object for it to
-            # be fully functional.
-            self.principalsResource = DirectoryPrincipalProvisioningResource(
-                "/principals/", ds
-            )
-        self._directoryChangeHooks = [_setUpPrincipals]
-
-    directoryService = None
-    principalsResource = None
-
-    def addDirectoryService(self, newService):
-        """
-        Add an L{IDirectoryService} to this test case.
-
-        If this test case does not have a directory service yet, create it and
-        assign C{directoryService} and C{principalsResource} attributes to this
-        test case.
-
-        If the test case already has a directory service, create an
-        L{AggregateDirectoryService} and re-assign the C{self.directoryService}
-        attribute to point at it instead, while setting the C{realmName} of the
-        new service to match the old one.
-
-        If the test already has an L{AggregateDirectoryService}, create a
-        I{new} L{AggregateDirectoryService} with the same list of services,
-        after adjusting the new service's realm to match the existing ones.
-        """
-
-        if self.directoryService is None:
-            directoryService = newService
-        else:
-            newService.realmName = self.directoryService.realmName
-            if isinstance(self.directoryService, AggregateDirectoryService):
-                directories = set(self.directoryService._recordTypes.items())
-                directories.add(newService)
-            else:
-                directories = [newService, self.directoryService]
-            directoryService = AggregateDirectoryService(directories, None)
-
-        self.directoryService = directoryService
-        # FIXME: see FIXME in DirectoryPrincipalProvisioningResource.__init__;
-        # this performs a necessary modification to the directory service object
-        # for it to be fully functional.
-        for hook in self._directoryChangeHooks:
-            hook(directoryService)
-
-
-    def whenDirectoryServiceChanges(self, callback):
-        """
-        When the C{directoryService} attribute is changed by
-        L{TestCase.addDirectoryService}, call the given callback in order to
-        update any state which relies upon that service.
-
-        If there's already a directory, invoke the callback immediately.
-        """
-        self._directoryChangeHooks.append(callback)
-        if self.directoryService is not None:
-            callback(self.directoryService)
-
-
-
 class SimpleStoreRequest(SimpleRequest):
     """
     A SimpleRequest that automatically grabs the proper transaction for a test.
     """
-    def __init__(self, test, method, uri, headers=None, content=None, authid=None):
+    def __init__(self, test, method, uri, headers=None, content=None, authPrincipal=None):
         super(SimpleStoreRequest, self).__init__(test.site, method, uri, headers, content)
         self._test = test
         self._newStoreTransaction = test.transactionUnderTest(txn=transactionFromRequest(self, test.storeUnderTest()))
         self.credentialFactories = {}
 
         # Fake credentials if auth needed
-        if authid is not None:
-            record = self._test.directory.recordWithShortName(DirectoryService.recordType_users, authid)
-            if record:
-                self.authzUser = self.authnUser = element.Principal(element.HRef("/principals/__uids__/%s/" % (record.uid,)))
+        if authPrincipal is not None:
+            self.authzUser = self.authnUser = authPrincipal
 
 
     @inlineCallbacks
@@ -187,27 +103,21 @@ class SimpleStoreRequest(SimpleRequest):
 
 
 
-class StoreTestCase(CommonCommonTests, twext.web2.dav.test.util.TestCase):
+class StoreTestCase(CommonCommonTests, txweb2.dav.test.util.TestCase):
     """
-    A base class for tests that use the SQL store.
+    A base class for tests that use the SQL store and need to create and
+    verify filesystem hierarchies.
     """
 
     @inlineCallbacks
     def setUp(self):
         yield super(StoreTestCase, self).setUp()
 
-        self.configure()
+        yield self.buildStoreAndDirectory()
 
-        self._sqlCalendarStore = yield buildCalendarStore(self, self.notifierFactory, directoryFromConfig(config))
-        self.rootResource = getRootResource(config, self._sqlCalendarStore)
-        self.directory = self._sqlCalendarStore.directoryService()
-
-        self.principalsResource = DirectoryPrincipalProvisioningResource("/principals/", self.directory)
-        self.site.resource.putChild("principals", self.principalsResource)
-        self.calendarCollection = DirectoryCalendarHomeProvisioningResource(self.directory, "/calendars/", self._sqlCalendarStore)
-        self.site.resource.putChild("calendars", self.calendarCollection)
-        self.addressbookCollection = DirectoryAddressBookHomeProvisioningResource(self.directory, "/addressbooks/", self._sqlCalendarStore)
-        self.site.resource.putChild("addressbooks", self.addressbookCollection)
+        self.rootResource = getRootResource(config, self.store)
+        self.actualRoot = self.rootResource.resource.resource
+        self.site = Site(self.actualRoot)
 
         yield self.populate()
 
@@ -216,163 +126,20 @@ class StoreTestCase(CommonCommonTests, twext.web2.dav.test.util.TestCase):
         return succeed(None)
 
 
-    def storeUnderTest(self):
-        """
-        Return a store for testing.
-        """
-        return self._sqlCalendarStore
-
-
     def configure(self):
         """
         Adjust the global configuration for this test.
         """
-        self.serverRoot = self.mktemp()
-        os.mkdir(self.serverRoot)
-
-        config.reset()
-
-        config.ServerRoot = os.path.abspath(self.serverRoot)
-        config.ConfigRoot = "config"
-        config.LogRoot = "logs"
-        config.RunRoot = "logs"
-
-        if not os.path.exists(config.DataRoot):
-            os.makedirs(config.DataRoot)
-        if not os.path.exists(config.DocumentRoot):
-            os.makedirs(config.DocumentRoot)
-        if not os.path.exists(config.ConfigRoot):
-            os.makedirs(config.ConfigRoot)
-        if not os.path.exists(config.LogRoot):
-            os.makedirs(config.LogRoot)
+        super(StoreTestCase, self).configure()
 
         config.Memcached.Pools.Default.ClientEnabled = False
         config.Memcached.Pools.Default.ServerEnabled = False
         ClientFactory.allowTestCache = True
         memcacher.Memcacher.allowTestCache = True
-        memcacher.Memcacher.memoryCacheInstance = None
+        memcacher.Memcacher.reset()
         config.DirectoryAddressBook.Enabled = False
         config.UsePackageTimezones = True
 
-        accounts = FilePath(config.DataRoot).child("accounts.xml")
-        accounts.setContent(xmlFile.getContent())
-
-
-    @property
-    def directoryService(self):
-        """
-        Read-only alias for L{DirectoryFixture.directoryService} for
-        compatibility with older tests.  TODO: remove this.
-        """
-        return self.directory
-
-
-
-class TestCase(twext.web2.dav.test.util.TestCase):
-    resource_class = RootResource
-
-    def createDataStore(self):
-        """
-        Create an L{IDataStore} that can store calendars (but not
-        addressbooks.)  By default returns a L{CommonDataStore}, but this is a
-        hook for subclasses to override to provide different data stores.
-        """
-        return CommonDataStore(FilePath(config.DocumentRoot), None, None, True, False,
-                               quota=deriveQuota(self))
-
-
-    def createStockDirectoryService(self):
-        """
-        Create a stock C{directoryService} attribute and assign it.
-        """
-        self.xmlFile = FilePath(config.DataRoot).child("accounts.xml")
-        self.xmlFile.setContent(xmlFile.getContent())
-        self.directoryFixture.addDirectoryService(XMLDirectoryService({
-            "xmlFile": "accounts.xml",
-            "augmentService":
-                augment.AugmentXMLDB(xmlFiles=(augmentsFile.path,)),
-        }))
-
-
-    def setupCalendars(self):
-        """
-        When a directory service exists, set up the resources at C{/calendars}
-        and C{/addressbooks} (a L{DirectoryCalendarHomeProvisioningResource}
-        and L{DirectoryAddressBookHomeProvisioningResource} respectively), and
-        assign them to the C{self.calendarCollection} and
-        C{self.addressbookCollection} attributes.
-
-        A directory service may be associated with this L{TestCase} with
-        L{TestCase.createStockDirectoryService} or
-        L{TestCase.directoryFixture.addDirectoryService}.
-        """
-        newStore = self.createDataStore()
-        @self.directoryFixture.whenDirectoryServiceChanges
-        def putAllChildren(ds):
-            self.calendarCollection = (
-                DirectoryCalendarHomeProvisioningResource(
-                    ds, "/calendars/", newStore
-                ))
-            self.site.resource.putChild("calendars", self.calendarCollection)
-            self.addressbookCollection = (
-                DirectoryAddressBookHomeProvisioningResource(
-                    ds, "/addressbooks/", newStore
-                ))
-            self.site.resource.putChild("addressbooks",
-                                        self.addressbookCollection)
-
-
-    def configure(self):
-        """
-        Adjust the global configuration for this test.
-        """
-        config.reset()
-
-        config.ServerRoot = os.path.abspath(self.serverRoot)
-        config.ConfigRoot = "config"
-        config.LogRoot = "logs"
-        config.RunRoot = "logs"
-
-        config.Memcached.Pools.Default.ClientEnabled = False
-        config.Memcached.Pools.Default.ServerEnabled = False
-        ClientFactory.allowTestCache = True
-        memcacher.Memcacher.allowTestCache = True
-        memcacher.Memcacher.memoryCacheInstance = None
-        config.DirectoryAddressBook.Enabled = False
-        config.UsePackageTimezones = True
-
-
-    @property
-    def directoryService(self):
-        """
-        Read-only alias for L{DirectoryFixture.directoryService} for
-        compatibility with older tests.  TODO: remove this.
-        """
-        return self.directoryFixture.directoryService
-
-
-    def setUp(self):
-        super(TestCase, self).setUp()
-
-        self.directoryFixture = DirectoryFixture()
-
-        # FIXME: this is only here to workaround circular imports
-        doBind()
-
-        self.serverRoot = self.mktemp()
-        os.mkdir(self.serverRoot)
-
-        self.configure()
-
-        if not os.path.exists(config.DataRoot):
-            os.makedirs(config.DataRoot)
-        if not os.path.exists(config.DocumentRoot):
-            os.makedirs(config.DocumentRoot)
-        if not os.path.exists(config.ConfigRoot):
-            os.makedirs(config.ConfigRoot)
-        if not os.path.exists(config.LogRoot):
-            os.makedirs(config.LogRoot)
-
 
     def createHierarchy(self, structure, root=None):
         if root is None:
@@ -505,7 +272,7 @@ class TestCase(twext.web2.dav.test.util.TestCase):
                                     print("Xattr mismatch:", childPath, attr)
                                     print((xattr.getxattr(childPath, attr), " != ", value))
                                     return False
-                            else: # method
+                            else:  # method
                                 if not value(xattr.getxattr(childPath, attr)):
                                     return False
 
@@ -527,166 +294,97 @@ class TestCase(twext.web2.dav.test.util.TestCase):
 
 
 
+class TestCase(txweb2.dav.test.util.TestCase):
+    resource_class = RootResource
+
+    def createDataStore(self):
+        """
+        Create an L{IDataStore} that can store calendars (but not
+        addressbooks.)  By default returns a L{CommonDataStore}, but this is a
+        hook for subclasses to override to provide different data stores.
+        """
+        return CommonDataStore(FilePath(config.DocumentRoot), None, None, True, False,
+                               quota=deriveQuota(self))
+
+
+    def setupCalendars(self):
+        """
+        When a directory service exists, set up the resources at C{/calendars}
+        and C{/addressbooks} (a L{DirectoryCalendarHomeProvisioningResource}
+        and L{DirectoryAddressBookHomeProvisioningResource} respectively), and
+        assign them to the C{self.calendarCollection} and
+        C{self.addressbookCollection} attributes.
+
+        A directory service may be associated with this L{TestCase} with
+        L{TestCase.createStockDirectoryService} or
+        L{TestCase.directoryFixture.addDirectoryService}.
+        """
+        newStore = self.createDataStore()
+
+
+        @self.directoryFixture.whenDirectoryServiceChanges
+        def putAllChildren(ds):
+            self.calendarCollection = (
+                DirectoryCalendarHomeProvisioningResource(
+                    ds, "/calendars/", newStore
+                ))
+            self.site.resource.putChild("calendars", self.calendarCollection)
+            self.addressbookCollection = (
+                DirectoryAddressBookHomeProvisioningResource(
+                    ds, "/addressbooks/", newStore
+                ))
+            self.site.resource.putChild("addressbooks",
+                                        self.addressbookCollection)
+
+
+    def configure(self):
+        """
+        Adjust the global configuration for this test.
+        """
+        config.reset()
+
+        config.ServerRoot = os.path.abspath(self.serverRoot)
+        config.ConfigRoot = "config"
+        config.LogRoot = "logs"
+        config.RunRoot = "logs"
+
+        config.Memcached.Pools.Default.ClientEnabled = False
+        config.Memcached.Pools.Default.ServerEnabled = False
+        ClientFactory.allowTestCache = True
+        memcacher.Memcacher.allowTestCache = True
+        memcacher.Memcacher.reset()
+        config.DirectoryAddressBook.Enabled = False
+        config.UsePackageTimezones = True
+
+
+    def setUp(self):
+        super(TestCase, self).setUp()
+
+        # FIXME: this is only here to workaround circular imports
+        doBind()
+
+        self.serverRoot = self.mktemp()
+        os.mkdir(self.serverRoot)
+
+        self.configure()
+
+        if not os.path.exists(config.DataRoot):
+            os.makedirs(config.DataRoot)
+        if not os.path.exists(config.DocumentRoot):
+            os.makedirs(config.DocumentRoot)
+        if not os.path.exists(config.ConfigRoot):
+            os.makedirs(config.ConfigRoot)
+        if not os.path.exists(config.LogRoot):
+            os.makedirs(config.LogRoot)
+
+
+
 class norequest(object):
     def addResponseFilter(self, filter):
         "stub; ignore me"
 
 
 
-class HomeTestCase(TestCase):
-    """
-    Utility class for tests which wish to interact with a calendar home rather
-    than a top-level resource hierarchy.
-    """
-
-    def createDataStore(self):
-        # FIXME: AddressBookHomeTestCase needs the same treatment.
-        fp = FilePath(self.mktemp())
-        fp.createDirectory()
-        return CommonDataStore(fp, None, None, True, False)
-
-
-    def setUp(self):
-        """
-        Replace self.site.resource with an appropriately provisioned
-        L{CalendarHomeResource}, and, if the data store backing this test is a
-        file store, replace C{self.docroot} with a path pointing at the path
-        that stores the data for that L{CalendarHomeResource}.
-        """
-        super(HomeTestCase, self).setUp()
-        self.createStockDirectoryService()
-        @self.directoryFixture.whenDirectoryServiceChanges
-        def addHomeProvisioner(ds):
-            self.homeProvisioner = DirectoryCalendarHomeProvisioningResource(
-                ds, "/calendars/", self.createDataStore()
-            )
-
-        def _defer(user):
-            # Commit the transaction
-            self.addCleanup(self.noRenderCommit)
-            # FIXME: nothing should use docroot any more.
-            aPath = getattr(user._newStoreHome, "_path", None)
-            if aPath is not None:
-                self.docroot = aPath.path
-
-        return self._refreshRoot().addCallback(_defer)
-
-    committed = True
-
-    def noRenderCommit(self):
-        """
-        A resource was retrieved but will not be rendered, so commit.
-        """
-        if not self.committed:
-            self.committed = True
-            return self.site.resource._associatedTransaction.commit()
-
-
-    @inlineCallbacks
-    def _refreshRoot(self, request=None):
-        """
-        Refresh the user resource positioned at the root of this site, to give
-        it a new transaction.
-        """
-        yield self.noRenderCommit()
-        if request is None:
-            request = norequest()
-        users = yield self.homeProvisioner.getChild("users")
-
-        user, ignored = (yield users.locateChild(request, ["wsanchez"]))
-
-        # Force the request to succeed regardless of the implementation of
-        # accessControlList.
-        user.accessControlList = lambda request, *a, **k: succeed(
-            self.grantInherit(davxml.All())
-        )
-
-        # Fix the site to point directly at the user's calendar home so that we
-        # can focus on testing just that rather than hierarchy traversal..
-        self.site.resource = user
-        self.committed = False
-        returnValue(user)
-
-
-    @inlineCallbacks
-    def send(self, request, callback=None):
-        """
-        Override C{send} in order to refresh the 'user' resource each time, to
-        get a new transaction to associate with the calendar home.
-        """
-        yield self.noRenderCommit()
-        yield self._refreshRoot(request)
-        result = (yield super(HomeTestCase, self).send(request))
-        self.committed = True
-        yield self._refreshRoot()
-        if callback is not None:
-            result = yield callback(result)
-        returnValue(result)
-
-
-
-class AddressBookHomeTestCase(TestCase):
-    """
-    Utility class for tests which wish to interact with a addressbook home rather
-    than a top-level resource hierarchy.
-    """
-
-    def setUp(self):
-        """
-        Replace self.site.resource with an appropriately provisioned
-        AddressBookHomeFile, and replace self.docroot with a path pointing at that
-        file.
-        """
-        super(AddressBookHomeTestCase, self).setUp()
-        self.createStockDirectoryService()
-        @self.directoryFixture.whenDirectoryServiceChanges
-        def addHomeProvisioner(ds):
-            self.homeProvisioner = DirectoryAddressBookHomeProvisioningResource(
-                ds, "/calendars/", self.createDataStore()
-            )
-
-        @inlineCallbacks
-        def _defer(user):
-            # Commit the transaction
-            yield self.site.resource._associatedTransaction.commit()
-            self.docroot = user._newStoreHome._path.path
-
-        return self._refreshRoot().addCallback(_defer)
-
-
-    @inlineCallbacks
-    def _refreshRoot(self):
-        """
-        Refresh the user resource positioned at the root of this site, to give
-        it a new transaction.
-        """
-        users = self.homeProvisioner.getChild("users")
-        user, ignored = (yield users.locateChild(norequest(), ["wsanchez"]))
-
-        # Force the request to succeed regardless of the implementation of
-        # accessControlList.
-        user.accessControlList = lambda request, *a, **k: succeed(
-            self.grantInherit(davxml.All())
-        )
-
-        # Fix the site to point directly at the user's calendar home so that we
-        # can focus on testing just that rather than hierarchy traversal..
-        self.site.resource = user
-        returnValue(user)
-
-
-    @inlineCallbacks
-    def send(self, request, callback=None):
-        """
-        Override C{send} in order to refresh the 'user' resource each time, to
-        get a new transaction to associate with the calendar home.
-        """
-        yield self._refreshRoot()
-        result = (yield super(AddressBookHomeTestCase, self).send(request, callback))
-        returnValue(result)
-
-
-
 class InMemoryPropertyStore(object):
     def __init__(self):
         class _FauxPath(object):
@@ -866,7 +564,7 @@ class CapturingProcessProtocol(ProcessProtocol):
         Some output was received on stderr.
         """
         # Ignore the Postgres "NOTICE" output
-        if "NOTICE" in data:
+        if not data or "NOTICE" in data or "PowmInsecureWarning" in data:
             return
 
         self.error.append(data)
@@ -886,4 +584,5 @@ class CapturingProcessProtocol(ProcessProtocol):
         if why.value.exitCode == 0 and not self.error:
             self.deferred.callback(''.join(self.output))
         else:
-            self.deferred.errback(ErrorOutput(repr(''.join(self.error))))
+            errtxt = "Code: {}, Errors: {}".format(why.value.exitCode, repr(''.join(self.error)))
+            self.deferred.errback(ErrorOutput(errtxt))
