@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,10 +19,11 @@ import os
 from twext.python.log import Logger
 
 from twisted.python.filepath import FilePath
+from twisted.python.lockfile import FilesystemLock
 
 from twistedcaldav.config import config
 
-from pycalendar.timezonedb import PyCalendarTimezoneDatabase
+from pycalendar.timezonedb import TimezoneDatabase
 
 log = Logger()
 
@@ -95,7 +96,7 @@ class TimezoneCache(object):
             TimezoneCache.dirName = None
             TimezoneCache.validatePath()
         TimezoneCache.version = TimezoneCache.getTZVersion(TimezoneCache.getDBPath())
-        PyCalendarTimezoneDatabase.createTimezoneDatabase(TimezoneCache.getDBPath())
+        TimezoneDatabase.createTimezoneDatabase(TimezoneCache.getDBPath())
 
 
     @staticmethod
@@ -103,7 +104,7 @@ class TimezoneCache(object):
         try:
             return open(os.path.join(dbpath, "version.txt")).read().strip()
         except IOError:
-            return TimezoneCache.IANA_VERSION_PREFIX + "Unknown"
+            return ""
 
 
     class FilteredFilePath(FilePath):
@@ -133,7 +134,7 @@ class TimezoneCache(object):
             # Check if pkg is more recent and copy over
             pkgversion = TimezoneCache.getTZVersion(TimezoneCache._getPackageDBPath())
             dbversion = TimezoneCache.getTZVersion(dbpath.path)
-            if pkgversion > dbversion:
+            if not dbversion or pkgversion > dbversion:
                 dbpath.remove()
                 TimezoneCache.copyPackage("Updating")
             else:
@@ -143,7 +144,7 @@ class TimezoneCache(object):
     @staticmethod
     def copyPackage(title):
         """
-        Copy package directory to db path using a temporary sibling to avoid potential
+        Copy package directory to db path using a file lock to avoid potential
         concurrency race conditions.
 
         @param title: string to use in log entry
@@ -151,27 +152,28 @@ class TimezoneCache(object):
         """
         dbpath = FilePath(TimezoneCache.getDBPath())
         pkgpath = TimezoneCache.FilteredFilePath(TimezoneCache._getPackageDBPath())
-        log.info(
-            "{title} timezones from {pkg} to {to}",
-            title=title,
-            pkg=pkgpath.path,
-            to=dbpath.path
-        )
 
-        # Use temp directory to copy to first
-        temp = dbpath.temporarySibling()
-        pkgpath.copyFilteredDirectoryTo(temp)
+        lockfile = FilesystemLock(dbpath.path + ".lock")
+        result = lockfile.lock()
+        try:
+            if result and not dbpath.exists():
+                log.info(
+                    "{title} timezones from {pkg} to {to}",
+                    title=title,
+                    pkg=pkgpath.path,
+                    to=dbpath.path
+                )
 
-        # Move to actual path if it stll does not exist
-        if not dbpath.exists():
-            temp.moveTo(dbpath)
-        else:
-            temp.remove()
+                # Copy over the entire package
+                pkgpath.copyFilteredDirectoryTo(dbpath)
+        finally:
+            if result:
+                lockfile.unlock()
 
 
     @staticmethod
     def clear():
-        PyCalendarTimezoneDatabase.clearTimezoneDatabase()
+        TimezoneDatabase.clearTimezoneDatabase()
 
 # zoneinfo never changes in a running instance so cache all this data as we use it
 cachedTZs = {}
@@ -184,6 +186,8 @@ def hasTZ(tzid):
     cannot be found.
     """
 
+    if isinstance(tzid, unicode):
+        tzid = tzid.encode("utf-8")
     if tzid not in cachedVTZs:
         readVTZ(tzid)
     return True
@@ -194,6 +198,8 @@ def addVTZ(tzid, tzcal):
     """
     Add a VTIMEZONE component to the cache.
     """
+    if isinstance(tzid, unicode):
+        tzid = tzid.encode("utf-8")
     if tzid not in cachedVTZs:
         cachedVTZs[tzid] = tzcal
         cachedTZs[tzid] = str(tzcal)
@@ -205,9 +211,11 @@ def readVTZ(tzid):
     Try to load the specified TZID as a calendar object from the database. Raise if not found.
     """
 
+    if isinstance(tzid, unicode):
+        tzid = tzid.encode("utf-8")
     if tzid not in cachedVTZs:
 
-        tzcal = PyCalendarTimezoneDatabase.getTimezoneInCalendar(tzid)
+        tzcal = TimezoneDatabase.getTimezoneInCalendar(tzid)
         if tzcal:
             cachedVTZs[tzid] = tzcal
         else:
@@ -222,6 +230,8 @@ def readTZ(tzid):
     Try to load the specified TZID as text from the database. Raise if not found.
     """
 
+    if isinstance(tzid, unicode):
+        tzid = tzid.encode("utf-8")
     if tzid not in cachedTZs:
 
         tzcal = readVTZ(tzid)
