@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 """
 Timezone service resource and operations.
 
-This is based on http://tools.ietf.org/html/draft-douglass-timezone-service which is the CalConnect
+This is based on http://tools.ietf.org/html/draft-ietf-tzdist-service which is the IETF
 proposal for a standard timezone service.
 """
 
@@ -26,13 +26,14 @@ __all__ = [
 ]
 
 from twext.python.log import Logger
-from twext.web2 import responsecode
-from twext.web2.dav.method.propfind import http_PROPFIND
-from twext.web2.dav.noneprops import NonePropertyStore
-from twext.web2.http import HTTPError, JSONResponse
-from twext.web2.http import Response
-from twext.web2.http_headers import MimeType
-from twext.web2.stream import MemoryStream
+from txweb2 import responsecode
+from txweb2.dav.method.propfind import http_PROPFIND
+from txweb2.dav.noneprops import NonePropertyStore
+from txweb2.dav.util import joinURL
+from txweb2.http import HTTPError, JSONResponse, StatusResponse
+from txweb2.http import Response
+from txweb2.http_headers import MimeType
+from txweb2.stream import MemoryStream
 from txdav.xml import element as davxml
 
 from twisted.internet.defer import succeed, inlineCallbacks, returnValue, \
@@ -47,16 +48,18 @@ from twistedcaldav.ical import tzexpandlocal
 from twistedcaldav.resource import ReadOnlyNoCopyResourceMixIn
 from twistedcaldav.timezones import TimezoneException, TimezoneCache, readVTZ, \
     addVTZ
+from twistedcaldav.util import bestAcceptType
 from twistedcaldav.xmlutil import addSubElement
 
-from pycalendar.calendar import PyCalendar
-from pycalendar.datetime import PyCalendarDateTime
-from pycalendar.exceptions import PyCalendarInvalidData
+from pycalendar.icalendar.calendar import Calendar
+from pycalendar.datetime import DateTime
+from pycalendar.exceptions import InvalidData
 
 import hashlib
 import itertools
 import json
 import os
+import urllib
 
 log = Logger()
 
@@ -89,6 +92,12 @@ class TimezoneStdServiceResource (ReadOnlyNoCopyResourceMixIn, DAVResourceWithou
         else:
             raise ValueError("Invalid TimezoneService mode: %s" % (config.TimezoneService.Mode,))
 
+        self.formats = []
+        self.formats.append("text/calendar")
+        if config.EnableJSONData:
+            self.formats.append("application/calendar+json")
+        self.formats.append("text/plain")
+
 
     def _initPrimaryService(self):
         tzpath = TimezoneCache.getDBPath()
@@ -140,15 +149,17 @@ class TimezoneStdServiceResource (ReadOnlyNoCopyResourceMixIn, DAVResourceWithou
 
 
     def defaultAccessControlList(self):
-        return davxml.ACL(
-            # DAV:Read for all principals (includes anonymous)
-            davxml.ACE(
-                davxml.Principal(davxml.All()),
-                davxml.Grant(
-                    davxml.Privilege(davxml.Read()),
+        return succeed(
+            davxml.ACL(
+                # DAV:Read for all principals (includes anonymous)
+                davxml.ACE(
+                    davxml.Principal(davxml.All()),
+                    davxml.Grant(
+                        davxml.Privilege(davxml.Read()),
+                    ),
+                    davxml.Protected(),
                 ),
-                davxml.Protected(),
-            ),
+            )
         )
 
 
@@ -188,135 +199,168 @@ class TimezoneStdServiceResource (ReadOnlyNoCopyResourceMixIn, DAVResourceWithou
 
     http_PROPFIND = http_PROPFIND
 
-    def http_GET(self, request):
-        """
-        The timezone service POST method.
-        """
-
-        # GET and POST do the same thing
-        return self.http_POST(request)
+    def problemReport(self, code, description, status):
+        raise HTTPError(JSONResponse(
+            status,
+            {
+                "type": "https://datatracker.ietf.org/doc/draft-ietf-tzdist-service/",
+                "error-code": code,
+                "title": description,
+                "status": status,
+            },
+            contentType="application/problem+json",
+            pretty=config.TimezoneService.PrettyPrintJSON,
+        ))
 
 
     def http_POST(self, request):
+        raise HTTPError(StatusResponse(responsecode.NOT_ALLOWED, "Method not allowed"))
+
+
+    @inlineCallbacks
+    def http_GET(self, request):
         """
-        The timezone service POST method.
+        The timezone service GET method.
         """
 
-        # Check authentication and access controls
-        def _gotResult(_):
+        yield self.authorize(request, (davxml.Read(),))
 
-            if not request.args:
-                # Do normal GET behavior
-                return self.render(request)
+        urlbits = map(urllib.unquote, request.path.strip("/").split("/")[1:])
+        child = urlbits[0] if len(urlbits) > 0 else ""
 
-            action = request.args.get("action", ("",))
-            if len(action) != 1:
-                raise HTTPError(JSONResponse(
-                    responsecode.BAD_REQUEST,
-                    {
-                        "error": "invalid-action",
-                        "description": "Invalid action query parameter",
-                    },
-                ))
-            action = action[0]
+        if not child:
+            if len(request.args) != 0:
+                self.problemReport("invalid-action", "Invalid request-URI query parameters", responsecode.BAD_REQUEST)
 
-            action = {
-                "capabilities"  : self.doCapabilities,
-                "list"          : self.doList,
-                "get"           : self.doGet,
-                "expand"        : self.doExpand,
-            }.get(action, None)
+            # Do normal GET behavior
+            returnValue(self.render(request))
 
-            if action is None:
-                raise HTTPError(JSONResponse(
-                    responsecode.BAD_REQUEST,
-                    {
-                        "error": "invalid-action",
-                        "description": "Unknown action query parameter",
-                    },
-                ))
+        childResponder = {
+            "capabilities"  : self.childCapabilities,
+            "zones"         : self.childZones,
+            "observances"   : self.childObservances,
+        }.get(child, None)
 
-            return action(request)
+        if childResponder is None:
+            self.problemReport("invalid-action", "Invalid action", responsecode.BAD_REQUEST)
 
-        d = self.authorize(request, (davxml.Read(),))
-        d.addCallback(_gotResult)
-        return d
+        returnValue(childResponder(request, urlbits))
 
 
-    def doCapabilities(self, request):
+    def childCapabilities(self, request, urlbits):
         """
-        Return a list of all timezones known to the server.
+        Request on {/service-prefix}/capabilities.
+        """
+        if len(urlbits) != 1:
+            self.problemReport("invalid-action", "Invalid path segment", responsecode.BAD_REQUEST)
+        return self.actionCapabilities(request)
+
+
+    def childZones(self, request, urlbits):
+        """
+        Request on {/service-prefix}/zones.
+        """
+        if len(urlbits) == 2:
+            return self.actionGet(request, urlbits[1])
+        elif len(urlbits) == 1:
+            pattern = request.args.get("pattern", ())
+            if len(pattern) == 0:
+                return self.actionList(request)
+            else:
+                return self.actionFind(request)
+        else:
+            self.problemReport("invalid-action", "Invalid path segment", responsecode.BAD_REQUEST)
+
+
+    def childObservances(self, request, urlbits):
+        """
+        Request on {/service-prefix}/observances.
+        """
+        if len(urlbits) == 2:
+            return self.actionExpand(request, urlbits[1])
+        elif len(urlbits) == 1:
+            self.problemReport("invalid-action", "Missing {tzid} path segment", responsecode.BAD_REQUEST)
+        else:
+            self.problemReport("invalid-action", "Invalid path segment", responsecode.BAD_REQUEST)
+
+
+    def actionCapabilities(self, request):
+        """
+        Return the capabilities of this server.
         """
 
+        if len(request.args) != 0:
+            self.problemReport("invalid-action", "Invalid request-URI query parameters", responsecode.BAD_REQUEST)
+
+        urlbase = request.path.rsplit("/", 1)[0]
         result = {
+            "version": "1",
             "info" : {
-                "version": "1",
                 "primary-source" if self.primary else "secondary_source": self.info_source,
+                "formats": self.formats,
                 "contacts" : [],
             },
             "actions" : [
                 {
                     "name": "capabilities",
+                    "uri-template": joinURL(urlbase, "capabilities"),
                     "parameters": [],
                 },
                 {
                     "name": "list",
+                    "uri-template": joinURL(urlbase, "zones{?changedsince}"),
                     "parameters": [
                         {"name": "changedsince", "required": False, "multi": False, },
                     ],
                 },
                 {
                     "name": "get",
+                    "uri-template": joinURL(urlbase, "zones{/tzid}{?start,end}"),
                     "parameters": [
-                        {"name": "format", "required": False, "multi": False, "values": ["text/calendar", "text/plain", ], },
-                        {"name": "tzid", "required": True, "multi": False, },
+                        {"name": "start", "required": False, "multi": False},
+                        {"name": "stop", "required": False, "multi": False, },
                     ],
                 },
                 {
                     "name": "expand",
+                    "uri-template": joinURL(urlbase, "zones{/tzid}/observances{?start,end}"),
                     "parameters": [
-                        {"name": "tzid", "required": True, "multi": False, },
-                        {"name": "start", "required": False, "multi": False, },
-                        {"name": "end", "required": False, "multi": False, },
+                        {"name": "start", "required": True, "multi": False, },
+                        {"name": "end", "required": True, "multi": False, },
+                    ],
+                },
+                {
+                    "name": "find",
+                    "uri-template": joinURL(urlbase, "zones{?pattern}"),
+                    "parameters": [
+                        {"name": "pattern", "required": True, "multi": False, },
                     ],
                 },
             ]
         }
-        return JSONResponse(responsecode.OK, result)
+        return JSONResponse(responsecode.OK, result, pretty=config.TimezoneService.PrettyPrintJSON)
 
 
-    def doList(self, request):
+    def actionList(self, request):
         """
         Return a list of all timezones known to the server.
         """
 
+        if set(request.args.keys()) - set(("changedsince",)):
+            self.problemReport("invalid-action", "Invalid request-URI query parameters", responsecode.BAD_REQUEST)
+
         changedsince = request.args.get("changedsince", ())
         if len(changedsince) > 1:
-            raise HTTPError(JSONResponse(
-                responsecode.BAD_REQUEST,
-                {
-                    "error": "invalid-changedsince",
-                    "description": "Invalid changedsince query parameter",
-                },
-            ))
+            self.problemReport("invalid-changedsince", "Too many changedsince request-URI query parameters", responsecode.BAD_REQUEST)
         if len(changedsince) == 1:
             # Validate a date-time stamp
             changedsince = changedsince[0]
             try:
-                dt = PyCalendarDateTime.parseText(changedsince)
+                dt = DateTime.parseText(changedsince, fullISO=True)
             except ValueError:
-                raise HTTPError(JSONResponse(
-                    responsecode.BAD_REQUEST,
-                    {
-                        "error": "invalid-changedsince",
-                        "description": "Invalid changedsince query parameter",
-                    },
-                ))
+                self.problemReport("invalid-changedsince", "Invalid changedsince request-URI query parameter value", responsecode.BAD_REQUEST)
             if not dt.utc():
-                raise HTTPError(JSONResponse(
-                    responsecode.BAD_REQUEST,
-                    "Invalid changedsince query parameter value",
-                ))
+                self.problemReport("invalid-changedsince", "Invalid changedsince request-URI query parameter value - not UTC", responsecode.BAD_REQUEST)
 
         timezones = []
         for tz in self.timezones.listTimezones(changedsince):
@@ -329,129 +373,83 @@ class TimezoneStdServiceResource (ReadOnlyNoCopyResourceMixIn, DAVResourceWithou
             "dtstamp": self.timezones.dtstamp,
             "timezones": timezones,
         }
-        return JSONResponse(responsecode.OK, result)
+        return JSONResponse(responsecode.OK, result, pretty=config.TimezoneService.PrettyPrintJSON)
 
 
-    def doGet(self, request):
+    def actionGet(self, request, tzid):
         """
         Return the specified timezone data.
         """
 
-        tzids = request.args.get("tzid", ())
-        if len(tzids) != 1:
-            raise HTTPError(JSONResponse(
-                responsecode.BAD_REQUEST,
-                {
-                    "error": "invalid-tzid",
-                    "description": "Invalid tzid query parameter",
-                },
-            ))
+        if set(request.args.keys()) - set(("start", "end",)):
+            self.problemReport("invalid-action", "Invalid request-URI query parameters", responsecode.BAD_REQUEST)
 
-        format = request.args.get("format", ("text/calendar",))
-        if len(format) != 1 or format[0] not in ("text/calendar", "text/plain",):
-            raise HTTPError(JSONResponse(
-                responsecode.BAD_REQUEST,
-                {
-                    "error": "invalid-format",
-                    "description": "Invalid format query parameter",
-                },
-            ))
-        format = format[0]
+        accepted_type = bestAcceptType(request.headers.getHeader("accept"), self.formats)
+        if accepted_type is None:
+            self.problemReport("invalid-format", "Accept header does not match available media types", responsecode.NOT_ACCEPTABLE)
 
-        calendar = self.timezones.getTimezone(tzids[0])
+        calendar = self.timezones.getTimezone(tzid)
         if calendar is None:
-            raise HTTPError(JSONResponse(
-                responsecode.NOT_FOUND,
-                {
-                    "error": "missing-tzid",
-                    "description": "Tzid could not be found",
-                }
-            ))
+            self.problemReport("tzid-not-found", "Time zone identifier not found", responsecode.NOT_FOUND)
 
-        tzdata = calendar.getText()
+        tzdata = calendar.getText(format=accepted_type if accepted_type != "text/plain" else None)
 
         response = Response()
         response.stream = MemoryStream(tzdata)
-        response.headers.setHeader("content-type", MimeType.fromString("%s; charset=utf-8" % (format,)))
+        response.headers.setHeader("content-type", MimeType.fromString("%s; charset=utf-8" % (accepted_type,)))
         return response
 
 
-    def doExpand(self, request):
+    def actionExpand(self, request, tzid):
         """
         Expand a timezone within specified start/end dates.
         """
 
-        tzids = request.args.get("tzid", ())
-        if len(tzids) != 1:
-            raise HTTPError(JSONResponse(
-                responsecode.BAD_REQUEST,
-                {
-                    "error": "invalid-tzid",
-                    "description": "Invalid tzid query parameter",
-                },
-            ))
+        if set(request.args.keys()) - set(("start", "end",)):
+            self.problemReport("invalid-action", "Invalid request-URI query parameters", responsecode.BAD_REQUEST)
 
-        try:
-            start = request.args.get("start", ())
-            if len(start) > 1:
-                raise ValueError()
-            elif len(start) == 1:
-                start = PyCalendarDateTime.parseText(start[0])
-            else:
-                start = PyCalendarDateTime.getToday()
-                start.setDay(1)
-                start.setMonth(1)
-        except ValueError:
-            raise HTTPError(JSONResponse(
-                responsecode.BAD_REQUEST,
-                {
-                    "error": "invalid-start",
-                    "description": "Invalid start query parameter",
-                }
-            ))
+        start = request.args.get("start", ())
+        if len(start) == 0:
+            self.problemReport("invalid-start", "Missing start request-URI query parameter", responsecode.BAD_REQUEST)
+        if len(start) > 1:
+            self.problemReport("invalid-start", "Too many start request-URI query parameters", responsecode.BAD_REQUEST)
+        elif len(start) == 1:
+            try:
+                if len(start[0]) != 20:
+                    raise ValueError()
+                start = DateTime.parseText(start[0], fullISO=True)
+            except ValueError:
+                self.problemReport("invalid-start", "Invalid start request-URI query parameter value", responsecode.BAD_REQUEST)
 
-        try:
-            end = request.args.get("end", ())
-            if len(end) > 1:
-                raise ValueError()
-            elif len(end) == 1:
-                end = PyCalendarDateTime.parseText(end[0])
-            else:
-                end = PyCalendarDateTime.getToday()
-                end.setDay(1)
-                end.setMonth(1)
-                end.offsetYear(10)
+        end = request.args.get("end", ())
+        if len(end) == 0:
+            self.problemReport("invalid-end", "Missing end request-URI query parameter", responsecode.BAD_REQUEST)
+        if len(end) > 1:
+            self.problemReport("invalid-end", "Too many end request-URI query parameters", responsecode.BAD_REQUEST)
+        elif len(end) == 1:
+            try:
+                if len(end[0]) != 20:
+                    raise ValueError()
+                end = DateTime.parseText(end[0], fullISO=True)
+            except ValueError:
+                self.problemReport("invalid-end", "Invalid end request-URI query parameter value", responsecode.BAD_REQUEST)
             if end <= start:
-                raise ValueError()
-        except ValueError:
-            raise HTTPError(JSONResponse(
-                responsecode.BAD_REQUEST,
-                {
-                    "error": "invalid-end",
-                    "description": "Invalid end query parameter",
-                }
-            ))
+                self.problemReport("invalid-end", "Invalid end request-URI query parameter value - earlier than start", responsecode.BAD_REQUEST)
 
-        tzid = tzids[0]
         tzdata = self.timezones.getTimezone(tzid)
         if tzdata is None:
-            raise HTTPError(JSONResponse(
-                responsecode.NOT_FOUND,
-                {
-                    "error": "missing-tzid",
-                    "description": "Tzid could not be found",
-                }
-            ))
+            self.problemReport("tzid-not-found", "Time zone identifier not found", responsecode.NOT_FOUND)
 
         # Now do the expansion (but use a cache to avoid re-calculating TZs)
         observances = self.expandcache.get((tzid, start, end), None)
         if observances is None:
-            observances = tzexpandlocal(tzdata, start, end)
+            observances = tzexpandlocal(tzdata, start, end, utc_onset=True)
             self.expandcache[(tzid, start, end)] = observances
 
         # Turn into JSON
         result = {
             "dtstamp": self.timezones.dtstamp,
+            "tzid": tzid,
             "observances": [
                 {
                     "name": name,
@@ -461,7 +459,72 @@ class TimezoneStdServiceResource (ReadOnlyNoCopyResourceMixIn, DAVResourceWithou
                 } for onset, utc_offset_from, utc_offset_to, name in observances
             ],
         }
-        return JSONResponse(responsecode.OK, result)
+        return JSONResponse(responsecode.OK, result, pretty=config.TimezoneService.PrettyPrintJSON)
+
+
+    def actionFind(self, request):
+        """
+        Return a list of all timezones matching a pattern.
+        """
+
+        if set(request.args.keys()) - set(("pattern",)):
+            self.problemReport("invalid-action", "Invalid request-URI query parameters", responsecode.BAD_REQUEST)
+
+        pattern = request.args.get("pattern", ())
+        if len(pattern) == 0:
+            self.problemReport("invalid-pattern", "Missing pattern request-URI query parameter", responsecode.BAD_REQUEST)
+        elif len(pattern) > 1:
+            self.problemReport("invalid-pattern", "Too many pattern request-URI query parameters", responsecode.BAD_REQUEST)
+        pattern = pattern[0]
+
+        def _comp_is(pattern, s):
+            return pattern == s
+        def _comp_startswith(pattern, s):
+            return s.startswith(pattern)
+        def _comp_endswith(pattern, s):
+            return s.endswith(pattern)
+        def _comp_contains(pattern, s):
+            return pattern in s
+
+        def _normalize(s):
+            return s.replace("_", " ").lower()
+
+        if pattern.startswith("*") and pattern.endswith("*"):
+            pattern = pattern[1:-1]
+            comparator = _comp_contains
+        elif pattern.endswith("*"):
+            pattern = pattern[:-1]
+            comparator = _comp_startswith
+        elif pattern.startswith("*"):
+            pattern = pattern[1:]
+            comparator = _comp_endswith
+        else:
+            comparator = _comp_is
+        pattern = _normalize(pattern)
+
+        if not pattern:
+            self.problemReport("invalid-pattern", "Invalid pattern request-URI query parameter value", responsecode.BAD_REQUEST)
+
+        timezones = []
+        for tz in self.timezones.listTimezones(None):
+            matched = comparator(pattern, _normalize(tz.tzid))
+            if not matched:
+                for alias in tz.aliases:
+                    if comparator(pattern, _normalize(alias)):
+                        matched = True
+                        break
+            if matched:
+                timezones.append({
+                    "tzid": tz.tzid,
+                    "last-modified": tz.dtstamp,
+                    "aliases": tz.aliases,
+                })
+
+        result = {
+            "dtstamp": self.timezones.dtstamp,
+            "timezones": timezones,
+        }
+        return JSONResponse(responsecode.OK, result, pretty=config.TimezoneService.PrettyPrintJSON)
 
 
 
@@ -558,7 +621,7 @@ class CommonTimezoneDatabase(object):
         Generate a PyCalendar containing the requested timezone.
         """
         # We will just use our existing TimezoneCache here
-        calendar = PyCalendar()
+        calendar = Calendar()
         try:
             vtz = readVTZ(tzid)
             calendar.addComponent(vtz.getComponents()[0].duplicate())
@@ -618,7 +681,7 @@ class PrimaryTimezoneDatabase(CommonTimezoneDatabase):
         Create a new DB xml file from scratch by scanning zoneinfo.
         """
 
-        self.dtstamp = PyCalendarDateTime.getNowUTC().getXMLText()
+        self.dtstamp = DateTime.getNowUTC().getXMLText()
         self._scanTZs("")
         self._dumpTZs()
 
@@ -655,7 +718,7 @@ class PrimaryTimezoneDatabase(CommonTimezoneDatabase):
 
         try:
             for alias in aliases.splitlines():
-                alias_from, alias_to = alias.split()
+                alias_from, alias_to = alias.split("\t")
                 tzinfo = self.timezones.get(alias_to)
                 if tzinfo:
                     if alias_from != alias_to:
@@ -672,7 +735,7 @@ class PrimaryTimezoneDatabase(CommonTimezoneDatabase):
         """
         Update existing DB info by comparing md5's.
         """
-        self.dtstamp = PyCalendarDateTime.getNowUTC().getXMLText()
+        self.dtstamp = DateTime.getNowUTC().getXMLText()
         self.changeCount = 0
         self.changed = set()
         self._scanTZs("", checkIfChanged=True)
@@ -847,8 +910,8 @@ class SecondaryTimezoneDatabase(CommonTimezoneDatabase):
 
         ical = response.data
         try:
-            calendar = PyCalendar.parseText(ical)
-        except PyCalendarInvalidData:
+            calendar = Calendar.parseText(ical)
+        except InvalidData:
             log.error("Invalid calendar data for tzid: %s" % (tzinfo.tzid,))
             returnValue(None)
         ical = calendar.getText()
