@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2007-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2007-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ import os
 import re
 import sys
 import base64
+import itertools
 
 from subprocess import Popen, PIPE, STDOUT
 from hashlib import md5, sha1
@@ -29,20 +30,26 @@ from twext.python.log import Logger
 
 log = Logger()
 from twext.internet.gaiendpoint import GAIEndpoint
+from twext.internet.adaptendpoint import connect
 
 ##
 # System Resources (Memory size and processor count)
 ##
 
 try:
-    from ctypes import *
-    import ctypes.util
+    from ctypes import (
+        cdll,
+        c_int, c_uint64, c_ulong,
+        c_char_p, c_void_p,
+        addressof, sizeof, c_size_t,
+    )
+    from ctypes.util import find_library
     hasCtypes = True
 except ImportError:
     hasCtypes = False
 
 if sys.platform == "darwin" and hasCtypes:
-    libc = cdll.LoadLibrary(ctypes.util.find_library("libc"))
+    libc = cdll.LoadLibrary(find_library("libc"))
 
     def getNCPU():
         """
@@ -56,8 +63,8 @@ if sys.platform == "darwin" and hasCtypes:
         ]
         libc.sysctlbyname(
             "hw.ncpu",
-            c_voidp(addressof(ncpu)),
-            c_voidp(addressof(size)),
+            c_void_p(addressof(ncpu)),
+            c_void_p(addressof(size)),
             None,
             0
         )
@@ -77,8 +84,8 @@ if sys.platform == "darwin" and hasCtypes:
         ]
         libc.sysctlbyname(
             "hw.memsize",
-            c_voidp(addressof(memsize)),
-            c_voidp(addressof(size)),
+            c_void_p(addressof(memsize)),
+            c_void_p(addressof(size)),
             None,
             0
         )
@@ -87,7 +94,7 @@ if sys.platform == "darwin" and hasCtypes:
 
 
 elif sys.platform == "linux2" and hasCtypes:
-    libc = cdll.LoadLibrary(ctypes.util.find_library("libc"))
+    libc = cdll.LoadLibrary(find_library("libc"))
 
     def getNCPU():
         return libc.get_nprocs()
@@ -489,31 +496,103 @@ class AuthorizedHTTPGetter(client.HTTPPageGetter):
 
 
 
-def normalizationLookup(cuaddr, principalFunction, config):
+def bestAcceptType(accepts, allowedTypes):
     """
-    Lookup function to be passed to ical.normalizeCalendarUserAddresses.
-    Returns a tuple of (Full name, guid, and calendar user address list)
-    for the given cuaddr.  The principalFunction is called to retrieve the
-    principal for the cuaddr.
+    Given a set of Accept headers and the set of types the server can return, determine the best choice
+    of format to return to the client.
+
+    @param accepts: parsed accept headers
+    @type accepts: C{dict}
+    @param allowedTypes: list of allowed types in server preferred order
+    @type allowedTypes: C{list}
     """
-    try:
-        principal = principalFunction(cuaddr)
-    except Exception, e:
-        log.debug("Lookup of %s failed: %s" % (cuaddr, e))
-        principal = None
 
-    if principal is None:
-        return (None, None, None)
-    else:
-        rec = principal.record
+    # If no Accept present just use the first allowed type - the server's preference
+    if not accepts:
+        return allowedTypes[0]
 
-        # RFC5545 syntax does not allow backslash escaping in
-        # parameter values. A double-quote is thus not allowed
-        # in a parameter value except as the start/end delimiters.
-        # Single quotes are allowed, so we convert any double-quotes
-        # to single-quotes.
-        fullName = rec.fullName.replace('"', "'")
+    # Get mapping for ordered top-level types for use in subtype wildcard match
+    toptypes = {}
+    for allowed in allowedTypes:
+        mediaType = allowed.split("/")[0]
+        if mediaType not in toptypes:
+            toptypes[mediaType] = allowed
 
-        cuas = principal.record.calendarUserAddresses
+    result = None
+    result_qval = 0.0
+    for content_type, qval in accepts.items():
+        # Exact match
+        ctype = "%s/%s" % (content_type.mediaType, content_type.mediaSubtype,)
+        if ctype in allowedTypes:
+            if qval > result_qval:
+                result = ctype
+                result_qval = qval
 
-        return (fullName, rec.guid, cuas)
+        # Subtype wildcard match
+        elif content_type.mediaType != "*" and content_type.mediaSubtype == "*":
+            if content_type.mediaType in toptypes:
+                if qval > result_qval:
+                    result = toptypes[content_type.mediaType]
+                    result_qval = qval
+
+        # Full wildcard match
+        elif content_type.mediaType == "*" and content_type.mediaSubtype == "*":
+            if qval > result_qval:
+                result = allowedTypes[0]
+                result_qval = qval
+
+    return result
+
+
+
+def userAgentProductTokens(user_agent):
+    """
+    Parse an HTTP User-Agent header to extract the product tokens and ignore
+    any parenthesized comment strings in the header.
+
+    @param user_agent: text of User-Agent header value
+    @type user_agent: L{str}
+
+    @return: list of product tokens extracted from the header
+    @rtype: L{list}
+    """
+
+    ua_hdr = user_agent.split()
+    ua_tokens = []
+    comment = False
+    for token in ua_hdr:
+        if comment:
+            if token.endswith(")"):
+                comment = False
+        elif token.startswith("("):
+            if not token.endswith(")"):
+                comment = True
+        else:
+            ua_tokens.append(token)
+
+    return ua_tokens
+
+
+
+def matchClientFixes(config, user_agent):
+    """
+    Given a user-agent string, see if it matches any of the configured client fixes.
+
+    @param config: the L{config} to match against.
+    @type config: L{ConfigDict}
+    @param user_agent: the HTTP User-Agent header value to test.
+    @type user_agent: L{str}
+    """
+
+    if len(config.ClientFixesCompiled) == 0 or not user_agent:
+        return set()
+
+    ua_tokens = userAgentProductTokens(user_agent)
+
+    client_fixes = set()
+    for fix, patterns in config.ClientFixesCompiled.items():
+        for pattern, token in itertools.product(patterns, ua_tokens):
+            if pattern.match(token) is not None:
+                client_fixes.add(fix)
+                break
+    return client_fixes
