@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,12 +28,14 @@ import cStringIO as StringIO
 import codecs
 
 from twext.python.log import Logger
-from twext.web2.stream import IStream
-from twext.web2.dav.util import allDataFromStream
+from txweb2.stream import IStream
+from txweb2.dav.util import allDataFromStream
 
-from pycalendar.attribute import PyCalendarAttribute
-from pycalendar.componentbase import PyCalendarComponentBase
-from pycalendar.exceptions import PyCalendarError
+from twistedcaldav.config import config
+
+from pycalendar.parameter import Parameter
+from pycalendar.componentbase import ComponentBase
+from pycalendar.exceptions import ErrorBase
 from pycalendar.vcard.card import Card
 from pycalendar.vcard.property import Property as pyProperty
 
@@ -44,6 +46,8 @@ vCardProductID = "-//CALENDARSERVER.ORG//NONSGML Version 1//EN"
 class InvalidVCardDataError(ValueError):
     pass
 
+
+
 class Property (object):
     """
     vCard Property
@@ -56,7 +60,7 @@ class Property (object):
             values are (possibly empty) lists of parameter values.
         """
         if name is None:
-            assert value  is None
+            assert value is None
             assert params is None
 
             pyobj = kwargs["pycard"]
@@ -73,30 +77,54 @@ class Property (object):
             for attrname, attrvalue in params.items():
                 if isinstance(attrvalue, unicode):
                     attrvalue = attrvalue.encode("utf-8")
-                self._pycard.addAttribute(PyCalendarAttribute(attrname, attrvalue))
+                self._pycard.addParameter(Parameter(attrname, attrvalue))
 
-    def __str__ (self): return str(self._pycard)
-    def __repr__(self): return "<%s: %r: %r>" % (self.__class__.__name__, self.name(), self.value())
 
-    def __hash__(self): return hash(str(self))
+    def __str__(self):
+        return str(self._pycard)
+
+
+    def __repr__(self):
+        return "<%s: %r: %r>" % (self.__class__.__name__, self.name(), self.value())
+
+
+    def __hash__(self):
+        return hash(str(self))
+
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
 
-    def __ne__(self, other): return not self.__eq__(other)
     def __eq__(self, other):
-        if not isinstance(other, Property): return False
+        if not isinstance(other, Property):
+            return False
         return self._pycard == other._pycard
 
-    def __gt__(self, other): return not (self.__eq__(other) or self.__lt__(other))
+
+    def __gt__(self, other):
+        return not (self.__eq__(other) or self.__lt__(other))
+
+
     def __lt__(self, other):
         my_name = self.name()
         other_name = other.name()
 
-        if my_name < other_name: return True
-        if my_name > other_name: return False
+        if my_name < other_name:
+            return True
+        if my_name > other_name:
+            return False
 
         return self.value() < other.value()
 
-    def __ge__(self, other): return self.__eq__(other) or self.__gt__(other)
-    def __le__(self, other): return self.__eq__(other) or self.__lt__(other)
+
+    def __ge__(self, other):
+        return self.__eq__(other) or self.__gt__(other)
+
+
+    def __le__(self, other):
+        return self.__eq__(other) or self.__lt__(other)
+
 
     def duplicate(self):
         """
@@ -105,35 +133,45 @@ class Property (object):
         """
         return Property(None, None, params=None, pycard=self._pycard.duplicate())
 
-    def name  (self): return self._pycard.getName()
 
-    def value (self): return self._pycard.getValue().getValue()
+    def name(self):
+        return self._pycard.getName()
+
+
+    def value(self):
+        return self._pycard.getValue().getValue()
+
+
+    def strvalue(self):
+        return str(self._pycard.getValue())
 
-    def strvalue (self): return str(self._pycard.getValue())
 
     def setValue(self, value):
         self._pycard.setValue(value)
 
+
     def parameterNames(self):
         """
         Returns a set containing parameter names for this property.
         """
         result = set()
-        for pyattrlist in self._pycard.getAttributes().values():
+        for pyattrlist in self._pycard.getParameters().values():
             for pyattr in pyattrlist:
                 result.add(pyattr.getName())
         return result
 
+
     def parameterValue(self, name, default=None):
         """
         Returns a single value for the given parameter.  Raises
         InvalidICalendarDataError if the parameter has more than one value.
         """
         try:
-            return self._pycard.getAttributeValue(name)
+            return self._pycard.getParameterValue(name)
         except KeyError:
             return default
 
+
     def parameterValues(self, name):
         """
         Returns a single value for the given parameter.  Raises
@@ -141,7 +179,7 @@ class Property (object):
         """
         results = []
         try:
-            attrs = self._pycard.getAttributes()[name.upper()]
+            attrs = self._pycard.getParameters()[name.upper()]
         except KeyError:
             return []
 
@@ -149,28 +187,33 @@ class Property (object):
             results.extend(attr.getValues())
         return results
 
+
     def hasParameter(self, paramname):
-        return self._pycard.hasAttribute(paramname)
+        return self._pycard.hasParameter(paramname)
+
 
     def setParameter(self, paramname, paramvalue):
-        self._pycard.replaceAttribute(PyCalendarAttribute(paramname, paramvalue))
+        self._pycard.replaceParameter(Parameter(paramname, paramvalue))
+
 
     def removeParameter(self, paramname):
-        self._pycard.removeAttributes(paramname)
+        self._pycard.removeParameters(paramname)
+
 
     def removeAllParameters(self):
-        self._pycard.setAttributes({})
+        self._pycard.setParameters({})
+
 
     def removeParameterValue(self, paramname, paramvalue):
 
         paramname = paramname.upper()
         for attrName in self.parameterNames():
             if attrName.upper() == paramname:
-                for attr in tuple(self._pycard.getAttributes()[attrName]):
+                for attr in tuple(self._pycard.getParameters()[attrName]):
                     for value in attr.getValues():
                         if value == paramvalue:
                             if not attr.removeValue(value):
-                                self._pycard.removeAttributes(paramname)
+                                self._pycard.removeParameters(paramname)
 
 
 
@@ -178,8 +221,20 @@ class Component (object):
     """
     X{vCard} component.
     """
+    allowedTypesList = None
+
+
     @classmethod
-    def allFromString(clazz, string):
+    def allowedTypes(cls):
+        if cls.allowedTypesList is None:
+            cls.allowedTypesList = ["text/vcard"]
+            if config.EnableJSONData:
+                cls.allowedTypesList.append("application/vcard+json")
+        return cls.allowedTypesList
+
+
+    @classmethod
+    def allFromString(clazz, string, format=None):
         """
         FIXME: Just default to reading a single VCARD - actually need more
         """
@@ -193,62 +248,85 @@ class Component (object):
         if string[:3] == codecs.BOM_UTF8:
             string = string[3:]
 
-        return clazz.allFromStream(StringIO.StringIO(string))
+        return clazz.allFromStream(StringIO.StringIO(string), format)
+
 
     @classmethod
-    def allFromStream(clazz, stream):
+    def allFromStream(clazz, stream, format=None):
         """
         FIXME: Just default to reading a single VCARD - actually need more
         """
         try:
-            results = Card.parseMultiple(stream)
-        except PyCalendarError:
+            results = Card.parseMultipleData(stream, format)
+        except ErrorBase:
             results = None
         if not results:
             stream.seek(0)
             raise InvalidVCardDataError("%s" % (stream.read(),))
         return [clazz(None, pycard=result) for result in results]
 
+
     @classmethod
-    def fromString(clazz, string):
+    def fromString(clazz, string, format=None):
         """
         Construct a L{Component} from a string.
         @param string: a string containing vCard data.
         @return: a L{Component} representing the first component described by
             C{string}.
         """
-        if type(string) is unicode:
-            string = string.encode("utf-8")
-        else:
-            # Valid utf-8 please
-            string.decode("utf-8")
+        return clazz._fromData(string, False, format)
 
-        # No BOMs please
-        if string[:3] == codecs.BOM_UTF8:
-            string = string[3:]
-
-        return clazz.fromStream(StringIO.StringIO(string))
 
     @classmethod
-    def fromStream(clazz, stream):
+    def fromStream(clazz, stream, format=None):
         """
         Construct a L{Component} from a stream.
         @param stream: a C{read()}able stream containing vCard data.
         @return: a L{Component} representing the first component described by
             C{stream}.
         """
-        cal = Card()
-        try:
-            result = cal.parse(stream)
-        except PyCalendarError:
-            result = None
-        if not result:
-            stream.seek(0)
-            raise InvalidVCardDataError("%s" % (stream.read(),))
-        return clazz(None, pycard=cal)
+        return clazz._fromData(stream, True, format)
+
 
     @classmethod
-    def fromIStream(clazz, stream):
+    def _fromData(clazz, data, isstream, format=None):
+        """
+        Construct a L{Component} from a stream.
+        @param stream: a C{read()}able stream containing vCard data.
+        @param format: a C{str} indicating whether the data is vCard or jCard
+        @return: a L{Component} representing the first component described by
+            C{stream}.
+        """
+
+        if isstream:
+            pass
+        else:
+            if type(data) is unicode:
+                data = data.encode("utf-8")
+            else:
+                # Valid utf-8 please
+                data.decode("utf-8")
+
+            # No BOMs please
+            if data[:3] == codecs.BOM_UTF8:
+                data = data[3:]
+
+        errmsg = "Unknown"
+        try:
+            result = Card.parseData(data, format)
+        except ErrorBase, e:
+            errmsg = "%s: %s" % (e.mReason, e.mData,)
+            result = None
+        if not result:
+            if isstream:
+                data.seek(0)
+                data = data.read()
+            raise InvalidVCardDataError("%s\n%s" % (errmsg, data,))
+        return clazz(None, pycard=result)
+
+
+    @classmethod
+    def fromIStream(clazz, stream, format=None):
         """
         Construct a L{Component} from a stream.
         @param stream: an L{IStream} containing vCard data.
@@ -261,9 +339,11 @@ class Component (object):
         #   A better solution would parse directly and incrementally from the
         #   request stream.
         #
-        def parse(data): return clazz.fromString(data)
+        def parse(data):
+            return clazz.fromString(data, format)
         return allDataFromStream(IStream(stream), parse)
 
+
     def __init__(self, name, **kwargs):
         """
         Use this constructor to initialize an empty L{Component}.
@@ -277,8 +357,8 @@ class Component (object):
                 pyobj = kwargs["pycard"]
 
                 if pyobj is not None:
-                    if not isinstance(pyobj, PyCalendarComponentBase):
-                        raise TypeError("Not a PyCalendarComponentBase: %r" % (pyobj,))
+                    if not isinstance(pyobj, ComponentBase):
+                        raise TypeError("Not a ComponentBase: %r" % (pyobj,))
 
                 self._pycard = pyobj
             else:
@@ -302,28 +382,53 @@ class Component (object):
         else:
             raise ValueError("VCards have no child components")
 
-    def __str__ (self): return str(self._pycard)
-    def __repr__(self): return "<%s: %r>" % (self.__class__.__name__, str(self._pycard))
+
+    def __str__(self):
+        return str(self._pycard)
+
+
+    def __repr__(self):
+        return "<%s: %r>" % (self.__class__.__name__, str(self._pycard))
+
 
     def __hash__(self):
         return hash(str(self))
 
-    def __ne__(self, other): return not self.__eq__(other)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+
     def __eq__(self, other):
         if not isinstance(other, Component):
             return False
         return self._pycard == other._pycard
 
+
+    def getText(self, format=None):
+        """
+        Return text representation
+        """
+        assert self.name() == "VCARD", "Must be a VCARD: %r" % (self,)
+
+        result = self._pycard.getText(format)
+        if result is None:
+            raise ValueError("Unknown format requested for address data.")
+        return result
+
+
     # FIXME: Should this not be in __eq__?
     def same(self, other):
         return self._pycard == other._pycard
 
+
     def name(self):
         """
-        @return: the name of the iCalendar type of this component.
+        @return: the name of the vCard type of this component.
         """
         return self._pycard.getType()
 
+
     def duplicate(self):
         """
         Duplicate this object and all its contents.
@@ -331,6 +436,7 @@ class Component (object):
         """
         return Component(None, pycard=self._pycard.duplicate())
 
+
     def hasProperty(self, name):
         """
         @param name: the name of the property whose existence is being tested.
@@ -338,6 +444,7 @@ class Component (object):
         """
         return self._pycard.hasProperty(name)
 
+
     def getProperty(self, name):
         """
         Get one property from the property list.
@@ -346,10 +453,13 @@ class Component (object):
         @raise: L{ValueError} if there is more than one property of the given name.
         """
         properties = tuple(self.properties(name))
-        if len(properties) == 1: return properties[0]
-        if len(properties) > 1: raise InvalidVCardDataError("More than one %s property in component %r" % (name, self))
+        if len(properties) == 1:
+            return properties[0]
+        if len(properties) > 1:
+            raise InvalidVCardDataError("More than one %s property in component %r" % (name, self))
         return None
 
+
     def properties(self, name=None):
         """
         @param name: if given and not C{None}, restricts the returned properties
@@ -368,6 +478,7 @@ class Component (object):
             for p in properties
         )
 
+
     def propertyValue(self, name):
         properties = tuple(self.properties(name))
         if len(properties) == 1:
@@ -385,6 +496,7 @@ class Component (object):
         self._pycard.addProperty(property._pycard)
         self._pycard.finalise()
 
+
     def removeProperty(self, property):
         """
         Remove a property from this component.
@@ -393,6 +505,7 @@ class Component (object):
         self._pycard.removeProperty(property._pycard)
         self._pycard.finalise()
 
+
     def removeProperties(self, name):
         """
         remove all properties with name
@@ -400,6 +513,7 @@ class Component (object):
         """
         self._pycard.removeProperties(name)
 
+
     def replaceProperty(self, property):
         """
         Add or replace a property in this component.
@@ -410,6 +524,7 @@ class Component (object):
         self._pycard.removeProperties(property.name())
         self.addProperty(property)
 
+
     def resourceUID(self):
         """
         @return: the UID of the subcomponents in this component.
@@ -421,6 +536,7 @@ class Component (object):
 
         return self._resource_uid
 
+
     def resourceKind(self):
         """
         @return: the kind of the subcomponents in this component.
@@ -432,6 +548,7 @@ class Component (object):
 
         return self._resource_kind
 
+
     def resourceMemberAddresses(self):
         """
         @return: an iterable of X-ADDRESSBOOKSERVER-MEMBER property values
@@ -440,6 +557,7 @@ class Component (object):
 
         return [prop.value() for prop in list(self.properties("X-ADDRESSBOOKSERVER-MEMBER"))]
 
+
     def validVCardData(self, doFix=True, doRaise=True):
         """
         @return: tuple of fixed, unfixed issues
@@ -454,17 +572,19 @@ class Component (object):
         if unfixed:
             log.debug("vCard data had unfixable problems:\n  %s" % ("\n  ".join(unfixed),))
             if doRaise:
-                raise InvalidVCardDataError("Calendar data had unfixable problems:\n  %s" % ("\n  ".join(unfixed),))
+                raise InvalidVCardDataError("Address data had unfixable problems:\n  %s" % ("\n  ".join(unfixed),))
         if fixed:
             log.debug("vCard data had fixable problems:\n  %s" % ("\n  ".join(fixed),))
 
         return fixed, unfixed
 
+
     def validForCardDAV(self):
         """
         @raise ValueError: if the given vcard data is not valid.
         """
-        if self.name() != "VCARD": raise InvalidVCardDataError("Not a vcard")
+        if self.name() != "VCARD":
+            raise InvalidVCardDataError("Not a vcard")
 
         version = self.propertyValue("VERSION")
         if version != "3.0":
