@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav -*-
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,13 @@
 ##
 
 """
-WebDAV support for Twisted.
+Calendar & contacts data store.
 """
 
-# Make sure we have twext's required Twisted patches loaded before we do
-# anything at all.
-__import__("twext")
+#
+# FIXME: 'txdav' should be renamed to 'datastore' and shoul not be
+# WebDAV-specific.
+#
+# (That means txdav.xml should be pulled out, for example, as that is
+# WebDAV-specific.)
+#
