@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.base.datastore.test.test_subpostgres -*-
 # #
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,21 +23,22 @@ import os
 import pwd
 import re
 import signal
-
 from hashlib import md5
+from pipes import quote as shell_quote
+
 
 from twisted.python.procutils import which
 from twisted.internet.protocol import ProcessProtocol
 
+from twext.enterprise.dal.parseschema import splitSQLString
 from twext.python.log import Logger
 from twext.python.filepath import CachingFilePath
 
-import pgdb
-
 from twisted.protocols.basic import LineReceiver
-from twisted.internet.defer import Deferred
+from twisted.internet.defer import Deferred, succeed
 from txdav.base.datastore.dbapiclient import DBAPIConnector
-from txdav.base.datastore.dbapiclient import postgresPreflight
+from txdav.base.datastore.dbapiclient import postgres
+from txdav.common.icommondatastore import InternalDataStoreError
 
 from twisted.application.service import MultiService
 
@@ -48,10 +49,12 @@ log = Logger()
 _MAGIC_READY_COOKIE = "database system is ready to accept connections"
 
 
-class _PostgresMonitor(ProcessProtocol):
+
+class PostgresMonitor(ProcessProtocol):
     """
     A monitoring protocol which watches the postgres subprocess.
     """
+    log = Logger()
 
     def __init__(self, svc=None):
         self.lineReceiver = LineReceiver()
@@ -77,17 +80,24 @@ class _PostgresMonitor(ProcessProtocol):
 
 
     def outReceived(self, out):
-        log.warn("received postgres stdout {out!r}", out=out)
+        for line in out.split("\n"):
+            if line:
+                self.log.info("{message}", message=line)
         # self.lineReceiver.dataReceived(out)
 
 
     def errReceived(self, err):
-        log.warn("received postgres stderr {err}", err=err)
+        for line in err.split("\n"):
+            if line:
+                self.log.error("{message}", message=line)
         self.lineReceiver.dataReceived(err)
 
 
     def processEnded(self, reason):
-        log.warn("postgres process ended with status {status}", status=reason.value.status)
+        self.log.info(
+            "pg_ctl process ended with status={status}",
+            status=reason.value.status
+        )
         # If pg_ctl exited with zero, we were successful in starting postgres
         # If pg_ctl exited with nonzero, we need to give up.
         self.lineReceiver.connectionLost(reason)
@@ -95,7 +105,7 @@ class _PostgresMonitor(ProcessProtocol):
         if reason.value.status == 0:
             self.completionDeferred.callback(None)
         else:
-            log.warn("Could not start postgres; see postgres.log")
+            self.log.error("Could not start postgres; see postgres.log")
             self.completionDeferred.errback(reason)
 
 
@@ -158,25 +168,29 @@ class CapturingProcessProtocol(ProcessProtocol):
         """
         The process is over, fire the Deferred with the output.
         """
-        self.deferred.callback(''.join(self.output))
+        self.deferred.callback("".join(self.output))
 
 
 
 class PostgresService(MultiService):
 
-    def __init__(self, dataStoreDirectory, subServiceFactory,
-                 schema, resetSchema=False, databaseName='subpostgres',
-                 clusterName="cluster",
-                 logFile="postgres.log", socketDir="",
-                 listenAddresses=[], sharedBuffers=30,
-                 maxConnections=20, options=[],
-                 testMode=False,
-                 uid=None, gid=None,
-                 spawnedDBUser="caldav",
-                 importFileName=None,
-                 pgCtl="pg_ctl",
-                 initDB="initdb",
-                 reactor=None):
+    def __init__(
+        self, dataStoreDirectory, subServiceFactory,
+        schema, resetSchema=False, databaseName="subpostgres",
+        clusterName="cluster",
+        logFile="postgres.log",
+        logDirectory="",
+        socketDir="",
+        socketName="",
+        listenAddresses=[], sharedBuffers=30,
+        maxConnections=20, options=[],
+        testMode=False,
+        uid=None, gid=None,
+        spawnedDBUser="caldav",
+        pgCtl="pg_ctl",
+        initDB="initdb",
+        reactor=None,
+    ):
         """
         Initialize a L{PostgresService} pointed at a data store directory.
 
@@ -189,8 +203,6 @@ class PostgresService(MultiService):
 
         @param spawnedDBUser: the postgres role
         @type spawnedDBUser: C{str}
-        @param importFileName: path to SQL file containing previous data to import
-        @type importFileName: C{str}
         """
 
         # FIXME: By default there is very little (4MB) shared memory available,
@@ -218,34 +230,66 @@ class PostgresService(MultiService):
         # Make logFile absolute in case the working directory of postgres is
         # elsewhere:
         self.logFile = os.path.abspath(logFile)
-        if listenAddresses:
-            self.socketDir = None
-            self.host, self.port = listenAddresses[0].split(":") if ":" in listenAddresses[0] else (listenAddresses[0], None,)
-            self.listenAddresses = [addr.split(":")[0] for addr in listenAddresses]
+        if logDirectory:
+            self.logDirectory = os.path.abspath(logDirectory)
         else:
-            if not socketDir:
-                # Socket directory was not specified, so come up with one
-                # in /tmp and based on a hash of the data store directory
-                digest = md5(dataStoreDirectory.path).hexdigest()
-                socketDir = "/tmp/ccs_postgres_" + digest
+            self.logDirectory = ""
 
-            self.socketDir = CachingFilePath(socketDir)
+        # Always use our own configured socket dir in case the built-in
+        # postgres tries to use a directory we don't have permissions for
+        if not socketDir:
+            # Socket directory was not specified, so come up with one
+            # in /tmp and based on a hash of the data store directory
+            digest = md5(dataStoreDirectory.path).hexdigest()
+            socketDir = "/tmp/ccs_postgres_" + digest
+        self.socketDir = CachingFilePath(socketDir)
+        self.socketName = socketName
+
+        if listenAddresses:
+            if ":" in listenAddresses[0]:
+                self.host, self.port = listenAddresses[0].split(":")
+            else:
+                self.host, self.port = (listenAddresses[0], None)
+
+            self.listenAddresses = [
+                addr.split(":")[0] for addr in listenAddresses
+            ]
+        else:
             self.host = self.socketDir.path
             self.port = None
             self.listenAddresses = []
+
+        self.testMode = testMode
         self.sharedBuffers = sharedBuffers if not testMode else 16
-        self.maxConnections = maxConnections if not testMode else 4
+        self.maxConnections = maxConnections if not testMode else 8
         self.options = options
 
         self.uid = uid
         self.gid = gid
         self.spawnedDBUser = spawnedDBUser
-        self.importFileName = importFileName
         self.schema = schema
         self.monitor = None
         self.openConnections = []
-        self._pgCtl = pgCtl
-        self._initdb = initDB
+
+        def locateCommand(name, cmd):
+            for found in which(cmd):
+                return found
+
+            raise InternalDataStoreError(
+                "Unable to locate {} command: {}".format(name, cmd)
+            )
+
+        self._pgCtl = locateCommand("pg_ctl", pgCtl)
+
+        # Make note of the inode for the pg_ctl script; if it changes or is
+        # missing when it comes time to stop postgres, instead send SIGTERM
+        # to stop our postgres (since we can't do a graceful shutdown)
+        try:
+            self._pgCtlInode = os.stat(self._pgCtl).st_ino
+        except:
+            self._pgCtlInode = 0
+
+        self._initdb = locateCommand("initdb", initDB)
         self._reactor = reactor
         self._postgresPid = None
 
@@ -258,21 +302,10 @@ class PostgresService(MultiService):
         return self._reactor
 
 
-    def pgCtl(self):
-        """
-        Locate the path to pg_ctl.
-        """
-        return which(self._pgCtl)[0]
-
-
-    def initdb(self):
-        return which(self._initdb)[0]
-
-
     def activateDelayedShutdown(self):
         """
-        Call this when starting database initialization code to protect against
-        shutdown.
+        Call this when starting database initialization code to
+        protect against shutdown.
 
         Sets the delayedShutdown flag to True so that if reactor shutdown
         commences, the shutdown will be delayed until deactivateDelayedShutdown
@@ -295,26 +328,50 @@ class PostgresService(MultiService):
         if databaseName is None:
             databaseName = self.databaseName
 
-        if self.spawnedDBUser:
-            dsn = "%s:dbname=%s:%s" % (self.host, databaseName, self.spawnedDBUser)
-        elif self.uid is not None:
-            dsn = "%s:dbname=%s:%s" % (self.host, databaseName,
-                pwd.getpwuid(self.uid).pw_name)
+        kwargs = {
+            "database": databaseName,
+        }
+
+        if self.host.startswith("/"):
+            kwargs["endpoint"] = "unix:{}".format(self.host)
         else:
-            dsn = "%s:dbname=%s" % (self.host, databaseName)
+            kwargs["endpoint"] = "tcp:{}".format(self.host)
+            if self.port:
+                kwargs["endpoint"] = "{}:{}".format(kwargs["endpoint"], self.port)
+        if self.spawnedDBUser:
+            kwargs["user"] = self.spawnedDBUser
+        elif self.uid is not None:
+            kwargs["user"] = pwd.getpwuid(self.uid).pw_name
 
-        kwargs = {}
-        if self.port:
-            kwargs["host"] = "%s:%s" % (self.host, self.port,)
-
-        return DBAPIConnector(pgdb, postgresPreflight, dsn, **kwargs)
+        return DBAPIConnector.connectorFor("postgres", **kwargs)
 
 
     def produceConnection(self, label="<unlabeled>", databaseName=None):
         """
         Produce a DB-API 2.0 connection pointed at this database.
         """
-        return self._connectorFor(databaseName).connect(label)
+        connection = self._connectorFor(databaseName).connect(label)
+
+        if postgres.__name__ == "pg8000":
+            # Patch pg8000 behavior to match what we need wrt text processing
+
+            def my_text_out(v):
+                return v.encode("utf-8") if isinstance(v, unicode) else str(v)
+            connection.realConnection.py_types[str] = (705, postgres.core.FC_TEXT, my_text_out)
+            connection.realConnection.py_types[postgres.six.text_type] = (705, postgres.core.FC_TEXT, my_text_out)
+
+            def my_text_recv(data, offset, length):
+                return str(data[offset: offset + length])
+            connection.realConnection.default_factory = lambda: (postgres.core.FC_TEXT, my_text_recv)
+            connection.realConnection.pg_types[19] = (postgres.core.FC_BINARY, my_text_recv)
+            connection.realConnection.pg_types[25] = (postgres.core.FC_BINARY, my_text_recv)
+            connection.realConnection.pg_types[705] = (postgres.core.FC_BINARY, my_text_recv)
+            connection.realConnection.pg_types[829] = (postgres.core.FC_TEXT, my_text_recv)
+            connection.realConnection.pg_types[1042] = (postgres.core.FC_BINARY, my_text_recv)
+            connection.realConnection.pg_types[1043] = (postgres.core.FC_BINARY, my_text_recv)
+            connection.realConnection.pg_types[2275] = (postgres.core.FC_BINARY, my_text_recv)
+
+        return connection
 
 
     def ready(self, createDatabaseConn, createDatabaseCursor):
@@ -323,45 +380,43 @@ class PostgresService(MultiService):
         If the database has not been created and there is a dump file,
         then the dump file is imported.
         """
-
         if self.resetSchema:
             try:
                 createDatabaseCursor.execute(
-                    "drop database %s" % (self.databaseName)
+                    "drop database {}".format(self.databaseName)
                 )
-            except pgdb.DatabaseError:
+            except postgres.DatabaseError:
                 pass
 
         try:
             createDatabaseCursor.execute(
-                "create database %s with encoding 'UTF8'" % (self.databaseName)
+                "create database {} with encoding 'UTF8'"
+                .format(self.databaseName)
             )
         except:
             # database already exists
-            executeSQL = False
+            sqlToExecute = None
         else:
-            # database does not yet exist; if dump file exists, execute it, otherwise
-            # execute schema
-            executeSQL = True
+            # database does not yet exist; if dump file exists, execute it,
+            # otherwise execute schema
             sqlToExecute = self.schema
-            if self.importFileName:
-                importFilePath = CachingFilePath(self.importFileName)
-                if importFilePath.exists():
-                    sqlToExecute = importFilePath.getContent()
 
         createDatabaseCursor.close()
         createDatabaseConn.close()
 
-        if executeSQL:
+        if sqlToExecute is not None:
             connection = self.produceConnection()
             cursor = connection.cursor()
-            cursor.execute(sqlToExecute)
+            for statement in splitSQLString(sqlToExecute):
+                cursor.execute(statement)
             connection.commit()
             connection.close()
 
         if self.shutdownDeferred is None:
             # Only continue startup if we've not begun shutdown
-            self.subServiceFactory(self.produceConnection, self).setServiceParent(self)
+            self.subServiceFactory(
+                self.produceConnection, self
+            ).setServiceParent(self)
 
 
     def pauseMonitor(self):
@@ -391,47 +446,84 @@ class PostgresService(MultiService):
         """
         Start the database and initialize the subservice.
         """
-
         def createConnection():
-            createDatabaseConn = self.produceConnection(
-                'schema creation', 'postgres'
-            )
+            try:
+                createDatabaseConn = self.produceConnection(
+                    "schema creation", "postgres"
+                )
+            except postgres.DatabaseError as e:
+                log.error(
+                    "Unable to connect to database for schema creation:"
+                    " {error}",
+                    error=e
+                )
+                raise
+
             createDatabaseCursor = createDatabaseConn.cursor()
-            createDatabaseCursor.execute("commit")
+
+            if postgres.__name__ == "pg8000":
+                createDatabaseConn.realConnection.autocommit = True
+            elif postgres.__name__ == "pgdb":
+                createDatabaseCursor.execute("commit")
+            else:
+                raise InternalDataStoreError(
+                    "Unknown Postgres DBM module: {}".format(postgres)
+                )
+
             return createDatabaseConn, createDatabaseCursor
 
-        monitor = _PostgresMonitor(self)
-        pgCtl = self.pgCtl()
+        monitor = PostgresMonitor(self)
         # check consistency of initdb and postgres?
 
         options = []
         options.append(
-            "-c listen_addresses='%s'" % (",".join(self.listenAddresses))
+            "-c listen_addresses={}"
+            .format(shell_quote(",".join(self.listenAddresses)))
         )
         if self.socketDir:
-            options.append("-k '%s'" % (self.socketDir.path,))
+            options.append(
+                "-c unix_socket_directories={}"
+                .format(shell_quote(self.socketDir.path))
+            )
         if self.port:
-            options.append("-c port=%s" % (self.port,))
-        options.append("-c shared_buffers=%d" % (self.sharedBuffers,))
-        options.append("-c max_connections=%d" % (self.maxConnections,))
+            options.append(
+                "-c port={}".format(shell_quote(self.port))
+            )
+        options.append(
+            "-c shared_buffers={:d}"
+            .format(self.sharedBuffers)  # int: don't quote
+        )
+        options.append(
+            "-c max_connections={:d}"
+            .format(self.maxConnections)  # int: don't quote
+        )
         options.append("-c standard_conforming_strings=on")
         options.append("-c unix_socket_permissions=0770")
         options.extend(self.options)
+        if self.logDirectory:  # tell postgres to rotate logs
+            options.append(
+                "-c log_directory={}".format(shell_quote(self.logDirectory))
+            )
+            options.append("-c log_truncate_on_rotation=on")
+            options.append("-c log_filename=postgresql_%w.log")
+            options.append("-c log_rotation_age=1440")
+            options.append("-c logging_collector=on")
 
-        log.warn("Requesting postgres start via {cmd}", cmd=pgCtl)
+        options.append("-c log_line_prefix=%t")
+        if self.testMode:
+            options.append("-c log_statement=all")
+
+        args = [
+            self._pgCtl, "start",
+            "--log={}".format(self.logFile),
+            "--timeout=86400",  # Plenty of time for a long cluster upgrade
+            "-w",  # Wait for startup to complete
+            "-o", " ".join(options),  # Options passed to postgres
+        ]
+
+        log.info("Requesting postgres start via: {args}", args=args)
         self.reactor.spawnProcess(
-            monitor, pgCtl,
-            [
-                pgCtl,
-                "start",
-                "-l", self.logFile,
-                "-t 86400",  # Give plenty of time for a long cluster upgrade
-                "-w",
-                # XXX what are the quoting rules for '-o'?  do I need to repr()
-                # the path here?
-                "-o",
-                " ".join(options),
-            ],
+            monitor, self._pgCtl, args,
             env=self.env, path=self.workingDir.path,
             uid=self.uid, gid=self.gid,
         )
@@ -445,7 +537,7 @@ class PostgresService(MultiService):
             removed/renamed/unmounted.
             """
             reResult = re.search("PID: (\d+)\D", result)
-            if reResult != None:
+            if reResult is not None:
                 self._postgresPid = int(reResult.group(1))
             self.ready(*createConnection())
             self.deactivateDelayedShutdown()
@@ -455,12 +547,12 @@ class PostgresService(MultiService):
             We started postgres; we're responsible for stopping it later.
             Call pgCtl status to get the pid.
             """
-            log.warn("{cmd} exited", cmd=pgCtl)
+            log.info("{cmd} exited", cmd=self._pgCtl)
             self.shouldStopDatabase = True
             d = Deferred()
             statusMonitor = CapturingProcessProtocol(d, None)
             self.reactor.spawnProcess(
-                statusMonitor, pgCtl, [pgCtl, "status"],
+                statusMonitor, self._pgCtl, [self._pgCtl, "status"],
                 env=self.env, path=self.workingDir.path,
                 uid=self.uid, gid=self.gid,
             )
@@ -475,7 +567,7 @@ class PostgresService(MultiService):
             d = Deferred()
             statusMonitor = CapturingProcessProtocol(d, None)
             self.reactor.spawnProcess(
-                statusMonitor, pgCtl, [pgCtl, "status"],
+                statusMonitor, self._pgCtl, [self._pgCtl, "status"],
                 env=self.env, path=self.workingDir.path,
                 uid=self.uid, gid=self.gid,
             )
@@ -486,7 +578,10 @@ class PostgresService(MultiService):
             We can't start postgres or connect to a running instance.  Shut
             down.
             """
-            log.failure("Can't start or connect to postgres", f)
+            log.critical(
+                "Can't start or connect to postgres: {failure.value}",
+                failure=f
+            )
             self.deactivateDelayedShutdown()
             self.reactor.stop()
 
@@ -503,35 +598,56 @@ class PostgresService(MultiService):
         env.update(PGDATA=clusterDir.path,
                    PGHOST=self.host,
                    PGUSER=self.spawnedDBUser)
-        initdb = self.initdb()
+
         if self.socketDir:
             if not self.socketDir.isdir():
+                log.info("Creating {dir}", dir=self.socketDir.path.decode("utf-8"))
                 self.socketDir.createDirectory()
+
             if self.uid and self.gid:
                 os.chown(self.socketDir.path, self.uid, self.gid)
-        if self.dataStoreDirectory.isdir():
-            self.startDatabase()
-        else:
+
+            os.chmod(self.socketDir.path, 0770)
+
+        if not self.dataStoreDirectory.isdir():
+            log.info("Creating {dir}", dir=self.dataStoreDirectory.path.decode("utf-8"))
             self.dataStoreDirectory.createDirectory()
-            if not self.workingDir.isdir():
-                self.workingDir.createDirectory()
-            if self.uid and self.gid:
-                os.chown(self.dataStoreDirectory.path, self.uid, self.gid)
-                os.chown(self.workingDir.path, self.uid, self.gid)
+
+        if not self.workingDir.isdir():
+            log.info("Creating {dir}", dir=self.workingDir.path.decode("utf-8"))
+            self.workingDir.createDirectory()
+
+        if self.uid and self.gid:
+            os.chown(self.dataStoreDirectory.path, self.uid, self.gid)
+            os.chown(self.workingDir.path, self.uid, self.gid)
+
+        if not clusterDir.isdir():
+            # No cluster directory, run initdb
+            log.info("Running initdb for {dir}", dir=clusterDir.path.decode("utf-8"))
             dbInited = Deferred()
             self.reactor.spawnProcess(
                 CapturingProcessProtocol(dbInited, None),
-                initdb, [initdb, "-E", "UTF8", "-U", self.spawnedDBUser],
+                self._initdb,
+                [self._initdb, "-E", "UTF8", "-U", self.spawnedDBUser],
                 env=env, path=self.workingDir.path,
                 uid=self.uid, gid=self.gid,
             )
+
             def doCreate(result):
                 if result.find("FATAL:") != -1:
                     log.error(result)
-                    raise RuntimeError("Unable to initialize postgres database: %s" % (result,))
+                    raise InternalDataStoreError(
+                        "Unable to initialize postgres database: {}"
+                        .format(result)
+                    )
                 self.startDatabase()
+
             dbInited.addCallback(doCreate)
 
+        else:
+            log.info("Cluster already exists at {dir}", dir=clusterDir.path.decode("utf-8"))
+            self.startDatabase()
+
 
     def stopService(self):
         """
@@ -550,24 +666,41 @@ class PostgresService(MultiService):
             # If pg_ctl's startup wasn't successful, don't bother to stop the
             # database.  (This also happens in command-line tools.)
             if self.shouldStopDatabase:
-                monitor = _PostgresMonitor()
-                pgCtl = self.pgCtl()
-                # FIXME: why is this 'logfile' and not self.logfile?
-                self.reactor.spawnProcess(monitor, pgCtl,
-                    [pgCtl, '-l', 'logfile', 'stop'],
-                    env=self.env, path=self.workingDir.path,
-                    uid=self.uid, gid=self.gid,
-                )
-                return monitor.completionDeferred
-        return d.addCallback(superStopped)
 
-#        def maybeStopSubprocess(result):
-#            if self.monitor is not None:
-#                self.monitor.transport.signalProcess("INT")
-#                return self.monitor.completionDeferred
-#            return result
-#        d.addCallback(maybeStopSubprocess)
-#        return d
+                # Compare pg_ctl inode with one we saw at the start; if different
+                # (or missing), fall back to SIGTERM
+                try:
+                    newInode = os.stat(self._pgCtl).st_ino
+                except OSError:
+                    # Missing
+                    newInode = -1
+
+                if self._pgCtlInode != newInode:
+                    # send SIGTERM to postgres
+                    log.info("Postgres control script mismatch")
+                    if self._postgresPid:
+                        log.info("Sending SIGTERM to Postgres")
+                        try:
+                            os.kill(self._postgresPid, signal.SIGTERM)
+                        except OSError:
+                            pass
+                    return succeed(None)
+                else:
+                    # use pg_ctl stop
+                    monitor = PostgresMonitor()
+                    args = [
+                        self._pgCtl, "stop",
+                        "--log={}".format(self.logFile),
+                    ]
+                    log.info("Requesting postgres stop via: {args}", args=args)
+                    self.reactor.spawnProcess(
+                        monitor, self._pgCtl,
+                        args,
+                        env=self.env, path=self.workingDir.path,
+                        uid=self.uid, gid=self.gid,
+                    )
+                    return monitor.completionDeferred
+        return d.addCallback(superStopped)
 
 
     def hardStop(self):
