@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -29,6 +29,8 @@ from txdav.base.datastore.subpostgres import PostgresService
 from twisted.internet.defer import inlineCallbacks, Deferred
 from twisted.application.service import Service
 
+
+
 class SubprocessStartup(TestCase):
     """
     Tests for starting and stopping the subprocess.
@@ -44,6 +46,7 @@ class SubprocessStartup(TestCase):
         """
 
         test = self
+
         class SimpleService1(Service):
 
             instances = []
@@ -82,7 +85,7 @@ class SubprocessStartup(TestCase):
         cursor = connection.cursor()
         cursor.execute("select * from test_dummy_table")
         values = cursor.fetchall()
-        self.assertEquals(values, [["dummy"]])
+        self.assertEquals(map(list, values), [["dummy"]])
 
 
     @inlineCallbacks
@@ -95,6 +98,7 @@ class SubprocessStartup(TestCase):
         """
 
         test = self
+
         class SimpleService2(Service):
 
             instances = []
@@ -134,58 +138,4 @@ class SubprocessStartup(TestCase):
         cursor = connection.cursor()
         cursor.execute("select * from test_dummy_table")
         values = cursor.fetchall()
-        self.assertEquals(values, [["dummy"]])
-
-
-    @inlineCallbacks
-    def test_startService_withDumpFile(self):
-        """
-        Assuming a properly configured environment ($PATH points at an 'initdb'
-        and 'postgres', $PYTHONPATH includes pgdb), starting a
-        L{PostgresService} will start the service passed to it, after importing
-        an existing dump file.
-        """
-
-        test = self
-        class SimpleService1(Service):
-
-            instances = []
-            ready = Deferred()
-
-            def __init__(self, connectionFactory, storageService):
-                self.connection = connectionFactory()
-                test.addCleanup(self.connection.close)
-                self.instances.append(self)
-
-
-            def startService(self):
-                cursor = self.connection.cursor()
-                try:
-                    cursor.execute(
-                        "insert into import_test_table values ('value2')"
-                    )
-                except:
-                    self.ready.errback()
-                else:
-                    self.ready.callback(None)
-                finally:
-                    cursor.close()
-
-        # The SQL in importFile.sql will get executed, including the insertion of "value1"
-        importFileName = CachingFilePath(__file__).parent().child("importFile.sql").path
-        svc = PostgresService(
-            CachingFilePath("postgres_3.pgdb"),
-            SimpleService1,
-            "",
-            databaseName="dummy_db",
-            testMode=True,
-            importFileName=importFileName
-        )
-        svc.startService()
-        self.addCleanup(svc.stopService)
-        yield SimpleService1.ready
-        connection = SimpleService1.instances[0].connection
-        cursor = connection.cursor()
-        cursor.execute("select * from import_test_table")
-        values = cursor.fetchall()
-        self.assertEquals(values, [["value1"], ["value2"]])
+        self.assertEquals(map(list, values), [["dummy"]])
