@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -43,6 +43,12 @@ class PropertyStoreTest(base.PropertyStoreTest):
         self.propertyStore1 = self.propertyStore
         self.propertyStore2 = PropertyStore("user01", lambda : tempFile)
         self.propertyStore2._setPerUserUID("user02")
+        self.propertyStore2._setProxyUID("user02")
+        self.propertyStore3 = PropertyStore("user01", lambda : tempFile)
+        self.propertyStore3._setProxyUID("user03")
+        self.propertyStore4 = PropertyStore("user01", lambda : tempFile)
+        self.propertyStore4._setPerUserUID("user02")
+        self.propertyStore4._setProxyUID("user04")
 
 
     def test_init(self):
