@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.caldav.datastore.test.test_file -*-
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,18 +35,18 @@ from errno import ENOENT
 
 from twisted.internet.defer import inlineCallbacks, returnValue, succeed, fail
 
-from twext.python.vcomponent import VComponent
+from twistedcaldav.ical import Component as VComponent
 from txdav.xml import element as davxml
 from txdav.xml.rfc2518 import GETContentType
-from twext.web2.dav.resource import TwistedGETContentMD5
-from twext.web2.http_headers import generateContentType, MimeType
+from txweb2.dav.resource import TwistedGETContentMD5
+from txweb2.http_headers import generateContentType, MimeType
 
-from twistedcaldav import caldavxml, customxml
-from twistedcaldav.caldavxml import ScheduleCalendarTransp, Opaque
+from twistedcaldav import caldavxml, customxml, ical
+from twistedcaldav.caldavxml import ScheduleCalendarTransp, Opaque, Transparent
 from twistedcaldav.config import config
 from twistedcaldav.ical import InvalidICalendarDataError
 
-from txdav.caldav.icalendarstore import IAttachment
+from txdav.caldav.icalendarstore import IAttachment, AttachmentSizeTooLarge
 from txdav.caldav.icalendarstore import ICalendar, ICalendarObject
 from txdav.caldav.icalendarstore import ICalendarHome
 
@@ -78,12 +78,22 @@ CalendarStoreTransaction = CommonStoreTransaction
 
 IGNORE_NAMES = ('dropbox', 'notification', 'freebusy')
 
+
+
 class CalendarHome(CommonHome):
     implements(ICalendarHome)
 
     _topPath = "calendars"
     _notifierPrefix = "CalDAV"
 
+    _componentCalendarName = {
+        "VEVENT": "calendar",
+        "VTODO": "tasks",
+        "VJOURNAL": "journals",
+        "VAVAILABILITY": "available",
+        "VPOLL": "polls",
+    }
+
     def __init__(self, uid, path, calendarStore, transaction):
         super(CalendarHome, self).__init__(uid, path, calendarStore, transaction)
 
@@ -163,8 +173,10 @@ class CalendarHome(CommonHome):
         for calendar in self.calendars():
             for calendarObject in calendar.calendarObjects():
                 component = calendarObject.component()
-                if (component.hasPropertyInAnyComponent("X-APPLE-DROPBOX") or
-                    component.hasPropertyInAnyComponent("ATTACH")):
+                if (
+                    component.hasPropertyInAnyComponent("X-APPLE-DROPBOX") or
+                    component.hasPropertyInAnyComponent("ATTACH")
+                ):
                     dropboxID = (yield calendarObject.dropboxID())
                     dropboxIDs.append(dropboxID)
         returnValue(dropboxIDs)
@@ -177,19 +189,18 @@ class CalendarHome(CommonHome):
 
     def createdHome(self):
 
-        # Default calendar
-        defaultCal = self.createCalendarWithName("calendar")
-        props = defaultCal.properties()
-        props[PropertyName(*ScheduleCalendarTransp.qname())] = ScheduleCalendarTransp(Opaque())
-
         # Check whether components type must be separate
         if config.RestrictCalendarsToOneComponentType:
-            defaultCal.setSupportedComponents("VEVENT")
-
-            # Default tasks
-            defaultTasks = self.createCalendarWithName("tasks")
-            props = defaultTasks.properties()
-            defaultTasks.setSupportedComponents("VTODO")
+            for name in ical.allowedStoreComponents:
+                cal = self.createCalendarWithName(self._componentCalendarName[name])
+                cal.setSupportedComponents(name)
+                props = cal.properties()
+                if name not in ("VEVENT", "VAVAILABILITY",):
+                    props[PropertyName(*ScheduleCalendarTransp.qname())] = ScheduleCalendarTransp(Transparent())
+                else:
+                    props[PropertyName(*ScheduleCalendarTransp.qname())] = ScheduleCalendarTransp(Opaque())
+        else:
+            cal = self.createCalendarWithName("calendar")
 
         self.createCalendarWithName("inbox")
 
@@ -310,6 +321,7 @@ class Calendar(CommonHomeChild):
                 PropertyName.fromElement(customxml.GETCTag),
                 PropertyName.fromElement(caldavxml.SupportedCalendarComponentSet),
             ),
+            (),
         )
 
 
@@ -739,6 +751,7 @@ class CalendarObject(CommonObjectResource, CalendarObjectBase):
                 PropertyName.fromElement(customxml.TwistedCalendarHasPrivateCommentsProperty),
                 PropertyName.fromElement(customxml.ScheduleChanges),
             ),
+            (),
         )
 
 
@@ -794,6 +807,13 @@ class AttachmentStorageTransport(StorageTransportBase):
         newSize = self._file.tell()
         # FIXME: do anything
         self._file.close()
+
+        # Check max size for attachment
+        if newSize > config.MaximumAttachmentSize:
+            self._path.remove()
+            return fail(AttachmentSizeTooLarge())
+
+        # Check overall user quota
         allowed = home.quotaAllowedBytes()
         if allowed is not None and allowed < (home.quotaUsedBytes()
                                               + (newSize - oldSize)):
