@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.test.test_index -*-
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
 # limitations under the License.
 ##
 
+
 """
 CalDAV Index.
 
@@ -43,21 +44,23 @@ from twisted.internet.defer import maybeDeferred, succeed
 
 from twext.python.log import Logger
 
+from txdav.caldav.datastore.query.builder import buildExpression
+from txdav.caldav.datastore.query.filter import Filter
+from txdav.common.datastore.query.filegenerator import sqllitegenerator
 from txdav.common.icommondatastore import SyncTokenValidException, \
     ReservationError, IndexedSearchException
 
-from twistedcaldav.dateops import pyCalendarTodatetime
+from twistedcaldav.dateops import pyCalendarToSQLTimestamp
 from twistedcaldav.ical import Component
-from twistedcaldav.query import calendarquery, calendarqueryfilter
 from twistedcaldav.sql import AbstractSQLDatabase
 from twistedcaldav.sql import db_prefix
 from twistedcaldav.instance import InvalidOverriddenInstanceError
 from twistedcaldav.config import config
 from twistedcaldav.memcachepool import CachePoolUserMixIn
 
-from pycalendar.datetime import PyCalendarDateTime
-from pycalendar.duration import PyCalendarDuration
-from pycalendar.timezone import PyCalendarTimezone
+from pycalendar.datetime import DateTime
+from pycalendar.duration import Duration
+from pycalendar.timezone import Timezone
 
 log = Logger()
 
@@ -275,6 +278,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
 
         changed = []
         deleted = []
+        invalid = []
         for name, wasdeleted in results:
             if name:
                 if wasdeleted == 'Y':
@@ -285,7 +289,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
             else:
                 raise SyncTokenValidException
 
-        return changed, deleted,
+        return (changed, deleted, invalid)
 
 
     def lastRevision(self):
@@ -320,7 +324,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
 
         # Make sure we have a proper Filter element and get the partial SQL
         # statement to use.
-        if isinstance(filter, calendarqueryfilter.Filter):
+        if isinstance(filter, Filter):
             if fbtype:
                 # Lookup the useruid - try the empty (default) one if needed
                 dbuseruid = self._db_value_for_sql(
@@ -330,7 +334,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
             else:
                 dbuseruid = ""
 
-            qualifiers = calendarquery.sqlcalendarquery(filter, None, dbuseruid, fbtype)
+            qualifiers = sqlcalendarquery(filter, None, dbuseruid, fbtype)
             if qualifiers is not None:
                 # Determine how far we need to extend the current expansion of
                 # events. If we have an open-ended time-range we will expand one
@@ -341,7 +345,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
                     maxDate = maxDate.duplicate()
                     maxDate.setDateOnly(True)
                     if isStartDate:
-                        maxDate += PyCalendarDuration(days=365)
+                        maxDate += Duration(days=365)
                     self.testAndUpdateIndex(maxDate)
             else:
                 # We cannot handle this filter in an indexed search
@@ -437,6 +441,24 @@ class AbstractCalendarIndex(AbstractSQLDatabase):
 
 
 
+def sqlcalendarquery(filter, calendarid=None, userid=None, freebusy=False):
+    """
+    Convert the supplied calendar-query into a partial SQL statement.
+
+    @param filter: the L{Filter} for the calendar-query to convert.
+    @return: a C{tuple} of (C{str}, C{list}), where the C{str} is the partial SQL statement,
+            and the C{list} is the list of argument substitutions to use with the SQL API execute method.
+            Or return C{None} if it is not possible to create an SQL query to fully match the calendar-query.
+    """
+    try:
+        expression = buildExpression(filter, sqllitegenerator.FIELDS)
+        sql = sqllitegenerator(expression, calendarid, userid, freebusy)
+        return sql.generate()
+    except ValueError:
+        return None
+
+
+
 class CalendarIndex (AbstractCalendarIndex):
     """
     Calendar index - abstract class for indexer that indexes calendar objects in a collection.
@@ -636,7 +658,7 @@ class CalendarIndex (AbstractCalendarIndex):
         Gives all resources which have not been expanded beyond a given date
         in the index
         """
-        return self._db_values_for_sql("select NAME from RESOURCE where RECURRANCE_MAX < :1", pyCalendarTodatetime(minDate))
+        return self._db_values_for_sql("select NAME from RESOURCE where RECURRANCE_MAX < :1", pyCalendarToSQLTimestamp(minDate))
 
 
     def reExpandResource(self, name, expand_until):
@@ -671,7 +693,7 @@ class CalendarIndex (AbstractCalendarIndex):
         if master is None or not calendar.isRecurring():
             # When there is no master we have a set of overridden components - index them all.
             # When there is one instance - index it.
-            expand = PyCalendarDateTime(2100, 1, 1, 0, 0, 0, tzid=PyCalendarTimezone(utc=True))
+            expand = DateTime(2100, 1, 1, 0, 0, 0, tzid=Timezone.UTCTimezone)
             doInstanceIndexing = True
         else:
             # If migrating or re-creating or config option for delayed indexing is off, always index
@@ -682,8 +704,8 @@ class CalendarIndex (AbstractCalendarIndex):
             # by default.  This is a caching parameter which affects the size of the index;
             # it does not affect search results beyond this period, but it may affect
             # performance of such a search.
-            expand = (PyCalendarDateTime.getToday() +
-                      PyCalendarDuration(days=config.FreeBusyIndexExpandAheadDays))
+            expand = (DateTime.getToday() +
+                      Duration(days=config.FreeBusyIndexExpandAheadDays))
 
             if expand_until and expand_until > expand:
                 expand = expand_until
@@ -700,8 +722,8 @@ class CalendarIndex (AbstractCalendarIndex):
             # occurrences into some obscenely far-in-the-future date, so we cap the caching
             # period.  Searches beyond this period will always be relatively expensive for
             # resources with occurrences beyond this period.
-            if expand > (PyCalendarDateTime.getToday() +
-                         PyCalendarDuration(days=config.FreeBusyIndexExpandMaxDays)):
+            if expand > (DateTime.getToday() +
+                         Duration(days=config.FreeBusyIndexExpandMaxDays)):
                 raise IndexedSearchException()
 
         # Always do recurrence expansion even if we do not intend to index - we need this to double-check the
@@ -716,7 +738,7 @@ class CalendarIndex (AbstractCalendarIndex):
         # Now coerce indexing to off if needed
         if not doInstanceIndexing:
             instances = None
-            recurrenceLimit = PyCalendarDateTime(1900, 1, 1, 0, 0, 0, tzid=PyCalendarTimezone(utc=True))
+            recurrenceLimit = DateTime(1900, 1, 1, 0, 0, 0, tzid=Timezone.UTCTimezone)
 
         self._delete_from_db(name, uid, False)
 
@@ -725,7 +747,7 @@ class CalendarIndex (AbstractCalendarIndex):
             """
             insert into RESOURCE (NAME, UID, TYPE, RECURRANCE_MAX, ORGANIZER)
             values (:1, :2, :3, :4, :5)
-            """, name, uid, calendar.resourceType(), pyCalendarTodatetime(recurrenceLimit) if recurrenceLimit else None, organizer
+            """, name, uid, calendar.resourceType(), pyCalendarToSQLTimestamp(recurrenceLimit) if recurrenceLimit else None, organizer
         )
         resourceid = self.lastrowid
 
@@ -763,14 +785,14 @@ class CalendarIndex (AbstractCalendarIndex):
                     """,
                     resourceid,
                     float,
-                    pyCalendarTodatetime(start),
-                    pyCalendarTodatetime(end),
+                    pyCalendarToSQLTimestamp(start),
+                    pyCalendarToSQLTimestamp(end),
                     icalfbtype_to_indexfbtype.get(instance.component.getFBType(), 'F'),
                     transp
                 )
                 instanceid = self.lastrowid
-                peruserdata = calendar.perUserTransparency(instance.rid)
-                for useruid, transp in peruserdata:
+                peruserdata = calendar.perUserData(instance.rid)
+                for useruid, (transp, _ignore_adjusted_start, _ignore_adjusted_end) in peruserdata:
                     peruserid = useruidmap[useruid]
                     self._db_execute(
                         """
@@ -782,18 +804,18 @@ class CalendarIndex (AbstractCalendarIndex):
             # Special - for unbounded recurrence we insert a value for "infinity"
             # that will allow an open-ended time-range to always match it.
             if calendar.isRecurringUnbounded():
-                start = PyCalendarDateTime(2100, 1, 1, 0, 0, 0, tzid=PyCalendarTimezone(utc=True))
-                end = PyCalendarDateTime(2100, 1, 1, 1, 0, 0, tzid=PyCalendarTimezone(utc=True))
+                start = DateTime(2100, 1, 1, 0, 0, 0, tzid=Timezone.UTCTimezone)
+                end = DateTime(2100, 1, 1, 1, 0, 0, tzid=Timezone.UTCTimezone)
                 float = 'N'
                 self._db_execute(
                     """
                     insert into TIMESPAN (RESOURCEID, FLOAT, START, END, FBTYPE, TRANSPARENT)
                     values (:1, :2, :3, :4, :5, :6)
-                    """, resourceid, float, pyCalendarTodatetime(start), pyCalendarTodatetime(end), '?', '?'
+                    """, resourceid, float, pyCalendarToSQLTimestamp(start), pyCalendarToSQLTimestamp(end), '?', '?'
                 )
                 instanceid = self.lastrowid
-                peruserdata = calendar.perUserTransparency(None)
-                for useruid, transp in peruserdata:
+                peruserdata = calendar.perUserData(None)
+                for useruid, (transp, _ignore_adjusted_start, _ignore_adjusted_end) in peruserdata:
                     peruserid = useruidmap[useruid]
                     self._db_execute(
                         """
@@ -852,15 +874,16 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
     def reserveUID(self, uid):
         uid = uid.encode('utf-8')
         self.log.debug("Reserving UID %r @ %r" % (
-                uid,
-                self.index.resource.fp.path))
+            uid,
+            self.index.resource.fp.path)
+        )
 
         def _handleFalse(result):
             if result is False:
                 raise ReservationError(
                     "UID %s already reserved for calendar collection %s."
                     % (uid, self.index.resource)
-                    )
+                )
 
         d = self.getCachePool().add(self._key(uid),
                                     'reserved',
@@ -872,15 +895,16 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
     def unreserveUID(self, uid):
         uid = uid.encode('utf-8')
         self.log.debug("Unreserving UID %r @ %r" % (
-                uid,
-                self.index.resource.fp.path))
+            uid,
+            self.index.resource.fp.path)
+        )
 
         def _handleFalse(result):
             if result is False:
                 raise ReservationError(
                     "UID %s is not reserved for calendar collection %s."
                     % (uid, self.index.resource)
-                    )
+                )
 
         d = self.getCachePool().delete(self._key(uid))
         d.addCallback(_handleFalse)
@@ -890,8 +914,9 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
     def isReservedUID(self, uid):
         uid = uid.encode('utf-8')
         self.log.debug("Is reserved UID %r @ %r" % (
-                uid,
-                self.index.resource.fp.path))
+            uid,
+            self.index.resource.fp.path)
+        )
 
         def _checkValue((flags, value)):
             if value is None:
@@ -945,7 +970,7 @@ class SQLUIDReserver(object):
                 raise ReservationError(
                     "UID %s is not reserved for calendar collection %s."
                     % (uid, self.index.resource)
-                    )
+                )
             else:
                 try:
                     self.index._db_execute(
@@ -1092,7 +1117,7 @@ class Index (CalendarIndex):
             except ValueError:
                 log.error("Non-calendar resource: %s" % (name,))
             else:
-                #log.info("Indexing resource: %s" % (name,))
+                # og.info("Indexing resource: %s" % (name,))
                 self.addResource(name, calendar, True, reCreate=True)
             finally:
                 stream.close()
@@ -1204,7 +1229,7 @@ class IndexSchedule (CalendarIndex):
             except ValueError:
                 log.error("Non-calendar resource: %s" % (name,))
             else:
-                #log.info("Indexing resource: %s" % (name,))
+                # log.info("Indexing resource: %s" % (name,))
                 self.addResource(name, calendar, True, reCreate=True)
             finally:
                 stream.close()
