@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,21 +16,17 @@
 
 from difflib import unified_diff
 
-from pycalendar.datetime import PyCalendarDateTime
-from pycalendar.period import PyCalendarPeriod
+from pycalendar.datetime import DateTime
+from pycalendar.period import Period
 
 from twext.python.log import Logger
 
 from twistedcaldav import accounting
 from twistedcaldav.config import config
 from twistedcaldav.ical import Component, Property
-from txdav.caldav.datastore.scheduling.cuaddress import normalizeCUAddr
+from txdav.caldav.datastore.scheduling.utils import normalizeCUAddr
 from txdav.caldav.datastore.scheduling.itip import iTipGenerator
 
-"""
-Class that handles diff'ing two calendar objects.
-"""
-
 __all__ = [
     "iCalDiff",
 ]
@@ -38,19 +34,32 @@ __all__ = [
 log = Logger()
 
 class iCalDiff(object):
+    """
+    This object is used for doing comparisons between two calendar objects to
+    work out what the changes are, in order to determine whether a scheduling
+    operation might be needed. The behavior will varying based on whether the
+    change is being triggered by an Organizer or an Attendee.
+    """
 
-    def __init__(self, oldcalendar, newcalendar, smart_merge):
+    def __init__(self, oldcalendar, newcalendar, smart_merge, forceTRANSP=False):
         """
+        Note that this object will always duplicate the calendar objects when doing
+        comparisons so as not to change the calendar objects passed in.
 
-        @param oldcalendar:
-        @type oldcalendar:
-        @param newcalendar:
-        @type newcalendar:
+        @param oldcalendar: the existing calendar object to compare to
+        @type oldcalendar: L{Component}
+        @param newcalendar: the new calendar object
+        @type newcalendar: L{Component}
+        @param smart_merge: whether or not a "smart" CalDAV merge is done (If-Schedule-Tag-Match)
+        @type smart_merge: L{bool}
+        @param forceTRANSP: whether or not to apply a fix for clients failing to set TRANSP properly
+        @type forceTRANSP: L{bool}
         """
 
         self.oldcalendar = oldcalendar
         self.newcalendar = newcalendar
         self.smart_merge = smart_merge
+        self.forceTRANSP = forceTRANSP
 
 
     def organizerDiff(self):
@@ -334,7 +343,7 @@ class iCalDiff(object):
                     overridden = returnCalendar.overriddenComponent(rid)
                     if self._attendeeDecline(overridden):
                         changeCausesReply = True
-                        changedRids.append(rid.getText() if rid else "")
+                        changedRids.append(rid)
 
                     # When a master component is present we keep the missing override in place but mark it as hidden.
                     # When no master is present we now do the same so we can track updates to the override correctly.
@@ -346,7 +355,7 @@ class iCalDiff(object):
 
                     # If smart_merge is happening, then derive an instance in the new data as the change in the old
                     # data is valid and likely due to some other attendee changing their status.
-                    if  self.smart_merge:
+                    if self.smart_merge:
                         newOverride = self.newcalendar.deriveInstance(rid, allowCancelled=True)
                         if newOverride is None:
                             self._logDiffError("attendeeMerge: Could not derive instance for uncancelled component: %s" % (key,))
@@ -426,10 +435,10 @@ class iCalDiff(object):
                 # We used to generate a 403 here - but instead we now ignore this error and let the server data
                 # override the client
                 self._logDiffError("attendeeMerge: Mismatched calendar objects")
-                #return False, False, (), None
+                # return False, False, (), None
             changeCausesReply |= reply
             if reply:
-                changedRids.append(rid.getText() if rid else "")
+                changedRids.append(rid)
 
         # We need to derive instances for any declined using an EXDATE
         for decline in sorted(declines):
@@ -439,7 +448,7 @@ class iCalDiff(object):
                 if overridden is not None:
                     if self._attendeeDecline(overridden):
                         changeCausesReply = True
-                        changedRids.append(decline.getText() if decline else "")
+                        changedRids.append(decline)
 
                     # When a master component is present we keep the missing override in place but mark it as hidden.
                     # When no master is present we remove the override,
@@ -496,10 +505,18 @@ class iCalDiff(object):
 
             # If PARTSTAT was changed by the attendee, add a timestamp if needed
             if config.Scheduling.Options.TimestampAttendeePartStatChanges:
-                serverAttendee.setParameter("X-CALENDARSERVER-DTSTAMP", PyCalendarDateTime.getNowUTC().getText())
+                serverAttendee.setParameter("X-CALENDARSERVER-DTSTAMP", DateTime.getNowUTC().getText())
+            serverAttendee.removeParameter("X-CALENDARSERVER-AUTO")
 
             replyNeeded = True
 
+            # Apply client fix only if the PARTSTAT was changed
+            if self.forceTRANSP:
+                if clientAttendee.parameterValue("PARTSTAT", "NEEDS-ACTION") in ("ACCEPTED", "TENTATIVE",):
+                    clientComponent.replaceProperty(Property("TRANSP", "OPAQUE"))
+                else:
+                    clientComponent.replaceProperty(Property("TRANSP", "TRANSPARENT"))
+
         if serverAttendee.parameterValue("RSVP", "FALSE") != clientAttendee.parameterValue("RSVP", "FALSE"):
             if clientAttendee.parameterValue("RSVP", "FALSE") == "FALSE":
                 try:
@@ -528,6 +545,10 @@ class iCalDiff(object):
             if comp.name() == "VALARM":
                 serverComponent.addComponent(comp)
 
+        # VPOLL
+        if serverComponent.name() == "VPOLL":
+            replyNeeded = self._transferVPOLLData(serverComponent, clientComponent)
+
         return True, replyNeeded
 
 
@@ -568,6 +589,47 @@ class iCalDiff(object):
             return True
 
 
+    def _transferVPOLLData(self, serverComponent, clientComponent):
+
+        changed = False
+
+        # Get the matching VVOTER component in each VPOLL
+        serverVoter = serverComponent.voterComponentForVoter(self.attendee)
+        clientVoter = clientComponent.voterComponentForVoter(self.attendee)
+
+        # Now get a map of each response
+        serverMap = serverVoter.voteMap()
+        clientMap = clientVoter.voteMap()
+
+        # Remove missing
+        for poll_id in set(serverMap.keys()) - set(clientMap.keys()):
+            serverVoter.removeComponent(serverMap[poll_id])
+            changed = True
+
+        # Add new ones
+        for poll_id in set(clientMap.keys()) - set(serverMap.keys()):
+            vote = clientMap[poll_id].duplicate()
+            vote.replaceProperty(Property("LAST-MODIFIED", DateTime.getNowUTC()))
+            serverVoter.addComponent(vote)
+            changed = True
+
+        # Look for response change
+        for poll_id in set(serverMap.keys()) & set(clientMap.keys()):
+            server_vote = serverMap[poll_id]
+            client_vote = clientMap[poll_id]
+            server_response = server_vote.propertyValue("RESPONSE")
+            client_response = client_vote.propertyValue("RESPONSE")
+            if server_response != client_response:
+                if client_response is not None:
+                    server_vote.replaceProperty(Property("RESPONSE", client_response))
+                else:
+                    server_vote.removeProperty("RESPONSE")
+                server_vote.replaceProperty(Property("LAST-MODIFIED", DateTime.getNowUTC()))
+                changed = True
+
+        return changed
+
+
     def _checkInvalidChanges(self, serverComponent, clientComponent, declines):
 
         # Properties we care about: DTSTART, DTEND, DURATION, RRULE, RDATE, EXDATE
@@ -597,14 +659,14 @@ class iCalDiff(object):
     def _getNormalizedDateTimeProperties(self, component):
 
         # Basic time properties
-        if component.name() in ("VEVENT", "VJOURNAL",):
+        if component.name() in ("VEVENT", "VJOURNAL", "VPOLL"):
             dtstart = component.getProperty("DTSTART")
             dtend = component.getProperty("DTEND")
             duration = component.getProperty("DURATION")
 
-            timeRange = PyCalendarPeriod(
-                start=dtstart.value()  if dtstart  is not None else None,
-                end=dtend.value()    if dtend    is not None else None,
+            timeRange = Period(
+                start=dtstart.value() if dtstart is not None else None,
+                end=dtend.value() if dtend is not None else None,
                 duration=duration.value() if duration is not None else None,
             )
             newdue = None
@@ -614,16 +676,19 @@ class iCalDiff(object):
             duration = component.getProperty("DURATION")
 
             if dtstart or duration:
-                timeRange = PyCalendarPeriod(
-                    start=dtstart.value()  if dtstart  is not None else None,
+                timeRange = Period(
+                    start=dtstart.value() if dtstart is not None else None,
                     duration=duration.value() if duration is not None else None,
                 )
             else:
-                timeRange = PyCalendarPeriod()
+                timeRange = Period()
 
             newdue = component.getProperty("DUE")
             if newdue is not None:
                 newdue = newdue.value().duplicate().adjustToUTC()
+        else:
+            timeRange = Period()
+            newdue = None
 
         # Recurrence rules - we need to normalize the order of the value parts
         newrrules = set()
@@ -639,7 +704,7 @@ class iCalDiff(object):
         rdates = component.properties("RDATE")
         for rdate in rdates:
             for value in rdate.value():
-                if isinstance(value, PyCalendarDateTime):
+                if isinstance(value, DateTime):
                     value = value.duplicate().adjustToUTC()
                 newrdates.add(value)
 
@@ -647,7 +712,7 @@ class iCalDiff(object):
         newexdates = set()
         exdates = component.properties("EXDATE")
         for exdate in exdates:
-            newexdates.update([value.getValue().duplicate().adjustToUTC() for value in exdate.value()])
+            newexdates.update([exvalue.getValue().duplicate().adjustToUTC() for exvalue in exdate.value()])
 
         return timeRange.getStart(), timeRange.getEnd(), newdue, newrrules, newrdates, newexdates
 
@@ -691,7 +756,7 @@ class iCalDiff(object):
         return partstatChanged
 
 
-    def whatIsDifferent(self):
+    def whatIsDifferent(self, isiTip=True):
         """
         Compare the two calendar objects in their entirety and return a list of properties
         and PARTSTAT parameters that are different.
@@ -710,6 +775,7 @@ class iCalDiff(object):
             return map
 
         rids = {}
+        needs_action_changes = {}
 
         oldmap = mapComponents(self.oldcalendar)
         oldset = set(oldmap.keys())
@@ -720,40 +786,156 @@ class iCalDiff(object):
         for key in (oldset & newset):
             component1 = oldmap[key]
             component2 = newmap[key]
-            self._diffComponents(component1, component2, rids)
+            self._diffComponents(component1, component2, rids, needs_action_changes, isiTip)
 
         # Now verify that each additional component in oldset matches a derived component in newset
         for key in oldset - newset:
+            rid = key[2]
             oldcomponent = oldmap[key]
-            newcomponent = self.newcalendar.deriveInstance(key[2])
+            newcomponent = self.newcalendar.deriveInstance(rid)
             if newcomponent is None:
+                # For the non iTIP case we must report missing components on either side. Marking
+                # the DTSTART as changed is enough to trigger logic in the caller to treat this
+                # as a significant change.
+                if not isiTip:
+                    rids[rid.getText() if rid is not None else ""] = {"DTSTART": set()}
                 continue
-            self._diffComponents(oldcomponent, newcomponent, rids)
+            self._diffComponents(oldcomponent, newcomponent, rids, needs_action_changes, isiTip)
 
         # Now verify that each additional component in oldset matches a derived component in newset
         for key in newset - oldset:
-            oldcomponent = self.oldcalendar.deriveInstance(key[2])
+            rid = key[2]
+            oldcomponent = self.oldcalendar.deriveInstance(rid)
             if oldcomponent is None:
+                # For the non iTIP case we must report missing components on either side. Marking
+                # the DTSTART as changed is enough to trigger logic in the caller to treat this
+                # as a significant change.
+                if not isiTip:
+                    rids[rid.getText() if rid is not None else ""] = {"DTSTART": set()}
                 continue
             newcomponent = newmap[key]
-            self._diffComponents(oldcomponent, newcomponent, rids)
+            self._diffComponents(oldcomponent, newcomponent, rids, needs_action_changes, isiTip)
 
-        return rids
+        return (rids, needs_action_changes,)
 
 
-    def _componentDuplicateAndNormalize(self, comp):
+    TRPROPS = frozenset((
+        "DTSTART",
+        "DTEND",
+        "DURATION",
+        "DUE",
+        "RECURRENCE-ID",
+        "RRULE",
+        "RDATE",
+        "EXDATE",
+        "STATUS",
+        "TRANSP",
+        "X-APPLE-TRAVEL-DURATION",
+        "X-APPLE-TRAVEL-RETURN-DURATION",
+    ))
+
+    def timeRangeDifference(self):
+        """
+        Is there a difference between the two components that implies a change to the time or
+        transparency/status of any instance.
+
+        @return: L{True} if there is such a change, L{False} otherwise
+        @rtype: L{bool}
+        """
+
+        rids, _ignore_changes = self.whatIsDifferent(isiTip=False)
+        for props in rids.values():
+            props = frozenset(props.keys())
+            if props & self.TRPROPS:
+                return True
+        else:
+            return False
+
+
+    def attendeeNeedsAction(self, diffs):
+        """
+        Given a set of results from L{whatIsDifferent}, determine which recurrence-id's
+        have changed in such a way that we need to force the attendee to NEEDS-ACTION
+        state. We need this because with async processing of iTIP messages, it is possible
+        that the incoming iTIP message does not gave the most recent partstat change an
+        attendee made, but we need to preserve the attendee partstat whenever possible.
+
+        @param diffs: mapping of rid to property changes
+        @type diffs: C{dict}
+        """
+
+        date_changed_rids = set()
+        recurrence_reschedule = False
+
+        for rid, props in diffs.iteritems():
+            if any([testprop in props for testprop in (
+                "DTSTART",
+                "DTEND",
+                "DURATION",
+                "DUE",
+                "RECURRENCE-ID",
+            )]):
+                date_changed_rids.add(rid)
+
+            # Check to see whether a change to R-ID's happened
+            if rid is None:
+
+                if "DTSTART" in props and self.newcalendar.masterComponent().hasProperty("RRULE"):
+                    # DTSTART change with RRULE present is always a reschedule
+                    recurrence_reschedule = True
+
+                elif "RRULE" in props:
+
+                    # Need to see if the RRULE change is a simple truncation or expansion - i.e. a change to
+                    # COUNT or UNTIL only. If so we don't need to treat this as a complete re-schedule.
+
+                    # Start off assuming they are different
+                    recurrence_reschedule = True
+
+                    # Get each RRULE (can be only one in the master)
+                    oldrrule = tuple(self.oldcalendar.masterComponent().properties("RRULE"))
+                    oldrrule = oldrrule[0].value() if len(oldrrule) else None
+                    newrrule = tuple(self.newcalendar.masterComponent().properties("RRULE"))
+                    newrrule = newrrule[0].value() if len(newrrule) else None
+
+                    if newrrule is not None and oldrrule is not None:
+
+                        # Normalize the rrules by removing COUNT/UNTIL and then compare
+                        oldrrule = oldrrule.duplicate()
+                        newrrule = newrrule.duplicate()
+
+                        oldrrule.setUseUntil(False)
+                        oldrrule.setUntil(None)
+                        oldrrule.setUseCount(False)
+                        oldrrule.setCount(0)
+
+                        newrrule.setUseUntil(False)
+                        newrrule.setUntil(None)
+                        newrrule.setUseCount(False)
+                        newrrule.setCount(0)
+
+                        # If they are equal we have a simple change - no overall reschedule
+                        if newrrule == oldrrule:
+                            recurrence_reschedule = False
+
+        return (date_changed_rids, recurrence_reschedule,)
+
+
+    def _componentDuplicateAndNormalize(self, comp, isiTip=True):
         comp = comp.duplicate()
         comp.normalizePropertyValueLists("EXDATE")
-        comp.removePropertyParameters("ORGANIZER", ("SCHEDULE-STATUS",))
-        comp.removePropertyParameters("ATTENDEE", ("SCHEDULE-STATUS", "SCHEDULE-FORCE-SEND",))
         comp.removeAlarms()
         comp.normalizeAll()
         comp.normalizeAttachments()
-        iTipGenerator.prepareSchedulingMessage(comp, reply=True)
+        if isiTip:
+            comp.removePropertyParameters("ORGANIZER", ("SCHEDULE-STATUS",))
+            comp.removePropertyParameters("ATTENDEE", ("SCHEDULE-STATUS", "SCHEDULE-FORCE-SEND",))
+            comp.removePropertyParameters("VOTER", ("SCHEDULE-STATUS", "SCHEDULE-FORCE-SEND",))
+            iTipGenerator.prepareSchedulingMessage(comp, reply=True)
         return comp
 
 
-    def _diffComponents(self, comp1, comp2, rids):
+    def _diffComponents(self, comp1, comp2, rids, needs_action_rids, isiTip=True):
 
         assert isinstance(comp1, Component) and isinstance(comp2, Component)
 
@@ -762,23 +944,25 @@ class iCalDiff(object):
             return
 
         # Duplicate then normalize for comparison
-        comp1 = self._componentDuplicateAndNormalize(comp1)
-        comp2 = self._componentDuplicateAndNormalize(comp2)
+        comp1 = self._componentDuplicateAndNormalize(comp1, isiTip)
+        comp2 = self._componentDuplicateAndNormalize(comp2, isiTip)
 
         # Diff all the properties
         propdiff = set(comp1.properties()) ^ set(comp2.properties())
         addedChanges = False
+        attendeeChanges = False
 
         propsChanged = {}
         for prop in propdiff:
             if prop.name() in (
-                "TRANSP",
                 "DTSTAMP",
                 "CREATED",
                 "LAST-MODIFIED",
                 "X-CALENDARSERVER-PRIVATE-COMMENT",
             ):
                 continue
+            if prop.name() == "ATTENDEE":
+                attendeeChanges = True
             propsChanged.setdefault(prop.name(), set())
             addedChanges = True
             prop1s = tuple(comp1.properties(prop.name()))
@@ -794,7 +978,30 @@ class iCalDiff(object):
 
         if addedChanges:
             rid = comp1.getRecurrenceIDUTC()
-            rids[rid.getText() if rid is not None else ""] = propsChanged
+            rids[rid] = propsChanged
+
+        if attendeeChanges:
+            self._diffNeedsAction(comp1, comp2, needs_action_rids)
+
+
+    def _diffNeedsAction(self, comp1, comp2, needs_action_rids):
+
+        rid = comp1.getRecurrenceIDUTC()
+
+        # Get all the properties
+        attendees1 = comp1.getAllAttendeeProperties()
+        attendees2 = comp2.getAllAttendeeProperties()
+
+        # Invert
+        attendees1ByValue = dict([(attendee.value(), attendee) for attendee in attendees1])
+        attendees2ByValue = dict([(attendee.value(), attendee) for attendee in attendees2])
+
+        for attendee in attendees2ByValue.keys():
+            if attendee not in attendees1ByValue or (
+                attendees2ByValue[attendee].parameterValue("PARTSTAT", "NEEDS-ACTION") == "NEEDS-ACTION" and
+                attendees1ByValue[attendee].parameterValue("PARTSTAT", "NEEDS-ACTION") != "NEEDS-ACTION"
+            ):
+                needs_action_rids.setdefault(rid, set()).add(attendee)
 
 
     def _logDiffError(self, title):
