@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,8 +19,7 @@ Inbound IMIP mail handling for Calendar Server
 """
 
 from twext.enterprise.dal.record import fromTable
-from twext.enterprise.dal.syntax import Delete
-from twext.enterprise.queue import WorkItem
+from twext.enterprise.jobs.workitem import WorkItem, RegeneratingWorkItem
 from twext.internet.gaiendpoint import GAIEndpoint
 from twext.python.log import Logger, LegacyLogger
 
@@ -28,7 +27,6 @@ from twisted.application import service
 from twisted.internet import protocol, defer, ssl
 from twisted.internet.defer import inlineCallbacks, returnValue, succeed
 from twisted.mail import pop3client, imap4
-from twisted.mail.smtp import messageid
 
 from twistedcaldav.config import config
 from twistedcaldav.ical import Property, Component
@@ -39,6 +37,8 @@ from txdav.caldav.datastore.scheduling.itip import iTIPRequestStatus
 from txdav.common.datastore.sql_tables import schema
 
 import datetime
+import dateutil.parser
+import dateutil.tz
 import email.utils
 
 
@@ -77,29 +77,37 @@ class IMIPReplyWork(WorkItem, fromTable(schema.IMIP_REPLY_WORK)):
     @inlineCallbacks
     def doWork(self):
         calendar = Component.fromString(self.icalendarText)
-        yield injectMessage(self.transaction, self.organizer, self.attendee, calendar)
+        try:
+            yield injectMessage(self.transaction, self.organizer, self.attendee, calendar)
+        except:
+            log.error("Unable to process reply")
 
 
 
-class IMIPPollingWork(WorkItem, fromTable(schema.IMIP_POLLING_WORK)):
+class IMIPPollingWork(RegeneratingWorkItem, fromTable(schema.IMIP_POLLING_WORK)):
 
     # FIXME: purge all old tokens here
     group = "imip_polling"
 
+    def regenerateInterval(self):
+        """
+        Return the interval in seconds between regenerating instances.
+        """
+        mailRetriever = self.transaction._mailRetriever
+        if mailRetriever is not None:
+            return mailRetriever.settings["PollingSeconds"]
+
+        # The lack of mailRetriever means IMIP polling is turned off.
+        # Returning None will cause this work item to no longer be scheduled.
+        return None
+
+
     @inlineCallbacks
     def doWork(self):
 
-        # Delete all other work items
-        yield Delete(From=self.table, Where=None).on(self.transaction)
-
         mailRetriever = self.transaction._mailRetriever
         if mailRetriever is not None:
-            try:
-                yield mailRetriever.fetchMail()
-            except Exception, e:
-                log.error("Failed to fetch mail (%s)" % (e,))
-            finally:
-                yield mailRetriever.scheduleNextPoll()
+            yield mailRetriever.fetchMail()
 
 
 
@@ -115,7 +123,8 @@ class MailRetriever(service.Service):
         # If we're using our dedicated account on our local server, we're free
         # to delete all messages that arrive in the inbox so as to not let
         # cruft build up
-        self.deleteAllMail = shouldDeleteAllMail(config.ServerHostName,
+        self.deleteAllMail = shouldDeleteAllMail(
+            config.ServerHostName,
             settings.Server, settings.Username)
         self.mailReceiver = MailReceiver(store, directory)
         mailType = settings['Type']
@@ -127,12 +136,14 @@ class MailRetriever(service.Service):
         contextFactory = None
         if settings["UseSSL"]:
             contextFactory = ssl.ClientContextFactory()
-        self.point = GAIEndpoint(self.reactor, settings.Server,
+        self.point = GAIEndpoint(
+            self.reactor, settings.Server,
             settings.Port, contextFactory=contextFactory)
 
 
     def fetchMail(self):
-        return self.point.connect(self.factory(self.settings, self.mailReceiver,
+        return self.point.connect(self.factory(
+            self.settings, self.mailReceiver,
             self.deleteAllMail))
 
 
@@ -140,7 +151,7 @@ class MailRetriever(service.Service):
     def scheduleNextPoll(self, seconds=None):
         if seconds is None:
             seconds = self.settings["PollingSeconds"]
-        yield scheduleNextMailPoll(self.store, seconds)
+        yield IMIPPollingWork.reschedule(self.store, seconds)
 
 
 
@@ -169,7 +180,7 @@ def shouldDeleteAllMail(serverHostName, inboundServer, username):
 
 @inlineCallbacks
 def scheduleNextMailPoll(store, seconds):
-    txn = store.newTransaction()
+    txn = store.newTransaction(label="scheduleNextMailPoll")
     notBefore = datetime.datetime.utcnow() + datetime.timedelta(seconds=seconds)
     log.debug("Scheduling next mail poll: %s" % (notBefore,))
     yield txn.enqueue(IMIPPollingWork, notBefore=notBefore)
@@ -177,16 +188,41 @@ def scheduleNextMailPoll(store, seconds):
 
 
 
+def sanitizeCalendar(calendar):
+    """
+    Clean up specific issues seen in the wild from third party IMIP capable
+    servers.
+
+    @param calendar: the calendar Component to sanitize
+    @type calendar: L{Component}
+    """
+    # Don't let a missing PRODID prevent the reply from being processed
+    if not calendar.hasProperty("PRODID"):
+        calendar.addProperty(
+            Property(
+                "PRODID", "Unknown"
+            )
+        )
+
+    # For METHOD:REPLY we can remove STATUS properties
+    methodProperty = calendar.getProperty("METHOD")
+    if methodProperty is not None:
+        if methodProperty.value() == "REPLY":
+            calendar.removeAllPropertiesWithName("STATUS")
+
+
+
 class MailReceiver(object):
 
     NO_TOKEN = 0
     UNKNOWN_TOKEN = 1
-    MALFORMED_TO_ADDRESS = 2
-    NO_ORGANIZER_ADDRESS = 3
-    REPLY_FORWARDED_TO_ORGANIZER = 4
-    INJECTION_SUBMITTED = 5
-    INCOMPLETE_DSN = 6
-    UNKNOWN_FAILURE = 7
+    UNKNOWN_TOKEN_OLD = 2
+    MALFORMED_TO_ADDRESS = 3
+    NO_ORGANIZER_ADDRESS = 4
+    REPLY_FORWARDED_TO_ORGANIZER = 5
+    INJECTION_SUBMITTED = 6
+    INCOMPLETE_DSN = 7
+    UNKNOWN_FAILURE = 8
 
     def __init__(self, store, directory):
         self.store = store
@@ -207,7 +243,7 @@ class MailReceiver(object):
                 deliveryStatus = part
                 continue
             elif contentType == "message/rfc822":
-                #original = part
+                # original = part
                 continue
             elif contentType == "text/calendar":
                 calBody = part.get_payload(decode=True)
@@ -252,24 +288,25 @@ class MailReceiver(object):
             log.error("Mail gateway can't find token in DSN %s" % (msgId,))
             return
 
-        txn = self.store.newTransaction()
-        result = (yield txn.imipLookupByToken(token))
+        txn = self.store.newTransaction(label="MailReceiver.processDSN")
+        records = (yield txn.imipLookupByToken(token))
         yield txn.commit()
         try:
             # Note the results are returned as utf-8 encoded strings
-            organizer, attendee, _ignore_icaluid = result[0]
+            record = records[0]
         except:
             # This isn't a token we recognize
-            log.error("Mail gateway found a token (%s) but didn't "
-                           "recognize it in message %s"
-                           % (token, msgId))
+            log.error(
+                "Mail gateway found a token (%s) but didn't recognize it in message %s"
+                % (token, msgId))
             returnValue(self.UNKNOWN_TOKEN)
 
-        calendar.removeAllButOneAttendee(attendee)
+        calendar.removeAllButOneAttendee(record.attendee)
         calendar.getOrganizerProperty().setValue(organizer)
         for comp in calendar.subcomponents():
             if comp.name() == "VEVENT":
-                comp.addProperty(Property("REQUEST-STATUS",
+                comp.addProperty(Property(
+                    "REQUEST-STATUS",
                     ["5.1", "Service unavailable"]))
                 break
         else:
@@ -278,9 +315,13 @@ class MailReceiver(object):
             pass
 
         log.warn("Mail gateway processing DSN %s" % (msgId,))
-        txn = self.store.newTransaction()
-        yield txn.enqueue(IMIPReplyWork, organizer=organizer, attendee=attendee,
-            icalendarText=str(calendar))
+        txn = self.store.newTransaction(label="MailReceiver.processDSN")
+        yield txn.enqueue(
+            IMIPReplyWork,
+            organizer=record.organizer,
+            attendee=record.attendee,
+            icalendarText=str(calendar)
+        )
         yield txn.commit()
         returnValue(self.INJECTION_SUBMITTED)
 
@@ -293,25 +334,44 @@ class MailReceiver(object):
             # addr looks like: server_address+token@example.com
             token = self._extractToken(addr)
             if not token:
-                log.error("Mail gateway didn't find a token in message "
-                               "%s (%s)" % (msg['Message-ID'], msg['To']))
+                log.error(
+                    "Mail gateway didn't find a token in message "
+                    "%s (%s)" % (msg['Message-ID'], msg['To']))
                 returnValue(self.NO_TOKEN)
         else:
-            log.error("Mail gateway couldn't parse To: address (%s) in "
-                           "message %s" % (msg['To'], msg['Message-ID']))
+            log.error(
+                "Mail gateway couldn't parse To: address (%s) in "
+                "message %s" % (msg['To'], msg['Message-ID']))
             returnValue(self.MALFORMED_TO_ADDRESS)
 
-        txn = self.store.newTransaction()
-        result = (yield txn.imipLookupByToken(token))
+        txn = self.store.newTransaction(label="MailReceiver.processReply")
+        records = (yield txn.imipLookupByToken(token))
         yield txn.commit()
         try:
             # Note the results are returned as utf-8 encoded strings
-            organizer, attendee, _ignore_icaluid = result[0]
+            record = records[0]
         except:
             # This isn't a token we recognize
-            log.error("Mail gateway found a token (%s) but didn't "
-                           "recognize it in message %s"
-                           % (token, msg['Message-ID']))
+            log.info(
+                "Mail gateway found a token (%s) but didn't "
+                "recognize it in message %s"
+                % (token, msg['Message-ID']))
+            # Any email with an unknown token which was sent over 72 hours ago
+            # is deleted.  If we can't parse the date we leave it in the inbox.
+            dateString = msg.get("Date")
+            if dateString is not None:
+                try:
+                    dateSent = dateutil.parser.parse(dateString)
+                except Exception, e:
+                    log.info(
+                        "Could not parse date in IMIP email '{}' ({})".format(
+                            dateString, e
+                        )
+                    )
+                    returnValue(self.UNKNOWN_TOKEN)
+                now = datetime.datetime.now(dateutil.tz.tzutc())
+                if dateSent < now - datetime.timedelta(hours=72):
+                    returnValue(self.UNKNOWN_TOKEN_OLD)
             returnValue(self.UNKNOWN_TOKEN)
 
         for part in msg.walk():
@@ -320,26 +380,32 @@ class MailReceiver(object):
                 break
         else:
             # No icalendar attachment
-            log.warn("Mail gateway didn't find an icalendar attachment "
-                          "in message %s" % (msg['Message-ID'],))
+            log.warn(
+                "Mail gateway didn't find an icalendar attachment "
+                "in message %s" % (msg['Message-ID'],))
 
             toAddr = None
-            fromAddr = attendee[7:]
-            if organizer.startswith("mailto:"):
-                toAddr = organizer[7:]
-            elif organizer.startswith("urn:uuid:"):
-                guid = organizer[9:]
-                record = self.directory.recordWithGUID(guid)
-                if record and record.emailAddresses:
-                    toAddr = list(record.emailAddresses)[0]
+            fromAddr = record.attendee[7:]
+            if record.organizer.startswith("mailto:"):
+                toAddr = record.organizer[7:]
+            elif record.organizer.startswith("urn:x-uid:"):
+                uid = record.organizer[10:]
+                record = yield self.directory.recordWithUID(uid)
+                try:
+                    if record and record.emailAddresses:
+                        toAddr = list(record.emailAddresses)[0]
+                except AttributeError:
+                    pass
 
             if toAddr is None:
-                log.error("Don't have an email address for the organizer; "
-                               "ignoring reply.")
+                log.error(
+                    "Don't have an email address for the organizer; "
+                    "ignoring reply.")
                 returnValue(self.NO_ORGANIZER_ADDRESS)
 
             settings = config.Scheduling["iMIP"]["Sending"]
-            smtpSender = SMTPSender(settings.Username, settings.Password,
+            smtpSender = SMTPSender(
+                settings.Username, settings.Password,
                 settings.UseSSL, settings.Server, settings.Port)
 
             del msg["From"]
@@ -349,7 +415,7 @@ class MailReceiver(object):
             del msg["To"]
             msg["To"] = toAddr
             log.warn("Mail gateway forwarding reply back to organizer")
-            yield smtpSender.sendMessage(fromAddr, toAddr, messageid(), msg)
+            yield smtpSender.sendMessage(fromAddr, toAddr, SMTPSender.betterMessageID(), msg.as_string())
             returnValue(self.REPLY_FORWARDED_TO_ORGANIZER)
 
         # Process the imip attachment; inject to calendar server
@@ -358,21 +424,25 @@ class MailReceiver(object):
         calendar = Component.fromString(calBody)
         event = calendar.mainComponent()
 
-        calendar.removeAllButOneAttendee(attendee)
+        sanitizeCalendar(calendar)
+
+        calendar.removeAllButOneAttendee(record.attendee)
         organizerProperty = calendar.getOrganizerProperty()
         if organizerProperty is None:
             # ORGANIZER is required per rfc2446 section 3.2.3
-            log.warn("Mail gateway didn't find an ORGANIZER in REPLY %s"
-                          % (msg['Message-ID'],))
-            event.addProperty(Property("ORGANIZER", organizer))
+            log.warn(
+                "Mail gateway didn't find an ORGANIZER in REPLY %s"
+                % (msg['Message-ID'],))
+            event.addProperty(Property("ORGANIZER", record.organizer))
         else:
-            organizerProperty.setValue(organizer)
+            organizerProperty.setValue(record.organizer)
 
         if not calendar.getAttendees():
             # The attendee we're expecting isn't there, so add it back
             # with a SCHEDULE-STATUS of SERVICE_UNAVAILABLE.
             # The organizer will then see that the reply was not successful.
-            attendeeProp = Property("ATTENDEE", attendee,
+            attendeeProp = Property(
+                "ATTENDEE", record.attendee,
                 params={
                     "SCHEDULE-STATUS": iTIPRequestStatus.SERVICE_UNAVAILABLE,
                 }
@@ -384,9 +454,13 @@ class MailReceiver(object):
             # the appropriate ATTENDEE.  This will require a new localizable
             # email template for the message.
 
-        txn = self.store.newTransaction()
-        yield txn.enqueue(IMIPReplyWork, organizer=organizer, attendee=attendee,
-            icalendarText=str(calendar))
+        txn = self.store.newTransaction(label="MailReceiver.processReply")
+        yield txn.enqueue(
+            IMIPReplyWork,
+            organizer=record.organizer,
+            attendee=record.attendee,
+            icalendarText=str(calendar)
+        )
         yield txn.commit()
         returnValue(self.INJECTION_SUBMITTED)
 
@@ -399,6 +473,7 @@ class MailReceiver(object):
 
         NO_TOKEN - there was no token in the To address
         UNKNOWN_TOKEN - there was an unknown token in the To address
+        UNKNOWN_TOKEN_OLD - there was an unknown token and it's an old email
         MALFORMED_TO_ADDRESS - we could not parse the To address at all
         NO_ORGANIZER_ADDRESS - no ics attachment and no email to forward to
         REPLY_FORWARDED_TO_ORGANIZER - no ics attachment, but reply forwarded
@@ -420,18 +495,20 @@ class MailReceiver(object):
                     return self.processDSN(calBody, msg['Message-ID'])
                 else:
                     # It's a DSN without enough to go on
-                    log.error("Mail gateway can't process DSN %s"
-                                   % (msg['Message-ID'],))
+                    log.error(
+                        "Mail gateway can't process DSN %s"
+                        % (msg['Message-ID'],))
                     return succeed(self.INCOMPLETE_DSN)
 
-            log.info("Mail gateway received message %s from %s to %s" %
-                (msg['Message-ID'], msg['From'], msg['To']))
+            log.info(
+                "Mail gateway received message {msgid} from {fromAddr} to {toAddr}",
+                msgid=msg['Message-ID'], fromAddr=msg['From'], toAddr=msg['To'])
 
             return self.processReply(msg)
 
         except Exception, e:
             # Don't let a failure of any kind stop us
-            log.error("Failed to process message: %s" % (e,))
+            log.error("Failed to process message: {error}", error=str(e))
         return succeed(self.UNKNOWN_FAILURE)
 
 
@@ -442,9 +519,9 @@ def injectMessage(txn, organizer, attendee, calendar):
     try:
         scheduler = IMIPScheduler(txn, None)
         results = (yield scheduler.doSchedulingDirectly("iMIP", attendee, [organizer, ], calendar,))
-        log.info("Successfully injected iMIP response from %s to %s" % (attendee, organizer))
+        log.info("Successfully injected iMIP response from {attendee} to {organizer}", attendee=attendee, organizer=organizer)
     except Exception, e:
-        log.error("Failed to inject iMIP response (%s)" % (e,))
+        log.error("Failed to inject iMIP response ({error})", error=str(e))
         raise
 
     returnValue(results)
@@ -463,14 +540,16 @@ class POP3DownloadProtocol(pop3client.POP3Client):
     def serverGreeting(self, greeting):
         self.log.debug("POP servergreeting")
         pop3client.POP3Client.serverGreeting(self, greeting)
-        login = self.login(self.factory.settings["Username"],
+        login = self.login(
+            self.factory.settings["Username"],
             self.factory.settings["Password"])
         login.addCallback(self.cbLoggedIn)
         login.addErrback(self.cbLoginFailed)
 
 
     def cbLoginFailed(self, reason):
-        self.log.error("POP3 login failed for %s" %
+        self.log.error(
+            "POP3 login failed for %s" %
             (self.factory.settings["Username"],))
         return self.quit()
 
@@ -549,9 +628,11 @@ class IMAP4DownloadProtocol(imap4.IMAP4Client):
 
     def serverGreeting(self, capabilities):
         self.log.debug("IMAP servergreeting")
-        return self.authenticate(self.factory.settings["Password"]
-            ).addCallback(self.cbLoggedIn
-            ).addErrback(self.ebAuthenticateFailed)
+        return self.authenticate(
+            self.factory.settings["Password"]
+        ).addCallback(
+            self.cbLoggedIn
+        ).addErrback(self.ebAuthenticateFailed)
 
 
     def ebLogError(self, error):
@@ -559,12 +640,15 @@ class IMAP4DownloadProtocol(imap4.IMAP4Client):
 
 
     def ebAuthenticateFailed(self, reason):
-        self.log.debug("IMAP authenticate failed for {name}, trying login",
+        self.log.debug(
+            "IMAP authenticate failed for {name}, trying login",
             name=self.factory.settings["Username"])
-        return self.login(self.factory.settings["Username"],
+        return self.login(
+            self.factory.settings["Username"],
             self.factory.settings["Password"]
-            ).addCallback(self.cbLoggedIn
-            ).addErrback(self.ebLoginFailed)
+        ).addCallback(
+            self.cbLoggedIn
+        ).addErrback(self.ebLoginFailed)
 
 
     def ebLoginFailed(self, reason):
@@ -608,16 +692,15 @@ class IMAP4DownloadProtocol(imap4.IMAP4Client):
         if self.messageUIDs:
             nextUID = self.messageUIDs.pop(0)
             messageListToFetch = imap4.MessageSet(nextUID)
-            self.log.debug("Downloading message %d of %d (%s)" %
-                (self.messageCount - len(self.messageUIDs), self.messageCount,
-                nextUID))
+            self.log.debug(
+                "Downloading message %d of %d (%s)" %
+                (self.messageCount - len(self.messageUIDs), self.messageCount, nextUID))
             self.fetchMessage(messageListToFetch, True).addCallback(
                 self.cbGotMessage, messageListToFetch).addErrback(
                     self.ebLogError)
         else:
-            self.log.debug("Seeing if anything new has arrived")
-            # Go back and see if any more messages have come in
-            self.expunge().addCallback(self.cbInboxSelected)
+            # We're done for this polling interval
+            self.expunge()
 
 
     @inlineCallbacks
@@ -625,7 +708,7 @@ class IMAP4DownloadProtocol(imap4.IMAP4Client):
         self.log.debug("IMAP in cbGotMessage")
         try:
             messageData = results.values()[0]['RFC822']
-        except IndexError:
+        except (IndexError, KeyError):
             # results will be empty unless the "twistedmail-imap-flags-anywhere"
             # patch from http://twistedmatrix.com/trac/ticket/1105 is applied
             self.log.error("Skipping empty results -- apply twisted patch!")
@@ -637,16 +720,35 @@ class IMAP4DownloadProtocol(imap4.IMAP4Client):
             # Delete all mail we see
             yield self.cbFlagDeleted(messageList)
         else:
-            # Delete only mail we've processed; the rest are left flagged Seen
+            # Delete only mail we've processed; the rest are left flagged \Seen
             if actionTaken == MailReceiver.INJECTION_SUBMITTED:
                 yield self.cbFlagDeleted(messageList)
+            elif actionTaken == MailReceiver.UNKNOWN_TOKEN:
+                # It's not a token we recognize (probably meant for another pod)
+                # so remove the \Seen flag
+                yield self.cbFlagUnseen(messageList)
+            elif actionTaken == MailReceiver.UNKNOWN_TOKEN_OLD:
+                # It's not a token we recognize, but it's old, so delete it
+                yield self.cbFlagDeleted(messageList)
             else:
                 self.fetchNextMessage()
 
 
+    def cbFlagUnseen(self, messageList):
+        self.removeFlags(
+            messageList, ("\\Seen",), uid=True
+        ).addCallback(self.cbMessageUnseen, messageList)
+
+
+    def cbMessageUnseen(self, results, messageList):
+        self.log.debug("Removed \\Seen flag from message")
+        self.fetchNextMessage()
+
+
     def cbFlagDeleted(self, messageList):
-        self.addFlags(messageList, ("\\Deleted",),
-            uid=True).addCallback(self.cbMessageDeleted, messageList)
+        self.addFlags(
+            messageList, ("\\Deleted",), uid=True
+        ).addCallback(self.cbMessageDeleted, messageList)
 
 
     def cbMessageDeleted(self, results, messageList):
