@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,20 +26,20 @@ import os
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
 import email.utils
-from pycalendar.datetime import PyCalendarDateTime
-from pycalendar.duration import PyCalendarDuration
+from pycalendar.datetime import DateTime
+from pycalendar.duration import Duration
 from twext.enterprise.dal.record import fromTable
-from twext.enterprise.queue import WorkItem
+from twext.enterprise.jobs.workitem import WorkItem
 from twext.python.log import Logger
 from twisted.internet.defer import inlineCallbacks, returnValue
-from twisted.mail.smtp import messageid, rfc822date
+from twisted.mail.smtp import rfc822date
 from twisted.web.microdom import Text as DOMText, Element as DOMElement
 from twisted.web.microdom import parseString
 from twisted.web.template import XMLString, TEMPLATE_NAMESPACE, Element, renderer, flattenString, tags
 from twistedcaldav.config import config
 from twistedcaldav.ical import Component
 from twistedcaldav.localization import translationTo, _, getLanguage
-from txdav.caldav.datastore.scheduling.cuaddress import normalizeCUAddr
+from txdav.caldav.datastore.scheduling.utils import normalizeCUAddr
 from txdav.caldav.datastore.scheduling.imip.smtpsender import SMTPSender
 from txdav.common.datastore.sql_tables import schema
 
@@ -77,9 +77,11 @@ class IMIPInvitationWork(WorkItem, fromTable(schema.IMIP_INVITATION_WORK)):
         if cls.mailSender is None:
             if config.Scheduling.iMIP.Enabled:
                 settings = config.Scheduling.iMIP.Sending
-                smtpSender = SMTPSender(settings.Username, settings.Password,
+                smtpSender = SMTPSender(
+                    settings.Username, settings.Password,
                     settings.UseSSL, settings.Server, settings.Port)
-                cls.mailSender = MailSender(settings.Address,
+                cls.mailSender = MailSender(
+                    settings.Address,
                     settings.SuppressionDays, smtpSender, getLanguage(config))
         return cls.mailSender
 
@@ -92,7 +94,8 @@ class IMIPInvitationWork(WorkItem, fromTable(schema.IMIP_INVITATION_WORK)):
         mailSender = self.getMailSender()
         if mailSender is not None:
             calendar = Component.fromString(self.icalendarText)
-            yield mailSender.outbound(self.transaction,
+            yield mailSender.outbound(
+                self.transaction,
                 self.fromAddr, self.toAddr, calendar)
 
 #
@@ -320,8 +323,8 @@ class MailSender(object):
         """
 
         if onlyAfter is None:
-            duration = PyCalendarDuration(days=self.suppressionDays)
-            onlyAfter = PyCalendarDateTime.getNowUTC() - duration
+            duration = Duration(days=self.suppressionDays)
+            onlyAfter = DateTime.getNowUTC() - duration
 
         icaluid = calendar.resourceUID()
         method = calendar.propertyValue("METHOD")
@@ -361,28 +364,29 @@ class MailSender(object):
 
             # Reuse or generate a token based on originator, toAddr, and
             # event uid
-            token = (yield txn.imipGetToken(originator, toAddr.lower(), icaluid))
-            if token is None:
+            record = (yield txn.imipGetToken(originator, toAddr.lower(), icaluid))
+            if record is None:
 
                 # Because in the past the originator was sometimes in mailto:
                 # form, lookup an existing token by mailto: as well
                 organizerProperty = calendar.getOrganizerProperty()
                 organizerEmailAddress = organizerProperty.parameterValue("EMAIL", None)
                 if organizerEmailAddress is not None:
-                    token = (yield txn.imipGetToken("mailto:%s" % (organizerEmailAddress.lower(),), toAddr.lower(), icaluid))
+                    record = (yield txn.imipGetToken("mailto:%s" % (organizerEmailAddress.lower(),), toAddr.lower(), icaluid))
 
-            if token is None:
-                token = (yield txn.imipCreateToken(originator, toAddr.lower(), icaluid))
+            if record is None:
+                record = (yield txn.imipCreateToken(originator, toAddr.lower(), icaluid))
                 self.log.debug("Mail gateway created token %s for %s "
                                "(originator), %s (recipient) and %s (icaluid)"
-                               % (token, originator, toAddr, icaluid))
+                               % (record.token, originator, toAddr, icaluid))
                 inviteState = "new"
 
             else:
                 self.log.debug("Mail gateway reusing token %s for %s "
                                "(originator), %s (recipient) and %s (icaluid)"
-                               % (token, originator, toAddr, icaluid))
+                               % (record.token, originator, toAddr, icaluid))
                 inviteState = "update"
+            token = record.token
 
             fullServerAddress = self.address
             _ignore_name, serverAddress = email.utils.parseaddr(fullServerAddress)
@@ -449,6 +453,8 @@ class MailSender(object):
             orgEmail = organizerMailto[7:]
 
             orgCN = calendar.getOrganizerProperty().parameterValue('CN', None)
+            if orgCN:
+                orgCN = orgCN.decode("utf-8")
             addressWithToken = formattedFrom
 
         # At the point we've created the token in the db, which we always
@@ -469,13 +475,14 @@ class MailSender(object):
                     if emailAddress:
                         attendeeProp.setValue("mailto:%s" % (emailAddress,))
 
-        msgId, message = self.generateEmail(inviteState, calendar, orgEmail,
+        msgId, message = self.generateEmail(
+            inviteState, calendar, orgEmail,
             orgCN, attendees, formattedFrom, addressWithToken, recipient,
             language=self.language)
 
         try:
-            success = (yield self.smtpSender.sendMessage(fromAddr, toAddr,
-                msgId, message))
+            success = (yield self.smtpSender.sendMessage(
+                fromAddr, toAddr, msgId, message))
             returnValue(success)
         except Exception, e:
             self.log.error("Failed to send IMIP message (%s)" % (str(e),))
@@ -543,8 +550,9 @@ class MailSender(object):
         plainText = self.renderPlainText(details, (orgCN, orgEmail),
                                          attendees, canceled)
 
-        htmlText = self.renderHTML(details, (orgCN, orgEmail),
-                                              attendees, canceled)
+        htmlText = self.renderHTML(
+            details, (orgCN, orgEmail), attendees, canceled
+        )
 
         msg = MIMEMultipart()
         msg["From"] = fromAddress
@@ -552,7 +560,7 @@ class MailSender(object):
         msg["Reply-To"] = replyToAddress
         msg["To"] = toAddress
         msg["Date"] = rfc822date()
-        msgId = messageid()
+        msgId = SMTPSender.betterMessageID()
         msg["Message-ID"] = msgId
 
         msgAlt = MIMEMultipart("alternative")
@@ -581,7 +589,8 @@ class MailSender(object):
         method = calendar.propertyValue("METHOD").lower()
         msgIcal.set_param("method", method)
         msgIcal.add_header("Content-ID", "<invitation.ics>")
-        msgIcal.add_header("Content-Disposition",
+        msgIcal.add_header(
+            "Content-Disposition",
             "inline;filename=invitation.ics")
         msg.attach(msgIcal)
 
@@ -600,15 +609,17 @@ class MailSender(object):
         plainAttendeeList = []
         for cn, mailto in attendees:
             if cn:
-                plainAttendeeList.append(cn if not mailto else
-                    "%s <%s>" % (cn, mailto))
+                plainAttendeeList.append(
+                    cn if not mailto else "%s <%s>" % (cn, mailto)
+                )
             elif mailto:
                 plainAttendeeList.append("<%s>" % (mailto,))
 
         details['plainAttendees'] = ", ".join(plainAttendeeList)
 
-        details['plainOrganizer'] = (orgCN if not orgEmail else
-            "%s <%s>" % (orgCN, orgEmail))
+        details['plainOrganizer'] = (
+            orgCN if not orgEmail else "%s <%s>" % (orgCN, orgEmail)
+        )
 
         # plain text version
         if canceled:
@@ -654,8 +665,9 @@ class MailSender(object):
         # template stuff, and once again, it's just a 'mailto:'.
         # tags.a(href="mailto:"+email)[cn]
         if orgEmail:
-            details['htmlOrganizer'] = tags.a(href="mailto:%s" % (orgEmail,))(
-                orgCN)
+            if not orgCN:
+                orgCN = orgEmail
+            details['htmlOrganizer'] = tags.a(href="mailto:%s" % (orgEmail,))(orgCN)
         else:
             details['htmlOrganizer'] = orgCN
 
