@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,8 +17,9 @@ from __future__ import print_function
 
 from cStringIO import StringIO
 
-from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.datetime import DateTime
 
+from twisted.internet import reactor
 from twisted.internet.defer import inlineCallbacks, succeed
 from twisted.trial import unittest
 from twisted.web.template import Element, renderer, flattenString
@@ -31,6 +32,8 @@ from txdav.caldav.datastore.scheduling.imip.outbound import MailSender
 from txdav.caldav.datastore.scheduling.imip.outbound import StringFormatTemplateLoader
 from txdav.common.datastore.test.util import buildStore
 
+from twext.enterprise.jobs.jobitem import JobItem
+
 import email
 from email.iterators import typed_subpart_iterator
 import os
@@ -57,7 +60,7 @@ SUMMARY:t\xe9sting outbound( )
 DESCRIPTION:awesome description with "<" and "&"
 END:VEVENT
 END:VCALENDAR
-"""
+""".encode("utf-8")
 
 inviteTextNoTimezone = u"""BEGIN:VCALENDAR
 VERSION:2.0
@@ -80,7 +83,7 @@ SUMMARY:t\xe9sting outbound( )
 DESCRIPTION:awesome description with "<" and "&"
 END:VEVENT
 END:VCALENDAR
-"""
+""".encode("utf-8")
 
 inviteTextWithTimezone = u"""BEGIN:VCALENDAR
 VERSION:2.0
@@ -246,7 +249,7 @@ SUMMARY:t\xe9sting outbound( )
 DESCRIPTION:awesome description with "<" and "&"
 END:VEVENT
 END:VCALENDAR
-"""
+""".encode("utf-8")
 
 ORGANIZER = "urn:uuid:C3B38B00-4166-11DD-B22C-A07C87E02F6A"
 ATTENDEE = "mailto:attendee@example.com"
@@ -283,45 +286,39 @@ class OutboundTests(unittest.TestCase):
     def setUp(self):
         self.store = yield buildStore(self, None)
         self.directory = self.store.directoryService()
-        self.sender = MailSender("server@example.com", 7, DummySMTPSender(),
+        self.sender = MailSender(
+            "server@example.com", 7, DummySMTPSender(),
             language="en")
 
         def _getSender(ignored):
             return self.sender
         self.patch(IMIPInvitationWork, "getMailSender", _getSender)
 
-        self.wp = None
-        self.store.queuer.callWithNewProposals(self._proposalCallback)
-
-
-    def _proposalCallback(self, wp):
-        self.wp = wp
-
 
     @inlineCallbacks
     def test_work(self):
         txn = self.store.newTransaction()
-        wp = (yield txn.enqueue(IMIPInvitationWork,
+        yield txn.enqueue(
+            IMIPInvitationWork,
             fromAddr=ORGANIZER,
             toAddr=ATTENDEE,
             icalendarText=initialInviteText.replace("\n", "\r\n"),
-        ))
-        self.assertEquals(wp, self.wp)
+        )
         yield txn.commit()
-        yield wp.whenExecuted()
+        yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
         txn = self.store.newTransaction()
-        token = (yield txn.imipGetToken(
+        record = (yield txn.imipGetToken(
             ORGANIZER,
             ATTENDEE,
             ICALUID
         ))
-        self.assertTrue(token)
-        organizer, attendee, icaluid = (yield txn.imipLookupByToken(token))[0]
+        self.assertTrue(record is not None)
+        record = (yield txn.imipLookupByToken(record.token))[0]
         yield txn.commit()
-        self.assertEquals(organizer, ORGANIZER)
-        self.assertEquals(attendee, ATTENDEE)
-        self.assertEquals(icaluid, ICALUID)
+        self.assertEquals(record.organizer, ORGANIZER)
+        self.assertEquals(record.attendee, ATTENDEE)
+        self.assertEquals(record.icaluid, ICALUID)
 
 
     @inlineCallbacks
@@ -329,19 +326,20 @@ class OutboundTests(unittest.TestCase):
         self.sender.smtpSender.shouldSucceed = False
 
         txn = self.store.newTransaction()
-        wp = (yield txn.enqueue(IMIPInvitationWork,
+        yield txn.enqueue(
+            IMIPInvitationWork,
             fromAddr=ORGANIZER,
             toAddr=ATTENDEE,
             icalendarText=initialInviteText.replace("\n", "\r\n"),
-        ))
+        )
         yield txn.commit()
-        yield wp.whenExecuted()
-        # Verify a new work proposal was not created
-        self.assertEquals(wp, self.wp)
+        yield JobItem.waitEmpty(self.store.newTransaction, reactor, 60)
 
 
-    def _interceptEmail(self, inviteState, calendar, orgEmail, orgCn,
-        attendees, fromAddress, replyToAddress, toAddress, language="en"):
+    def _interceptEmail(
+        self, inviteState, calendar, orgEmail, orgCn,
+        attendees, fromAddress, replyToAddress, toAddress, language="en"
+    ):
         self.inviteState = inviteState
         self.calendar = calendar
         self.orgEmail = orgEmail
@@ -351,7 +349,8 @@ class OutboundTests(unittest.TestCase):
         self.replyToAddress = replyToAddress
         self.toAddress = toAddress
         self.language = language
-        self.results = self._actualGenerateEmail(inviteState, calendar,
+        self.results = self._actualGenerateEmail(
+            inviteState, calendar,
             orgEmail, orgCn, attendees, fromAddress, replyToAddress, toAddress,
             language=language)
         return self.results
@@ -391,63 +390,63 @@ class OutboundTests(unittest.TestCase):
 
             # Update
             (
-                """BEGIN:VCALENDAR
+                u"""BEGIN:VCALENDAR
 VERSION:2.0
 METHOD:REQUEST
 BEGIN:VEVENT
 UID:CFDD5E46-4F74-478A-9311-B3FF905449C3
 DTSTART:20100325T154500Z
 DTEND:20100325T164500Z
-ATTENDEE;CN=The Attendee;CUTYPE=INDIVIDUAL;PARTSTAT=NEEDS-ACTION;RSVP=TRUE:
+ATTENDEE;CN=Th\xe9 Attendee;CUTYPE=INDIVIDUAL;PARTSTAT=NEEDS-ACTION;RSVP=TRUE:
  mailto:attendee@example.com
-ATTENDEE;CN=The Organizer;CUTYPE=INDIVIDUAL;EMAIL=organizer@example.com;PAR
+ATTENDEE;CN=Th\xe9 Organizer;CUTYPE=INDIVIDUAL;EMAIL=organizer@example.com;PAR
  TSTAT=ACCEPTED:urn:uuid:C3B38B00-4166-11DD-B22C-A07C87E02F6A
-ORGANIZER;CN=The Organizer;EMAIL=organizer@example.com:urn:uuid:C3B38B00-41
+ORGANIZER;CN=Th\xe9 Organizer;EMAIL=organizer@example.com:urn:uuid:C3B38B00-41
  66-11DD-B22C-A07C87E02F6A
-SUMMARY:testing outbound( ) *update*
+SUMMARY:t\xe9sting outbound( ) *update*
 END:VEVENT
 END:VCALENDAR
-""",
+""".encode("utf-8"),
                 "CFDD5E46-4F74-478A-9311-B3FF905449C3",
                 "urn:uuid:C3B38B00-4166-11DD-B22C-A07C87E02F6A",
                 "mailto:attendee@example.com",
                 "update",
                 "organizer@example.com",
-                "The Organizer",
+                u"Th\xe9 Organizer",
                 [
-                    (u'The Attendee', u'attendee@example.com'),
-                    (u'The Organizer', u'organizer@example.com')
+                    (u'Th\xe9 Attendee', u'attendee@example.com'),
+                    (u'Th\xe9 Organizer', u'organizer@example.com')
                 ],
-                "The Organizer <organizer@example.com>",
-                "The Organizer <organizer@example.com>",
+                u"Th\xe9 Organizer <organizer@example.com>",
+                "=?utf-8?q?Th=C3=A9_Organizer_=3Corganizer=40example=2Ecom=3E?=",
                 "attendee@example.com",
             ),
 
             # Reply
             (
-                """BEGIN:VCALENDAR
+                u"""BEGIN:VCALENDAR
 VERSION:2.0
 METHOD:REPLY
 BEGIN:VEVENT
 UID:DFDD5E46-4F74-478A-9311-B3FF905449C4
 DTSTART:20100325T154500Z
 DTEND:20100325T164500Z
-ATTENDEE;CN=The Attendee;CUTYPE=INDIVIDUAL;EMAIL=attendee@example.com;PARTST
+ATTENDEE;CN=Th\xe9 Attendee;CUTYPE=INDIVIDUAL;EMAIL=attendee@example.com;PARTST
  AT=ACCEPTED:urn:uuid:C3B38B00-4166-11DD-B22C-A07C87E02F6A
-ORGANIZER;CN=The Organizer;EMAIL=organizer@example.com:mailto:organizer@exam
+ORGANIZER;CN=Th\xe9 Organizer;EMAIL=organizer@example.com:mailto:organizer@exam
  ple.com
-SUMMARY:testing outbound( ) *reply*
+SUMMARY:t\xe9sting outbound( ) *reply*
 END:VEVENT
 END:VCALENDAR
-""",
+""".encode("utf-8"),
                 None,
                 "urn:uuid:C3B38B00-4166-11DD-B22C-A07C87E02F6A",
                 "mailto:organizer@example.com",
                 "reply",
                 "organizer@example.com",
-                "The Organizer",
+                u"Th\xe9 Organizer",
                 [
-                    (u'The Attendee', u'attendee@example.com'),
+                    (u'Th\xe9 Attendee', u'attendee@example.com'),
                 ],
                 "attendee@example.com",
                 "attendee@example.com",
@@ -455,9 +454,11 @@ END:VCALENDAR
             ),
 
         )
-        for (inputCalendar, UID, inputOriginator, inputRecipient, inviteState,
+        for (
+            inputCalendar, UID, inputOriginator, inputRecipient, inviteState,
             outputOrganizerEmail, outputOrganizerName, outputAttendeeList,
-            outputFrom, encodedFrom, outputRecipient) in data:
+            outputFrom, encodedFrom, outputRecipient
+        ) in data:
 
             txn = self.store.newTransaction()
             yield self.sender.outbound(
@@ -465,7 +466,7 @@ END:VCALENDAR
                 inputOriginator,
                 inputRecipient,
                 Component.fromString(inputCalendar.replace("\n", "\r\n")),
-                onlyAfter=PyCalendarDateTime(2010, 1, 1, 0, 0, 0)
+                onlyAfter=DateTime(2010, 1, 1, 0, 0, 0)
             )
             yield txn.commit()
 
@@ -481,12 +482,12 @@ END:VCALENDAR
             if UID: # The organizer is local, and server is sending to remote
                     # attendee
                 txn = self.store.newTransaction()
-                token = (yield txn.imipGetToken(inputOriginator, inputRecipient,
-                    UID))
+                record = (yield txn.imipGetToken(inputOriginator, inputRecipient, UID))
                 yield txn.commit()
-                self.assertNotEquals(token, None)
-                self.assertEquals(msg["Reply-To"],
-                    "server+%s@example.com" % (token,))
+                self.assertNotEquals(record, None)
+                self.assertEquals(
+                    msg["Reply-To"],
+                    "server+%s@example.com" % (record.token,))
 
                 # Make sure attendee property for organizer exists and matches
                 # the CUA of the organizer property
@@ -509,7 +510,7 @@ END:VCALENDAR
                 inputOriginator,
                 inputRecipient,
                 Component.fromString(inputCalendar.replace("\n", "\r\n")),
-                onlyAfter=PyCalendarDateTime(2021, 1, 1, 0, 0, 0)
+                onlyAfter=DateTime(2021, 1, 1, 0, 0, 0)
             )
             yield txn.commit()
             self.assertFalse(self.sender.smtpSender.sendMessageCalled)
@@ -518,30 +519,31 @@ END:VCALENDAR
     @inlineCallbacks
     def test_tokens(self):
         txn = self.store.newTransaction()
-        token = (yield txn.imipLookupByToken("xyzzy"))
-        yield txn.commit()
-        self.assertEquals(token, [])
-
-        txn = self.store.newTransaction()
-        token1 = (yield txn.imipCreateToken("organizer", "attendee", "icaluid"))
+        self.assertEquals((yield txn.imipLookupByToken("xyzzy")), [])
         yield txn.commit()
 
         txn = self.store.newTransaction()
-        token2 = (yield txn.imipGetToken("organizer", "attendee", "icaluid"))
-        yield txn.commit()
-        self.assertEquals(token1, token2)
-
-        txn = self.store.newTransaction()
-        self.assertEquals((yield txn.imipLookupByToken(token1)),
-            [["organizer", "attendee", "icaluid"]])
+        record1 = (yield txn.imipCreateToken("organizer", "attendee", "icaluid"))
         yield txn.commit()
 
         txn = self.store.newTransaction()
-        yield txn.imipRemoveToken(token1)
+        record2 = (yield txn.imipGetToken("organizer", "attendee", "icaluid"))
+        yield txn.commit()
+        self.assertEquals(record1.token, record2.token)
+
+        txn = self.store.newTransaction()
+        record = (yield txn.imipLookupByToken(record1.token))[0]
+        self.assertEquals(
+            [record.organizer, record.attendee, record.icaluid],
+            ["organizer", "attendee", "icaluid"])
         yield txn.commit()
 
         txn = self.store.newTransaction()
-        self.assertEquals((yield txn.imipLookupByToken(token1)), [])
+        yield txn.imipRemoveToken(record1.token)
+        yield txn.commit()
+
+        txn = self.store.newTransaction()
+        self.assertEquals((yield txn.imipLookupByToken(record1.token)), [])
         yield txn.commit()
 
 
@@ -556,11 +558,11 @@ END:VCALENDAR
         # Explictly store a token with mailto: CUA for organizer
         # (something that doesn't happen any more, but did in the past)
         txn = self.store.newTransaction()
-        origToken = (yield txn.imipCreateToken(organizerEmail,
+        origRecord = (yield txn.imipCreateToken(
+            organizerEmail,
             "mailto:attendee@example.com",
             "CFDD5E46-4F74-478A-9311-B3FF905449C3"
-            )
-        )
+        ))
         yield txn.commit()
 
         inputCalendar = initialInviteText
@@ -569,21 +571,23 @@ END:VCALENDAR
         inputRecipient = "mailto:attendee@example.com"
 
         txn = self.store.newTransaction()
-        yield self.sender.outbound(txn, inputOriginator, inputRecipient,
+        yield self.sender.outbound(
+            txn, inputOriginator, inputRecipient,
             Component.fromString(inputCalendar.replace("\n", "\r\n")),
-            onlyAfter=PyCalendarDateTime(2010, 1, 1, 0, 0, 0))
+            onlyAfter=DateTime(2010, 1, 1, 0, 0, 0))
+        yield txn.commit()
 
         # Verify we didn't create a new token...
         txn = self.store.newTransaction()
-        token = (yield txn.imipGetToken(inputOriginator, inputRecipient, UID))
+        record = (yield txn.imipGetToken(inputOriginator, inputRecipient, UID))
         yield txn.commit()
-        self.assertEquals(token, None)
+        self.assertEquals(record, None)
 
         # But instead kept the old one...
         txn = self.store.newTransaction()
-        token = (yield txn.imipGetToken(organizerEmail, inputRecipient, UID))
+        record = (yield txn.imipGetToken(organizerEmail, inputRecipient, UID))
         yield txn.commit()
-        self.assertEquals(token, origToken)
+        self.assertEquals(record.token, origRecord.token)
 
 
     def generateSampleEmail(self, caltext=initialInviteText):
@@ -621,6 +625,59 @@ END:VCALENDAR
         self.assertEquals(actualTypes, expectedTypes)
 
 
+    def test_generateEmail_noOrganizerCN(self):
+        """
+        L{MailHandler.generateEmail} generates a MIME-formatted email when
+        the organizer property has no CN parameter.
+        """
+        calendar = Component.fromString(initialInviteText)
+        _ignore_msgID, msgTxt = self.sender.generateEmail(
+            inviteState='new',
+            calendar=calendar,
+            orgEmail=u"user01@localhost",
+            orgCN=None,
+            attendees=[(u"Us\xe9r One", "user01@localhost"),
+                       (u"User 2", "user02@localhost")],
+            fromAddress="user01@localhost",
+            replyToAddress="imip-system@localhost",
+            toAddress="user03@localhost",
+        )
+        message = email.message_from_string(msgTxt)
+        self.assertTrue(message is not None)
+
+
+    def test_generateEmail_noAttendeeCN(self):
+        """
+        L{MailHandler.generateEmail} generates a MIME-formatted email when
+        the attendee property has no CN parameter.
+        """
+        calendar = Component.fromString(initialInviteText)
+        _ignore_msgID, msgTxt = self.sender.generateEmail(
+            inviteState='new',
+            calendar=calendar,
+            orgEmail=u"user01@localhost",
+            orgCN=u"User Z\xe9ro One",
+            attendees=[(None, "user01@localhost"),
+                       (None, "user02@localhost")],
+            fromAddress="user01@localhost",
+            replyToAddress="imip-system@localhost",
+            toAddress="user03@localhost",
+        )
+        message = email.message_from_string(msgTxt)
+        self.assertTrue(message is not None)
+
+
+    def test_messageID(self):
+        """
+        L{SMTPSender.betterMessageID} generates a Message-ID domain matching
+        the L{config.ServerHostName} value.
+        """
+        self.patch(config, "ServerHostName", "calendar.example.com")
+        msgID, message = self.generateSampleEmail()
+        self.assertEquals(message['Message-ID'], msgID)
+        self.assertEqual(msgID[:-1].split("@")[1], config.ServerHostName)
+
+
     def test_alwaysIncludeTimezones(self):
         """
         L{MailHandler.generateEmail} generates a MIME-formatted email with a
@@ -691,7 +748,7 @@ END:VCALENDAR
         """
         class StubElement(Element):
             loader = StringFormatTemplateLoader(
-                lambda : StringIO(
+                lambda: StringIO(
                     "<test><alpha>%(slot1)s</alpha>%(other)s</test>"
                 ),
                 "testRenderHere"
@@ -703,8 +760,10 @@ END:VCALENDAR
                                      other="world")
         result = []
         flattenString(None, StubElement()).addCallback(result.append)
-        self.assertEquals(result,
-                          ["<test><alpha>hello</alpha>world</test>"])
+        self.assertEquals(
+            list(result),
+            ["<test><alpha>hello</alpha>world</test>"]
+        )
 
 
     def test_templateLoaderWithAttributes(self):
@@ -715,7 +774,7 @@ END:VCALENDAR
         """
         class StubElement(Element):
             loader = StringFormatTemplateLoader(
-                lambda : StringIO(
+                lambda: StringIO(
                     '<test><alpha beta="before %(slot1)s after">inner</alpha>'
                     '%(other)s</test>'
                 ),
@@ -728,9 +787,13 @@ END:VCALENDAR
                                      other="world")
         result = []
         flattenString(None, StubElement()).addCallback(result.append)
-        self.assertEquals(result,
-                          ['<test><alpha beta="before hello after">'
-                           'inner</alpha>world</test>'])
+        self.assertEquals(
+            result,
+            [
+                '<test><alpha beta="before hello after">'
+                'inner</alpha>world</test>'
+            ]
+        )
 
 
     def test_templateLoaderTagSoup(self):
@@ -742,7 +805,7 @@ END:VCALENDAR
         """
         class StubElement(Element):
             loader = StringFormatTemplateLoader(
-                lambda : StringIO(
+                lambda: StringIO(
                     '<test><alpha beta="before %(slot1)s after">inner</alpha>'
                     '%(other)s'
                 ),
