@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2012-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,11 +15,11 @@
 ##
 
 from twext.python.log import Logger
-from twext.web2.client.http import ClientRequest
-from twext.web2.dav.util import allDataFromStream, joinURL
-from twext.web2.http import Response
-from twext.web2.http_headers import MimeType
-from twext.web2.stream import MemoryStream
+from txweb2.client.http import ClientRequest
+from txweb2.dav.util import allDataFromStream, joinURL
+from txweb2.http import Response
+from txweb2.http_headers import MimeType
+from txweb2.stream import MemoryStream
 
 from twisted.internet.defer import inlineCallbacks, returnValue, succeed
 from twistedcaldav.client.geturl import getURL
@@ -294,8 +294,16 @@ class DKIMUtils(object):
 
     @staticmethod
     def canonicalizeBody(data):
-        if not data.endswith("\r\n"):
-            data += "\r\n"
+        """
+        DKIM simple body canonicalization: remove empty lines at the end
+        and ensure it ends with one \r\n.
+
+        @param data: data to canonicalize
+        @type data: L{str}
+        """
+        while data.endswith("\r\n"):
+            data = data[:-2]
+        data += "\r\n"
         return data
 
 
@@ -516,7 +524,7 @@ class DKIMVerifier(object):
     def __init__(self, headers, body, key_lookup=None, protocol_debug=False):
         """
         @param headers: The HTTP request headers to process
-        @type headers: L{twext.web2.http_headers.Headers}
+        @type headers: L{txweb2.http_headers.Headers}
         @param body: The HTTP request body to process
         @type body: C{str}
         """
@@ -812,7 +820,8 @@ class PublicKeyLookup(object):
             return None
 
 
-    def flushCache(self):
+    @staticmethod
+    def flushCache():
         PublicKeyLookup.keyCache = {}
 
 
@@ -862,7 +871,7 @@ class PublicKeyLookup_HTTP_WellKnown(PublicKeyLookup):
         """
 
         # First we do an SRV lookup for _domainkey to get the public key server host/port
-        result = (yield lookupServerViaSRV(self.dkim_tags["d"], service="_domainkey"))
+        result = (yield lookupServerViaSRV(self.dkim_tags["d"], service="_domainkey_lookup"))
         if result is None:
             log.debug("DKIM: SRV _domainkey failed on: %s trying domain directly" % (self.dkim_tags["d"],))
             host = self.dkim_tags["d"]
