@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2012-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,9 @@
 
 from Crypto.PublicKey import RSA
 
-from twext.web2.dav.util import allDataFromStream
-from twext.web2.http_headers import Headers, MimeType
-from twext.web2.stream import MemoryStream
+from txweb2.dav.util import allDataFromStream
+from txweb2.http_headers import Headers, MimeType
+from txweb2.stream import MemoryStream
 
 from twisted.internet.defer import inlineCallbacks, succeed
 from twisted.names import client
@@ -328,7 +328,8 @@ class TestDKIMVerifier (TestDKIMBase):
 
         data = (
             # Count on Recipient
-            ("""Host:example.com
+            (
+                """Host:example.com
 Content-Type: text/calendar  ; charset =  "utf-8"
 Originator:  mailto:user01@example.com
 Recipient:  mailto:user02@example.com  ,\t mailto:user03@example.com\t\t
@@ -337,14 +338,15 @@ DKIM-Signature:  v=1;\t\t d=example.com; s = dkim; t\t=\t1234; a=rsa-sha1; \t\tq
 Cache-Control:no-cache
 Connection:close
 """,
-            """content-type:text/calendar ; charset = "utf-8"
+                """content-type:text/calendar ; charset = "utf-8"
 originator:mailto:user01@example.com
 recipient:mailto:user02@example.com,mailto:user03@example.com
 ischedule-version:1.0
 dkim-signature:v=1; d=example.com; s = dkim; t = 1234; a=rsa-sha1; q=dns/txt:http/well-known ; c=ischedule-relaxed/simple; h=Content-Type:Originator:Recipient:iSchedule-Version:iSchedule-Message-ID; bh=abc; b="""
             ),
             # Exact count on Recipient
-            ("""Host:example.com
+            (
+                """Host:example.com
 Content-Type: text/calendar  ; charset =  "utf-8"
 Originator:  mailto:user01@example.com
 Recipient:  mailto:user02@example.com  ,\t mailto:user03@example.com\t\t
@@ -354,14 +356,15 @@ DKIM-Signature:  v=1;\t\t d=example.com; s = dkim; t\t=\t1234; a=rsa-sha1; \t\tq
 Cache-Control:no-cache
 Connection:close
 """,
-            """content-type:text/calendar ; charset = "utf-8"
+                """content-type:text/calendar ; charset = "utf-8"
 originator:mailto:user01@example.com
 recipient:mailto:user02@example.com,mailto:user03@example.com,mailto:user04@example.com
 ischedule-version:1.0
 dkim-signature:v=1; d=example.com; s = dkim; t = 1234; a=rsa-sha1; q=dns/txt:http/well-known ; c=ischedule-relaxed/simple; h=Content-Type:Originator:Recipient:iSchedule-Version:iSchedule-Message-ID; bh=abc; b="""
             ),
             # Re-ordered Content-Type
-            ("""Host:example.com
+            (
+                """Host:example.com
 iSchedule-Version: 1.0
 Originator:  mailto:user01@example.com
 Recipient:  mailto:user02@example.com  ,\t mailto:user03@example.com\t\t
@@ -370,7 +373,7 @@ Content-Type: text/calendar  ; charset =  "utf-8"
 Cache-Control:no-cache
 Connection:close
 """,
-            """content-type:text/calendar ; charset = "utf-8"
+                """content-type:text/calendar ; charset = "utf-8"
 originator:mailto:user01@example.com
 recipient:mailto:user02@example.com,mailto:user03@example.com
 ischedule-version:1.0
@@ -386,6 +389,34 @@ dkim-signature:v=1; d=example.com; s = dkim; t = 1234; a=rsa-sha1; q=dns/txt:htt
             self.assertEqual(extracted, result.replace("\n", "\r\n"))
 
 
+    def test_canonicalize_body(self):
+        """
+        L{DKIMUtils.canonicalizeBody} correctly canonicalizes bodies.
+        """
+
+        data = (
+            (
+                """Simple""",
+                """Simple\n""",
+            ),
+            (
+                """Simple\n""",
+                """Simple\n""",
+            ),
+            (
+                """Simple\n\n""",
+                """Simple\n""",
+            ),
+        )
+
+        for text, result in data:
+            self.assertEqual(
+                DKIMUtils.canonicalizeBody(text.replace("\n", "\r\n")),
+                result.replace("\n", "\r\n"),
+            )
+
+
+    @inlineCallbacks
     def test_locate_public_key(self):
         """
         L{DKIMVerifier.locatePublicKey} correctly finds key matching headers.
@@ -393,7 +424,8 @@ dkim-signature:v=1; d=example.com; s = dkim; t = 1234; a=rsa-sha1; q=dns/txt:htt
 
         data = (
             # Valid
-            ("""Host:example.com
+            (
+                """Host:example.com
 Content-Type: text/calendar  ; charset =  "utf-8"
 Originator:  mailto:user01@example.com
 Recipient:  mailto:user02@example.com  ,\t mailto:user03@example.com\t\t
@@ -401,11 +433,12 @@ DKIM-Signature:  v=1;\t\t d=example.com; s = dkim; t\t=\t1234; a=rsa-sha1; \t\tq
 Cache-Control:no-cache
 Connection:close
 """,
-            [DKIMUtils.extractTags("v=DKIM1; p=%s" % (self.public_key_data,))],
-            True,
+                [DKIMUtils.extractTags("v=DKIM1; p=%s" % (self.public_key_data,))],
+                True,
             ),
             # Invalid - no method
-            ("""Host:example.com
+            (
+                """Host:example.com
 Content-Type: text/calendar  ; charset =  "utf-8"
 Originator:  mailto:user01@example.com
 Recipient:  mailto:user02@example.com  ,\t mailto:user03@example.com\t\t
@@ -413,11 +446,12 @@ DKIM-Signature:  v=1;\t\t d=example.com; s = dkim; t\t=\t1234; a=rsa-sha1; \t\tq
 Cache-Control:no-cache
 Connection:close
 """,
-            [DKIMUtils.extractTags("v=DKIM1; p=%s" % (self.public_key_data,))],
-            False,
+                [DKIMUtils.extractTags("v=DKIM1; p=%s" % (self.public_key_data,))],
+                False,
             ),
             # Invalid - wrong algorithm
-            ("""Host:example.com
+            (
+                """Host:example.com
 Content-Type: text/calendar  ; charset =  "utf-8"
 Originator:  mailto:user01@example.com
 Recipient:  mailto:user02@example.com  ,\t mailto:user03@example.com\t\t
@@ -425,21 +459,22 @@ DKIM-Signature:  v=1;\t\t d=example.com; s = dkim; t\t=\t1234; a=rsa-sha1; \t\tq
 Cache-Control:no-cache
 Connection:close
 """,
-            [DKIMUtils.extractTags("v=DKIM1; h=sha-1; p=%s" % (self.public_key_data,))],
-            False,
+                [DKIMUtils.extractTags("v=DKIM1; h=sha-1; p=%s" % (self.public_key_data,))],
+                False,
             ),
         )
 
         for hdrs, keys, result in data:
             headers = [hdr.split(":", 1) for hdr in hdrs.splitlines()]
             TestPublicKeyLookup.PublicKeyLookup_Testing.keys = keys
+            TestPublicKeyLookup.PublicKeyLookup_Testing.flushCache()
             verifier = DKIMVerifier(self._makeHeaders(headers), "", key_lookup=(TestPublicKeyLookup.PublicKeyLookup_Testing,))
             verifier.processDKIMHeader()
             pkey = (yield verifier.locatePublicKey())
             if result:
-                self.assertNotEqual(pkey, None)
+                self.assertTrue(pkey is not None)
             else:
-                self.assertEqual(pkey, None)
+                self.assertTrue(pkey is None)
 
 
     @inlineCallbacks
@@ -467,7 +502,7 @@ Connection:close
                 TestPublicKeyLookup.PublicKeyLookup_Testing.keys = keys
                 data = (yield allDataFromStream(request.stream))
                 verifier = DKIMVerifier(request.headers, data, key_lookup=(TestPublicKeyLookup.PublicKeyLookup_Testing,))
-                TestPublicKeyLookup.PublicKeyLookup_Testing({}).flushCache()
+                TestPublicKeyLookup.PublicKeyLookup_Testing.flushCache()
                 try:
                     yield verifier.verify()
                 except Exception, e:
