@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2012-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2012-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,8 +27,7 @@ class Test_IScheduleServersParser(unittest.TestCase):
     def test_readXML(self):
 
         fp = FilePath(self.mktemp())
-        fp.open("w").write(
-"""<?xml version="1.0" encoding="utf-8"?>
+        fp.open("w").write("""<?xml version="1.0" encoding="utf-8"?>
 <!DOCTYPE servers SYSTEM "servertoserver.dtd">
 <servers>
   <server>
@@ -43,8 +42,7 @@ class Test_IScheduleServersParser(unittest.TestCase):
     </hosts>
   </server>
 </servers>
-"""
-)
+""")
 
         parser = IScheduleServersParser(fp)
         self.assertEqual(len(parser.servers), 1)
