@@ -1,5 +1,5 @@
 
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,11 +19,11 @@ from twisted.python.failure import Failure
 
 from twext.enterprise.locking import NamedLock
 from twext.python.log import Logger
-from twext.web2 import responsecode
-from twext.web2.http import HTTPError, Response
-from twext.web2.http_headers import MimeType
+from txweb2 import responsecode
+from txweb2.http import HTTPError, Response
+from txweb2.http_headers import MimeType
 from txdav.xml import element as davxml
-from twext.web2.dav.http import messageForFailure, statusForFailure, \
+from txweb2.dav.http import messageForFailure, statusForFailure, \
     ErrorResponse
 
 from twistedcaldav import caldavxml
@@ -33,16 +33,19 @@ from twistedcaldav.config import config
 from twistedcaldav.ical import Component
 from txdav.caldav.datastore.scheduling import addressmapping
 from txdav.caldav.datastore.scheduling.caldav.delivery import ScheduleViaCalDAV
+from txdav.caldav.datastore.scheduling.cuaddress import EmailCalendarUser
 from txdav.caldav.datastore.scheduling.cuaddress import InvalidCalendarUser, \
-    calendarUserFromPrincipal, OtherServerCalendarUser
+    OtherServerCalendarUser, calendarUserFromCalendarUserAddress
 from txdav.caldav.datastore.scheduling.cuaddress import LocalCalendarUser
 from txdav.caldav.datastore.scheduling.cuaddress import RemoteCalendarUser
-from txdav.caldav.datastore.scheduling.cuaddress import EmailCalendarUser
-from txdav.caldav.datastore.scheduling.cuaddress import PartitionedCalendarUser
 from txdav.caldav.datastore.scheduling.imip.delivery import ScheduleViaIMip
 from txdav.caldav.datastore.scheduling.ischedule.delivery import ScheduleViaISchedule
 from txdav.caldav.datastore.scheduling.itip import iTIPRequestStatus
+
+from pycalendar.period import Period
+
 import hashlib
+from collections import namedtuple
 
 """
 CalDAV/Server-to-Server scheduling behavior.
@@ -50,8 +53,8 @@ CalDAV/Server-to-Server scheduling behavior.
 This module handles the delivery of scheduling messages to organizer and attendees. The basic idea is to first
 confirm the integrity of the incoming scheduling message, check authorization. Appropriate L{DeliveryService}s
 are then used to deliver the message to attendees or organizer. Delivery responses are processed and returned.
-This takes into account partitioning and podding of users by detecting the appropriate host for a calendar
-user and then dispatching the delivery accordingly.
+This takes into account podding of users by detecting the appropriate host for a calendar user and then
+dispatching the delivery accordingly.
 
 The L{Scheduler} class defines the basic behavior for processing deliveries. Sub-classes are defined for the
 different ways a deliver can be triggered.
@@ -161,7 +164,7 @@ class Scheduler(object):
         """
 
         self.calendar = calendar
-        self.preProcessCalendarData()
+        yield self.preProcessCalendarData()
 
         if self.logItems is not None:
             self.logItems["recipients"] = len(recipients)
@@ -171,7 +174,7 @@ class Scheduler(object):
         # of data for all events with the same UID. So detect this and use a lock
         if calendar.resourceType() != "VFREEBUSY":
             uid = calendar.resourceUID()
-            yield NamedLock.acquire(self.txn, "ImplicitUIDLock:%s" % (hashlib.md5(uid).hexdigest(),))
+            yield NamedLock.acquire(self.txn, "ImplicitUIDLock:{}".format(hashlib.md5(uid).hexdigest(),))
 
         result = (yield self.doSchedulingDirectly("POST", originator, recipients, calendar))
         returnValue(result)
@@ -275,7 +278,12 @@ class Scheduler(object):
             try:
                 self.calendar.validCalendarData()
             except ValueError, e:
-                log.error("%s request calendar component is not valid:%s %s" % (self.method, e, self.calendar,))
+                log.error(
+                    "{method} request calendar component is not valid:{exc} {cal}",
+                    method=self.method,
+                    exc=e,
+                    cal=self.calendar,
+                )
                 raise HTTPError(self.errorResponse(
                     responsecode.FORBIDDEN,
                     self.errorElements["invalid-calendar-data"],
@@ -284,7 +292,11 @@ class Scheduler(object):
 
             # Must have a METHOD
             if not self.calendar.isValidMethod():
-                log.error("%s request must have valid METHOD property in calendar component: %s" % (self.method, self.calendar,))
+                log.error(
+                    "{method} request must have valid METHOD property in calendar component: {cal}",
+                    method=self.method,
+                    cal=self.calendar,
+                )
                 raise HTTPError(self.errorResponse(
                     responsecode.FORBIDDEN,
                     self.errorElements["invalid-scheduling-message"],
@@ -293,7 +305,11 @@ class Scheduler(object):
 
             # Verify iTIP behavior
             if not self.calendar.isValidITIP():
-                log.error("%s request must have a calendar component that satisfies iTIP requirements: %s" % (self.method, self.calendar,))
+                log.error(
+                    "{method} request must have a calendar component that satisfies iTIP requirements: {cal}",
+                    method=self.method,
+                    cal=self.calendar,
+                )
                 raise HTTPError(self.errorResponse(
                     responsecode.FORBIDDEN,
                     self.errorElements["invalid-scheduling-message"],
@@ -302,7 +318,11 @@ class Scheduler(object):
 
             # X-CALENDARSERVER-ACCESS is not allowed in Outbox POSTs
             if self.calendar.hasProperty(Component.ACCESS_PROPERTY):
-                log.error("X-CALENDARSERVER-ACCESS not allowed in a calendar component %s request: %s" % (self.method, self.calendar,))
+                log.error(
+                    "X-CALENDARSERVER-ACCESS not allowed in a calendar component {method} request: {cal}",
+                    method=self.method,
+                    cal=self.calendar,
+                )
                 raise HTTPError(self.errorResponse(
                     responsecode.FORBIDDEN,
                     (calendarserver_namespace, "no-access-restrictions"),
@@ -321,7 +341,10 @@ class Scheduler(object):
 
             # Must have only one
             if len(attendees) != 1:
-                log.error("Wrong number of ATTENDEEs in calendar data: %s" % (str(self.calendar),))
+                log.error(
+                    "Wrong number of ATTENDEEs in calendar data: {cal}",
+                    cal=str(self.calendar),
+                )
                 raise HTTPError(self.errorResponse(
                     responsecode.FORBIDDEN,
                     self.errorElements["invalid-scheduling-message"],
@@ -330,7 +353,7 @@ class Scheduler(object):
             self.attendee = attendees[0]
 
         else:
-            msg = "Unknown iTIP METHOD: %s" % (self.calendar.propertyValue("METHOD"),)
+            msg = "Unknown iTIP METHOD: {}".format(self.calendar.propertyValue("METHOD"),)
             log.error(msg)
             raise HTTPError(self.errorResponse(
                 responsecode.FORBIDDEN,
@@ -345,7 +368,10 @@ class Scheduler(object):
                 # Extract time range from VFREEBUSY object
                 vfreebusies = [v for v in self.calendar.subcomponents() if v.name() == "VFREEBUSY"]
                 if len(vfreebusies) != 1:
-                    log.error("iTIP data is not valid for a VFREEBUSY request: %s" % (self.calendar,))
+                    log.error(
+                        "iTIP data is not valid for a VFREEBUSY request: {cal}",
+                        cal=str(self.calendar),
+                    )
                     raise HTTPError(self.errorResponse(
                         responsecode.FORBIDDEN,
                         self.errorElements["invalid-scheduling-message"],
@@ -354,7 +380,10 @@ class Scheduler(object):
                 dtstart = vfreebusies[0].getStartDateUTC()
                 dtend = vfreebusies[0].getEndDateUTC()
                 if dtstart is None or dtend is None:
-                    log.error("VFREEBUSY start/end not valid: %s" % (self.calendar,))
+                    log.error(
+                        "VFREEBUSY start/end not valid: {cal}",
+                        cal=str(self.calendar),
+                    )
                     raise HTTPError(self.errorResponse(
                         responsecode.FORBIDDEN,
                         self.errorElements["invalid-scheduling-message"],
@@ -363,16 +392,17 @@ class Scheduler(object):
 
                 # Some clients send floating instead of UTC - coerce to UTC
                 if not dtstart.utc() or not dtend.utc():
-                    log.error("VFREEBUSY start or end not UTC: %s" % (self.calendar,))
+                    log.error(
+                        "VFREEBUSY start or end not UTC: {cal}",
+                        cal=self.calendar,
+                    )
                     raise HTTPError(self.errorResponse(
                         responsecode.FORBIDDEN,
                         self.errorElements["invalid-scheduling-message"],
                         "VFREEBUSY start or end not UTC",
                     ))
 
-                self.timeRange = caldavxml.TimeRange(start=dtstart.getText(), end=dtend.getText())
-                self.timeRange.start = dtstart
-                self.timeRange.end = dtend
+                self.timeRange = Period(dtstart, dtend)
 
                 # Look for masked UID
                 self.excludeUID = self.calendar.getMaskUID()
@@ -401,15 +431,15 @@ class Scheduler(object):
         #
         if isinstance(self.organizer, LocalCalendarUser):
             accountingType = "iTIP-VFREEBUSY" if self.calendar.mainType() == "VFREEBUSY" else "iTIP"
-            if accountingEnabled(accountingType, self.organizer.principal):
+            if accountingEnabled(accountingType, self.organizer.record):
                 emitAccounting(
-                    accountingType, self.organizer.principal,
-                    "Originator: %s\nRecipients:\n%sMethod:%s\n\n%s"
-                    % (
-                        str(self.originator),
-                        str("".join(["    %s\n" % (recipient,) for recipient in self.recipients])),
-                        str(self.method),
-                        str(self.calendar),
+                    accountingType,
+                    self.organizer.record,
+                    "Originator: {o}\nRecipients:\n{r}Method:{method}\n\n{cal}".format(
+                        o=str(self.originator),
+                        r=str("".join(["    {}\n".format(recipient,) for recipient in self.recipients])),
+                        method=str(self.method),
+                        cal=str(self.calendar),
                     )
                 )
 
@@ -424,7 +454,11 @@ class Scheduler(object):
     @inlineCallbacks
     def generateSchedulingResponse(self):
 
-        log.info("METHOD: %s, Component: %s" % (self.calendar.propertyValue("METHOD"), self.calendar.mainType(),))
+        log.info(
+            "METHOD: {method}, Component: {comp}",
+            method=self.calendar.propertyValue("METHOD"),
+            comp=self.calendar.mainType(),
+        )
 
         # For free-busy do immediate determination of iTIP result rather than fan-out
         freebusy = self.checkForFreeBusy()
@@ -434,7 +468,6 @@ class Scheduler(object):
 
         # Loop over each recipient and aggregate into lists by service types.
         caldav_recipients = []
-        partitioned_recipients = []
         otherserver_recipients = []
         remote_recipients = []
         imip_recipients = []
@@ -456,9 +489,6 @@ class Scheduler(object):
             elif isinstance(recipient, LocalCalendarUser):
                 caldav_recipients.append(recipient)
 
-            elif isinstance(recipient, PartitionedCalendarUser):
-                partitioned_recipients.append(recipient)
-
             elif isinstance(recipient, OtherServerCalendarUser):
                 otherserver_recipients.append(recipient)
 
@@ -480,16 +510,12 @@ class Scheduler(object):
         if caldav_recipients:
             yield self.generateLocalSchedulingResponses(caldav_recipients, responses, freebusy)
 
-        # Now process partitioned recipients
-        if partitioned_recipients:
-            yield self.generateRemoteSchedulingResponses(partitioned_recipients, responses, freebusy, getattr(self.txn, 'doing_attendee_refresh', False))
-
         # Now process other server recipients
         if otherserver_recipients:
             yield self.generateRemoteSchedulingResponses(otherserver_recipients, responses, freebusy, getattr(self.txn, 'doing_attendee_refresh', False))
 
         # To reduce chatter, we suppress certain messages
-        if not self.suppress_refresh:
+        if not self.suppress_refresh or self.calendar.mainType() == "VPOLL":
 
             # Now process remote recipients
             if remote_recipients:
@@ -556,34 +582,44 @@ class RemoteScheduler(Scheduler):
 
         results = []
         for recipient in self.recipients:
-            # Get the principal resource for this recipient
-            principal = self.txn.directoryService().recordWithCalendarUserAddress(recipient)
+            # Get the calendar user object for this recipient
+            recipientAddress = yield calendarUserFromCalendarUserAddress(recipient, self.txn)
 
-            # If no principal we may have a remote recipient but we should check whether
+            # If no calendar user we may have a remote recipient but we should check whether
             # the address is one that ought to be on our server and treat that as a missing
             # user. Also if server-to-server is not enabled then remote addresses are not allowed.
-            if principal is None:
+            if not recipientAddress.hosted():
                 localUser = (yield addressmapping.mapper.isCalendarUserInMyDomain(recipient))
                 if localUser:
-                    log.error("No principal for calendar user address: %s" % (recipient,))
+                    log.error(
+                        "No record for calendar user address: {r}",
+                        r=recipient,
+                    )
                 else:
-                    log.error("Unknown calendar user address: %s" % (recipient,))
+                    log.error(
+                        "Unknown calendar user address: {r}",
+                        r=recipient,
+                    )
                 results.append(InvalidCalendarUser(recipient))
             else:
-                # Map recipient to their inbox
+                # Map recipient to their inbox and cache on calendar user object
                 inbox = None
-                if principal.calendarsEnabled() and principal.thisServer():
-                    if principal.locallyHosted():
-                        recipient_home = yield self.txn.calendarHomeWithUID(principal.uid, create=True)
+                if recipientAddress.validRecipient():
+                    if isinstance(recipientAddress, LocalCalendarUser):
+                        recipient_home = yield self.txn.calendarHomeWithUID(recipientAddress.record.uid, create=True)
                         if recipient_home:
                             inbox = (yield recipient_home.calendarWithName("inbox"))
                     else:
                         inbox = "dummy"
+                    recipientAddress.inbox = inbox
 
                 if inbox:
-                    results.append(calendarUserFromPrincipal(recipient, principal, inbox))
+                    results.append(recipientAddress)
                 else:
-                    log.error("No schedule inbox for principal: %s" % (principal,))
+                    log.error(
+                        "No scheduling for calendar user: {r}",
+                        r=recipient,
+                    )
                     results.append(InvalidCalendarUser(recipient))
 
         self.recipients = results
@@ -663,6 +699,11 @@ class ScheduleResponseQueue (object):
     response_description_element = davxml.ResponseDescription
     calendar_data_element = caldavxml.CalendarData
 
+    ScheduleResonseDetails = namedtuple(
+        "ScheduleResonseDetails",
+        ["recipient", "reqstatus", "calendar", "error", "message", ]
+    )
+
     def __init__(self, method, success_response, recipient_mapper=None):
         """
         @param method: the name of the method generating the queue.
@@ -704,24 +745,27 @@ class ScheduleResponseQueue (object):
             error = self.errorForFailure(what)
             message = messageForFailure(what)
         else:
-            raise AssertionError("Unknown data type: %r" % (what,))
+            raise AssertionError("Unknown data type: {}".format(what,))
 
         if self.recipient_mapper is not None:
             recipient = self.recipient_mapper(recipient)
 
         if not suppressErrorLog and code > 400: # Error codes only
-            self.log.error("Error during %s for %s: %s" % (self.method, recipient, message))
+            self.log.error(
+                "Error during {method} for {r}: {msg}",
+                method=self.method,
+                r=recipient,
+                msg=message,
+            )
 
-        children = []
-        children.append(self.recipient_element(davxml.HRef.fromString(recipient)) if self.recipient_uses_href else self.recipient_element.fromString(recipient))
-        children.append(self.request_status_element(reqstatus))
-        if calendar is not None:
-            children.append(self.calendar_data_element.fromCalendar(calendar))
-        if error is not None:
-            children.append(error)
-        if message is not None:
-            children.append(self.response_description_element(message))
-        self.responses.append(self.response_element(*children))
+        details = ScheduleResponseQueue.ScheduleResonseDetails(
+            self.recipient_element(davxml.HRef.fromString(recipient)) if self.recipient_uses_href else self.recipient_element.fromString(recipient),
+            self.request_status_element(reqstatus),
+            calendar,
+            error,
+            self.response_description_element(message) if message is not None else None,
+        )
+        self.responses.append(details)
 
 
     def errorForFailure(self, failure):
@@ -737,19 +781,17 @@ class ScheduleResponseQueue (object):
         @param clone: the response to clone.
         """
 
-        children = []
-        children.append(self.recipient_element(davxml.HRef.fromString(recipient)) if self.recipient_uses_href else self.recipient_element.fromString(recipient))
-        children.append(self.request_status_element.fromString(request_status))
-        if calendar_data is not None:
-            children.append(self.calendar_data_element.fromCalendar(calendar_data))
-        if error is not None:
-            children.append(self.error_element(*error))
-        if desc is not None:
-            children.append(self.response_description_element.fromString(desc))
-        self.responses.append(self.response_element(*children))
+        details = ScheduleResponseQueue.ScheduleResonseDetails(
+            self.recipient_element(davxml.HRef.fromString(recipient)) if self.recipient_uses_href else self.recipient_element.fromString(recipient),
+            self.request_status_element.fromString(request_status),
+            calendar_data,
+            self.error_element(*error) if error is not None else None,
+            self.response_description_element.fromString(desc) if desc is not None else None,
+        )
+        self.responses.append(details)
 
 
-    def response(self):
+    def response(self, format=None):
         """
         Generate a L{ScheduleResponseResponse} with the responses contained in the
         queue or, if no such responses, return the C{success_response} provided
@@ -757,6 +799,20 @@ class ScheduleResponseQueue (object):
         @return: the response.
         """
         if self.responses:
-            return ScheduleResponseResponse(self.schedule_response_element, self.responses, self.location)
+            # Convert our queue to all XML elements
+            xml_responses = []
+            for response in self.responses:
+                children = []
+                children.append(response.recipient)
+                children.append(response.reqstatus)
+                if response.calendar is not None:
+                    children.append(self.calendar_data_element.fromCalendar(response.calendar, format))
+                if response.error is not None:
+                    children.append(response.error)
+                if response.message is not None:
+                    children.append(response.message)
+                xml_responses.append(self.response_element(*children))
+
+            return ScheduleResponseResponse(self.schedule_response_element, xml_responses, self.location)
         else:
             return self.success_response
