@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2013-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2013-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # limitations under the License.
 ##
 
-from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.datetime import DateTime
 from twisted.trial import unittest
 from twistedcaldav.stdconfig import config
 from twistedcaldav.ical import Component
@@ -31,10 +31,12 @@ class ICalSplitter (unittest.TestCase):
 
         self.subs = {}
 
-        self.now = PyCalendarDateTime.getNowUTC()
+        self.now = DateTime.getNowUTC()
         self.now.setHHMMSS(0, 0, 0)
 
         self.subs["now"] = self.now
+        self.subs["now_1"] = self.now.duplicate()
+        self.subs["now_1"].offsetSeconds(-1)
 
         for i in range(30):
             attrname = "now_back%s" % (i + 1,)
@@ -68,6 +70,10 @@ class ICalSplitter (unittest.TestCase):
             getattr(self, attrname_1).offsetSeconds(-1)
             self.subs[attrname_1] = getattr(self, attrname_1)
 
+        setattr(self, "now_future", self.now.duplicate())
+        getattr(self, "now_future").offsetYear(2)
+        self.subs["now_future"] = getattr(self, "now_future")
+
         self.patch(config, "MaxAllowedInstances", 500)
 
 
@@ -89,7 +95,7 @@ ATTENDEE:mailto:user2@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
             ),
             (
                 "#1.2 Large, old, non-recurring component",
@@ -129,7 +135,7 @@ ATTENDEE:mailto:user25@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
             ),
             (
                 "#2.1 Small, old, simple recurring component",
@@ -147,7 +153,7 @@ RRULE:FREQ=DAILY
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
             ),
             (
                 "#2.2 Large, old, simple recurring component",
@@ -188,7 +194,45 @@ RRULE:FREQ=DAILY
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
+            ),
+            (
+                "#2.3 Small, more than a year in the future, simple recurring component",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890
+DTSTART:%(now_future)s
+DURATION:PT1H
+ORGANIZER:mailto:user1@example.com
+ATTENDEE:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+RRULE:FREQ=DAILY
+END:VEVENT
+END:VCALENDAR
+""",
+                (False, True),
+            ),
+            (
+                "#2.4 Small, all cancelled, simple recurring component",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890
+DTSTART:%(now_back30)s
+DURATION:PT1H
+ORGANIZER:mailto:user1@example.com
+ATTENDEE:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+EXDATE:%(now_back30)s
+EXDATE:%(now_back29)s
+RRULE:FREQ=DAILY;COUNT=2
+END:VEVENT
+END:VCALENDAR
+""",
+                (False, True),
             ),
             (
                 "#3.1 Small, old, recurring with future override",
@@ -215,7 +259,7 @@ ATTENDEE:mailto:user2@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
             ),
             (
                 "#3.2 Large, old, recurring component with future override",
@@ -265,7 +309,7 @@ ATTENDEE:mailto:user2@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
             ),
             (
                 "#4.1 Small, old, recurring with past override",
@@ -292,7 +336,7 @@ ATTENDEE:mailto:user2@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
             ),
             (
                 "#4.2 Large, old, recurring component with one past override",
@@ -342,7 +386,7 @@ ATTENDEE:mailto:user2@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
             ),
             (
                 "#4.2 Large, old, recurring component with two past overrides",
@@ -401,7 +445,7 @@ ATTENDEE:mailto:user2@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                True,
+                (True, False),
             ),
             (
                 "#5.1 Small, old, limited recurring with past override",
@@ -428,7 +472,7 @@ ATTENDEE:mailto:user2@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
             ),
             (
                 "#5.2 Large, old, limited recurring component with past override",
@@ -478,7 +522,7 @@ ATTENDEE:mailto:user2@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
             ),
             (
                 "#6.1 Small, old, limited future recurring with past override",
@@ -505,7 +549,7 @@ ATTENDEE:mailto:user2@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                False,
+                (False, False),
             ),
             (
                 "#6.2 Large, old, limited future recurring component with two past overrides",
@@ -564,7 +608,7 @@ ATTENDEE:mailto:user2@example.com
 END:VEVENT
 END:VCALENDAR
 """,
-                True,
+                (True, False),
             ),
         )
 
@@ -572,8 +616,8 @@ END:VCALENDAR
             ical = Component.fromString(calendar % self.subs)
 
             splitter = iCalSplitter(1024, 14)
-            will_split = splitter.willSplit(ical)
-            self.assertEqual(will_split, result, msg="Failed: %s" % (description,))
+            willSplitResult = splitter.willSplit(ical)
+            self.assertEqual(willSplitResult, result, msg="Failed: %s" % (description,))
 
 
     def test_split(self):
@@ -678,7 +722,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
 ATTENDEE:mailto:user1@example.com
@@ -711,7 +755,7 @@ RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 RRULE:FREQ=DAILY;UNTIL=%(now_back14_1)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back25)s
 DTSTART:%(now_back25)s
 DURATION:PT1H
@@ -721,7 +765,7 @@ ORGANIZER:mailto:user1@example.com
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back24)s
 DTSTART:%(now_back24)s
 DURATION:PT1H
@@ -842,7 +886,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
 ATTENDEE:mailto:user1@example.com
@@ -877,7 +921,7 @@ RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 RRULE:FREQ=DAILY;UNTIL=%(now_back14_1)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back25)s
 DTSTART:%(now_back25)s
 DURATION:PT1H
@@ -887,7 +931,7 @@ ORGANIZER:mailto:user1@example.com
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back24)s
 DTSTART:%(now_back24)s
 DURATION:PT1H
@@ -999,7 +1043,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
 ATTENDEE:mailto:user1@example.com
@@ -1033,7 +1077,7 @@ RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 RRULE:FREQ=DAILY;UNTIL=%(now_back13_1)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back25)s
 DTSTART:%(now_back25)s
 DURATION:PT1H
@@ -1043,7 +1087,7 @@ ORGANIZER:mailto:user1@example.com
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back24)s
 DTSTART:%(now_back24)s
 DURATION:PT1H
@@ -1178,7 +1222,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
 ATTENDEE:mailto:user1@example.com
@@ -1212,7 +1256,7 @@ RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 RRULE:FREQ=DAILY;UNTIL=%(now_back14_1)s;INTERVAL=10
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back30)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
@@ -1222,7 +1266,7 @@ ORGANIZER:mailto:user1@example.com
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back20)s
 DTSTART:%(now_back20)s
 DURATION:PT1H
@@ -1379,7 +1423,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
 ATTENDEE:mailto:user1@example.com
@@ -1412,7 +1456,7 @@ RDATE:%(now_back15)s
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back30)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
@@ -1422,7 +1466,7 @@ ORGANIZER:mailto:user1@example.com
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back15)s
 DTSTART:%(now_back15)s
 DURATION:PT1H
@@ -1558,7 +1602,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
 ATTENDEE:mailto:user1@example.com
@@ -1591,7 +1635,7 @@ RDATE:%(now_back15)s
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back30)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
@@ -1601,7 +1645,7 @@ ORGANIZER:mailto:user1@example.com
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back15)s
 DTSTART:%(now_back15)s
 DURATION:PT1H
@@ -1763,7 +1807,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
 ATTENDEE:mailto:user1@example.com
@@ -1796,7 +1840,7 @@ RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 RRULE:FREQ=DAILY;UNTIL=%(now_back14_1)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back25)s
 DTSTART:%(now_back25)s
 DURATION:PT1H
@@ -1806,7 +1850,7 @@ ORGANIZER:mailto:user1@example.com
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back24)s
 DTSTART:%(now_back24)s
 DURATION:PT1H
@@ -1816,7 +1860,7 @@ ORGANIZER:mailto:user1@example.com
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:X-CALENDARSERVER-PERUSER
-UID:%(relID)s
+UID:%(uid)s
 X-CALENDARSERVER-PERUSER-UID:user01
 BEGIN:X-CALENDARSERVER-PERINSTANCE
 TRANSP:OPAQUE
@@ -1938,7 +1982,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 DTSTART:%(now_back30)s
 DURATION:PT1H
 ATTENDEE:mailto:user1@example.com
@@ -1971,7 +2015,7 @@ RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 RRULE:FREQ=DAILY;UNTIL=%(now_back14_1)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back25)s
 DTSTART:%(now_back25)s
 DURATION:PT1H
@@ -1981,7 +2025,7 @@ ORGANIZER:mailto:user1@example.com
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back24)s
 DTSTART:%(now_back24)s
 DURATION:PT1H
@@ -2037,7 +2081,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back25)s
 DTSTART:%(now_back25)s
 DURATION:PT1H
@@ -2083,7 +2127,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back25)s
 DTSTART:%(now_back25)s
 DURATION:PT1H
@@ -2093,7 +2137,7 @@ ORGANIZER:mailto:user1@example.com
 RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_back24)s
 DTSTART:%(now_back24)s
 DURATION:PT1H
@@ -2167,16 +2211,120 @@ END:VCALENDAR
             ical = Component.fromString(calendar % self.subs)
             splitter = iCalSplitter(1024, 14)
             if title[0] == "1":
-                self.assertTrue(splitter.willSplit(ical), "Failed will split: %s" % (title,))
+                self.assertTrue(splitter.willSplit(ical)[0], "Failed will split: %s" % (title,))
             icalOld, icalNew = splitter.split(ical)
             relsubs = dict(self.subs)
-            relsubs["relID"] = icalOld.resourceUID()
+            relsubs["uid"] = icalOld.resourceUID()
+            relsubs["relID"] = icalOld.mainComponent().propertyValue("RELATED-TO") if icalOld.mainComponent() else icalNew.mainComponent().propertyValue("RELATED-TO")
             self.assertEqual(str(icalNew).replace("\r\n ", ""), split_future.replace("\n", "\r\n") % relsubs, "Failed future: %s" % (title,))
             self.assertEqual(str(icalOld).replace("\r\n ", ""), split_past.replace("\n", "\r\n") % relsubs, "Failed past: %s" % (title,))
 
             # Make sure new items won't split again
-            self.assertFalse(splitter.willSplit(icalNew), "Failed future will split: %s" % (title,))
-            self.assertFalse(splitter.willSplit(icalOld), "Failed past will split: %s" % (title,))
+            self.assertFalse(splitter.willSplit(icalNew)[0], "Failed future will split: %s" % (title,))
+            self.assertFalse(splitter.willSplit(icalOld)[0], "Failed past will split: %s" % (title,))
+
+
+    def test_split_negative_count(self):
+
+        data = (
+            (
+                "1.1 - RRULE with override",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890
+DTSTART:%(now_back30)s
+DURATION:PT1H
+ATTENDEE:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+ORGANIZER:mailto:user1@example.com
+RRULE:FREQ=WEEKLY;COUNT=100
+END:VEVENT
+BEGIN:VEVENT
+UID:12345-67890
+RECURRENCE-ID:%(now_back23)s
+DTSTART:%(now_back23)s
+DURATION:PT1H
+ATTENDEE:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+ORGANIZER:mailto:user1@example.com
+END:VEVENT
+BEGIN:VEVENT
+UID:12345-67890
+RECURRENCE-ID:%(now_back16)s
+DTSTART:%(now_back16)s
+DURATION:PT1H
+ATTENDEE:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+ORGANIZER:mailto:user1@example.com
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890
+DTSTART:%(now_fwd5)s
+DURATION:PT1H
+ATTENDEE:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+ORGANIZER:mailto:user1@example.com
+RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
+RRULE:FREQ=WEEKLY;COUNT=95
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:%(uid)s
+DTSTART:%(now_back30)s
+DURATION:PT1H
+ATTENDEE:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+ORGANIZER:mailto:user1@example.com
+RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
+RRULE:FREQ=WEEKLY;UNTIL=%(now_fwd5_1)s
+END:VEVENT
+BEGIN:VEVENT
+UID:%(uid)s
+RECURRENCE-ID:%(now_back23)s
+DTSTART:%(now_back23)s
+DURATION:PT1H
+ATTENDEE:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+ORGANIZER:mailto:user1@example.com
+RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
+END:VEVENT
+BEGIN:VEVENT
+UID:%(uid)s
+RECURRENCE-ID:%(now_back16)s
+DTSTART:%(now_back16)s
+DURATION:PT1H
+ATTENDEE:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+ORGANIZER:mailto:user1@example.com
+RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
+END:VEVENT
+END:VCALENDAR
+""",
+            ),
+        )
+
+        for title, calendar, split_future, split_past in data:
+            ical = Component.fromString(calendar % self.subs)
+            splitter = iCalSplitter(100, 0)
+            if title[0] == "1":
+                self.assertTrue(splitter.willSplit(ical)[0], "Failed will split: %s" % (title,))
+            icalOld, icalNew = splitter.split(ical)
+            relsubs = dict(self.subs)
+            relsubs["uid"] = icalOld.resourceUID()
+            relsubs["relID"] = icalOld.mainComponent().propertyValue("RELATED-TO") if icalOld.mainComponent() else icalNew.mainComponent().propertyValue("RELATED-TO")
+            self.assertEqual(str(icalNew).replace("\r\n ", ""), split_future.replace("\n", "\r\n") % relsubs, "Failed future: %s" % (title,))
+            self.assertEqual(str(icalOld).replace("\r\n ", ""), split_past.replace("\n", "\r\n") % relsubs, "Failed past: %s" % (title,))
 
 
     def test_future_split(self):
@@ -2291,7 +2439,7 @@ END:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 DTSTART:%(now)s
 DURATION:PT1H
 ATTENDEE:mailto:user1@example.com
@@ -2324,7 +2472,7 @@ RELATED-TO;RELTYPE=X-CALENDARSERVER-RECURRENCE-SET:%(relID)s
 RRULE:FREQ=DAILY;UNTIL=%(now_fwd11_1)s
 END:VEVENT
 BEGIN:VEVENT
-UID:%(relID)s
+UID:%(uid)s
 RECURRENCE-ID:%(now_fwd10)s
 DTSTART:%(now_fwd10)s
 DURATION:PT1H
@@ -2341,24 +2489,26 @@ END:VCALENDAR
         for title, calendar, split_future, split_past in data:
             ical = Component.fromString(calendar % self.subs)
             splitter = iCalSplitter(1024, 14)
-            icalOld, icalNew = splitter.split(ical, rid=PyCalendarDateTime.parseText("%(now_fwd11)s" % self.subs))
+            icalOld, icalNew = splitter.split(ical, rid=DateTime.parseText("%(now_fwd11)s" % self.subs))
             relsubs = dict(self.subs)
-            relsubs["relID"] = icalOld.resourceUID()
+            relsubs["uid"] = icalOld.resourceUID()
+            relsubs["relID"] = icalOld.mainComponent().propertyValue("RELATED-TO") if icalOld.mainComponent() else icalNew.mainComponent().propertyValue("RELATED-TO")
             self.assertEqual(str(icalNew).replace("\r\n ", ""), split_future.replace("\n", "\r\n") % relsubs, "Failed future: %s" % (title,))
             self.assertEqual(str(icalOld).replace("\r\n ", ""), split_past.replace("\n", "\r\n") % relsubs, "Failed past: %s" % (title,))
 
             # Make sure new items won't split again
-            self.assertFalse(splitter.willSplit(icalNew), "Failed future will split: %s" % (title,))
-            self.assertFalse(splitter.willSplit(icalOld), "Failed past will split: %s" % (title,))
+            self.assertFalse(splitter.willSplit(icalNew)[0], "Failed future will split: %s" % (title,))
+            self.assertFalse(splitter.willSplit(icalOld)[0], "Failed past will split: %s" % (title,))
 
             ical = Component.fromString(calendar % self.subs)
             splitter = iCalSplitter(1024, 14)
-            icalOld, icalNew = splitter.split(ical, rid=splitter.whereSplit(ical, break_point=PyCalendarDateTime.parseText("%(now_fwd10_12h)s" % self.subs)))
+            icalOld, icalNew = splitter.split(ical, rid=splitter.whereSplit(ical, break_point=DateTime.parseText("%(now_fwd10_12h)s" % self.subs)))
             relsubs = dict(self.subs)
-            relsubs["relID"] = icalOld.resourceUID()
+            relsubs["uid"] = icalOld.resourceUID()
+            relsubs["relID"] = icalOld.mainComponent().propertyValue("RELATED-TO") if icalOld.mainComponent() else icalNew.mainComponent().propertyValue("RELATED-TO")
             self.assertEqual(str(icalNew).replace("\r\n ", ""), split_future.replace("\n", "\r\n") % relsubs, "Failed future: %s" % (title,))
             self.assertEqual(str(icalOld).replace("\r\n ", ""), split_past.replace("\n", "\r\n") % relsubs, "Failed past: %s" % (title,))
 
             # Make sure new items won't split again
-            self.assertFalse(splitter.willSplit(icalNew), "Failed future will split: %s" % (title,))
-            self.assertFalse(splitter.willSplit(icalOld), "Failed past will split: %s" % (title,))
+            self.assertFalse(splitter.willSplit(icalNew)[0], "Failed future will split: %s" % (title,))
+            self.assertFalse(splitter.willSplit(icalOld)[0], "Failed past will split: %s" % (title,))
