@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.caldav.datastore -*-
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ from twisted.internet.protocol import Protocol
 from twisted.python import hashlib
 
 from twext.python.clsprop import classproperty
-from twext.python.vcomponent import VComponent
+from twistedcaldav.ical import Component as VComponent
 from twext.python.filepath import CachingFilePath as FilePath
 from twext.enterprise.ienterprise import AlreadyFinishedError
 
@@ -35,26 +35,27 @@ from txdav.xml.element import WebDAVUnknownElement
 from txdav.idav import IPropertyStore, IDataStore
 from txdav.base.propertystore.base import PropertyName
 from txdav.common.icommondatastore import HomeChildNameAlreadyExistsError, \
-    ICommonTransaction, InvalidComponentForStoreError, InvalidUIDError
+    ICommonTransaction, InvalidComponentForStoreError, InvalidUIDError, \
+    ObjectResourceNameNotAllowedError
 from txdav.common.icommondatastore import InvalidObjectResourceError
 from txdav.common.icommondatastore import NoSuchHomeChildError
 from txdav.common.icommondatastore import ObjectResourceNameAlreadyExistsError
 from txdav.common.inotifications import INotificationObject
 from txdav.common.datastore.test.util import CommonCommonTests
-from txdav.common.datastore.sql_tables import _BIND_MODE_WRITE, _BIND_MODE_READ
 
 from txdav.caldav.icalendarstore import (
     ICalendarObject, ICalendarHome,
     ICalendar, ICalendarTransaction,
     ComponentUpdateState)
 
-from twistedcaldav.customxml import InviteNotification, InviteSummary
 from txdav.common.datastore.test.util import transactionClean
 from txdav.common.icommondatastore import ConcurrentModification
 from twistedcaldav.ical import Component
 from twistedcaldav.config import config
 from calendarserver.push.util import PushPriority
 
+import json
+
 
 storePath = FilePath(__file__).parent().child("calendar_store")
 
@@ -76,6 +77,7 @@ calendar1_objectNames = [
     "2.ics",
     "3.ics",
     "4.ics",
+    "5.ics",
 ]
 
 home1_calendarNames = [
@@ -240,51 +242,53 @@ class CommonTests(CommonCommonTests):
         metadata2 = cls.metadata2.copy()
         metadata3 = cls.metadata3.copy()
         metadata4 = cls.metadata4.copy()
+        metadata5 = cls.metadata5.copy()
         return {
-        "home1": {
-            "calendar_1": {
-                "1.ics": (cal1Root.child("1.ics").getContent(), metadata1),
-                "2.ics": (cal1Root.child("2.ics").getContent(), metadata2),
-                "3.ics": (cal1Root.child("3.ics").getContent(), metadata3),
-                "4.ics": (cal1Root.child("4.ics").getContent(), metadata4),
+            "home1": {
+                "calendar_1": {
+                    "1.ics": (cal1Root.child("1.ics").getContent(), metadata1),
+                    "2.ics": (cal1Root.child("2.ics").getContent(), metadata2),
+                    "3.ics": (cal1Root.child("3.ics").getContent(), metadata3),
+                    "4.ics": (cal1Root.child("4.ics").getContent(), metadata4),
+                    "5.ics": (cal1Root.child("5.ics").getContent(), metadata5),
+                },
+                "calendar_2": {},
+                "calendar_empty": {},
+                "not_a_calendar": None
             },
-            "calendar_2": {},
-            "calendar_empty": {},
-            "not_a_calendar": None
-        },
-        "not_a_home": None,
-        "home_splits": {
-            "calendar_1": {
-                "1.ics": (cal1SplitsRoot.child("1.ics").getContent(), metadata1),
-                "2.ics": (cal1SplitsRoot.child("2.ics").getContent(), metadata2),
-                "3.ics": (cal1SplitsRoot.child("3.ics").getContent(), metadata3),
+            "home_splits": {
+                "calendar_1": {
+                    "1.ics": (cal1SplitsRoot.child("1.ics").getContent(), metadata1),
+                    "2.ics": (cal1SplitsRoot.child("2.ics").getContent(), metadata2),
+                    "3.ics": (cal1SplitsRoot.child("3.ics").getContent(), metadata3),
+                },
+                "calendar_2": {
+                    "1.ics": (cal2SplitsRoot.child("1.ics").getContent(), metadata1),
+                    "2.ics": (cal2SplitsRoot.child("2.ics").getContent(), metadata2),
+                    "3.ics": (cal2SplitsRoot.child("3.ics").getContent(), metadata3),
+                    "4.ics": (cal2SplitsRoot.child("4.ics").getContent(), metadata4),
+                    "5.ics": (cal2SplitsRoot.child("5.ics").getContent(), metadata4),
+                },
             },
-            "calendar_2": {
-                "1.ics": (cal2SplitsRoot.child("1.ics").getContent(), metadata1),
-                "2.ics": (cal2SplitsRoot.child("2.ics").getContent(), metadata2),
-                "3.ics": (cal2SplitsRoot.child("3.ics").getContent(), metadata3),
-                "4.ics": (cal2SplitsRoot.child("4.ics").getContent(), metadata4),
-                "5.ics": (cal2SplitsRoot.child("5.ics").getContent(), metadata4),
+            "home_no_splits": {
+                "calendar_1": {
+                    "1.ics": (cal1NoSplitsRoot.child("1.ics").getContent(), metadata1),
+                    "2.ics": (cal1NoSplitsRoot.child("2.ics").getContent(), metadata2),
+                    "3.ics": (cal1NoSplitsRoot.child("3.ics").getContent(), metadata3),
+                },
             },
-        },
-        "home_no_splits": {
-            "calendar_1": {
-                "1.ics": (cal1NoSplitsRoot.child("1.ics").getContent(), metadata1),
-                "2.ics": (cal1NoSplitsRoot.child("2.ics").getContent(), metadata2),
-                "3.ics": (cal1NoSplitsRoot.child("3.ics").getContent(), metadata3),
+            "home_splits_shared": {
+                "calendar_1": {},
             },
-        },
-        "home_splits_shared": {
-            "calendar_1": {},
-        },
-        "home_defaults": {
-            "calendar_1": {
-                "1.ics": (cal1DefaultsRoot.child("1.ics").getContent(), metadata1),
-                "3.ics": (cal1DefaultsRoot.child("3.ics").getContent(), metadata3),
+            "home_defaults": {
+                "calendar_1": {
+                    "1.ics": (cal1DefaultsRoot.child("1.ics").getContent(), metadata1),
+                    "3.ics": (cal1DefaultsRoot.child("3.ics").getContent(), metadata3),
+                },
+                "inbox" : {},
             },
-            "inbox" : {},
-        },
-    }
+        }
+
     md5s = {
         "home1": {
             "calendar_1": {
@@ -297,7 +301,6 @@ class CommonTests(CommonCommonTests):
             "calendar_empty": {},
             "not_a_calendar": None
         },
-        "not_a_home": None,
         "home_splits": {
             "calendar_1": {
                 "1.ics": md5Values[0],
@@ -373,10 +376,12 @@ class CommonTests(CommonCommonTests):
     @inlineCallbacks
     def notificationUnderTest(self):
         txn = self.transactionUnderTest()
-        notifications = yield txn.notificationsWithUID("home1")
-        inviteNotification = InviteNotification()
-        yield notifications.writeNotificationObject("abc", inviteNotification,
-            inviteNotification.toxml())
+        notifications = yield txn.notificationsWithUID("home1", create=True)
+        yield notifications.writeNotificationObject(
+            "abc",
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+        )
         notificationObject = yield notifications.notificationObjectWithUID("abc")
         returnValue(notificationObject)
 
@@ -398,21 +403,30 @@ class CommonTests(CommonCommonTests):
         objects changed or deleted since
         """
         txn = self.transactionUnderTest()
-        coll = yield txn.notificationsWithUID("home1")
-        invite1 = InviteNotification()
-        yield coll.writeNotificationObject("1", invite1, invite1.toxml())
+        coll = yield txn.notificationsWithUID("home1", create=True)
+        yield coll.writeNotificationObject(
+            "1",
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+        )
         st = yield coll.syncToken()
-        yield coll.writeNotificationObject("2", invite1, invite1.toxml())
+        yield coll.writeNotificationObject(
+            "2",
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+        )
         rev = self.token2revision(st)
         yield coll.removeNotificationObjectWithUID("1")
         st2 = yield coll.syncToken()
         rev2 = self.token2revision(st2)
-        changed, deleted = yield coll.resourceNamesSinceToken(rev)
+        changed, deleted, invalid = yield coll.resourceNamesSinceToken(rev)
         self.assertEquals(set(changed), set(["2.xml"]))
         self.assertEquals(set(deleted), set(["1.xml"]))
-        changed, deleted = yield coll.resourceNamesSinceToken(rev2)
+        self.assertEquals(len(invalid), 0)
+        changed, deleted, invalid = yield coll.resourceNamesSinceToken(rev2)
         self.assertEquals(set(changed), set([]))
         self.assertEquals(set(deleted), set([]))
+        self.assertEquals(len(invalid), 0)
 
 
     @inlineCallbacks
@@ -422,16 +436,23 @@ class CommonTests(CommonCommonTests):
         overwrite the notification object.
         """
         notifications = yield self.transactionUnderTest().notificationsWithUID(
-            "home1"
+            "home1", create=True
         )
-        inviteNotification = InviteNotification()
-        yield notifications.writeNotificationObject("abc", inviteNotification,
-            inviteNotification.toxml())
-        inviteNotification2 = InviteNotification(InviteSummary("a summary"))
         yield notifications.writeNotificationObject(
-            "abc", inviteNotification, inviteNotification2.toxml())
+            "abc",
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+        )
+        yield notifications.writeNotificationObject(
+            "abc",
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+            json.loads("{\"notification-type\":\"invite-notification\",\"summary\":\"a summary\"}"),
+        )
         abc = yield notifications.notificationObjectWithUID("abc")
-        self.assertEquals((yield abc.xmldata()), inviteNotification2.toxml())
+        self.assertEquals(
+            (yield abc.notificationData()),
+            json.loads("{\"notification-type\":\"invite-notification\",\"summary\":\"a summary\"}"),
+        )
 
 
     @inlineCallbacks
@@ -442,7 +463,7 @@ class CommonTests(CommonCommonTests):
         """
         # Prime the home collection first
         yield self.transactionUnderTest().notificationsWithUID(
-            "home1"
+            "home1", create=True
         )
         yield self.commit()
 
@@ -450,9 +471,11 @@ class CommonTests(CommonCommonTests):
             "home1"
         )
         self.notifierFactory.reset()
-        inviteNotification = InviteNotification()
-        yield notifications.writeNotificationObject("abc", inviteNotification,
-            inviteNotification.toxml())
+        yield notifications.writeNotificationObject(
+            "abc",
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+        )
 
         # notify is called prior to commit
         self.assertEquals(
@@ -490,14 +513,18 @@ class CommonTests(CommonCommonTests):
         overwrite the notification object.
         """
         notifications = yield self.transactionUnderTest().notificationsWithUID(
-            "home1"
+            "home1", create=True
         )
-        inviteNotification = InviteNotification()
-        yield notifications.writeNotificationObject("abc", inviteNotification,
-            inviteNotification.toxml())
-        inviteNotification2 = InviteNotification(InviteSummary("a summary"))
         yield notifications.writeNotificationObject(
-            "def", inviteNotification, inviteNotification2.toxml())
+            "abc",
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+        )
+        yield notifications.writeNotificationObject(
+            "def",
+            json.loads("{\"notification-type\":\"invite-notification\"}"),
+            json.loads("{\"notification-type\":\"invite-notification\",\"summary\":\"a summary\"}"),
+        )
 
         yield self.commit()
 
@@ -529,7 +556,7 @@ class CommonTests(CommonCommonTests):
         L{INotificationCollection} that the object was retrieved from.
         """
         txn = self.transactionUnderTest()
-        collection = yield txn.notificationsWithUID("home1")
+        collection = yield txn.notificationsWithUID("home1", create=True)
         notification = yield self.notificationUnderTest()
         self.assertIdentical(collection, notification.notificationCollection())
 
@@ -730,6 +757,7 @@ class CommonTests(CommonCommonTests):
         exists.
         """
         home = yield self.homeUnderTest()
+        trash = yield home.getTrash(create=True)
 
         # FIXME: test transactions
         for name in home1_calendarNames:
@@ -740,14 +768,20 @@ class CommonTests(CommonCommonTests):
         yield self.commit()
 
         # Make sure notification fired after commit
+        expected = [
+            ("/CalDAV/example.com/home1/", PushPriority.high),
+            ("/CalDAV/example.com/home1/calendar_1/", PushPriority.high),
+            ("/CalDAV/example.com/home1/calendar_2/", PushPriority.high),
+            ("/CalDAV/example.com/home1/calendar_empty/", PushPriority.high),
+        ]
+        if trash:
+            expected.append(
+                ("/CalDAV/example.com/home1/{}/".format(trash.name()), PushPriority.high),
+            )
+
         self.assertEquals(
             set(self.notifierFactory.history),
-            set([
-                ("/CalDAV/example.com/home1/", PushPriority.high),
-                ("/CalDAV/example.com/home1/calendar_1/", PushPriority.high),
-                ("/CalDAV/example.com/home1/calendar_2/", PushPriority.high),
-                ("/CalDAV/example.com/home1/calendar_empty/", PushPriority.high),
-            ])
+            set(expected)
         )
 
 
@@ -850,7 +884,7 @@ class CommonTests(CommonCommonTests):
         ctxn = self.concurrentTransaction()
         calendar1prime = yield self.calendarUnderTest(ctxn)
         obj1 = yield calendar1prime.calendarObjectWithName("1.ics")
-        yield obj1.remove()
+        yield obj1.purge()
         yield ctxn.commit()
         try:
             retrieval = yield calendarObject.component()
@@ -901,6 +935,8 @@ class CommonTests(CommonCommonTests):
         """
         Remove an existing calendar object.
         """
+        home = yield self.homeUnderTest()
+        trash = yield home.getTrash(create=True)
         calendar = yield self.calendarUnderTest()
         for name in calendar1_objectNames:
             uid = (u'uid' + name.rstrip(".ics"))
@@ -916,13 +952,18 @@ class CommonTests(CommonCommonTests):
                 None
             )
 
+        expected = [
+            ("/CalDAV/example.com/home1/", PushPriority.high),
+            ("/CalDAV/example.com/home1/calendar_1/", PushPriority.high),
+        ]
+        if trash:
+            expected.append(
+                ("/CalDAV/example.com/home1/{}/".format(trash.name()), PushPriority.high),
+            )
         # notify is called prior to commit
         self.assertEquals(
             set(self.notifierFactory.history),
-            set([
-                ("/CalDAV/example.com/home1/", PushPriority.high),
-                ("/CalDAV/example.com/home1/calendar_1/", PushPriority.high),
-            ])
+            set(expected)
         )
         yield self.commit()
 
@@ -950,156 +991,6 @@ class CommonTests(CommonCommonTests):
         self.assertEquals((yield self.calendarUnderTest()).name(), "calendar_1")
 
 
-    @inlineCallbacks
-    def test_shareWith(self):
-        """
-        L{ICalendar.shareWith} will share a calendar with a given home UID.
-        """
-        cal = yield self.calendarUnderTest()
-        other = yield self.homeUnderTest(name=OTHER_HOME_UID)
-        newCalName = yield cal.shareWith(other, _BIND_MODE_WRITE)
-        self.sharedName = newCalName
-        yield self.commit()
-        normalCal = yield self.calendarUnderTest()
-        otherHome = yield self.homeUnderTest(name=OTHER_HOME_UID)
-        otherCal = yield otherHome.childWithName(newCalName)
-        self.assertNotIdentical(otherCal, None)
-        self.assertEqual(
-            (yield
-             (yield otherCal.calendarObjectWithName("1.ics")).component()),
-            (yield
-             (yield normalCal.calendarObjectWithName("1.ics")).component())
-        )
-
-
-    @inlineCallbacks
-    def test_shareAgainChangesMode(self):
-        """
-        If a calendar is already shared with a given calendar home,
-        L{ICalendar.shareWith} will change the sharing mode.
-        """
-        yield self.test_shareWith()
-        # yield self.commit() # txn is none? why?
-        cal = yield self.calendarUnderTest()
-        other = yield self.homeUnderTest(name=OTHER_HOME_UID)
-        newName = yield cal.shareWith(other, _BIND_MODE_READ)
-        otherCal = yield other.childWithName(self.sharedName)
-
-        # Name should not change just because we updated the mode.
-        self.assertEqual(newName, self.sharedName)
-        self.assertNotIdentical(otherCal, None)
-
-        invitedCals = yield cal.sharingInvites()
-        self.assertEqual(len(invitedCals), 1)
-        self.assertEqual(invitedCals[0].mode(), _BIND_MODE_READ)
-
-
-    @inlineCallbacks
-    def test_unshareWith(self, commit=False):
-        """
-        L{ICalendar.unshareWith} will remove a previously-shared calendar from
-        another user's calendar home.
-        """
-        yield self.test_shareWith()
-        if commit:
-            yield self.commit()
-        cal = yield self.calendarUnderTest()
-        other = yield self.homeUnderTest(name=OTHER_HOME_UID)
-        newName = yield cal.unshareWith(other)
-        otherCal = yield other.childWithName(newName)
-        self.assertIdentical(otherCal, None)
-        invitedCals = yield cal.sharingInvites()
-        self.assertEqual(len(invitedCals), 0)
-
-
-    @inlineCallbacks
-    def test_unshareSharerSide(self, commit=False):
-        """
-        Verify the coll.unshare( ) method works when called from the
-        sharer's copy
-        """
-        yield self.test_shareWith()
-        if commit:
-            yield self.commit()
-        cal = yield self.calendarUnderTest()
-        other = yield self.homeUnderTest(name=OTHER_HOME_UID)
-        otherCal = yield other.childWithName(self.sharedName)
-        self.assertNotEqual(otherCal, None)
-        yield cal.unshare()
-        otherCal = yield other.childWithName(self.sharedName)
-        self.assertEqual(otherCal, None)
-        invitedCals = yield cal.sharingInvites()
-        self.assertEqual(len(invitedCals), 0)
-
-
-    @inlineCallbacks
-    def test_unshareShareeSide(self, commit=False):
-        """
-        Verify the coll.unshare( ) method works when called from the
-        sharee's copy
-        """
-        yield self.test_shareWith()
-        if commit:
-            yield self.commit()
-        cal = yield self.calendarUnderTest()
-        other = yield self.homeUnderTest(name=OTHER_HOME_UID)
-        otherCal = yield other.childWithName(self.sharedName)
-        self.assertNotEqual(otherCal, None)
-        yield otherCal.unshare()
-        otherCal = yield other.childWithName(self.sharedName)
-        self.assertEqual(otherCal, None)
-        invitedCals = yield cal.sharingInvites()
-        self.assertEqual(len(invitedCals), 0)
-
-
-    @inlineCallbacks
-    def test_unshareWithInDifferentTransaction(self):
-        """
-        L{ICalendar.unshareWith} will remove a previously-shared calendar from
-        another user's calendar home, assuming the sharing was committed in a
-        previous transaction.
-        """
-        yield self.test_unshareWith(True)
-
-
-    @inlineCallbacks
-    def test_sharingInvites(self):
-        """
-        L{ICalendar.sharingInvites} returns an iterable of all versions of a shared
-        calendar.
-        """
-        cal = yield self.calendarUnderTest()
-        sharedBefore = yield cal.sharingInvites()
-        # It's not shared yet; make sure sharingInvites doesn't include owner version.
-        self.assertEqual(len(sharedBefore), 0)
-        yield self.test_shareWith()
-        # FIXME: don't know why this separate transaction is needed; remove it.
-        yield self.commit()
-        cal = yield self.calendarUnderTest()
-        sharedAfter = yield cal.sharingInvites()
-        self.assertEqual(len(sharedAfter), 1)
-        self.assertEqual(sharedAfter[0].mode(), _BIND_MODE_WRITE)
-        self.assertEqual(sharedAfter[0].shareeUID(), OTHER_HOME_UID)
-
-
-    @inlineCallbacks
-    def test_sharedNotifierID(self):
-        yield self.test_shareWith()
-        yield self.commit()
-
-        home = yield self.homeUnderTest()
-        self.assertEquals(home.notifierID(), ("CalDAV", "home1",))
-        calendar = yield home.calendarWithName("calendar_1")
-        self.assertEquals(calendar.notifierID(), ("CalDAV", "home1/calendar_1",))
-        yield self.commit()
-
-        home = yield self.homeUnderTest(name=OTHER_HOME_UID)
-        self.assertEquals(home.notifierID(), ("CalDAV", OTHER_HOME_UID,))
-        calendar = yield home.calendarWithName(self.sharedName)
-        self.assertEquals(calendar.notifierID(), ("CalDAV", "home1/calendar_1",))
-        yield self.commit()
-
-
     @inlineCallbacks
     def test_hasCalendarResourceUIDSomewhereElse(self):
         """
@@ -1339,7 +1230,7 @@ END:VCALENDAR
     @inlineCallbacks
     def test_iCalendarText(self):
         """
-        L{ICalendarObject.iCalendarText} returns a C{str} describing the same
+        L{ICalendarObject._text} returns a C{str} describing the same
         data provided by L{ICalendarObject.component}.
         """
         text = yield (yield self.calendarObjectUnderTest())._text()
@@ -1360,14 +1251,18 @@ END:VCALENDAR
         )
 
 
+    @inlineCallbacks
     def test_organizer(self):
         """
         L{ICalendarObject.organizer} returns a C{str} describing the calendar
         user address of the C{ORGANIZER} property of the calendar object's
         component.
         """
+
+        calobj = yield self.calendarObjectUnderTest(name="5.ics")
+        organizer = yield calobj.organizer()
         self.assertEquals(
-            (yield self.calendarObjectUnderTest()).organizer(),
+            organizer,
             "mailto:wsanchez@example.com"
         )
 
@@ -1498,6 +1393,36 @@ END:VCALENDAR
         )
 
 
+    @inlineCallbacks
+    def test_createCalendarObjectWithName_invalidName(self):
+        """
+        L{ICalendar.createCalendarObjectWithName} raises
+        L{CalendarObjectNameAlreadyExistsError} if the calendar object name
+        starts with a ".".
+        """
+        cal = yield self.calendarUnderTest()
+        comp = VComponent.fromString(test_event_text)
+        yield self.failUnlessFailure(
+            maybeDeferred(cal.createCalendarObjectWithName, ".1.ics", comp),
+            ObjectResourceNameNotAllowedError,
+        )
+
+
+    @inlineCallbacks
+    def test_createCalendarObjectWithName_longName(self):
+        """
+        L{ICalendar.createCalendarObjectWithName} raises
+        L{CalendarObjectNameAlreadyExistsError} if the calendar object name
+        is too long.
+        """
+        cal = yield self.calendarUnderTest()
+        comp = VComponent.fromString(test_event_text)
+        yield self.failUnlessFailure(
+            maybeDeferred(cal.createCalendarObjectWithName, "A" * 256 + ".ics", comp),
+            ObjectResourceNameNotAllowedError,
+        )
+
+
     @inlineCallbacks
     def test_createCalendarObjectWithName_invalid(self):
         """
@@ -1506,8 +1431,9 @@ END:VCALENDAR
         text.
         """
         yield self.failUnlessFailure(
-            maybeDeferred((yield self.calendarUnderTest()).createCalendarObjectWithName,
-            "new", VComponent.fromString(test_event_notCalDAV_text)),
+            maybeDeferred(
+                (yield self.calendarUnderTest()).createCalendarObjectWithName,
+                "new", VComponent.fromString(test_event_notCalDAV_text)),
             InvalidObjectResourceError,
             InvalidComponentForStoreError,
         )
@@ -1718,9 +1644,8 @@ END:VCALENDAR
         cal = yield self.calendarUnderTest()
         st = yield home.syncToken()
         yield cal.createCalendarObjectWithName("new.ics", VComponent.fromString(
-                test_event_text
-            )
-        )
+            test_event_text
+        ))
 
         obj1 = yield cal.calendarObjectWithName("2.ics")
         yield obj1.remove()
@@ -1730,19 +1655,33 @@ END:VCALENDAR
 
         home = yield self.homeUnderTest()
 
-        changed, deleted = yield home.resourceNamesSinceToken(
+        expected = [
+            "calendar_1/",
+            "calendar_1/new.ics",
+            "calendar_1/2.ics",
+            "other-calendar/"
+        ]
+
+        trash = yield home.getTrash()
+        if trash is not None:
+            trashed = yield trash.calendarObjects()
+            expected.extend([
+                "{}/".format(trash.name()),
+                "{}/{}".format(trash.name(), trashed[0].name()),
+            ])
+
+        changed, deleted, invalid = yield home.resourceNamesSinceToken(
             self.token2revision(st), "infinity")
 
-        self.assertEquals(set(changed), set(["calendar_1/",
-                                             "calendar_1/new.ics",
-                                             "calendar_1/2.ics",
-                                             "other-calendar/"]))
+        self.assertEquals(set(changed), set(expected))
         self.assertEquals(set(deleted), set(["calendar_1/2.ics"]))
+        self.assertEquals(invalid, [])
 
-        changed, deleted = yield home.resourceNamesSinceToken(
+        changed, deleted, invalid = yield home.resourceNamesSinceToken(
             self.token2revision(st2), "infinity")
         self.assertEquals(changed, [])
         self.assertEquals(deleted, [])
+        self.assertEquals(invalid, [])
 
 
     @inlineCallbacks
@@ -1755,19 +1694,20 @@ END:VCALENDAR
         st = yield cal.syncToken()
         rev = self.token2revision(st)
         yield cal.createCalendarObjectWithName("new.ics", VComponent.fromString(
-                test_event_text
-            )
-        )
+            test_event_text
+        ))
         obj1 = yield cal.calendarObjectWithName("2.ics")
         yield obj1.remove()
         st2 = yield cal.syncToken()
         rev2 = self.token2revision(st2)
-        changed, deleted = yield cal.resourceNamesSinceToken(rev)
+        changed, deleted, invalid = yield cal.resourceNamesSinceToken(rev)
         self.assertEquals(set(changed), set(["new.ics"]))
         self.assertEquals(set(deleted), set(["2.ics"]))
-        changed, deleted = yield cal.resourceNamesSinceToken(rev2)
+        self.assertEquals(len(invalid), 0)
+        changed, deleted, invalid = yield cal.resourceNamesSinceToken(rev2)
         self.assertEquals(set(changed), set([]))
         self.assertEquals(set(deleted), set([]))
+        self.assertEquals(len(invalid), 0)
 
 
     @inlineCallbacks
@@ -1831,7 +1771,7 @@ END:VCALENDAR
         L{ICalendarStore.withEachCalendarHomeDo} executes its C{action}
         argument repeatedly with all homes that have been created.
         """
-        additionalUIDs = set('alpha-uid home2 home3 beta-uid'.split())
+        additionalUIDs = set('user01 home2 home3 uid1'.split())
         txn = self.transactionUnderTest()
         for name in additionalUIDs:
             yield txn.calendarHomeWithUID(name, create=True)
