@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 
 from twisted.internet.defer import inlineCallbacks
 
-from twistedcaldav.ical import Component
+from twistedcaldav.ical import Component, normalize_iCalStr
 
 from txdav.common.datastore.test.util import CommonCommonTests, populateCalendarsFrom
 from twisted.trial.unittest import TestCase
@@ -27,9 +27,11 @@ from txdav.common.icommondatastore import ObjectResourceTooBigError, \
     InvalidObjectResourceError, InvalidComponentForStoreError, InvalidUIDError, \
     UIDExistsError, UIDExistsElsewhereError
 from txdav.caldav.icalendarstore import InvalidComponentTypeError, \
-    TooManyAttendeesError, InvalidCalendarAccessError, ComponentUpdateState
+    TooManyAttendeesError, InvalidCalendarAccessError, ComponentUpdateState, \
+    DuplicatePrivateCommentsError
 from txdav.common.datastore.sql_tables import _BIND_MODE_WRITE
-from txdav.caldav.datastore.test.util import buildCalendarStore
+from txdav.caldav.datastore.sql import CalendarObject
+
 
 class ImplicitRequests (CommonCommonTests, TestCase):
     """
@@ -39,7 +41,7 @@ class ImplicitRequests (CommonCommonTests, TestCase):
     @inlineCallbacks
     def setUp(self):
         yield super(ImplicitRequests, self).setUp()
-        self._sqlCalendarStore = yield buildCalendarStore(self, self.notifierFactory)
+        yield self.buildStoreAndDirectory()
         yield self.populate()
 
 
@@ -52,19 +54,19 @@ class ImplicitRequests (CommonCommonTests, TestCase):
     @classproperty(cache=False)
     def requirements(cls): #@NoSelf
         return {
-        "user01": {
-            "calendar_1": {
+            "user01": {
+                "calendar_1": {
+                },
+                "inbox": {
+                },
             },
-            "inbox": {
+            "user02": {
+                "calendar_1": {
+                },
+                "inbox": {
+                },
             },
-        },
-        "user02": {
-            "calendar_1": {
-            },
-            "inbox": {
-            },
-        },
-    }
+        }
 
 
     def storeUnderTest(self):
@@ -103,8 +105,8 @@ END:VCALENDAR
         calendar_resource1 = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
         calendar1 = (yield calendar_resource1.component())
         calendar1 = str(calendar1).replace("\r\n ", "")
-        self.assertTrue("urn:uuid:user01" in calendar1)
-        self.assertTrue("urn:uuid:user02" in calendar1)
+        self.assertTrue("urn:x-uid:user01" in calendar1)
+        self.assertTrue("urn:x-uid:user02" in calendar1)
         self.assertTrue("CN=" in calendar1)
         yield self.commit()
 
@@ -181,7 +183,7 @@ END:VEVENT
 END:VCALENDAR
 """),
 
-        Component.fromString("""BEGIN:VCALENDAR
+            Component.fromString("""BEGIN:VCALENDAR
 VERSION:2.0
 PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
 METHOD:PUBLISH
@@ -304,18 +306,18 @@ END:VCALENDAR
 """
 
         self.patch(config, "EnablePrivateEvents", True)
-        calendar_collection = (yield self.calendarUnderTest(home="user01"))
-        calendar = Component.fromString(data1)
         txn = self.transactionUnderTest()
         txn._authz_uid = "user02"
+        calendar_collection = (yield self.calendarUnderTest(home="user01"))
+        calendar = Component.fromString(data1)
         yield self.failUnlessFailure(calendar_collection.createCalendarObjectWithName("test.ics", calendar), InvalidCalendarAccessError)
         yield self.commit()
 
         # This one should be OK
-        calendar_collection = (yield self.calendarUnderTest(home="user01"))
-        calendar = Component.fromString(data1)
         txn = self.transactionUnderTest()
         txn._authz_uid = "user01"
+        calendar_collection = (yield self.calendarUnderTest(home="user01"))
+        calendar = Component.fromString(data1)
         yield calendar_collection.createCalendarObjectWithName("test.ics", calendar)
         yield self.commit()
 
@@ -333,10 +335,10 @@ END:VEVENT
 END:VCALENDAR
 """
 
-        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
-        calendar = Component.fromString(data2)
         txn = self.transactionUnderTest()
         txn._authz_uid = "user01"
+        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
+        calendar = Component.fromString(data2)
         yield calendar_resource.setComponent(calendar)
         yield self.commit()
 
@@ -514,10 +516,10 @@ END:VEVENT
 END:VCALENDAR
 """
 
-        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
-        calendar = Component.fromString(data2)
         txn = self.transactionUnderTest()
         txn._authz_uid = "user01"
+        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
+        calendar = Component.fromString(data2)
         yield calendar_resource.setComponent(calendar)
         yield self.commit()
 
@@ -543,7 +545,7 @@ UID:12345-67890-organizer
 DTSTAMP:20080601T120000Z
 DTSTART:20080601T120000Z
 DTEND:20080601T130000Z
-X-CALENDARSERVER-ATTENDEE-COMMENT;X-CALENDARSERVER-ATTENDEE-REF="urn:uuid:user01";
+X-CALENDARSERVER-ATTENDEE-COMMENT;X-CALENDARSERVER-ATTENDEE-REF="urn:x-uid:user01";
  X-CALENDARSERVER-DTSTAMP=20131101T100000Z:Someone else's comment
 END:VEVENT
 END:VCALENDAR
@@ -567,17 +569,17 @@ END:VEVENT
 END:VCALENDAR
 """
 
-        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
-        calendar = Component.fromString(data2)
         txn = self.transactionUnderTest()
         txn._authz_uid = "user01"
+        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
+        calendar = Component.fromString(data2)
         yield calendar_resource.setComponent(calendar)
         yield self.commit()
 
         calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
         calendar1 = (yield calendar_resource.component())
         calendar1 = str(calendar1).replace("\r\n ", "")
-        self.assertTrue("X-CALENDARSERVER-ATTENDEE-COMMENT;X-CALENDARSERVER-ATTENDEE-REF=\"urn:uuid:user01\";X-CALENDARSERVER-DTSTAMP=20131101T100000Z:Someone else's comment" in calendar1)
+        self.assertTrue("X-CALENDARSERVER-ATTENDEE-COMMENT;X-CALENDARSERVER-ATTENDEE-REF=\"urn:x-uid:user01\";X-CALENDARSERVER-DTSTAMP=20131101T100000Z:Someone else's comment" in calendar1)
         self.assertTrue("SUMMARY:Changed" in calendar1)
         yield self.commit()
 
@@ -621,10 +623,10 @@ END:VTODO
 END:VCALENDAR
 """
 
-        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
-        calendar = Component.fromString(data2)
         txn = self.transactionUnderTest()
         txn._authz_uid = "user01"
+        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
+        calendar = Component.fromString(data2)
         yield calendar_resource.setComponent(calendar)
         yield self.commit()
 
@@ -680,10 +682,10 @@ END:VTODO
 END:VCALENDAR
 """
 
-        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
-        calendar = Component.fromString(data2)
         txn = self.transactionUnderTest()
         txn._authz_uid = "user01"
+        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
+        calendar = Component.fromString(data2)
         yield calendar_resource.setComponent(calendar)
         yield self.commit()
 
@@ -787,10 +789,10 @@ END:VEVENT
 END:VCALENDAR
 """
 
-        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
-        calendar = Component.fromString(data2)
         txn = self.transactionUnderTest()
         txn._authz_uid = "user01"
+        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
+        calendar = Component.fromString(data2)
         result = (yield calendar_resource.setComponent(calendar))
         yield self.commit()
         self.assertTrue(result)
@@ -1096,3 +1098,179 @@ END:VCALENDAR
         yield calendar_resource._setComponentInternal(calendar, internal_state=ComponentUpdateState.ORGANIZER_ITIP_UPDATE)
         self.assertEqual(calendar_resource.scheduleTag, schedule_tag)
         yield self.commit()
+
+
+    @inlineCallbacks
+    def test_validation_duplicatePrivateCommentsOKWIthiTIP(self):
+        """
+        Test that an iTIP update to an organizer event with duplicate private comments
+        does not fail.
+        """
+
+        data1 = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-attendee-reply
+DTSTAMP:20080601T120000Z
+DTSTART:20080601T120000Z
+DTEND:20080601T130000Z
+ORGANIZER;CN="User 01":mailto:user01@example.com
+ATTENDEE:mailto:user01@example.com
+ATTENDEE:mailto:user02@example.com
+END:VEVENT
+END:VCALENDAR
+"""
+
+        data2 = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-attendee-reply
+DTSTAMP:20080601T120000Z
+DTSTART:20080601T120000Z
+DTEND:20080601T130000Z
+ORGANIZER;CN="User 01":mailto:user01@example.com
+ATTENDEE:mailto:user01@example.com
+ATTENDEE:mailto:user02@example.com
+X-CALENDARSERVER-ATTENDEE-COMMENT;X-CALENDARSERVER-ATTENDEE-REF="urn:uuid:
+ user02";X-CALENDARSERVER-DTSTAMP=20140224T181133Z:Comment
+X-CALENDARSERVER-ATTENDEE-COMMENT;X-CALENDARSERVER-ATTENDEE-REF="urn:uuid:
+ user02";X-CALENDARSERVER-DTSTAMP=20140224T181133Z:Comment
+END:VEVENT
+END:VCALENDAR
+"""
+
+        data3 = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-attendee-reply
+DTSTAMP:20080601T120000Z
+DTSTART:20080601T120000Z
+DTEND:20080601T130000Z
+ORGANIZER;CN="User 01":mailto:user01@example.com
+ATTENDEE:mailto:user01@example.com
+ATTENDEE;PARTSTAT=ACCEPTED:mailto:user02@example.com
+X-CALENDARSERVER-ATTENDEE-COMMENT;X-CALENDARSERVER-ATTENDEE-REF="urn:uuid:
+ user02";X-CALENDARSERVER-DTSTAMP=20140224T181133Z:Comment
+X-CALENDARSERVER-ATTENDEE-COMMENT;X-CALENDARSERVER-ATTENDEE-REF="urn:uuid:
+ user02";X-CALENDARSERVER-DTSTAMP=20140224T181133Z:Comment
+END:VEVENT
+END:VCALENDAR
+"""
+
+        calendar_collection = (yield self.calendarUnderTest(home="user01"))
+        calendar = Component.fromString(data1)
+        yield calendar_collection.createCalendarObjectWithName("test.ics", calendar)
+        yield self.commit()
+
+        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
+        calendar = Component.fromString(data2)
+        yield calendar_resource._setComponentInternal(calendar, internal_state=ComponentUpdateState.RAW)
+        yield self.commit()
+
+        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
+        calendar = Component.fromString(data3)
+        yield calendar_resource._setComponentInternal(calendar, internal_state=ComponentUpdateState.ORGANIZER_ITIP_UPDATE)
+        yield self.commit()
+
+        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
+        calendar = Component.fromString(data3)
+        yield self.failUnlessFailure(calendar_resource.setComponent(calendar), DuplicatePrivateCommentsError)
+        yield self.commit()
+
+
+    @inlineCallbacks
+    def test_validation_deleteWithDuplicatePrivateComments(self):
+        """
+        Test that attendee private comments are no longer restored.
+        """
+
+        data1 = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-attendee-reply
+DTSTAMP:20080601T120000Z
+DTSTART:20080601T120000Z
+DTEND:20080601T130000Z
+ORGANIZER;CN="User 01":mailto:user01@example.com
+ATTENDEE:mailto:user01@example.com
+ATTENDEE:mailto:user02@example.com
+END:VEVENT
+END:VCALENDAR
+"""
+
+        data2 = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-attendee-reply
+DTSTAMP:20080601T120000Z
+DTSTART:20080601T120000Z
+DTEND:20080601T130000Z
+ORGANIZER;CN="User 01":mailto:user01@example.com
+ATTENDEE:mailto:user01@example.com
+ATTENDEE:mailto:user02@example.com
+X-CALENDARSERVER-ATTENDEE-COMMENT;X-CALENDARSERVER-ATTENDEE-REF="urn:uuid:
+ user02";X-CALENDARSERVER-DTSTAMP=20140224T181133Z:Comment
+X-CALENDARSERVER-ATTENDEE-COMMENT;X-CALENDARSERVER-ATTENDEE-REF="urn:uuid:
+ user02";X-CALENDARSERVER-DTSTAMP=20140224T181133Z:Comment
+END:VEVENT
+END:VCALENDAR
+"""
+
+        data3 = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-attendee-reply
+DTSTAMP:20080601T120000Z
+DTSTART:20080601T120000Z
+DTEND:20080601T130000Z
+ORGANIZER;CN="User 01":mailto:user01@example.com
+ATTENDEE:mailto:user01@example.com
+ATTENDEE;PARTSTAT=ACCEPTED:mailto:user02@example.com
+END:VEVENT
+END:VCALENDAR
+"""
+
+        calendar_collection = (yield self.calendarUnderTest(home="user01"))
+        calendar = Component.fromString(data1)
+        yield calendar_collection.createCalendarObjectWithName("test.ics", calendar)
+        yield self.commit()
+
+        calendar_resource = (yield self.calendarObjectUnderTest(name="test.ics", home="user01",))
+        calendar = Component.fromString(data2)
+        yield calendar_resource._setComponentInternal(calendar, internal_state=ComponentUpdateState.RAW)
+        yield self.commit()
+
+        def raiseHere(otherself, component, inserting, internal_state):
+            if component.hasDuplicatePrivateComments(doFix=False):
+                raise ValueError
+
+        self.patch(CalendarObject, "preservePrivateComments", raiseHere)
+
+        calendar2 = (yield self.calendarUnderTest(name="calendar_1", home="user02"))
+        cobjs = (yield calendar2.calendarObjects())
+        self.assertTrue(len(cobjs) == 1)
+        yield cobjs[0].setComponent(Component.fromString(data3))
+        yield self.commit()
+
+        calendar2 = (yield self.calendarUnderTest(name="calendar_1", home="user02"))
+        cobjs = (yield calendar2.calendarObjects())
+        calendar = yield cobjs[0].component()
+        self.assertTrue('SCHEDULE-STATUS=5.0' in normalize_iCalStr(calendar))
+        yield self.commit()
+
+        calendar2 = (yield self.calendarUnderTest(name="calendar_1", home="user02"))
+        cobjs = (yield calendar2.calendarObjects())
+        self.assertTrue(len(cobjs) == 1)
+        yield cobjs[0].remove()
+        yield self.commit()
+
+        calendar2 = (yield self.calendarUnderTest(name="calendar_1", home="user02"))
+        cobjs = (yield calendar2.calendarObjects())
+        self.assertTrue(len(cobjs) == 0)
+        yield self.commit()
