@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@ from twisted.internet.defer import inlineCallbacks
 from twisted.internet.task import deferLater
 
 from txdav.caldav.datastore.index_file import Index, MemcachedUIDReserver
+from txdav.caldav.datastore.query.filter import Filter
 from txdav.common.icommondatastore import ReservationError, \
     InternalDataStoreError
 
@@ -26,11 +27,10 @@ from twistedcaldav import caldavxml
 from twistedcaldav.caldavxml import TimeRange
 from twistedcaldav.ical import Component, InvalidICalendarDataError
 from twistedcaldav.instance import InvalidOverriddenInstanceError
-from twistedcaldav.query import calendarqueryfilter
 from twistedcaldav.test.util import InMemoryMemcacheProtocol
 import twistedcaldav.test.util
 
-from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.datetime import DateTime
 
 import os
 
@@ -311,7 +311,7 @@ END:VCALENDAR
             else:
                 self.assertFalse(self.db.resourceExists(name), msg=description)
 
-        self.db.testAndUpdateIndex(PyCalendarDateTime(2020, 1, 1))
+        self.db.testAndUpdateIndex(DateTime(2020, 1, 1))
         for description, name, calendar_txt, reCreate, ok in data:
             if ok:
                 self.assertTrue(self.db.resourceExists(name), msg=description)
@@ -469,18 +469,18 @@ END:VCALENDAR
 
             # Create fake filter element to match time-range
             filter = caldavxml.Filter(
-                  caldavxml.ComponentFilter(
-                      caldavxml.ComponentFilter(
-                          TimeRange(
-                              start=trstart,
-                              end=trend,
-                          ),
-                          name=("VEVENT", "VFREEBUSY", "VAVAILABILITY"),
-                      ),
-                      name="VCALENDAR",
-                   )
-              )
-            filter = calendarqueryfilter.Filter(filter)
+                caldavxml.ComponentFilter(
+                    caldavxml.ComponentFilter(
+                        TimeRange(
+                            start=trstart,
+                            end=trend,
+                        ),
+                        name=("VEVENT", "VFREEBUSY", "VAVAILABILITY"),
+                    ),
+                    name="VCALENDAR",
+                )
+            )
+            filter = Filter(filter)
 
             resources = yield self.db.indexedSearch(filter)
             index_results = set()
@@ -512,7 +512,7 @@ END:VCALENDAR
 """,
                 "20080601T000000Z", "20080602T000000Z",
                 "mailto:user1@example.com",
-                (('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'F'),),
+                (('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'F'),),
             ),
             (
                 "#1.2 Simple component - transparent",
@@ -534,7 +534,7 @@ END:VCALENDAR
 """,
                 "20080602T000000Z", "20080603T000000Z",
                 "mailto:user1@example.com",
-                (('N', "2008-06-02 12:00:00+00:00", "2008-06-02 13:00:00+00:00", 'B', 'T'),),
+                (('N', "2008-06-02 12:00:00", "2008-06-02 13:00:00", 'B', 'T'),),
             ),
             (
                 "#1.3 Simple component - canceled",
@@ -556,7 +556,7 @@ END:VCALENDAR
 """,
                 "20080603T000000Z", "20080604T000000Z",
                 "mailto:user1@example.com",
-                (('N', "2008-06-03 12:00:00+00:00", "2008-06-03 13:00:00+00:00", 'F', 'F'),),
+                (('N', "2008-06-03 12:00:00", "2008-06-03 13:00:00", 'F', 'F'),),
             ),
             (
                 "#1.4 Simple component - tentative",
@@ -578,7 +578,7 @@ END:VCALENDAR
 """,
                 "20080604T000000Z", "20080605T000000Z",
                 "mailto:user1@example.com",
-                (('N', "2008-06-04 12:00:00+00:00", "2008-06-04 13:00:00+00:00", 'T', 'F'),),
+                (('N', "2008-06-04 12:00:00", "2008-06-04 13:00:00", 'T', 'F'),),
             ),
             (
                 "#2.1 Recurring component - busy",
@@ -601,8 +601,8 @@ END:VCALENDAR
                 "20080605T000000Z", "20080607T000000Z",
                 "mailto:user1@example.com",
                 (
-                    ('N', "2008-06-05 12:00:00+00:00", "2008-06-05 13:00:00+00:00", 'B', 'F'),
-                    ('N', "2008-06-06 12:00:00+00:00", "2008-06-06 13:00:00+00:00", 'B', 'F'),
+                    ('N', "2008-06-05 12:00:00", "2008-06-05 13:00:00", 'B', 'F'),
+                    ('N', "2008-06-06 12:00:00", "2008-06-06 13:00:00", 'B', 'F'),
                 ),
             ),
             (
@@ -637,8 +637,8 @@ END:VCALENDAR
                 "20080607T000000Z", "20080609T000000Z",
                 "mailto:user1@example.com",
                 (
-                    ('N', "2008-06-07 12:00:00+00:00", "2008-06-07 13:00:00+00:00", 'B', 'F'),
-                    ('N', "2008-06-08 14:00:00+00:00", "2008-06-08 15:00:00+00:00", 'B', 'T'),
+                    ('N', "2008-06-07 12:00:00", "2008-06-07 13:00:00", 'B', 'F'),
+                    ('N', "2008-06-08 14:00:00", "2008-06-08 15:00:00", 'B', 'T'),
                 ),
             ),
         )
@@ -655,18 +655,18 @@ END:VCALENDAR
 
             # Create fake filter element to match time-range
             filter = caldavxml.Filter(
-                  caldavxml.ComponentFilter(
-                      caldavxml.ComponentFilter(
-                          TimeRange(
-                              start=trstart,
-                              end=trend,
-                          ),
-                          name=("VEVENT", "VFREEBUSY", "VAVAILABILITY"),
-                      ),
-                      name="VCALENDAR",
-                   )
-              )
-            filter = calendarqueryfilter.Filter(filter)
+                caldavxml.ComponentFilter(
+                    caldavxml.ComponentFilter(
+                        TimeRange(
+                            start=trstart,
+                            end=trend,
+                        ),
+                        name=("VEVENT", "VFREEBUSY", "VAVAILABILITY"),
+                    ),
+                    name="VCALENDAR",
+                )
+            )
+            filter = Filter(filter)
 
             resources = yield self.db.indexedSearch(filter, fbtype=True)
             index_results = set()
@@ -714,11 +714,11 @@ END:VCALENDAR
                 (
                     (
                         "user01",
-                        (('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'T'),),
+                        (('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'T'),),
                     ),
                     (
                         "user02",
-                        (('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'F'),),
+                        (('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'F'),),
                     ),
                 ),
             ),
@@ -767,15 +767,15 @@ END:VCALENDAR
                 (
                     (
                         "user01",
-                        (('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'T'),),
+                        (('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'T'),),
                     ),
                     (
                         "user02",
-                        (('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'F'),),
+                        (('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'F'),),
                     ),
                     (
                         "user03",
-                        (('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'F'),),
+                        (('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'F'),),
                     ),
                 ),
             ),
@@ -815,15 +815,15 @@ END:VCALENDAR
                     (
                         "user01",
                         (
-                            ('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'T'),
-                            ('N', "2008-06-02 12:00:00+00:00", "2008-06-02 13:00:00+00:00", 'B', 'T'),
+                            ('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'T'),
+                            ('N', "2008-06-02 12:00:00", "2008-06-02 13:00:00", 'B', 'T'),
                         ),
                     ),
                     (
                         "user02",
                         (
-                            ('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-02 12:00:00+00:00", "2008-06-02 13:00:00+00:00", 'B', 'F'),
+                            ('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'F'),
+                            ('N', "2008-06-02 12:00:00", "2008-06-02 13:00:00", 'B', 'F'),
                         ),
                     ),
                 ),
@@ -875,22 +875,22 @@ END:VCALENDAR
                     (
                         "user01",
                         (
-                            ('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'T'),
-                            ('N', "2008-06-02 12:00:00+00:00", "2008-06-02 13:00:00+00:00", 'B', 'T'),
+                            ('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'T'),
+                            ('N', "2008-06-02 12:00:00", "2008-06-02 13:00:00", 'B', 'T'),
                         ),
                     ),
                     (
                         "user02",
                         (
-                            ('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-02 12:00:00+00:00", "2008-06-02 13:00:00+00:00", 'B', 'F'),
+                            ('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'F'),
+                            ('N', "2008-06-02 12:00:00", "2008-06-02 13:00:00", 'B', 'F'),
                         ),
                     ),
                     (
                         "user03",
                         (
-                            ('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-02 12:00:00+00:00", "2008-06-02 13:00:00+00:00", 'B', 'F'),
+                            ('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'F'),
+                            ('N', "2008-06-02 12:00:00", "2008-06-02 13:00:00", 'B', 'F'),
                         ),
                     ),
                 ),
@@ -945,17 +945,17 @@ END:VCALENDAR
                     (
                         "user01",
                         (
-                            ('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'T'),
-                            ('N', "2008-06-02 13:00:00+00:00", "2008-06-02 14:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-03 12:00:00+00:00", "2008-06-03 13:00:00+00:00", 'B', 'T'),
+                            ('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'T'),
+                            ('N', "2008-06-02 13:00:00", "2008-06-02 14:00:00", 'B', 'F'),
+                            ('N', "2008-06-03 12:00:00", "2008-06-03 13:00:00", 'B', 'T'),
                         ),
                     ),
                     (
                         "user02",
                         (
-                            ('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-02 13:00:00+00:00", "2008-06-02 14:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-03 12:00:00+00:00", "2008-06-03 13:00:00+00:00", 'B', 'F'),
+                            ('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'F'),
+                            ('N', "2008-06-02 13:00:00", "2008-06-02 14:00:00", 'B', 'F'),
+                            ('N', "2008-06-03 12:00:00", "2008-06-03 13:00:00", 'B', 'F'),
                         ),
                     ),
                 ),
@@ -1025,25 +1025,25 @@ END:VCALENDAR
                     (
                         "user01",
                         (
-                            ('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'T'),
-                            ('N', "2008-06-02 13:00:00+00:00", "2008-06-02 14:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-03 12:00:00+00:00", "2008-06-03 13:00:00+00:00", 'B', 'T'),
+                            ('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'T'),
+                            ('N', "2008-06-02 13:00:00", "2008-06-02 14:00:00", 'B', 'F'),
+                            ('N', "2008-06-03 12:00:00", "2008-06-03 13:00:00", 'B', 'T'),
                         ),
                     ),
                     (
                         "user02",
                         (
-                            ('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-02 13:00:00+00:00", "2008-06-02 14:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-03 12:00:00+00:00", "2008-06-03 13:00:00+00:00", 'B', 'T'),
+                            ('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'F'),
+                            ('N', "2008-06-02 13:00:00", "2008-06-02 14:00:00", 'B', 'F'),
+                            ('N', "2008-06-03 12:00:00", "2008-06-03 13:00:00", 'B', 'T'),
                         ),
                     ),
                     (
                         "user03",
                         (
-                            ('N', "2008-06-01 12:00:00+00:00", "2008-06-01 13:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-02 13:00:00+00:00", "2008-06-02 14:00:00+00:00", 'B', 'F'),
-                            ('N', "2008-06-03 12:00:00+00:00", "2008-06-03 13:00:00+00:00", 'B', 'F'),
+                            ('N', "2008-06-01 12:00:00", "2008-06-01 13:00:00", 'B', 'F'),
+                            ('N', "2008-06-02 13:00:00", "2008-06-02 14:00:00", 'B', 'F'),
+                            ('N', "2008-06-03 12:00:00", "2008-06-03 13:00:00", 'B', 'F'),
                         ),
                     ),
                 ),
@@ -1062,18 +1062,18 @@ END:VCALENDAR
 
             # Create fake filter element to match time-range
             filter = caldavxml.Filter(
-                  caldavxml.ComponentFilter(
-                      caldavxml.ComponentFilter(
-                          TimeRange(
-                              start=trstart,
-                              end=trend,
-                          ),
-                          name=("VEVENT", "VFREEBUSY", "VAVAILABILITY"),
-                      ),
-                      name="VCALENDAR",
-                   )
-              )
-            filter = calendarqueryfilter.Filter(filter)
+                caldavxml.ComponentFilter(
+                    caldavxml.ComponentFilter(
+                        TimeRange(
+                            start=trstart,
+                            end=trend,
+                        ),
+                        name=("VEVENT", "VFREEBUSY", "VAVAILABILITY"),
+                    ),
+                    name="VCALENDAR",
+                )
+            )
+            filter = Filter(filter)
 
             for useruid, instances in peruserinstances:
                 resources = yield self.db.indexedSearch(filter, useruid=useruid, fbtype=True)
@@ -1142,12 +1142,12 @@ END:VCALENDAR
         self.db.deleteResource("data3.ics")
 
         tests = (
-            (0, (["data1.ics", "data2.ics", ], [],)),
-            (1, (["data2.ics", ], ["data3.ics", ],)),
-            (2, ([], ["data3.ics", ],)),
-            (3, ([], ["data3.ics", ],)),
-            (4, ([], [],)),
-            (5, ([], [],)),
+            (0, (["data1.ics", "data2.ics", ], [], [],)),
+            (1, (["data2.ics", ], ["data3.ics", ], [],)),
+            (2, ([], ["data3.ics", ], [],)),
+            (3, ([], ["data3.ics", ], [],)),
+            (4, ([], [], [],)),
+            (5, ([], [], [],)),
         )
 
         for revision, results in tests:
