@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.caldav.datastore.test.test_util -*-
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,11 +32,12 @@ from twisted.internet.protocol import Protocol
 
 from twext.python.log import Logger
 
-from twext.web2 import http_headers
+from txweb2 import http_headers
 
-from twext.python.vcomponent import InvalidICalendarDataError
-from twext.python.vcomponent import VComponent
+from twistedcaldav.ical import InvalidICalendarDataError
+from twistedcaldav.ical import Component as VComponent
 
+from twistedcaldav import ical
 from twistedcaldav.datafilters.hiddeninstance import HiddenInstanceFilter
 from twistedcaldav.datafilters.privateevents import PrivateEventFilter
 from twistedcaldav.ical import PERUSER_UID
@@ -82,7 +83,7 @@ def validateCalendarComponent(calendarObject, calendar, component, inserting, mi
             raise InvalidObjectResourceError(
                 "UID may not change (%s != %s)" % (
                     component.resourceUID(), calendarObject.uid()
-                 )
+                )
             )
     except NoSuchObjectResourceError:
         pass
@@ -99,32 +100,34 @@ def validateCalendarComponent(calendarObject, calendar, component, inserting, mi
 
 
 
+@inlineCallbacks
 def normalizationLookup(cuaddr, recordFunction, config):
     """
     Lookup function to be passed to ical.normalizeCalendarUserAddresses.
-    Returns a tuple of (Full name, guid, and calendar user address list)
-    for the given cuaddr.  The recordFunction is called to retrieve the
-    record for the cuaddr.
+    Returns a tuple of (Full name C{unicode}, uid C{unicode}, cudtype C{str}, and
+    calendar user address list C{unicode}) for the given cuaddr.
+    recordFunction is called to retrieve the record for the cuaddr.
     """
     try:
-        record = recordFunction(cuaddr)
+        record = yield recordFunction(cuaddr)
     except Exception, e:
         log.debug("Lookup of %s failed: %s" % (cuaddr, e))
         record = None
 
     if record is None:
-        return (None, None, None)
+        returnValue((None, None, None, None))
     else:
+
         # RFC5545 syntax does not allow backslash escaping in
         # parameter values. A double-quote is thus not allowed
         # in a parameter value except as the start/end delimiters.
         # Single quotes are allowed, so we convert any double-quotes
         # to single-quotes.
-        return (
-            record.fullName.replace('"', "'"),
-            record.uid,
-            record.calendarUserAddresses,
+        fullName = record.displayName.replace('"', "'").encode("utf-8")
+        cuas = set(
+            [cua.encode("utf-8") for cua in record.calendarUserAddresses]
         )
+        returnValue((fullName, record.uid, record.getCUType(), cuas))
 
 
 
@@ -138,8 +141,7 @@ def dropboxIDFromCalendarObject(calendarObject):
     """
 
     # Try "X-APPLE-DROPBOX" first
-    dropboxProperty = (yield calendarObject.component(
-        )).getFirstPropertyInAnyComponent("X-APPLE-DROPBOX")
+    dropboxProperty = (yield calendarObject.component()).getFirstPropertyInAnyComponent("X-APPLE-DROPBOX")
     if dropboxProperty is not None and dropboxProperty.value():
         componentDropboxID = dropboxProperty.value().rstrip("/").split("/")[-1]
         returnValue(componentDropboxID)
@@ -282,7 +284,7 @@ def _migrateCalendar(inCalendar, outCalendar, getComponent, merge=False):
 
 
 
-# MIME helpers - mostly copied from twext.web2.static
+# MIME helpers - mostly copied from txweb2.static
 
 def loadMimeTypes(mimetype_locations=['/etc/mime.types']):
     """
@@ -380,9 +382,11 @@ def migrateHome(inHome, outHome, getComponent=lambda x: x.component(), merge=Fal
     """
     from twistedcaldav.config import config
     if not merge:
-        yield outHome.removeCalendarWithName("calendar")
         if config.RestrictCalendarsToOneComponentType:
-            yield outHome.removeCalendarWithName("tasks")
+            for name in ical.allowedStoreComponents:
+                yield outHome.removeCalendarWithName(outHome._componentCalendarName[name])
+        else:
+            yield outHome.removeCalendarWithName("calendar")
         yield outHome.removeCalendarWithName("inbox")
 
     outHome.properties().update(inHome.properties())
@@ -504,7 +508,7 @@ class StorageTransportBase(object):
     def __init__(self, attachment, contentType, dispositionName):
         """
         Create a storage transport with a reference to an L{IAttachment} and a
-        L{twext.web2.http_headers.MimeType}.
+        L{txweb2.http_headers.MimeType}.
         """
         from twisted.internet import reactor
         self._clock = reactor
@@ -518,6 +522,11 @@ class StorageTransportBase(object):
             self._contentType = http_headers.MimeType.fromString(getType(self._attachment.name(), self.contentTypes))
 
 
+    def resetDetails(self, contentType, dispositionName):
+        self._contentType = contentType
+        self._dispositionName = dispositionName
+
+
     def write(self, data):
         """
         Children must override this to actually write the data, but should
@@ -669,7 +678,7 @@ def fixOneCalendarHome(home):
                 fixCount, comp = fixOneCalendarObject(comp)
                 fixedThisHome += fixCount
                 if fixCount:
-                    yield calObj.setComponent(comp)
+                    yield calObj._setComponentInternal(comp, internal_state=ComponentUpdateState.RAW)
             except:
                 log.failure(
                     "Error while processing calendar/object {calendarName} {calendarObject}",
