@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.caldav.datastore -*-
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -38,7 +38,7 @@ __all__ = [
     "IAttachment",
 
     # Exceptions
-   #"InvalidCalendarComponentError",
+    # "InvalidCalendarComponentError",
     "InvalidCalendarAccessError",
     "TooManyAttendeesError",
     "ResourceDeletedError",
@@ -363,9 +363,9 @@ class ICalendar(INotifier, IShareableCollection, IDataStoreObject):
         instances that occur within the time range that begins at
         C{start} and ends at C{end}.
 
-        @param start: a L{PyCalendarDateTime}.
-        @param end: a L{PyCalendarDateTime}.
-        @param timeZone: a L{PyCalendarTimezone}.
+        @param start: a L{DateTime}.
+        @param end: a L{DateTime}.
+        @param timeZone: a L{Timezone}.
         @return: an iterable of L{ICalendarObject}s.
         """
 
@@ -405,6 +405,18 @@ class ICalendar(INotifier, IShareableCollection, IDataStoreObject):
         """
         # TODO: implement this for the file store.
 
+    def effectiveShareMode(): #@NoSelf
+        """
+        The effective sharing mode of this calendar after group sharing is
+        considered
+
+        @see: L{ICalendar.viewerCalendarHome}
+        @return: a L{Deferred} which fires with C{BIND_*} constant or L{None}
+        if the mode is unknown
+
+        @rtype: L{Deferred}
+        """
+
     # FIXME: This should be calendarHome(), assuming we want to allow
     # back-references.
     def viewerCalendarHome(): #@NoSelf
@@ -658,7 +670,7 @@ class IAttachment(IDataStoreObject):
 
         @param contentType: The content type of the data which will be stored.
 
-        @type contentType: L{twext.web2.http_headers.MimeType}
+        @type contentType: L{txweb2.http_headers.MimeType}
 
         @return: A transport which stores the contents written to it.
 
@@ -775,6 +787,13 @@ class AttachmentStoreFailed(Exception):
 
 
 
+class AttachmentSizeTooLarge(Exception):
+    """
+    Unable to store an attachment because it is too large.
+    """
+
+
+
 class AttachmentStoreValidManagedID(Exception):
     """
     Specified attachment managed-id is not valid.
@@ -850,6 +869,13 @@ class InvalidSplit(CommonStoreError):
 
 
 
+class UnknownTimezone(CommonStoreError):
+    """
+    Unknown timezone id used.
+    """
+
+
+
 #
 # FIXME: These may belong elsewhere.
 #
@@ -880,7 +906,7 @@ class ComponentUpdateState(Names):
                             is done (more than RAW).
 
     RAW                   - store the supplied data as-is without any processing or validation. This is used
-                            for unit testing purposes only.
+                            for unit testing purposes only, or during migration.
     """
 
     NORMAL = NamedConstant()
@@ -914,7 +940,7 @@ class ComponentRemoveState(Names):
     NORMAL_NO_IMPLICIT -    this is an application layer (user) generated remove that deliberately turns
                             off implicit scheduling operations.
 
-    INTERNAL -              remove the resource without implicit scheduling.
+    INTERNAL -              remove the resource without implicit scheduling or attachment processing.
     """
 
     NORMAL = NamedConstant()
@@ -924,3 +950,34 @@ class ComponentRemoveState(Names):
     NORMAL.description = "normal"
     NORMAL_NO_IMPLICIT.description = "normal-no-implicit"
     INTERNAL.description = "internal"
+
+
+
+class SetComponentOptions(object):
+    """
+    Constants for keys used in the L{ICalendarObject.setComponent} method's
+    C{options} dict. The definitions below define the constant key name and
+    the type used for the dict entry's value.
+
+    @cvar smartMerge: Apply CalDAV smart merge to data (If-Schedule-Tag-Match)
+        Value: L{bool}
+
+    @cvar clientFixTRANSP: Apply fix for clients not setting TRANSP.
+        Value: L{bool}
+    """
+
+    # Smart Merge: CalDAV If-Schedule-Tag-Match behavior
+    smartMerge = "smartMerge"
+
+    # Fix for clients not setting TRANSP
+    clientFixTRANSP = "clientFixTRANSP"
+
+    _defaults = {
+        smartMerge: False,
+        clientFixTRANSP: False
+    }
+
+    @staticmethod
+    def value(options, key):
+        defaultValue = SetComponentOptions._defaults.get(key)
+        return options.get(key, defaultValue) if options is not None else defaultValue
