@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -37,10 +37,6 @@ except ImportError:
 
 from twisted.internet.defer import maybeDeferred
 
-from twistedcaldav import carddavxml
-from txdav.common.icommondatastore import SyncTokenValidException, \
-    ReservationError
-from twistedcaldav.query import addressbookquery
 from twistedcaldav.sql import AbstractSQLDatabase
 from twistedcaldav.sql import db_prefix
 from twistedcaldav.vcard import Component
@@ -49,6 +45,12 @@ from twext.python.log import Logger
 from twistedcaldav.config import config
 from twistedcaldav.memcachepool import CachePoolUserMixIn
 
+from txdav.carddav.datastore.query.builder import buildExpression
+from txdav.carddav.datastore.query.filter import Filter
+from txdav.common.datastore.query.filegenerator import sqllitegenerator
+from txdav.common.icommondatastore import SyncTokenValidException, \
+    ReservationError
+
 log = Logger()
 
 db_basename = db_prefix + "sqlite"
@@ -79,15 +81,16 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
     def reserveUID(self, uid):
         uid = uid.encode('utf-8')
         self.log.debug("Reserving UID %r @ %r" % (
-                uid,
-                self.index.resource.fp.path))
+            uid,
+            self.index.resource.fp.path
+        ))
 
         def _handleFalse(result):
             if result is False:
                 raise ReservationError(
                     "UID %s already reserved for address book collection %s."
                     % (uid, self.index.resource)
-                    )
+                )
 
         d = self.getCachePool().add(self._key(uid),
                                     'reserved',
@@ -99,15 +102,16 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
     def unreserveUID(self, uid):
         uid = uid.encode('utf-8')
         self.log.debug("Unreserving UID %r @ %r" % (
-                uid,
-                self.index.resource.fp.path))
+            uid,
+            self.index.resource.fp.path
+        ))
 
         def _handleFalse(result):
             if result is False:
                 raise ReservationError(
                     "UID %s is not reserved for address book collection %s."
                     % (uid, self.index.resource)
-                    )
+                )
 
         d = self.getCachePool().delete(self._key(uid))
         d.addCallback(_handleFalse)
@@ -117,8 +121,9 @@ class MemcachedUIDReserver(CachePoolUserMixIn):
     def isReservedUID(self, uid):
         uid = uid.encode('utf-8')
         self.log.debug("Is reserved UID %r @ %r" % (
-                uid,
-                self.index.resource.fp.path))
+            uid,
+            self.index.resource.fp.path
+        ))
 
         def _checkValue((flags, value)):
             if value is None:
@@ -172,7 +177,7 @@ class SQLUIDReserver(object):
                 raise ReservationError(
                     "UID %s is not reserved for address book collection %s."
                     % (uid, self.index.resource)
-                    )
+                )
             else:
                 try:
                     self.index._db_execute(
@@ -218,6 +223,24 @@ class SQLUIDReserver(object):
 
 
 
+def sqladdressbookquery(filter, addressbookid=None):
+    """
+    Convert the supplied addressbook-query into a partial SQL statement.
+
+    @param filter: the L{Filter} for the addressbook-query to convert.
+    @return: a C{tuple} of (C{str}, C{list}), where the C{str} is the partial SQL statement,
+            and the C{list} is the list of argument substitutions to use with the SQL API execute method.
+            Or return C{None} if it is not possible to create an SQL query to fully match the addressbook-query.
+    """
+    try:
+        expression = buildExpression(filter, sqllitegenerator.FIELDS)
+        sql = sqllitegenerator(expression, addressbookid, None)
+        return sql.generate()
+    except ValueError:
+        return None
+
+
+
 class AddressBookIndex(AbstractSQLDatabase):
     """
     AddressBook collection index abstract base class that defines the apis for the index.
@@ -410,6 +433,7 @@ class AddressBookIndex(AbstractSQLDatabase):
 
         changed = []
         deleted = []
+        invalid = []
         for name, wasdeleted in results:
             if name:
                 if wasdeleted == 'Y':
@@ -420,7 +444,7 @@ class AddressBookIndex(AbstractSQLDatabase):
             else:
                 raise SyncTokenValidException
 
-        return changed, deleted,
+        return (changed, deleted, invalid)
 
 
     def lastRevision(self):
@@ -445,8 +469,8 @@ class AddressBookIndex(AbstractSQLDatabase):
 
 
     def searchValid(self, filter):
-        if isinstance(filter, carddavxml.Filter):
-            qualifiers = addressbookquery.sqladdressbookquery(filter)
+        if isinstance(filter, Filter):
+            qualifiers = sqladdressbookquery(filter)
         else:
             qualifiers = None
 
@@ -466,8 +490,8 @@ class AddressBookIndex(AbstractSQLDatabase):
         # start caching...
 
         # Make sure we have a proper Filter element and get the partial SQL statement to use.
-        if isinstance(filter, carddavxml.Filter):
-            qualifiers = addressbookquery.sqladdressbookquery(filter)
+        if isinstance(filter, Filter):
+            qualifiers = sqladdressbookquery(filter)
         else:
             qualifiers = None
         if qualifiers is not None:
@@ -626,7 +650,7 @@ class AddressBookIndex(AbstractSQLDatabase):
                 except ValueError:
                     log.error("Non-addressbook resource: %s" % (name,))
                 else:
-                    #log.info("Indexing resource: %s" % (name,))
+                    # log.info("Indexing resource: %s" % (name,))
                     self.addResource(name, vcard, True)
             finally:
                 stream.close()
