@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -60,6 +60,7 @@ def setUpAddressBookStore(test):
     addressbookPath.parent().makedirs()
     storePath.copyTo(addressbookPath)
 
+    test.counter = 0
     test.notifierFactory = StubNotifierFactory()
     test.addressbookStore = AddressBookStore(storeRootPath, {"push": test.notifierFactory}, None)
     test.txn = test.addressbookStore.newTransaction(test.id() + " (old)")
@@ -98,9 +99,10 @@ class AddressBookStoreTest(unittest.TestCase):
         Filenames starting with "." are reserved by this
         implementation, so no UIDs may start with ".".
         """
-        self.assertEquals(
-            (yield self.addressbookStore.newTransaction(self.id()
-                ).addressbookHomeWithUID(".xyzzy")),
+        self.assertEquals((
+            yield self.addressbookStore.newTransaction(
+                self.id())
+            .addressbookHomeWithUID(".xyzzy")),
             None
         )
 
@@ -234,8 +236,7 @@ class AddressBookTest(unittest.TestCase):
         )
         self.assertEquals(
             addressbookObject.component(),
-            (yield self.addressbook1.addressbookObjectWithName("1.vcf")
-                ).component()
+            (yield self.addressbook1.addressbookObjectWithName("1.vcf")).component()
         )
 
 
@@ -354,20 +355,21 @@ class AddressBookTest(unittest.TestCase):
         )
 
 
+    @inlineCallbacks
     def test_modifyAddressBookObjectCaches(self):
         """
         Modifying a addressbook object should cache the modified component in
         memory, to avoid unnecessary parsing round-trips.
         """
         modifiedComponent = VComponent.fromString(vcard1modified_text)
-        (yield self.addressbook1.addressbookObjectWithName("1.vcf")).setComponent(
-            modifiedComponent
+        obj = yield self.addressbook1.addressbookObjectWithName("1.vcf")
+        yield obj.setComponent(modifiedComponent)
+        comp = yield obj.component()
+        self.assertEqual(
+            str(modifiedComponent),
+            str(comp)
         )
-        self.assertIdentical(
-            modifiedComponent,
-            (yield self.addressbook1.addressbookObjectWithName("1.vcf")).component()
-        )
-        self.txn.commit()
+        yield self.txn.commit()
 
 
     @testUnimplemented
