@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,28 +22,28 @@ L{txdav.carddav.datastore.test.common}.
 from twext.enterprise.dal.syntax import Select, Parameter
 
 from twisted.internet import reactor
-from twisted.internet.defer import inlineCallbacks, returnValue, DeferredList
+from twisted.internet.defer import inlineCallbacks, returnValue
 from twisted.internet.task import deferLater
 
 from twisted.trial import unittest
 
 from twistedcaldav import carddavxml
-from twistedcaldav.vcard import Component as VCard
+from twistedcaldav.vcard import Component as VCard, Component
 from twistedcaldav.vcard import Component as VComponent
 
 from txdav.base.propertystore.base import PropertyName
 
 from txdav.carddav.datastore.test.common import CommonTests as AddressBookCommonTests, \
-    vcard4_text
+    vcard4_text, adbk1Root
 from txdav.carddav.datastore.test.test_file import setUpAddressBookStore
 from txdav.carddav.datastore.util import _migrateAddressbook, migrateHome
 
 from txdav.common.icommondatastore import NoSuchObjectResourceError
 from txdav.common.datastore.sql import EADDRESSBOOKTYPE, CommonObjectResource
-from txdav.common.datastore.sql_tables import  _ABO_KIND_PERSON, _ABO_KIND_GROUP, \
-    schema, _BIND_MODE_DIRECT, _BIND_STATUS_ACCEPTED, _BIND_MODE_WRITE, \
-    _BIND_STATUS_INVITED
-from txdav.common.datastore.test.util import buildStore
+from txdav.common.datastore.sql_tables import _ABO_KIND_PERSON, _ABO_KIND_GROUP, schema
+from txdav.common.datastore.test.util import cleanStore, CommonCommonTests, \
+    populateAddressBooksFrom
+from txdav.carddav.datastore.sql import AddressBook
 
 from txdav.xml.rfc2518 import GETContentLanguage, ResourceType
 
@@ -57,7 +57,12 @@ class AddressBookSQLStorageTests(AddressBookCommonTests, unittest.TestCase):
     @inlineCallbacks
     def setUp(self):
         yield super(AddressBookSQLStorageTests, self).setUp()
-        self._sqlStore = yield buildStore(self, self.notifierFactory)
+        yield self.buildStoreAndDirectory(
+            extraUids=(
+                u"home_empty",
+                u"homeNew",
+            )
+        )
         yield self.populate()
 
 
@@ -66,8 +71,8 @@ class AddressBookSQLStorageTests(AddressBookCommonTests, unittest.TestCase):
         populateTxn = self.storeUnderTest().newTransaction()
         for homeUID in self.requirements:
             addressbooks = self.requirements[homeUID]
+            home = yield populateTxn.addressbookHomeWithUID(homeUID, create=True)
             if addressbooks is not None:
-                home = yield populateTxn.addressbookHomeWithUID(homeUID, True)
                 addressbook = home.addressbook()
 
                 addressbookObjNames = addressbooks[addressbook.name()]
@@ -82,13 +87,6 @@ class AddressBookSQLStorageTests(AddressBookCommonTests, unittest.TestCase):
         self.notifierFactory.reset()
 
 
-    def storeUnderTest(self):
-        """
-        Create and return a L{AddressBookStore} for testing.
-        """
-        return self._sqlStore
-
-
     @inlineCallbacks
     def assertAddressbooksSimilar(self, a, b, bAddressbookFilter=None):
         """
@@ -163,8 +161,8 @@ class AddressBookSQLStorageTests(AddressBookCommonTests, unittest.TestCase):
         toHome = yield self.transactionUnderTest().addressbookHomeWithUID(
             "new-home", create=True)
         toAddressbook = yield toHome.addressbookWithName("addressbook")
-        ok, bad = (yield _migrateAddressbook(fromAddressbook, toAddressbook,
-                                  lambda x: x.component()))
+        ok, bad = (yield _migrateAddressbook(
+            fromAddressbook, toAddressbook, lambda x: x.component()))
         self.assertEqual(ok, 1)
         self.assertEqual(bad, 1)
 
@@ -207,25 +205,27 @@ class AddressBookSQLStorageTests(AddressBookCommonTests, unittest.TestCase):
         self.assertPropertiesSimilar(fromHome, toHome,)
 
 
+    @inlineCallbacks
     def test_addressBookHomeVersion(self):
         """
         The DATAVERSION column for new addressbook homes must match the
         ADDRESSBOOK-DATAVERSION value.
         """
 
-        home = yield self.transactionUnderTest().addressbookHomeWithUID("home_version")
+        self.transactionUnderTest()._homeClass[EADDRESSBOOKTYPE]._dataVersionValue = None
+        home = yield self.transactionUnderTest().addressbookHomeWithUID("home_version", create=True)
         self.assertTrue(home is not None)
         yield self.transactionUnderTest().commit
 
         txn = yield self.transactionUnderTest()
-        version = yield txn.calendarserverValue("ADDRESSBOOK-DATAVERSION")[0][0]
+        version = yield txn.calendarserverValue("ADDRESSBOOK-DATAVERSION")
         ch = schema.ADDRESSBOOK_HOME
-        homeVersion = yield Select(
+        homeVersion = (yield Select(
             [ch.DATAVERSION, ],
             From=ch,
             Where=ch.OWNER_UID == "home_version",
-        ).on(txn)[0][0]
-        self.assertEqual(int(homeVersion, version))
+        ).on(txn))[0][0]
+        self.assertEqual(int(homeVersion), int(version))
 
 
     @inlineCallbacks
@@ -238,7 +238,7 @@ class AddressBookSQLStorageTests(AddressBookCommonTests, unittest.TestCase):
         operations, that we do not block other reads of the table.
         """
 
-        addressbookStore = self._sqlStore
+        addressbookStore = self.store
 
         txn1 = addressbookStore.newTransaction()
         txn2 = addressbookStore.newTransaction()
@@ -290,7 +290,7 @@ class AddressBookSQLStorageTests(AddressBookCommonTests, unittest.TestCase):
         Test that two concurrent attempts to PUT different address book object resources to the
         same address book home does not cause a deadlock.
         """
-        addressbookStore = yield buildStore(self, self.notifierFactory)
+        addressbookStore = self.store
 
         # Provision the home and addressbook now
         txn = addressbookStore.newTransaction()
@@ -359,19 +359,19 @@ END:VCARD
         C{INSERT} that violates a unique constraint.
         """
 
-        addressbookStore = self._sqlStore
+        addressbookStore = self.store
 
         txn1 = addressbookStore.newTransaction()
         txn2 = addressbookStore.newTransaction()
 
         notification_uid1_1 = yield txn1.notificationsWithUID(
-           "uid1",
+            "uid1", create=True,
         )
 
         @inlineCallbacks
         def _defer_notification_uid1_2():
             notification_uid1_2 = yield txn2.notificationsWithUID(
-                "uid1",
+                "uid1", create=True,
             )
             yield txn2.commit()
             returnValue(notification_uid1_2)
@@ -396,7 +396,7 @@ END:VCARD
         """
         Test that kind property UID is stored correctly in database
         """
-        addressbookStore = yield buildStore(self, self.notifierFactory)
+        addressbookStore = self.store
 
         # Provision the home and addressbook, one user and one group
         txn = addressbookStore.newTransaction()
@@ -418,7 +418,7 @@ item1.X-ABADR:us
 UID:uid1
 END:VCARD
 """.replace("\n", "\r\n")
-            )
+        )
         self.assertEqual(person.resourceUID(), "uid1")
         abObject = yield adbk.createAddressBookObjectWithName("1.vcf", person)
         self.assertEqual(abObject.uid(), "uid1")
@@ -442,7 +442,7 @@ END:VCARD
         """
         Test that kind property vCard is stored correctly in database
         """
-        addressbookStore = yield buildStore(self, self.notifierFactory)
+        addressbookStore = self.store
 
         # Provision the home and addressbook, one user and one group
         txn = addressbookStore.newTransaction()
@@ -464,7 +464,7 @@ item1.X-ABADR:us
 UID:uid1
 END:VCARD
 """.replace("\n", "\r\n")
-            )
+        )
         self.assertEqual(person.resourceKind(), None)
         abObject = yield adbk.createAddressBookObjectWithName("p.vcf", person)
         self.assertEqual(abObject.kind(), _ABO_KIND_PERSON)
@@ -481,7 +481,7 @@ X-ADDRESSBOOKSERVER-KIND:group
 X-ADDRESSBOOKSERVER-MEMBER:urn:uuid:uid1
 END:VCARD
 """.replace("\n", "\r\n")
-            )
+        )
         abObject = self.assertEqual(group.resourceKind(), "group")
         abObject = yield adbk.createAddressBookObjectWithName("g.vcf", group)
         self.assertEqual(abObject.kind(), _ABO_KIND_GROUP)
@@ -498,7 +498,7 @@ X-ADDRESSBOOKSERVER-KIND:badgroup
 X-ADDRESSBOOKSERVER-MEMBER:urn:uuid:uid1
 END:VCARD
 """.replace("\n", "\r\n")
-            )
+        )
         abObject = self.assertEqual(badgroup.resourceKind(), "badgroup")
         abObject = yield adbk.createAddressBookObjectWithName("bg.vcf", badgroup)
         self.assertEqual(abObject.kind(), _ABO_KIND_PERSON)
@@ -533,7 +533,8 @@ END:VCARD
         """
         Test that kind property vCard is stored correctly in database
         """
-        addressbookStore = yield buildStore(self, self.notifierFactory)
+        addressbookStore = self.store
+        yield cleanStore(self, addressbookStore)
 
         # Provision the home and addressbook, one user and one group
         txn = addressbookStore.newTransaction()
@@ -555,7 +556,7 @@ item1.X-ABADR:us
 UID:uid1
 END:VCARD
 """.replace("\n", "\r\n")
-            )
+        )
         self.assertEqual(person.resourceKind(), None)
         personObject = yield adbk.createAddressBookObjectWithName("p.vcf", person)
 
@@ -571,16 +572,16 @@ X-ADDRESSBOOKSERVER-KIND:group
 X-ADDRESSBOOKSERVER-MEMBER:urn:uuid:uid3
 END:VCARD
 """.replace("\n", "\r\n")
-            )
+        )
         groupObject = yield adbk.createAddressBookObjectWithName("g.vcf", group)
 
-        aboForeignMembers = schema.ABO_FOREIGN_MEMBERS
         aboMembers = schema.ABO_MEMBERS
-        memberRows = yield Select([aboMembers.GROUP_ID, aboMembers.MEMBER_ID], From=aboMembers,).on(txn)
-        self.assertEqual(memberRows, [])
+        memberRows = yield Select([aboMembers.GROUP_ID, aboMembers.MEMBER_ID], From=aboMembers, Where=aboMembers.REMOVED == False).on(txn)
+        self.assertEqual(list(memberRows), [])
 
+        aboForeignMembers = schema.ABO_FOREIGN_MEMBERS
         foreignMemberRows = yield Select([aboForeignMembers.GROUP_ID, aboForeignMembers.MEMBER_ADDRESS], From=aboForeignMembers).on(txn)
-        self.assertEqual(foreignMemberRows, [[groupObject._resourceID, "urn:uuid:uid3"]])
+        self.assertEqual(map(list, foreignMemberRows), [[groupObject._resourceID, "urn:uuid:uid3"]])
 
         subgroup = VCard.fromString(
             """BEGIN:VCARD
@@ -594,26 +595,46 @@ X-ADDRESSBOOKSERVER-KIND:group
 X-ADDRESSBOOKSERVER-MEMBER:urn:uuid:uid1
 END:VCARD
 """.replace("\n", "\r\n")
-            )
+        )
         subgroupObject = yield adbk.createAddressBookObjectWithName("sg.vcf", subgroup)
 
-        memberRows = yield Select([aboMembers.GROUP_ID, aboMembers.MEMBER_ID], From=aboMembers,).on(txn)
-        self.assertEqual(sorted(memberRows), sorted([
-                                                     [groupObject._resourceID, subgroupObject._resourceID],
-                                                     [subgroupObject._resourceID, personObject._resourceID],
-                                                    ]))
+        memberRows = yield Select([aboMembers.GROUP_ID, aboMembers.MEMBER_ID], From=aboMembers, Where=aboMembers.REMOVED == False).on(txn)
+        self.assertEqual(
+            map(list, sorted(memberRows)),
+            sorted([
+                [groupObject._resourceID, subgroupObject._resourceID],
+                [subgroupObject._resourceID, personObject._resourceID],
+            ])
+        )
 
         foreignMemberRows = yield Select([aboForeignMembers.GROUP_ID, aboForeignMembers.MEMBER_ADDRESS], From=aboForeignMembers).on(txn)
-        self.assertEqual(foreignMemberRows, [])
+        self.assertEqual(list(foreignMemberRows), [])
 
         yield subgroupObject.remove()
-        memberRows = yield Select([aboMembers.GROUP_ID, aboMembers.MEMBER_ID], From=aboMembers,).on(txn)
-        self.assertEqual(memberRows, [])
+        memberRows = yield Select([aboMembers.GROUP_ID, aboMembers.MEMBER_ID, aboMembers.REMOVED, aboMembers.REVISION], From=aboMembers).on(txn)
 
-        foreignMemberRows = yield Select([aboForeignMembers.GROUP_ID, aboForeignMembers.MEMBER_ADDRESS], From=aboForeignMembers,
-                                                 #Where=(aboForeignMembers.GROUP_ID == groupObject._resourceID),
-                                                 ).on(txn)
-        self.assertEqual(foreignMemberRows, [[groupObject._resourceID, "urn:uuid:uid3"]])
+        # combine by groupID
+        groupIDToMemberRowMap = {}
+        for groupID, id, removed, version in memberRows:
+            memberRow = groupIDToMemberRowMap.get(groupID, [])
+            memberRow.append((id, removed, version))
+            groupIDToMemberRowMap[groupID] = memberRow
+
+        # see if this object is in current version
+        groupIDs = set([
+            groupID for groupID, memberIDRemovedRevisionRows in groupIDToMemberRowMap.iteritems()
+            if AddressBook._currentMemberIDsFromMemberIDRemovedRevisionRows(memberIDRemovedRevisionRows)
+        ])
+
+        self.assertEqual(len(groupIDs), 0)
+
+        foreignMemberRows = yield Select(
+            [aboForeignMembers.GROUP_ID, aboForeignMembers.MEMBER_ADDRESS], From=aboForeignMembers,
+        ).on(txn)
+        self.assertEqual(
+            map(list, foreignMemberRows),
+            [[groupObject._resourceID, "urn:uuid:uid3"]]
+        )
 
         yield home.removeAddressBookWithName("addressbook")
         yield txn.commit()
@@ -627,9 +648,11 @@ END:VCARD
         """
 
         prop = schema.RESOURCE_PROPERTY
-        _allWithID = Select([prop.NAME, prop.VIEWER_UID, prop.VALUE],
-                        From=prop,
-                        Where=prop.RESOURCE_ID == Parameter("resourceID"))
+        _allWithID = Select(
+            [prop.NAME, prop.VIEWER_UID, prop.VALUE],
+            From=prop,
+            Where=prop.RESOURCE_ID == Parameter("resourceID")
+        )
 
         # Create address book and add a property
         home = yield self.homeUnderTest()
@@ -679,9 +702,11 @@ END:VCARD
         resourceID = addressobject._resourceID
 
         prop = schema.RESOURCE_PROPERTY
-        _allWithID = Select([prop.NAME, prop.VIEWER_UID, prop.VALUE],
-                        From=prop,
-                        Where=prop.RESOURCE_ID == Parameter("resourceID"))
+        _allWithID = Select(
+            [prop.NAME, prop.VIEWER_UID, prop.VALUE],
+            From=prop,
+            Where=prop.RESOURCE_ID == Parameter("resourceID")
+        )
 
         # No properties on existing address book object
         rows = yield _allWithID.on(self.transactionUnderTest(), resourceID=resourceID)
@@ -710,7 +735,7 @@ END:VCARD
         work concurrently without an exception.
         """
 
-        addressbookStore = self._sqlStore
+        addressbookStore = self.store
 
         # Provision the home and addressbook now
         txn = addressbookStore.newTransaction()
@@ -741,13 +766,13 @@ END:VCARD
 
         @inlineCallbacks
         def _defer1():
-            yield ab1.shareWith(shareeHome=sharerHome1, mode=_BIND_MODE_DIRECT, status=_BIND_STATUS_ACCEPTED, message="Shared Wiki AddressBook")
+            yield ab1.directShareWithUser("uid2")
             yield txn1.commit()
         d1 = _defer1()
 
         @inlineCallbacks
         def _defer2():
-            yield ab2.shareWith(shareeHome=sharerHome2, mode=_BIND_MODE_DIRECT, status=_BIND_STATUS_ACCEPTED, message="Shared Wiki AddressBook")
+            yield ab2.directShareWithUser("uid2")
             yield txn2.commit()
         d2 = _defer2()
 
@@ -770,7 +795,7 @@ END:VCARD
         self.assertTrue(resource._locked)
 
         # Setup a new transaction to verify the lock and also verify wait behavior
-        newTxn = self._sqlStore.newTransaction()
+        newTxn = self.store.newTransaction()
         newResource = yield self.addressbookObjectUnderTest(txn=newTxn)
         try:
             yield newResource.lock(wait=False)
@@ -789,7 +814,7 @@ END:VCARD
         self.assertTrue(resource._locked)
 
         # Setup a new transaction to verify the lock but pass in an alternative txn directly
-        newTxn = self._sqlStore.newTransaction()
+        newTxn = self.store.newTransaction()
 
         # FIXME: not sure why, but without this statement here, this portion of the test fails in a funny way.
         # Basically the query in the try block seems to execute twice, failing each time, one of which is caught,
@@ -868,247 +893,225 @@ END:VCARD
 
 
     @inlineCallbacks
-    def test_shareWithRevision(self):
+    def test_dataVersion(self):
         """
-        Verify that bindRevision on addressbooks and shared addressbooks has the correct value.
+        Make sure L{AddressBookObject}'s data version is set when object is created.
         """
-        ab = yield self.addressbookUnderTest()
-        self.assertEqual(ab._bindRevision, 0)
-        other = yield self.homeUnderTest(name="home2")
-        newABShareUID = yield ab.shareWith(other, _BIND_MODE_WRITE)
-        yield self.commit()
-
-        normalAB = yield self.addressbookUnderTest()
-        self.assertEqual(normalAB._bindRevision, 0)
-        otherHome = yield self.homeUnderTest(name="home2")
-        otherAB = yield otherHome.objectWithShareUID(newABShareUID)
-        self.assertNotEqual(otherAB._bindRevision, 0)
-
-
-    @inlineCallbacks
-    def test_shareGroupWithRevision(self):
-        """
-        Verify that bindRevision on addressbooks and shared groups has the correct value.
-        """
-        ab = yield self.addressbookUnderTest(home="home3")
-        self.assertEqual(ab._bindRevision, 0)
-        group = yield ab.objectResourceWithName("4.vcf")
-        other = yield self.homeUnderTest(name="home2")
-        newGroupShareUID = yield group.shareWith(other, _BIND_MODE_WRITE)
-        yield self.commit()
-
-        normalAB = yield self.addressbookUnderTest(home="home3")
-        self.assertEqual(normalAB._bindRevision, 0)
-        otherHome = yield self.homeUnderTest(name="home2")
-        otherGroup = yield otherHome.objectWithShareUID(newGroupShareUID)
-        otherAB = otherGroup.addressbook()
-        self.assertNotEqual(otherAB._bindRevision, 0)
-
-
-    @inlineCallbacks
-    def test_updateShareRevision(self):
-        """
-        Verify that bindRevision on addressbooks and shared addressbooks has the correct value.
-        """
-        ab = yield self.addressbookUnderTest()
-        self.assertEqual(ab._bindRevision, 0)
-        other = yield self.homeUnderTest(name="home2")
-        newABShareUID = yield ab.shareWith(other, _BIND_MODE_WRITE, status=_BIND_STATUS_INVITED)
-        yield self.commit()
-
-        normalAB = yield self.addressbookUnderTest()
-        self.assertEqual(normalAB._bindRevision, 0)
-        otherHome = yield self.homeUnderTest(name="home2")
-        otherAB = yield otherHome.invitedObjectWithShareUID(newABShareUID)
-        self.assertEqual(otherAB._bindRevision, 0)
-        yield self.commit()
-
-        normalAB = yield self.addressbookUnderTest()
-        otherHome = yield self.homeUnderTest(name="home2")
-        otherAB = yield otherHome.invitedObjectWithShareUID(newABShareUID)
-        yield normalAB.updateShare(otherAB, status=_BIND_STATUS_ACCEPTED)
-        yield self.commit()
-
-        normalAB = yield self.addressbookUnderTest()
-        self.assertEqual(normalAB._bindRevision, 0)
-        otherHome = yield self.homeUnderTest(name="home2")
-        otherAB = yield otherHome.objectWithShareUID(newABShareUID)
-        self.assertNotEqual(otherAB._bindRevision, 0)
-
-
-    @inlineCallbacks
-    def test_updateSharedGroupRevision(self):
-        """
-        Verify that bindRevision on addressbooks and shared addressbooks has the correct value.
-        """
-        ab = yield self.addressbookUnderTest(home="home3")
-        self.assertEqual(ab._bindRevision, 0)
-        group = yield ab.objectResourceWithName("4.vcf")
-        other = yield self.homeUnderTest(name="home2")
-        newGroupShareUID = yield group.shareWith(other, _BIND_MODE_WRITE, status=_BIND_STATUS_INVITED)
-        yield self.commit()
-
-        normalAB = yield self.addressbookUnderTest(home="home3")
-        self.assertEqual(normalAB._bindRevision, 0)
-        otherHome = yield self.homeUnderTest(name="home2")
-        otherGroup = yield otherHome.invitedObjectWithShareUID(newGroupShareUID)
-        otherAB = otherGroup.addressbook()
-        self.assertEqual(otherAB._bindRevision, 0)
-        yield self.commit()
-
-        normalAB = yield self.addressbookUnderTest(home="home3")
-        normalGroup = yield normalAB.objectResourceWithName("4.vcf")
-        otherHome = yield self.homeUnderTest(name="home2")
-        otherGroup = yield otherHome.invitedObjectWithShareUID(newGroupShareUID)
-        yield normalGroup.updateShare(otherGroup, status=_BIND_STATUS_ACCEPTED)
-        yield self.commit()
-
-        normalAB = yield self.addressbookUnderTest(home="home3")
-        self.assertEqual(normalAB._bindRevision, 0)
-        otherHome = yield self.homeUnderTest(name="home2")
-        otherGroup = yield otherHome.objectWithShareUID(newGroupShareUID)
-        otherAB = otherGroup.addressbook()
-        self.assertNotEqual(otherAB._bindRevision, 0)
-
-
-    @inlineCallbacks
-    def test_sharedRevisions(self):
-        """
-        Verify that resourceNamesSinceRevision returns all resources after initial bind and sync.
-        """
-        ab = yield self.addressbookUnderTest()
-        self.assertEqual(ab._bindRevision, 0)
-        other = yield self.homeUnderTest(name="home2")
-        newABShareUID = yield ab.shareWith(other, _BIND_MODE_WRITE)
-        yield self.commit()
-
-        normalAB = yield self.addressbookUnderTest()
-        self.assertEqual(normalAB._bindRevision, 0)
-        otherHome = yield self.homeUnderTest(name="home2")
-        otherAB = yield otherHome.objectWithShareUID(newABShareUID)
-        self.assertNotEqual(otherAB._bindRevision, 0)
-
-        changed, deleted = yield otherAB.resourceNamesSinceRevision(otherAB._bindRevision - 1)
-        self.assertNotEqual(len(changed), 0)
-        self.assertEqual(len(deleted), 0)
-
-        changed, deleted = yield otherAB.resourceNamesSinceRevision(otherAB._bindRevision)
-        self.assertEqual(len(changed), 0)
-        self.assertEqual(len(deleted), 0)
-
-        for depth in ("1", "infinity",):
-            changed, deleted = yield otherHome.resourceNamesSinceRevision(otherAB._bindRevision - 1, depth)
-            self.assertNotEqual(len(changed), 0)
-            self.assertEqual(len(deleted), 0)
-
-            changed, deleted = yield otherHome.resourceNamesSinceRevision(otherAB._bindRevision, depth)
-            self.assertEqual(len(changed), 0)
-            self.assertEqual(len(deleted), 0)
-
-
-    @inlineCallbacks
-    def test_sharedGroupRevisions(self):
-        """
-        Verify that resourceNamesSinceRevision returns all resources after initial bind and sync.
-        """
-        ab = yield self.addressbookUnderTest(home="home3")
-        self.assertEqual(ab._bindRevision, 0)
-        group = yield ab.objectResourceWithName("4.vcf")
-        other = yield self.homeUnderTest(name="home2")
-        newGroupShareUID = yield group.shareWith(other, _BIND_MODE_WRITE)
-        yield self.commit()
-
-        normalAB = yield self.addressbookUnderTest(home="home3")
-        self.assertEqual(normalAB._bindRevision, 0)
-        otherHome = yield self.homeUnderTest(name="home2")
-        otherGroup = yield otherHome.objectWithShareUID(newGroupShareUID)
-        otherAB = otherGroup.addressbook()
-        self.assertNotEqual(otherAB._bindRevision, 0)
-
-        changed, deleted = yield otherAB.resourceNamesSinceRevision(otherAB._bindRevision - 1)
-        self.assertNotEqual(len(changed), 0)
-        self.assertEqual(len(deleted), 0)
-
-        changed, deleted = yield otherAB.resourceNamesSinceRevision(otherAB._bindRevision)
-        self.assertEqual(len(changed), 0)
-        self.assertEqual(len(deleted), 0)
-
-        for depth in ("1", "infinity",):
-            changed, deleted = yield otherHome.resourceNamesSinceRevision(otherAB._bindRevision - 1, depth)
-            self.assertNotEqual(len(changed), 0)
-            self.assertEqual(len(deleted), 0)
-
-            changed, deleted = yield otherHome.resourceNamesSinceRevision(otherAB._bindRevision, depth)
-            self.assertEqual(len(changed), 0)
-            self.assertEqual(len(deleted), 0)
-
-
-    @inlineCallbacks
-    def test_addressbookRevisionChangeConcurrency(self):
-        """
-        Test that two concurrent attempts to add resources in two separate
-        calendar homes does not deadlock on the revision table update.
-        """
-
-        # Make sure homes are provisioned
-        txn = self.transactionUnderTest()
-        home_uid1 = yield txn.homeWithUID(EADDRESSBOOKTYPE, "user01", create=True)
-        home_uid2 = yield txn.homeWithUID(EADDRESSBOOKTYPE, "user02", create=True)
-        self.assertNotEqual(home_uid1, None)
-        self.assertNotEqual(home_uid2, None)
-        yield self.commit()
-
-        # Create first events in different calendar homes
-        txn1 = self._sqlStore.newTransaction()
-        txn2 = self._sqlStore.newTransaction()
-
-        addressbook_uid1_in_txn1 = yield self.addressbookUnderTest(txn1, "addressbook", "user01")
-        addressbook_uid2_in_txn2 = yield self.addressbookUnderTest(txn2, "addressbook", "user02")
-
-        data = """BEGIN:VCARD
+        olddata = """BEGIN:VCARD
 VERSION:3.0
-PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
-UID:data%(ctr)s
-FN:Data %(ctr)s
-N:Sub Group;;;;
-REV:20120503T194243Z
+N:Thompson;Default1;;;
+FN:Default1 Thompson
+EMAIL;type=INTERNET;type=WORK;type=pref:lthompson1@example.com
+TEL;type=WORK;type=pref:1-555-555-5555
+TEL;type=CELL:1-444-444-4444
+item1.ADR;type=WORK;type=pref:;;1245 Test;Sesame Street;California;11111;USA
+item1.X-ABADR:us
+UID:uid-dataversion-test
 END:VCARD
-
 """
 
-        component = VComponent.fromString(data % {"ctr": 1})
-        yield addressbook_uid1_in_txn1.createAddressBookObjectWithName("data1.ics", component)
+        yield self.homeUnderTest()
+        adbk = yield self.addressbookUnderTest(name="addressbook")
+        yield adbk.createAddressBookObjectWithName("data1.ics", VCard.fromString(olddata))
+        yield self.commit()
 
-        component = VComponent.fromString(data % {"ctr": 2})
-        yield addressbook_uid2_in_txn2.createAddressBookObjectWithName("data2.ics", component)
+        obj = yield self.addressbookObjectUnderTest(name="data1.ics", addressbook_name="addressbook")
+        self.assertEqual(obj._dataversion, obj._currentDataVersion)
+        yield self.commit()
 
-        # Setup deferreds to run concurrently and create second events in the calendar homes
-        # previously used by the other transaction - this could create the deadlock.
-        @inlineCallbacks
-        def _defer_uid3():
-            addressbook_uid1_in_txn2 = yield self.addressbookUnderTest(txn2, "addressbook", "user01")
-            component = VComponent.fromString(data % {"ctr": 3})
-            yield addressbook_uid1_in_txn2.createAddressBookObjectWithName("data3.ics", component)
-            yield txn2.commit()
-        d1 = _defer_uid3()
+
+
+class SyncTests(CommonCommonTests, unittest.TestCase):
+    """
+    Revision table/sync report tests.
+    """
+
+    @inlineCallbacks
+    def setUp(self):
+        yield super(SyncTests, self).setUp()
+        yield self.buildStoreAndDirectory()
+        yield self.populate()
+
+
+    requirements = {
+        "user01": {
+            "addressbook": {
+                "1.vcf": adbk1Root.child("1.vcf").getContent(),
+                "2.vcf": adbk1Root.child("2.vcf").getContent(),
+                "3.vcf": adbk1Root.child("3.vcf").getContent(),
+            },
+            "not_a_addressbook": None
+        },
+    }
+
+
+    @inlineCallbacks
+    def populate(self):
+        yield populateAddressBooksFrom(self.requirements, self.storeUnderTest())
+        self.notifierFactory.reset()
+
+
+    @inlineCallbacks
+    def test_updateAfterRevisionCleanup(self):
+        """
+        Make sure L{AddressBookObject}'s can be updated or removed after revision cleanup
+        removes their revision table entry..
+        """
+        person = """BEGIN:VCARD
+VERSION:3.0
+N:Thompson;Default1;;;
+FN:Default1 Thompson
+EMAIL;type=INTERNET;type=WORK;type=pref:lthompson1@example.com
+TEL;type=WORK;type=pref:1-555-555-5555
+TEL;type=CELL:1-444-444-4444
+item1.ADR;type=WORK;type=pref:;;1245 Test;Sesame Street;California;11111;USA
+item1.X-ABADR:us
+UID:uid-person
+X-ADDRESSBOOKSERVER-KIND:person
+END:VCARD
+"""
+        group = """BEGIN:VCARD
+VERSION:3.0
+N:Group;Fancy;;;
+FN:Fancy Group
+UID:uid-group
+X-ADDRESSBOOKSERVER-KIND:group
+X-ADDRESSBOOKSERVER-MEMBER:urn:uuid:uid-person
+END:VCARD
+"""
+        group_update = """BEGIN:VCARD
+VERSION:3.0
+N:Group2;Fancy;;;
+FN:Fancy Group2
+UID:uid-group
+X-ADDRESSBOOKSERVER-KIND:group
+X-ADDRESSBOOKSERVER-MEMBER:urn:uuid:uid-person
+END:VCARD
+"""
+
+        yield self.addressbookHomeUnderTest(name="user01")
+        adbk = yield self.addressbookUnderTest(home="user01", name="addressbook")
+        yield adbk.createAddressBookObjectWithName("person.vcf", VCard.fromString(person))
+        yield adbk.createAddressBookObjectWithName("group.vcf", VCard.fromString(group))
+        yield self.commit()
+
+        # Remove the revision
+        adbk = yield self.addressbookUnderTest(home="user01", name="addressbook")
+        yield adbk.syncToken()
+        yield self.transactionUnderTest().deleteRevisionsBefore(adbk._syncTokenRevision + 1)
+        yield self.commit()
+
+        # Update the object
+        obj = yield self.addressbookObjectUnderTest(name="group.vcf", addressbook_name="addressbook", home="user01")
+        yield obj.setComponent(VCard.fromString(group_update))
+        yield self.commit()
+
+        obj = yield self.addressbookObjectUnderTest(name="group.vcf", addressbook_name="addressbook", home="user01")
+        self.assertTrue(obj is not None)
+        obj = yield self.addressbookObjectUnderTest(name="person.vcf", addressbook_name="addressbook", home="user01")
+        self.assertTrue(obj is not None)
+        yield self.commit()
+
+
+    @inlineCallbacks
+    def test_removeAfterRevisionCleanup(self):
+        """
+        Make sure L{AddressBookObject}'s can be updated or removed after revision cleanup
+        removes their revision table entry..
+        """
+        person = """BEGIN:VCARD
+VERSION:3.0
+N:Thompson;Default1;;;
+FN:Default1 Thompson
+EMAIL;type=INTERNET;type=WORK;type=pref:lthompson1@example.com
+TEL;type=WORK;type=pref:1-555-555-5555
+TEL;type=CELL:1-444-444-4444
+item1.ADR;type=WORK;type=pref:;;1245 Test;Sesame Street;California;11111;USA
+item1.X-ABADR:us
+UID:uid-person
+X-ADDRESSBOOKSERVER-KIND:person
+END:VCARD
+"""
+        group = """BEGIN:VCARD
+VERSION:3.0
+N:Group;Fancy;;;
+FN:Fancy Group
+UID:uid-group
+X-ADDRESSBOOKSERVER-KIND:group
+X-ADDRESSBOOKSERVER-MEMBER:urn:uuid:uid-person
+END:VCARD
+"""
+
+        yield self.addressbookHomeUnderTest(name="user01")
+        adbk = yield self.addressbookUnderTest(home="user01", name="addressbook")
+        yield adbk.createAddressBookObjectWithName("person.vcf", VCard.fromString(person))
+        yield adbk.createAddressBookObjectWithName("group.vcf", VCard.fromString(group))
+        yield self.commit()
+
+        # Remove the revision
+        adbk = yield self.addressbookUnderTest(home="user01", name="addressbook")
+        yield adbk.syncToken()
+        yield self.transactionUnderTest().deleteRevisionsBefore(adbk._syncTokenRevision + 1)
+        yield self.commit()
+
+        # Remove the object
+        obj = yield self.addressbookObjectUnderTest(name="group.vcf", addressbook_name="addressbook", home="user01")
+        self.assertTrue(obj is not None)
+        yield obj.remove()
+        yield self.commit()
+
+        obj = yield self.addressbookObjectUnderTest(name="group.vcf", addressbook_name="addressbook", home="user01")
+        self.assertTrue(obj is None)
+        obj = yield self.addressbookObjectUnderTest(name="person.vcf", addressbook_name="addressbook", home="user01")
+        self.assertTrue(obj is not None)
+        yield self.commit()
+
+
+    @inlineCallbacks
+    def test_revisionModified(self):
+        """
+        Make sure the revision table MODIFIED value changes for an update or delete
+        """
 
         @inlineCallbacks
-        def _defer_uid4():
-            addressbook_uid2_in_txn1 = yield self.addressbookUnderTest(txn1, "addressbook", "user02")
-            component = VComponent.fromString(data % {"ctr": 4})
-            yield addressbook_uid2_in_txn1.createAddressBookObjectWithName("data4.ics", component)
-            yield txn1.commit()
-        d2 = _defer_uid4()
+        def _getModified():
+            home = yield self.addressbookHomeUnderTest(name="user01")
+            addressbook = yield self.addressbookUnderTest(home="user01", name="addressbook")
+            rev = addressbook._revisionsSchema
+            modified = yield Select(
+                [rev.MODIFIED, ],
+                From=rev,
+                Where=(
+                    rev.ADDRESSBOOK_HOME_RESOURCE_ID == Parameter("homeID")).And(
+                    rev.RESOURCE_NAME == Parameter("resourceName")
+                )
+            ).on(
+                home._txn,
+                homeID=home.id(),
+                resourceName="1.vcf",
+            )
+            yield self.commit()
+            returnValue(modified[0][0])
 
-        # Now do the concurrent provision attempt
-        yield DeferredList([d1, d2])
+        # Get current modified
+        old_modified = yield _getModified()
+        self.assertNotEqual(old_modified, None)
 
-        # Verify we did not have a deadlock and all resources have been created.
-        vcarddata1 = yield self.addressbookObjectUnderTest(name="data1.ics", addressbook_name="addressbook", home="user01")
-        vcarddata2 = yield self.addressbookObjectUnderTest(name="data2.ics", addressbook_name="addressbook", home="user02")
-        vcarddata3 = yield self.addressbookObjectUnderTest(name="data3.ics", addressbook_name="addressbook", home="user01")
-        vcarddata4 = yield self.addressbookObjectUnderTest(name="data4.ics", addressbook_name="addressbook", home="user02")
-        self.assertNotEqual(vcarddata1, None)
-        self.assertNotEqual(vcarddata2, None)
-        self.assertNotEqual(vcarddata3, None)
-        self.assertNotEqual(vcarddata4, None)
+        # Update resource
+        aobj = yield self.addressbookObjectUnderTest(home="user01", addressbook_name="addressbook", name="1.vcf")
+        yield aobj.setComponent(Component.fromString(adbk1Root.child("1.vcf").getContent()))
+        yield self.commit()
+
+        # Modified changed
+        update_modified = yield _getModified()
+        self.assertGreater(update_modified, old_modified)
+
+        # Delete resource
+        aobj = yield self.addressbookObjectUnderTest(home="user01", addressbook_name="addressbook", name="1.vcf")
+        yield aobj.remove()
+        yield self.commit()
+
+        # Modified changed
+        delete_modified = yield _getModified()
+        self.assertGreater(delete_modified, old_modified)
+        self.assertGreater(delete_modified, update_modified)
