@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.caldav.datastore.test.test_file -*-
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,13 +24,14 @@ from twext.internet.decorate import memoizedKey
 from twext.python.log import Logger
 from txdav.xml.rfc2518 import GETContentType, HRef
 from txdav.xml.rfc5842 import ResourceID
-from twext.web2.http_headers import generateContentType, MimeType
-from twext.web2.dav.resource import TwistedGETContentMD5, \
+from txweb2.http_headers import generateContentType, MimeType
+from txweb2.dav.resource import TwistedGETContentMD5, \
     TwistedQuotaUsedProperty
 
 from twisted.internet.defer import succeed, inlineCallbacks, returnValue
 from twisted.python.util import FancyEqMixin
 from twisted.python import hashlib
+from twisted.python.failure import Failure
 
 from twistedcaldav import customxml
 from twistedcaldav.customxml import NotificationType
@@ -43,7 +44,8 @@ from txdav.common.datastore.sql_tables import _BIND_MODE_OWN
 from txdav.common.icommondatastore import HomeChildNameNotAllowedError, \
     HomeChildNameAlreadyExistsError, NoSuchHomeChildError, \
     InternalDataStoreError, ObjectResourceNameNotAllowedError, \
-    ObjectResourceNameAlreadyExistsError, NoSuchObjectResourceError
+    ObjectResourceNameAlreadyExistsError, NoSuchObjectResourceError, \
+    ECALENDARTYPE, EADDRESSBOOKTYPE
 from txdav.common.idirectoryservice import IStoreDirectoryService
 from txdav.common.inotifications import INotificationCollection, \
     INotificationObject
@@ -58,20 +60,10 @@ from txdav.base.propertystore.xattr import PropertyStore as XattrPropertyStore
 from errno import EEXIST, ENOENT
 from zope.interface import implements, directlyProvides
 
+import json
 import uuid
 from twistedcaldav.sql import AbstractSQLDatabase, db_prefix
 import os
-import types
-
-ECALENDARTYPE = 0
-EADDRESSBOOKTYPE = 1
-
-# Labels used to identify the class of resource being modified, so that
-# notification systems can target the correct application
-NotifierPrefixes = {
-    ECALENDARTYPE : "CalDAV",
-    EADDRESSBOOKTYPE : "CardDAV",
-}
 
 TOPPATHS = (
     "calendars",
@@ -82,8 +74,7 @@ UIDPATH = "__uids__"
 
 
 class _StubQueuer(object):
-    def transferProposalCallbacks(self, otherQueuer):
-        return otherQueuer
+    pass
 
 
 
@@ -147,6 +138,10 @@ class CommonDataStore(DataStore):
         return self._directoryService
 
 
+    def setDirectoryService(self, directoryService):
+        self._directoryService = directoryService
+
+
     def callWithNewTransactions(self, callback):
         """
         Registers a method to be called whenever a new transaction is
@@ -176,6 +171,41 @@ class CommonDataStore(DataStore):
         return txn
 
 
+    @inlineCallbacks
+    def inTransaction(self, label, operation, transactionCreator=None):
+        """
+        Perform the given operation in a transaction, committing or aborting as
+        required.
+
+        @param label: the label to pass to the transaction creator
+
+        @param operation: a 1-arg callable that takes an L{IAsyncTransaction} and
+            returns a value.
+
+        @param transactionCreator: a 1-arg callable that takes a "label" arg and
+            returns a transaction
+
+        @return: a L{Deferred} that fires with C{operation}'s result or fails with
+            its error, unless there is an error creating, aborting or committing
+            the transaction.
+        """
+
+        if transactionCreator is None:
+            transactionCreator = self.newTransaction
+
+        txn = transactionCreator(label=label)
+
+        try:
+            result = yield operation(txn)
+        except:
+            f = Failure()
+            yield txn.abort()
+            returnValue(f)
+        else:
+            yield txn.commit()
+            returnValue(result)
+
+
     @inlineCallbacks
     def _withEachHomeDo(self, enumerator, action, batchSize):
         """
@@ -245,7 +275,8 @@ class CommonDataStore(DataStore):
                             continue
                         txn = self.newTransaction("enumerate home %r" % (uid,))
                         home = txn.homeWithUID(storeType, uid, False)
-                        yield (txn, home)
+                        if home is not None:
+                            yield (txn, home)
 
 
     def _eachCalendarHome(self):
@@ -303,15 +334,15 @@ class CommonStoreTransaction(DataStoreTransaction):
         CommonStoreTransaction._homeClass[EADDRESSBOOKTYPE] = AddressBookHome
 
 
-    def calendarHomeWithUID(self, uid, create=False):
-        return self.homeWithUID(ECALENDARTYPE, uid, create=create)
+    def calendarHomeWithUID(self, uid, status=None, create=False):
+        return self.homeWithUID(ECALENDARTYPE, uid, status=status, create=create)
 
 
-    def addressbookHomeWithUID(self, uid, create=False):
-        return self.homeWithUID(EADDRESSBOOKTYPE, uid, create=create)
+    def addressbookHomeWithUID(self, uid, status=None, create=False):
+        return self.homeWithUID(EADDRESSBOOKTYPE, uid, status=status, create=create)
 
 
-    def _determineMemo(self, storeType, uid, create=False):
+    def _determineMemo(self, storeType, uid, status=None, create=False):
         """
         Determine the memo dictionary to use for homeWithUID.
         """
@@ -334,7 +365,7 @@ class CommonStoreTransaction(DataStoreTransaction):
 
 
     @memoizedKey("uid", _determineMemo, deferredResult=False)
-    def homeWithUID(self, storeType, uid, create=False):
+    def homeWithUID(self, storeType, uid, status=None, create=False):
         if uid.startswith("."):
             return None
 
@@ -345,7 +376,7 @@ class CommonStoreTransaction(DataStoreTransaction):
 
 
     @memoizedKey("uid", "_notificationHomes", deferredResult=False)
-    def notificationsWithUID(self, uid, home=None):
+    def notificationsWithUID(self, uid, home=None, create=False):
 
         if home is None:
             home = self.homeWithUID(self._notificationHomeType, uid, create=True)
@@ -479,7 +510,8 @@ class SharedCollectionsDatabase(AbstractSQLDatabase):
 
     def addOrUpdateRecord(self, record):
 
-        self._db_execute("""insert or replace into SHARES (SHAREUID, SHARETYPE, HOSTURL, LOCALNAME, SUMMARY)
+        self._db_execute(
+            """insert or replace into SHARES (SHAREUID, SHARETYPE, HOSTURL, LOCALNAME, SUMMARY)
             values (:1, :2, :3, :4, :5)
             """, record.shareuid, record.sharetype, record.hosturl, record.localname, record.summary,
         )
@@ -572,7 +604,7 @@ class SharedCollectionsDatabase(AbstractSQLDatabase):
 
     def _makeRecord(self, row):
 
-        return SharedCollectionRecord(*[str(item) if type(item) == types.UnicodeType else item for item in row])
+        return SharedCollectionRecord(*[str(item) if isinstance(item, unicode) else item for item in row])
 
 
 
@@ -699,7 +731,7 @@ class CommonHome(FileMetaDataMixin):
 
 
     def directoryRecord(self):
-        return self.directoryService().recordWithUID(self.uid())
+        return self.directoryService().recordWithUID(self.uid().decode("utf-8"))
 
 
     def retrieveOldShares(self):
@@ -717,7 +749,7 @@ class CommonHome(FileMetaDataMixin):
             self.childWithName(name)
             for name in self._path.listdir()
             if not name.startswith(".") and
-                name not in self._removedChildren
+            name not in self._removedChildren
         )
 
     # For file store there is no efficient "bulk" load of all children so just
@@ -735,8 +767,8 @@ class CommonHome(FileMetaDataMixin):
             name
             for name in self._path.listdir()
             if not name.startswith(".") and
-                self._path.child(name).isdir() and
-                name not in self._removedChildren
+            self._path.child(name).isdir() and
+            name not in self._removedChildren
         ))
 
 
@@ -816,7 +848,7 @@ class CommonHome(FileMetaDataMixin):
 
 
     @writeOperation
-    def removeChildWithName(self, name):
+    def removeChildWithName(self, name, useTrash=True):
         if name.startswith(".") or name in self._removedChildren:
             raise NoSuchHomeChildError(name)
 
@@ -833,6 +865,10 @@ class CommonHome(FileMetaDataMixin):
                 self._removedChildren.add(name)
 
 
+    def getTrash(self, create=False):
+        return succeed(None)
+
+
     @inlineCallbacks
     def syncToken(self):
 
@@ -851,7 +887,8 @@ class CommonHome(FileMetaDataMixin):
     def resourceNamesSinceToken(self, token, depth):
         deleted = []
         changed = []
-        return succeed((changed, deleted))
+        invalid = []
+        return succeed((changed, deleted, invalid))
 
 
     # @cached
@@ -1042,6 +1079,13 @@ class CommonHomeChild(FileMetaDataMixin, FancyEqMixin, HomeChildBase):
         return _BIND_MODE_OWN
 
 
+    def effectiveShareMode(self):
+        """
+        Stub implementation of L{ICalendar.effectiveShareMode}; always returns L{_BIND_MODE_OWN}.
+        """
+        return _BIND_MODE_OWN
+
+
     def owned(self):
         return self._owned
 
@@ -1190,6 +1234,9 @@ class CommonHomeChild(FileMetaDataMixin, FancyEqMixin, HomeChildBase):
         if name.startswith("."):
             raise ObjectResourceNameNotAllowedError(name)
 
+        if len(name) > 255:
+            raise ObjectResourceNameNotAllowedError(name)
+
         objectResourcePath = self._path.child(name)
         if objectResourcePath.exists():
             raise ObjectResourceNameAlreadyExistsError(name)
@@ -1370,6 +1417,7 @@ class CommonObjectResource(FileMetaDataMixin, FancyEqMixin):
 
         self._parentCollection.removedObjectResource(self)
 
+    purge = remove
 
     def _text(self):
         raise NotImplementedError
@@ -1486,8 +1534,7 @@ class NotificationCollection(CommonHomeChild):
             # Return undo
             return lambda: home._path.child(collectionName).remove()
 
-        txn.addOperation(do, "create notification child %r" %
-                          (collectionName,))
+        txn.addOperation(do, "create notification child %r" % (collectionName,))
         return c
 
     notificationObjects = CommonHomeChild.objectResources
@@ -1499,17 +1546,17 @@ class NotificationCollection(CommonHomeChild):
         return self.notificationObjectWithName(name)
 
 
-    def writeNotificationObject(self, uid, xmltype, xmldata):
+    def writeNotificationObject(self, uid, notificationtype, notificationdata):
         name = uid + ".xml"
         if name.startswith("."):
             raise ObjectResourceNameNotAllowedError(name)
 
         objectResource = NotificationObject(name, self)
-        objectResource.setData(uid, xmltype, xmldata)
+        objectResource.setData(uid, notificationtype, notificationdata)
         self._cachedObjectResources[name] = objectResource
 
         # Update database
-        self.retrieveOldIndex().addOrUpdateRecord(NotificationRecord(uid, name, xmltype.name))
+        self.retrieveOldIndex().addOrUpdateRecord(NotificationRecord(uid, name, notificationtype))
 
         self.notifyChanged()
 
@@ -1572,15 +1619,16 @@ class NotificationObject(CommonObjectResource):
 
 
     @writeOperation
-    def setData(self, uid, xmltype, xmldata, inserting=False):
+    def setData(self, uid, notificationtype, notificationdata, inserting=False):
 
         rname = uid + ".xml"
         self._parentCollection.retrieveOldIndex().addOrUpdateRecord(
-            NotificationRecord(uid, rname, xmltype.name)
+            NotificationRecord(uid, rname, notificationtype)
         )
 
-        self._xmldata = xmldata
-        md5 = hashlib.md5(xmldata).hexdigest()
+        self._notificationdata = notificationdata
+        notificationtext = json.dumps(self._notificationdata)
+        md5 = hashlib.md5(notificationtext).hexdigest()
 
         def do():
             backup = None
@@ -1591,7 +1639,7 @@ class NotificationObject(CommonObjectResource):
             try:
                 # FIXME: concurrency problem; if this write is interrupted
                 # halfway through, the underlying file will be corrupt.
-                fh.write(xmldata)
+                fh.write(notificationtext)
             finally:
                 fh.close()
             def undo():
@@ -1610,7 +1658,7 @@ class NotificationObject(CommonObjectResource):
 
         props = self.properties()
         props[PropertyName(*GETContentType.qname())] = GETContentType.fromString(generateContentType(MimeType("text", "xml", params={"charset": "utf-8"})))
-        props[PropertyName.fromElement(NotificationType)] = NotificationType(xmltype)
+        props[PropertyName.fromElement(NotificationType)] = NotificationType(json.dumps(notificationtype))
         props[PropertyName.fromElement(TwistedGETContentMD5)] = TwistedGETContentMD5.fromString(md5)
 
         # FIXME: the property store's flush() method may already have been
@@ -1620,11 +1668,11 @@ class NotificationObject(CommonObjectResource):
         # manipulation methods won't work.
         self._transaction.addOperation(self.properties().flush, "post-update property flush")
 
-    _xmldata = None
+    _notificationdata = None
 
-    def xmldata(self):
-        if self._xmldata is not None:
-            return self._xmldata
+    def notificationData(self):
+        if self._notificationdata is not None:
+            return self._notificationdata
         try:
             fh = self._path.open()
         except IOError, e:
@@ -1638,14 +1686,14 @@ class NotificationObject(CommonObjectResource):
         finally:
             fh.close()
 
-        return text
+        return json.loads(text)
 
 
     def uid(self):
         return self._uid
 
 
-    def xmlType(self):
+    def notificationType(self):
         # NB This is the NotificationType property element
         return self.properties()[PropertyName.fromElement(NotificationType)]
 
@@ -1658,6 +1706,7 @@ class NotificationObject(CommonObjectResource):
             (
                 PropertyName.fromElement(customxml.NotificationType),
             ),
+            (),
         )
 
 
