@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.carddav.datastore.test -*-
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,25 +23,23 @@ from __future__ import print_function
 
 import os
 
-from zope.interface.declarations import implements
-from txdav.common.idirectoryservice import IStoreDirectoryService, \
-    IStoreDirectoryRecord
-
 # FIXME: Don't import from calendarserver in txdav
 from calendarserver.push.notifier import Notifier
 
 from hashlib import md5
 
-from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.datetime import DateTime
 
-from random import Random
+from random import Random, randint
 
 from twext.python.log import Logger
-from twext.python.filepath import CachingFilePath
-from twext.python.vcomponent import VComponent
+from twext.python.filepath import CachingFilePath as FilePath
 from twext.enterprise.adbapi2 import ConnectionPool
-from twext.enterprise.ienterprise import AlreadyFinishedError
-from twext.web2.dav.resource import TwistedGETContentMD5
+from twext.enterprise.ienterprise import AlreadyFinishedError, POSTGRES_DIALECT, \
+    ORACLE_DIALECT
+from twext.enterprise.jobs.jobitem import JobItem
+from twext.enterprise.jobs.queue import ControllerQueue
+from twext.who.directory import DirectoryRecord
 
 from twisted.application.service import Service
 from twisted.internet import reactor
@@ -50,17 +48,23 @@ from twisted.internet.defer import returnValue
 from twisted.internet.task import deferLater
 from twisted.trial.unittest import TestCase
 
-from twistedcaldav.config import config
+from twistedcaldav import ical
+from twistedcaldav.config import config, ConfigDict
+from twistedcaldav.ical import Component as VComponent
 from twistedcaldav.stdconfig import DEFAULT_CONFIG
 from twistedcaldav.vcard import Component as ABComponent
 
 from txdav.base.datastore.dbapiclient import DiagnosticConnectionWrapper
 from txdav.base.datastore.subpostgres import PostgresService
+from txdav.base.datastore.suboracle import OracleService
 from txdav.base.propertystore.base import PropertyName
 from txdav.caldav.icalendarstore import ComponentUpdateState
 from txdav.common.datastore.sql import CommonDataStore, current_sql_schema
 from txdav.common.datastore.sql_tables import schema
 from txdav.common.icommondatastore import NoSuchHomeChildError
+from txdav.who.util import buildDirectory
+
+from txweb2.dav.resource import TwistedGETContentMD5
 
 from zope.interface.exceptions import BrokenMethodImplementation, \
     DoesNotImplement
@@ -69,6 +73,8 @@ from zope.interface.verify import verifyObject
 import gc
 
 
+DB_TYPE = (POSTGRES_DIALECT, "pyformat", "current.sql",)
+#DB_TYPE = (ORACLE_DIALECT, "numeric", "current-oracle-dialect.sql",)
 
 log = Logger()
 
@@ -100,142 +106,132 @@ def dumpConnectionStatus():
 
 
 
-class TestStoreDirectoryService(object):
-
-    implements(IStoreDirectoryService)
-
-    def __init__(self):
-        self.records = {}
-
-
-    def recordWithUID(self, uid):
-        return self.records.get(uid)
-
-
-    def recordWithGUID(self, guid):
-        for record in self.records.itervalues():
-            if record.guid == guid:
-                return record
-        return None
-
-
-    def addRecord(self, record):
-        self.records[record.uid] = record
-
-
-
-class TestStoreDirectoryRecord(object):
-
-    implements(IStoreDirectoryRecord)
-
-    def __init__(self, uid, shortNames, fullName, extras={}):
-        self.uid = uid
-        self.guid = uid
-        self.shortNames = shortNames
-        self.fullName = fullName
-        self.displayName = self.fullName if self.fullName else self.shortNames[0]
-        self.extras = extras
-
-
-
 class SQLStoreBuilder(object):
     """
     Test-fixture-builder which can construct a PostgresStore.
     """
-    sharedService = None
-    currentTestID = None
+    def __init__(self, count=0, **options):
+        self.sharedService = None
+        self.currentTestID = None
 
-    SHARED_DB_PATH = "_test_sql_db" + str(os.getpid())
+        self.sharedDBPath = "_test_sql_db-{}-{}".format(
+            os.getpid(), count
+        )
+
+        self.options = options
 
 
-    @classmethod
-    def createService(cls, serviceFactory):
+    def createService(self, serviceFactory):
         """
         Create a L{PostgresService} to use for building a store.
         """
-        dbRoot = CachingFilePath(cls.SHARED_DB_PATH)
-        return PostgresService(
-            dbRoot, serviceFactory, current_sql_schema, resetSchema=True,
-            databaseName="caldav",
-            options=[
-                "-c log_lock_waits=TRUE",
-                "-c log_statement=all",
-                "-c log_line_prefix='%p.%x '",
-                "-c fsync=FALSE",
-                "-c synchronous_commit=off",
-                "-c full_page_writes=FALSE",
-            ],
-            testMode=True
-        )
+        dbRoot = FilePath(self.sharedDBPath)
+        if DB_TYPE[0] == POSTGRES_DIALECT:
+            return PostgresService(
+                dbRoot, serviceFactory, current_sql_schema, resetSchema=True,
+                databaseName="caldav",
+                options=[
+                    "-c log_lock_waits=TRUE",
+                    "-c log_statement=all",
+                    "-c log_line_prefix='%p.%x '",
+                    "-c fsync=FALSE",
+                    "-c synchronous_commit=off",
+                    "-c full_page_writes=FALSE",
+                    "-c client-min-messages=warning",
+                ],
+                testMode=True
+            )
+        elif DB_TYPE[0] == ORACLE_DIALECT:
+            return OracleService(
+                dbRoot, serviceFactory,
+                testMode=True,
+                dsnUser=self.options.get("dsnUser"),
+            )
 
 
-    @classmethod
-    def childStore(cls):
+    def childStore(self):
         """
         Create a store suitable for use in a child process, that is hooked up
         to the store that a parent test process is managing.
         """
         disableMemcacheForTest(TestCase())
         staticQuota = 3000
-        attachmentRoot = (CachingFilePath(cls.SHARED_DB_PATH)
-                          .child("attachments"))
-        stubsvc = cls.createService(lambda cf: Service())
+        attachmentRoot = (FilePath(self.sharedDBPath).child("attachments"))
+        stubsvc = self.createService(lambda cf: Service())
 
-        cp = ConnectionPool(stubsvc.produceConnection, maxConnections=1)
+        cp = ConnectionPool(
+            stubsvc.produceConnection,
+            maxConnections=1,
+            dialect=DB_TYPE[0],
+            paramstyle=DB_TYPE[1],
+        )
         # Attach the service to the running reactor.
         cp.startService()
         reactor.addSystemEventTrigger("before", "shutdown", cp.stopService)
         cds = CommonDataStore(
             cp.connection,
             {"push": StubNotifierFactory(), },
-            TestStoreDirectoryService(),
+            None,
             attachmentRoot, "",
             quota=staticQuota
         )
         return cds
 
 
-    def buildStore(self, testCase, notifierFactory, directoryService=None):
+    def buildStore(
+        self, testCase, notifierFactory,
+        directoryService=None, homes=None, enableJobProcessing=True,
+    ):
         """
         Do the necessary work to build a store for a particular test case.
 
         @return: a L{Deferred} which fires with an L{IDataStore}.
         """
         disableMemcacheForTest(testCase)
-        dbRoot = CachingFilePath(self.SHARED_DB_PATH)
+        dbRoot = FilePath(self.sharedDBPath)
         attachmentRoot = dbRoot.child("attachments")
-        if directoryService is None:
-            directoryService = TestStoreDirectoryService()
+        # The directory will be given to us later via setDirectoryService
         if self.sharedService is None:
             ready = Deferred()
+
             def getReady(connectionFactory, storageService):
                 self.makeAndCleanStore(
-                    testCase, notifierFactory, directoryService, attachmentRoot
+                    testCase, notifierFactory, directoryService,
+                    attachmentRoot, enableJobProcessing
                 ).chainDeferred(ready)
                 return Service()
+
             self.sharedService = self.createService(getReady)
             self.sharedService.startService()
+
             def startStopping():
                 log.info("Starting stopping.")
                 self.sharedService.unpauseMonitor()
                 return self.sharedService.stopService()
-            reactor.addSystemEventTrigger(#@UndefinedVariable
+
+            reactor.addSystemEventTrigger(
                 "before", "shutdown", startStopping)
             result = ready
         else:
             result = self.makeAndCleanStore(
-                testCase, notifierFactory, directoryService, attachmentRoot
+                testCase, notifierFactory, directoryService,
+                attachmentRoot, enableJobProcessing
             )
+
         def cleanUp():
             def stopit():
                 self.sharedService.pauseMonitor()
             return deferLater(reactor, 0.1, stopit)
+
         testCase.addCleanup(cleanUp)
         return result
 
 
     @inlineCallbacks
-    def makeAndCleanStore(self, testCase, notifierFactory, directoryService, attachmentRoot):
+    def makeAndCleanStore(
+        self, testCase, notifierFactory, directoryService,
+        attachmentRoot, enableJobProcessing=True
+    ):
         """
         Create a L{CommonDataStore} specific to the given L{TestCase}.
 
@@ -252,38 +248,73 @@ class SQLStoreBuilder(object):
         attachmentRoot.createDirectory()
 
         currentTestID = testCase.id()
-        cp = ConnectionPool(self.sharedService.produceConnection,
-                            maxConnections=5)
+        cp = ConnectionPool(
+            self.sharedService.produceConnection,
+            maxConnections=4,
+            dialect=DB_TYPE[0],
+            paramstyle=DB_TYPE[1],
+        )
         quota = deriveQuota(testCase)
         store = CommonDataStore(
             cp.connection,
             {"push": notifierFactory} if notifierFactory is not None else {},
             directoryService,
             attachmentRoot,
-            "https://example.com/calendars/__uids__/%(home)s/attachments/%(name)s",
+            "https://example.com/calendars/__uids__/%(home)s/dropbox/%(dropbox_id)s/%(name)s",
             quota=quota
         )
         store.label = currentTestID
         cp.startService()
+
+        @inlineCallbacks
         def stopIt():
+
+            if enableJobProcessing:
+                txn = store.newTransaction()
+                jobs = yield JobItem.all(txn)
+                yield txn.commit()
+                yield pool.stopService()
+            else:
+                jobs = ()
+
             # active transactions should have been shut down.
             wasBusy = len(cp._busy)
             busyText = repr(cp._busy)
-            stop = cp.stopService()
-            def checkWasBusy(ignored):
+            result = yield cp.stopService()
+
+            if deriveValue(testCase, _SPECIAL_TXN_CLEAN, lambda tc: False):
                 if wasBusy:
                     testCase.fail("Outstanding Transactions: " + busyText)
-                return ignored
-            if deriveValue(testCase, _SPECIAL_TXN_CLEAN, lambda tc: False):
-                stop.addBoth(checkWasBusy)
-            return stop
+                returnValue(result)
+
+            if len(jobs):
+                testCase.fail("Jobs left in job queue {}: {}".format(
+                    testCase,
+                    ",".join([job.workType for job in jobs])
+                ))
+
+            returnValue(result)
+
         testCase.addCleanup(stopIt)
         yield self.cleanStore(testCase, store)
+
+        # Start the job queue after store is up and cleaned
+        if enableJobProcessing:
+            pool = ControllerQueue(
+                reactor, store.newTransaction, useWorkerPool=False
+            )
+            store.queuer = store.pool = pool
+            pool.startService()
+
         returnValue(store)
 
 
     @inlineCallbacks
     def cleanStore(self, testCase, storeToClean):
+
+        if "noCleanup" in self.options:
+            returnValue(None)
+
         cleanupTxn = storeToClean.sqlTxnFactory(
             "%s schema-cleanup" % (testCase.id(),)
         )
@@ -292,29 +323,44 @@ class SQLStoreBuilder(object):
         # table' statements are issued, so it's not possible to reference a
         # later table.  Therefore it's OK to drop them in the (reverse) order
         # that they happen to be in.
-        tables = [t.name for t in schema.model.tables #@UndefinedVariable
-                  # All tables with rows _in_ the schema are populated
-                  # exclusively _by_ the schema and shouldn't be manipulated
-                  # while the server is running, so we leave those populated.
-                  if not t.schemaRows][::-1]
+        tables = [
+            t.name for t in schema.model.tables #@UndefinedVariable
+            # All tables with rows _in_ the schema are populated
+            # exclusively _by_ the schema and shouldn't be manipulated
+            # while the server is running, so we leave those populated.
+            if not t.schemaRows
+        ][::-1]
 
         for table in tables:
             try:
                 yield cleanupTxn.execSQL("delete from " + table, [])
             except:
                 log.failure("delete table {table} failed", table=table)
+
+        # Change the starting values of sequences to random values
+        for sequence in schema.model.sequences: #@UndefinedVariable
+            try:
+                if cleanupTxn.dialect == POSTGRES_DIALECT:
+                    curval = (yield cleanupTxn.execSQL("select nextval('{}')".format(sequence.name), []))[0][0]
+                    yield cleanupTxn.execSQL("select setval('{}', {})".format(sequence.name, curval + randint(1, 10000)), [])
+                elif cleanupTxn.dialect == ORACLE_DIALECT:
+                    yield cleanupTxn.execSQL("alter sequence {} increment by {}".format(sequence.name, randint(1, 10000)), [])
+                    yield cleanupTxn.execSQL("select {}.nextval from dual".format(sequence.name), [])
+                    yield cleanupTxn.execSQL("alter sequence {} increment by {}".format(sequence.name, 1), [])
+            except:
+                log.failure("setval sequence '{}' failed", sequence=sequence.name)
+        yield cleanupTxn.execSQL("update CALENDARSERVER set VALUE = '1' where NAME = 'MIN-VALID-REVISION'", [])
+
         yield cleanupTxn.commit()
 
         # Deal with memcached items that must be cleared
-        from txdav.caldav.datastore.sql import CalendarHome
-        CalendarHome._cacher.flushAll()
-        from txdav.carddav.datastore.sql import AddressBookHome
-        AddressBookHome._cacher.flushAll()
+        storeToClean.queryCacher.flushAll()
         from txdav.base.propertystore.sql import PropertyStore
         PropertyStore._cacher.flushAll()
 
 theStoreBuilder = SQLStoreBuilder()
 buildStore = theStoreBuilder.buildStore
+cleanStore = theStoreBuilder.cleanStore
 
 
 _notSet = object()
@@ -451,46 +497,61 @@ def populateCalendarsFrom(requirements, store, migrating=False):
         populateTxn._migrating = True
     for homeUID in requirements:
         calendars = requirements[homeUID]
+        home = yield populateTxn.calendarHomeWithUID(homeUID, create=True)
         if calendars is not None:
-            home = yield populateTxn.calendarHomeWithUID(homeUID, True)
             # We don't want the default calendar or inbox to appear unless it's
             # explicitly listed.
             try:
-                yield home.removeCalendarWithName("calendar")
-                # FIXME: this should be an argument to the function, not a
-                # global configuration variable.  Related: this needs
-                # independent tests.
                 if config.RestrictCalendarsToOneComponentType:
-                    yield home.removeCalendarWithName("tasks")
-                yield home.removeCalendarWithName("inbox")
+                    for name in ical.allowedStoreComponents:
+                        yield home.removeCalendarWithName(
+                            home._componentCalendarName[name],
+                            useTrash=False
+                        )
+                else:
+                    yield home.removeCalendarWithName("calendar", useTrash=False)
+                yield home.removeCalendarWithName("inbox", useTrash=False)
             except NoSuchHomeChildError:
                 pass
+
             for calendarName in calendars:
                 calendarObjNames = calendars[calendarName]
+
                 if calendarObjNames is not None:
                     # XXX should not be yielding!  this SQL will be executed
                     # first!
                     yield home.createCalendarWithName(calendarName)
                     calendar = yield home.calendarWithName(calendarName)
+
                     for objectName in calendarObjNames:
                         objData, metadata = calendarObjNames[objectName]
+
                         yield calendar._createCalendarObjectWithNameInternal(
                             objectName,
-                            VComponent.fromString(updateToCurrentYear(objData)),
+                            VComponent.fromString(
+                                updateToCurrentYear(objData)
+                            ),
                             internal_state=ComponentUpdateState.RAW,
                             options=metadata,
                         )
+
     yield populateTxn.commit()
 
 
 
 def updateToCurrentYear(data):
     """
-    Update the supplied iCalendar data so that all dates are updated to the current year.
+    Update the supplied iCalendar data so that all dates are updated to the
+    current year.
     """
 
-    nowYear = PyCalendarDateTime.getToday().getYear()
-    return data % {"now": nowYear}
+    subs = {}
+    nowYear = DateTime.getToday().getYear()
+    subs["now"] = nowYear
+    for i in range(1, 10):
+        subs["now-{}".format(i)] = nowYear - 1
+        subs["now+{}".format(i)] = nowYear + 1
+    return data % subs
 
 
 
@@ -508,7 +569,7 @@ def resetCalendarMD5s(md5s, store):
     for homeUID in md5s:
         calendars = md5s[homeUID]
         if calendars is not None:
-            home = yield populateTxn.calendarHomeWithUID(homeUID, True)
+            home = yield populateTxn.calendarHomeWithUID(homeUID, create=True)
             for calendarName in calendars:
                 calendarObjNames = calendars[calendarName]
                 if calendarObjNames is not None:
@@ -517,10 +578,12 @@ def resetCalendarMD5s(md5s, store):
                     calendar = yield home.calendarWithName(calendarName)
                     for objectName in calendarObjNames:
                         md5 = calendarObjNames[objectName]
-                        obj = yield calendar.calendarObjectWithName(
-                            objectName,
+                        obj = (
+                            yield calendar.calendarObjectWithName(objectName)
+                        )
+                        obj.properties()[md5key] = (
+                            TwistedGETContentMD5.fromString(md5)
                         )
-                        obj.properties()[md5key] = TwistedGETContentMD5.fromString(md5)
     yield populateTxn.commit()
 
 
@@ -539,7 +602,7 @@ def populateAddressBooksFrom(requirements, store):
     for homeUID in requirements:
         addressbooks = requirements[homeUID]
         if addressbooks is not None:
-            home = yield populateTxn.addressbookHomeWithUID(homeUID, True)
+            home = yield populateTxn.addressbookHomeWithUID(homeUID, create=True)
             # We don't want the default addressbook
             try:
                 yield home.removeAddressBookWithName("addressbook")
@@ -551,7 +614,9 @@ def populateAddressBooksFrom(requirements, store):
                     # XXX should not be yielding!  this SQL will be executed
                     # first!
                     yield home.createAddressBookWithName(addressbookName)
-                    addressbook = yield home.addressbookWithName(addressbookName)
+                    addressbook = (
+                        yield home.addressbookWithName(addressbookName)
+                    )
                     for objectName in addressbookObjNames:
                         objData = addressbookObjNames[objectName]
                         yield addressbook.createAddressBookObjectWithName(
@@ -576,19 +641,23 @@ def resetAddressBookMD5s(md5s, store):
     for homeUID in md5s:
         addressbooks = md5s[homeUID]
         if addressbooks is not None:
-            home = yield populateTxn.addressbookHomeWithUID(homeUID, True)
+            home = yield populateTxn.addressbookHomeWithUID(homeUID, create=True)
             for addressbookName in addressbooks:
                 addressbookObjNames = addressbooks[addressbookName]
                 if addressbookObjNames is not None:
                     # XXX should not be yielding!  this SQL will be executed
                     # first!
-                    addressbook = yield home.addressbookWithName(addressbookName)
+                    addressbook = (
+                        yield home.addressbookWithName(addressbookName)
+                    )
                     for objectName in addressbookObjNames:
                         md5 = addressbookObjNames[objectName]
                         obj = yield addressbook.addressbookObjectWithName(
                             objectName,
                         )
-                        obj.properties()[md5key] = TwistedGETContentMD5.fromString(md5)
+                        obj.properties()[md5key] = (
+                            TwistedGETContentMD5.fromString(md5)
+                        )
     yield populateTxn.commit()
 
 
@@ -610,6 +679,104 @@ def assertProvides(testCase, interface, provider):
 
 
 
+def buildTestDirectory(
+    store, dataRoot, accounts=None, resources=None, augments=None,
+    proxies=None, serversDB=None, cacheSeconds=0,
+):
+    """
+    @param store: the store for the directory to use
+
+    @param dataRoot: the directory to copy xml files to
+
+    @param accounts: path to the accounts.xml file
+    @type accounts: L{FilePath}
+
+    @param resources: path to the resources.xml file
+    @type resources: L{FilePath}
+
+    @param augments: path to the augments.xml file
+    @type augments: L{FilePath}
+
+    @param proxies: path to the proxies.xml file
+    @type proxies: L{FilePath}
+
+    @return: the directory service
+    @rtype: L{IDirectoryService}
+    """
+
+    defaultDirectory = FilePath(__file__).sibling("accounts")
+    if accounts is None:
+        accounts = defaultDirectory.child("accounts.xml")
+    if resources is None:
+        resources = defaultDirectory.child("resources.xml")
+    if augments is None:
+        augments = defaultDirectory.child("augments.xml")
+    if proxies is None:
+        proxies = defaultDirectory.child("proxies.xml")
+
+    if not os.path.exists(dataRoot):
+        os.makedirs(dataRoot)
+
+    accountsCopy = FilePath(dataRoot).child("accounts.xml")
+    accountsCopy.setContent(accounts.getContent())
+
+    resourcesCopy = FilePath(dataRoot).child("resources.xml")
+    resourcesCopy.setContent(resources.getContent())
+
+    augmentsCopy = FilePath(dataRoot).child("augments.xml")
+    augmentsCopy.setContent(augments.getContent())
+
+    proxiesCopy = FilePath(dataRoot).child("proxies.xml")
+    proxiesCopy.setContent(proxies.getContent())
+
+    servicesInfo = (
+        ConfigDict(
+            {
+                "Enabled": True,
+                "type": "xml",
+                "params": {
+                    "xmlFile": "accounts.xml",
+                    "recordTypes": ("users", "groups"),
+                },
+            }
+        ),
+        ConfigDict(
+            {
+                "Enabled": True,
+                "type": "xml",
+                "params": {
+                    "xmlFile": "resources.xml",
+                    "recordTypes": ("locations", "resources", "addresses"),
+                },
+            }
+        ),
+    )
+    augmentServiceInfo = ConfigDict(
+        {
+            "type": "twistedcaldav.directory.augment.AugmentXMLDB",
+            "params": {
+                "xmlFiles": ["augments.xml", ],
+                "statSeconds": 15,
+            },
+        }
+    )
+    wikiServiceInfo = ConfigDict(
+        {
+            "Enabled": True,
+            "EndpointDescriptor": "tcp:host=localhost:port=4444",
+        }
+    )
+    directory = buildDirectory(
+        store, dataRoot, servicesInfo, augmentServiceInfo, wikiServiceInfo,
+        serversDB, cacheSeconds
+    )
+
+    store.setDirectoryService(directory)
+
+    return directory
+
+
+
 class CommonCommonTests(object):
     """
     Common utility functionality for file/store combination tests.
@@ -619,6 +786,85 @@ class CommonCommonTests(object):
     savedStore = None
     assertProvides = assertProvides
 
+
+    @inlineCallbacks
+    def buildStoreAndDirectory(
+        self, accounts=None, resources=None, augments=None, proxies=None,
+        extraUids=None, serversDB=None, cacheSeconds=0,
+        storeBuilder=theStoreBuilder,
+    ):
+
+        self.serverRoot = self.mktemp()
+        os.mkdir(self.serverRoot)
+
+        self.counter = 0
+        self.notifierFactory = StubNotifierFactory()
+
+        config.reset()
+        self.configure()
+
+        self.store = yield self.buildStore(storeBuilder)
+        self._sqlCalendarStore = self.store  # FIXME: remove references to this
+
+        self.directory = buildTestDirectory(
+            self.store, config.DataRoot,
+            accounts=accounts, resources=resources,
+            augments=augments, proxies=proxies,
+            serversDB=serversDB, cacheSeconds=cacheSeconds
+        )
+        if extraUids:
+            for uid in extraUids:
+                yield self.addRecordFromFields(
+                    {
+                        self.directory.fieldName.uid:
+                            uid,
+                        self.directory.fieldName.recordType:
+                            self.directory.recordType.user,
+                    }
+                )
+
+
+    def configure(self):
+        """
+        Modify the configuration to suit unit tests, with a mktemp-created
+        ServerRoot
+        """
+
+        config.ServerRoot = os.path.abspath(self.serverRoot)
+        config.ConfigRoot = "config"
+        config.LogRoot = "logs"
+        config.RunRoot = "logs"
+
+        if not os.path.exists(config.DataRoot):
+            os.makedirs(config.DataRoot)
+        if not os.path.exists(config.DocumentRoot):
+            os.makedirs(config.DocumentRoot)
+        if not os.path.exists(config.ConfigRoot):
+            os.makedirs(config.ConfigRoot)
+        if not os.path.exists(config.LogRoot):
+            os.makedirs(config.LogRoot)
+
+        # Work queues for implicit scheduling slow down tests a lot and require
+        # them all to add "waits" for work to complete.
+        # Rewriting all the current tests to do that is not practical right
+        # now, so we will turn this off by default.
+        # Instead we will have a set of tests dedicated to work queue-based
+        # scheduling which will patch this option to True.
+        config.Scheduling.Options.WorkQueues.Enabled = False
+
+        self.config = config
+
+
+    def buildStore(self, storeBuilder=theStoreBuilder):
+        """
+        Builds and returns a store
+        """
+
+        # Build the store before the directory; the directory will be assigned
+        # to the store via setDirectoryService()
+        return storeBuilder.buildStore(self, self.notifierFactory, None)
+
+
     def transactionUnderTest(self, txn=None):
         """
         Create a transaction from C{storeUnderTest} and save it as
@@ -637,20 +883,25 @@ class CommonCommonTests(object):
         """
         if self.savedStore is None:
             self.savedStore = self.storeUnderTest()
+
         self.counter += 1
+
         if txn is None:
             txn = self.savedStore.newTransaction(
                 self.id() + " #" + str(self.counter)
             )
         else:
             txn._label = self.id() + " #" + str(self.counter)
+
         @inlineCallbacks
         def maybeCommitThis():
             try:
                 yield txn.commit()
             except AlreadyFinishedError:
                 pass
+
         self.addCleanup(maybeCommitThis)
+
         return txn
 
 
@@ -674,46 +925,133 @@ class CommonCommonTests(object):
         return result
 
 
-    def setUp(self):
-        self.counter = 0
-        self.notifierFactory = StubNotifierFactory()
-
-
     def storeUnderTest(self):
         """
-        Subclasses must implement this method.
+        Create and return the L{CalendarStore} for testing.
         """
-        raise NotImplementedError("CommonCommonTests subclasses must implement.")
+        return self.store
 
 
-    @inlineCallbacks
-    def homeUnderTest(self, txn=None, name="home1"):
+    def homeUnderTest(self, txn=None, name="home1", status=None, create=False):
         """
         Get the calendar home detailed by C{requirements['home1']}.
         """
         if txn is None:
             txn = self.transactionUnderTest()
-        returnValue((yield txn.calendarHomeWithUID(name)))
+        return txn.calendarHomeWithUID(name, status=status, create=create)
 
 
     @inlineCallbacks
-    def calendarUnderTest(self, txn=None, name="calendar_1", home="home1"):
+    def calendarUnderTest(self, txn=None, name="calendar_1", home="home1", status=None):
         """
         Get the calendar detailed by C{requirements['home1']['calendar_1']}.
         """
-        returnValue((yield
-            (yield self.homeUnderTest(txn, home)).calendarWithName(name))
-        )
+        home = yield self.homeUnderTest(txn, home, status=status)
+        calendar = yield home.calendarWithName(name)
+        returnValue(calendar)
 
 
     @inlineCallbacks
-    def calendarObjectUnderTest(self, txn=None, name="1.ics", calendar_name="calendar_1", home="home1"):
+    def calendarObjectUnderTest(
+        self, txn=None, name="1.ics", calendar_name="calendar_1", home="home1", status=None
+    ):
         """
         Get the calendar detailed by
         C{requirements[home][calendar_name][name]}.
         """
-        returnValue((yield (yield self.calendarUnderTest(txn, name=calendar_name, home=home))
-                     .calendarObjectWithName(name)))
+        calendar = yield self.calendarUnderTest(
+            txn, name=calendar_name, home=home, status=status
+        )
+        object = yield calendar.calendarObjectWithName(name)
+        returnValue(object)
+
+
+    @inlineCallbacks
+    def trashObjectsUnderTest(
+        self, txn=None, home="home1", status=None
+    ):
+        """
+        Get the calendar detailed by
+        C{requirements[home][calendar_name][name]}.
+        """
+        home = yield self.homeUnderTest(name=home)
+        trash = yield home.getTrash()
+        objects = yield trash.objectResources()
+        returnValue(objects)
+
+
+    def addressbookHomeUnderTest(self, txn=None, name="home1"):
+        """
+        Get the addressbook home detailed by C{requirements['home1']}.
+        """
+        if txn is None:
+            txn = self.transactionUnderTest()
+        return txn.addressbookHomeWithUID(name)
+
+
+    @inlineCallbacks
+    def addressbookUnderTest(self, txn=None, name="addressbook", home="home1"):
+        """
+        Get the addressbook detailed by
+        C{requirements['home1']['addressbook']}.
+        """
+        home = yield self.addressbookHomeUnderTest(txn=txn, name=home)
+        addressbook = yield home.addressbookWithName(name)
+        returnValue(addressbook)
+
+
+    @inlineCallbacks
+    def addressbookObjectUnderTest(
+        self, txn=None, name="1.vcf",
+        addressbook_name="addressbook", home="home1",
+    ):
+        """
+        Get the addressbook detailed by
+        C{requirements['home1']['addressbook']['1.vcf']}.
+        """
+        addressBook = yield self.addressbookUnderTest(
+            txn=txn, name=addressbook_name, home=home
+        )
+        object = yield addressBook.addressbookObjectWithName(name)
+        returnValue(object)
+
+
+    def notificationCollectionUnderTest(self, txn=None, name="home1", status=None, create=False):
+        if txn is None:
+            txn = self.transactionUnderTest()
+        return txn.notificationsWithUID(name, status=status, create=create)
+
+
+    def userRecordWithShortName(self, shortname):
+        return self.directory.recordWithShortName(
+            self.directory.recordType.user, shortname
+        )
+
+
+    @inlineCallbacks
+    def userUIDFromShortName(self, shortname):
+        record = yield self.directory.recordWithShortName(
+            self.directory.recordType.user, shortname
+        )
+        returnValue(record.uid if record is not None else None)
+
+
+    def addRecordFromFields(self, fields):
+        updatedRecord = DirectoryRecord(self.directory, fields)
+        return self.directory.updateRecords((updatedRecord,), create=True)
+
+
+    def removeRecord(self, uid):
+        return self.directory.removeRecords([uid])
+
+
+    def changeRecord(self, record, fieldname, value, directory=None):
+        if directory is None:
+            directory = self.directory
+        fields = record.fields.copy()
+        fields[fieldname] = value
+        updatedRecord = DirectoryRecord(directory, fields)
+        return directory.updateRecords((updatedRecord,))
 
 
 
