@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.common.datastore.upgrade.sql.test -*-
 ##
-# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,9 +21,9 @@ CALENDAR_HOME/ADDRESSBOOK_HOME/NOTIFICATION/APN_SUBSCRIPTIONS tables, as well
 as in calendar data and properties.
 """
 
-from txdav.common.datastore.sql import fixUUIDNormalization
 from twisted.internet.defer import inlineCallbacks
 from txdav.common.datastore.upgrade.sql.upgrades.util import updateCalendarDataVersion
+from txdav.common.datastore.sql_util import fixUUIDNormalization
 
 UPGRADE_TO_VERSION = 3
 
