@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ Tests for L{txdav.common.datastore.upgrade.migrate}.
 from twext.enterprise.adbapi2 import Pickle
 from twext.enterprise.dal.syntax import Delete
 from twext.python.filepath import CachingFilePath
-from twext.web2.http_headers import MimeType
+from txweb2.http_headers import MimeType
 
 from twisted.internet.defer import inlineCallbacks, Deferred, returnValue
 from twisted.internet.protocol import Protocol
@@ -39,17 +39,16 @@ from txdav.caldav.datastore.test.common import CommonTests
 from txdav.carddav.datastore.test.common import CommonTests as ABCommonTests
 from txdav.common.datastore.file import CommonDataStore
 from txdav.common.datastore.sql_tables import schema
-from txdav.common.datastore.test.util import SQLStoreBuilder, \
-    TestStoreDirectoryService
-from txdav.common.datastore.test.util import theStoreBuilder, \
-    populateCalendarsFrom, StubNotifierFactory, resetCalendarMD5s, \
-    populateAddressBooksFrom, resetAddressBookMD5s, deriveValue, \
-    withSpecialValue
+from txdav.common.datastore.test.util import SQLStoreBuilder
+from txdav.common.datastore.test.util import (
+    populateCalendarsFrom, StubNotifierFactory, resetCalendarMD5s,
+    populateAddressBooksFrom, resetAddressBookMD5s, deriveValue,
+    withSpecialValue, CommonCommonTests
+)
 from txdav.common.datastore.upgrade.migrate import UpgradeToDatabaseStep, \
     StoreSpawnerService, swapAMP
 from txdav.xml import element
 
-
 import copy
 
 
@@ -87,8 +86,8 @@ class StoreCreator(AMP):
 
     @PickleConfig.responder
     def pickleConfig(self, config, delegateTo):
-        #from twistedcaldav.config import config as globalConfig
-        #globalConfig._data = config._data
+        # from twistedcaldav.config import config as globalConfig
+        # globalConfig._data = config._data
         swapAMP(self, namedAny(delegateTo)(config))
         return {}
 
@@ -130,7 +129,7 @@ class StubSpawner(StoreSpawnerService):
 
 
 
-class HomeMigrationTests(TestCase):
+class HomeMigrationTests(CommonCommonTests, TestCase):
     """
     Tests for L{UpgradeToDatabaseStep}.
     """
@@ -162,15 +161,27 @@ END:VCALENDAR
         """
         Set up two stores to migrate between.
         """
+
+        yield super(HomeMigrationTests, self).setUp()
+        yield self.buildStoreAndDirectory(
+            extraUids=(
+                u"home1",
+                u"home2",
+                u"home3",
+                u"home_defaults",
+                u"home_no_splits",
+                u"home_splits",
+                u"home_splits_shared",
+            )
+        )
+        self.sqlStore = self.store
+
         # Add some files to the file store.
 
         self.filesPath = CachingFilePath(self.mktemp())
         self.filesPath.createDirectory()
         fileStore = self.fileStore = CommonDataStore(
-            self.filesPath, {"push": StubNotifierFactory()}, TestStoreDirectoryService(), True, True
-        )
-        self.sqlStore = yield theStoreBuilder.buildStore(
-            self, StubNotifierFactory()
+            self.filesPath, {"push": StubNotifierFactory()}, self.directory, True, True
         )
         self.upgrader = UpgradeToDatabaseStep(self.fileStore, self.sqlStore)
 
@@ -271,10 +282,10 @@ END:VCALENDAR
         self.addCleanup(txn.commit)
         self.assertIdentical(
             None,
-            (yield (yield (yield
-                (yield txn.calendarHomeWithUID("home1"))
-                          .calendarWithName("calendar_1")))
-                          .calendarObjectWithName("bogus.ics"))
+            (yield (yield (yield (
+                yield txn.calendarHomeWithUID("home1")
+            ).calendarWithName("calendar_1"))
+            ).calendarObjectWithName("bogus.ics"))
         )
 
 
@@ -284,6 +295,15 @@ END:VCALENDAR
         L{UpgradeToDatabaseService.startService} will do the upgrade, then
         start its dependent service by adding it to its service hierarchy.
         """
+
+        # Create a fake directory in the same place as a home, but with a non-existent uid
+        fake_dir = self.filesPath.child("calendars").child("__uids__").child("ho").child("me").child("foobar")
+        fake_dir.makedirs()
+
+        # Create a fake file in the same place as a home,with a name that matches the hash uid prefix
+        fake_file = self.filesPath.child("calendars").child("__uids__").child("ho").child("me").child("home_file")
+        fake_file.setContent("")
+
         yield self.upgrader.stepWithResult(None)
         txn = self.sqlStore.newTransaction()
         self.addCleanup(txn.commit)
@@ -309,6 +329,35 @@ END:VCALENDAR
             self.assertEquals(object.md5(), md5)
 
 
+    @withSpecialValue(
+        "extraRequirements",
+        {
+            "nonexistent": {
+                "calendar_1": {
+                }
+            }
+        }
+    )
+    @inlineCallbacks
+    def test_upgradeCalendarHomesMissingDirectoryRecord(self):
+        """
+        Test an upgrade where a directory record is missing for a home;
+        the original home directory will remain on disk.
+        """
+        yield self.upgrader.stepWithResult(None)
+        txn = self.sqlStore.newTransaction()
+        self.addCleanup(txn.commit)
+        for uid in CommonTests.requirements:
+            if CommonTests.requirements[uid] is not None:
+                self.assertNotIdentical(
+                    None, (yield txn.calendarHomeWithUID(uid))
+                )
+        self.assertIdentical(None, (yield txn.calendarHomeWithUID(u"nonexistent")))
+        # Skipped calendar homes are not deleted
+        self.assertTrue(self.filesPath.child("calendars").child(
+            "__uids__").child("no").child("ne").child("nonexistent").exists())
+
+
     @inlineCallbacks
     def test_upgradeExistingHome(self):
         """
