@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2015 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -87,7 +87,7 @@ class IPropertyStore(IMapping):
     using C{abort()}.
 
     Also, keys must be L{IPropertyName} providers and values must be
-    L{twext.web2.element.dav.base.WeDAVElement}s.
+    L{txweb2.element.dav.base.WeDAVElement}s.
     """
     # FIXME: the type for values isn't quite right, there should be some more
     # specific interface for that.
@@ -240,9 +240,9 @@ class ChangeCategory(Values):
     types of changes to the appropriate push priority level.
     TODO: make these values configurable in plist perhaps.
     """
-    default             = ValueConstant(PushPriority.high)
-    inbox               = ValueConstant(PushPriority.medium)
-    attendeeITIPUpdate  = ValueConstant(PushPriority.medium)
+    default = ValueConstant(PushPriority.high)
+    inbox = ValueConstant(PushPriority.medium)
+    attendeeITIPUpdate = ValueConstant(PushPriority.medium)
     organizerITIPUpdate = ValueConstant(PushPriority.medium)
 
 
