@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Computer, Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -7,10 +7,10 @@
 # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
-# 
+#
 # The above copyright notice and this permission notice shall be included in all
 # copies or substantial portions of the Software.
-# 
+#
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -33,8 +33,8 @@ See RFC 2518: http://www.ietf.org/rfc/rfc2518.txt
 __all__ = []
 
 
-from twext.web2 import responsecode
-from twext.web2.http_headers import MimeType
+from txweb2 import responsecode
+from txweb2.http_headers import MimeType
 
 from txdav.xml.base import WebDAVElement, WebDAVTextElement, PCDATAElement
 from txdav.xml.base import WebDAVEmptyElement, WebDAVOneShotElement
@@ -56,14 +56,15 @@ class ActiveLock (WebDAVElement):
 
     allowed_children = {
         (dav_namespace, "lockscope"): (1, 1),
-        (dav_namespace, "locktype" ): (1, 1),
-        (dav_namespace, "depth"    ): (1, 1),
-        (dav_namespace, "owner"    ): (0, 1),
-        (dav_namespace, "timeout"  ): (0, 1),
+        (dav_namespace, "locktype"): (1, 1),
+        (dav_namespace, "depth"): (1, 1),
+        (dav_namespace, "owner"): (0, 1),
+        (dav_namespace, "timeout"): (0, 1),
         (dav_namespace, "locktoken"): (0, 1),
     }
 
 
+
 @registerElement
 @registerElementClass
 class Depth (WebDAVTextElement):
@@ -77,7 +78,8 @@ class Depth (WebDAVTextElement):
 
         depth = str(self)
         if depth not in ("0", "1", "infinity"):
-            raise ValueError("Invalid depth: %s" % (depth,))
+            raise ValueError("Invalid depth: {0}".format(depth,))
+
 
 
 @registerElement
@@ -88,7 +90,8 @@ class LockToken (WebDAVElement):
     """
     name = "locktoken"
 
-    allowed_children = { (dav_namespace, "href"): (1, None) }
+    allowed_children = {(dav_namespace, "href"): (1, None)}
+
 
 
 @registerElement
@@ -100,6 +103,7 @@ class Timeout (WebDAVTextElement):
     name = "timeout"
 
 
+
 @registerElement
 @registerElementClass
 class Collection (WebDAVEmptyElement):
@@ -109,6 +113,7 @@ class Collection (WebDAVEmptyElement):
     name = "collection"
 
 
+
 @registerElement
 @registerElementClass
 class HRef (WebDAVTextElement):
@@ -118,6 +123,7 @@ class HRef (WebDAVTextElement):
     name = "href"
 
 
+
 @registerElement
 @registerElementClass
 class Link (WebDAVElement):
@@ -133,6 +139,7 @@ class Link (WebDAVElement):
     }
 
 
+
 @registerElement
 @registerElementClass
 class LinkDestination (WebDAVTextElement):
@@ -142,6 +149,7 @@ class LinkDestination (WebDAVTextElement):
     name = "dst"
 
 
+
 @registerElement
 @registerElementClass
 class LinkSource (WebDAVTextElement):
@@ -151,6 +159,7 @@ class LinkSource (WebDAVTextElement):
     name = "src"
 
 
+
 @registerElement
 @registerElementClass
 class LockEntry (WebDAVElement):
@@ -162,10 +171,11 @@ class LockEntry (WebDAVElement):
 
     allowed_children = {
         (dav_namespace, "lockscope"): (1, 1),
-        (dav_namespace, "locktype" ): (1, 1),
+        (dav_namespace, "locktype"): (1, 1),
     }
 
 
+
 @registerElement
 @registerElementClass
 class LockInfo (WebDAVElement):
@@ -177,11 +187,12 @@ class LockInfo (WebDAVElement):
 
     allowed_children = {
         (dav_namespace, "lockscope"): (1, 1),
-        (dav_namespace, "locktype" ): (1, 1),
-        (dav_namespace, "owner"    ): (0, 1),
+        (dav_namespace, "locktype"): (1, 1),
+        (dav_namespace, "owner"): (0, 1),
     }
 
 
+
 @registerElement
 @registerElementClass
 class LockScope (WebDAVOneShotElement):
@@ -193,10 +204,11 @@ class LockScope (WebDAVOneShotElement):
 
     allowed_children = {
         (dav_namespace, "exclusive"): (0, 1),
-        (dav_namespace, "shared"   ): (0, 1),
+        (dav_namespace, "shared"): (0, 1),
     }
 
 
+
 @registerElement
 @registerElementClass
 class Exclusive (WebDAVEmptyElement):
@@ -227,7 +239,8 @@ class LockType (WebDAVOneShotElement):
     """
     name = "locktype"
 
-    allowed_children = { (dav_namespace, "write"): (0, 1) }
+    allowed_children = {(dav_namespace, "write"): (0, 1)}
+
 
 
 @registerElement
@@ -244,6 +257,7 @@ class Write (WebDAVEmptyElement):
 LockType.write = LockType(Write())
 
 
+
 @registerElement
 @registerElementClass
 class MultiStatus (WebDAVElement):
@@ -253,11 +267,12 @@ class MultiStatus (WebDAVElement):
     name = "multistatus"
 
     allowed_children = {
-        (dav_namespace, "response"           ): (0, None),
+        (dav_namespace, "response"): (0, None),
         (dav_namespace, "responsedescription"): (0, 1),
     }
 
 
+
 @registerElement
 @registerElementClass
 class Response (WebDAVElement):
@@ -268,46 +283,64 @@ class Response (WebDAVElement):
     name = "response"
 
     allowed_children = {
-        (dav_namespace, "href"               ): (1, None),
-        (dav_namespace, "status"             ): (1, 1),
-        (dav_namespace, "propstat"           ): (1, None),
-        (dav_namespace, "error"              ): (0, 1),        # 2518bis
+        (dav_namespace, "href"): (1, None),
+        (dav_namespace, "status"): (1, 1),
+        (dav_namespace, "propstat"): (1, None),
+        (dav_namespace, "error"): (0, 1),        # 2518bis
         (dav_namespace, "responsedescription"): (0, 1),
     }
 
-    def __new__(clazz, *children):
-        if clazz is not Response: return WebDAVElement.__new__(clazz)
+    def __new__(cls, *children):
+        if cls is not Response:
+            return WebDAVElement.__new__(cls)
 
         resource_count = 0
-        status_count   = 0
+        status_count = 0
         propstat_count = 0
 
         for child in children:
-            if   isinstance(child, HRef          ): resource_count += 1
-            elif isinstance(child, Status        ): status_count   += 1
-            elif isinstance(child, PropertyStatus): propstat_count += 1
+            if isinstance(child, HRef):
+                resource_count += 1
+            elif isinstance(child, Status):
+                status_count += 1
+            elif isinstance(child, PropertyStatus):
+                propstat_count += 1
 
         if resource_count < 1:
-            raise ValueError("%s element must have at least one %s."
-                             % (clazz.sname(), HRef.sname()))
+            raise ValueError(
+                "{0} element must have at least one {1}.".format(
+                    cls.sname(), HRef.sname()
+                )
+            )
 
         if status_count is 0:
             if propstat_count is 0:
-                raise ValueError("%s element must have one of %s or %s"
-                                 % (clazz.sname(), Status.sname(), PropertyStatus.sname()))
+                raise ValueError(
+                    "{0} element must have one of {1} or {2}".format(
+                        cls.sname(), Status.sname(), PropertyStatus.sname()
+                    )
+                )
 
             if resource_count > 1:
-                raise ValueError("%s element with %s may only have one %s"
-                                 % (clazz.sname(), PropertyStatus.sname(), HRef.sname()))
+                raise ValueError(
+                    "{0} element with {1} may only have one {2}".format(
+                        cls.sname(), PropertyStatus.sname(), HRef.sname()
+                    )
+                )
 
             return PropertyStatusResponse.__new__(PropertyStatusResponse, *children)
 
         if status_count > 1:
-            raise ValueError("%s element may only have one %s" % (clazz.sname(), Status.sname()))
+            raise ValueError(
+                "{0} element may only have one {1}".format(
+                    cls.sname(), Status.sname()
+                )
+            )
 
         return StatusResponse.__new__(StatusResponse, *children)
 
 
+
 @registerElementClass
 class StatusResponse (Response):
     """
@@ -316,13 +349,14 @@ class StatusResponse (Response):
     unregistered = True
 
     allowed_children = {
-        (dav_namespace, "href"               ): (1, None),
-        (dav_namespace, "status"             ): (1, 1),
-        (dav_namespace, "error"              ): (0, 1),        # 2518bis
+        (dav_namespace, "href"): (1, None),
+        (dav_namespace, "status"): (1, 1),
+        (dav_namespace, "error"): (0, 1),        # 2518bis
         (dav_namespace, "responsedescription"): (0, 1),
     }
 
 
+
 @registerElementClass
 class PropertyStatusResponse (Response):
     """
@@ -331,13 +365,14 @@ class PropertyStatusResponse (Response):
     unregistered = True
 
     allowed_children = {
-        (dav_namespace, "href"               ): (1, 1),
-        (dav_namespace, "propstat"           ): (1, None),
-        (dav_namespace, "error"              ): (0, 1),        # 2518bis
+        (dav_namespace, "href"): (1, 1),
+        (dav_namespace, "propstat"): (1, None),
+        (dav_namespace, "error"): (0, 1),        # 2518bis
         (dav_namespace, "responsedescription"): (0, 1),
     }
 
 
+
 @registerElement
 @registerElementClass
 class PropertyStatus (WebDAVElement):
@@ -348,13 +383,14 @@ class PropertyStatus (WebDAVElement):
     name = "propstat"
 
     allowed_children = {
-        (dav_namespace, "prop"               ): (1, 1),
-        (dav_namespace, "status"             ): (1, 1),
-        (dav_namespace, "error"              ): (0, 1),        # 2518bis
+        (dav_namespace, "prop"): (1, 1),
+        (dav_namespace, "status"): (1, 1),
+        (dav_namespace, "error"): (0, 1),        # 2518bis
         (dav_namespace, "responsedescription"): (0, 1),
     }
 
 
+
 @registerElement
 @registerElementClass
 class Status (WebDAVTextElement):
@@ -363,32 +399,33 @@ class Status (WebDAVTextElement):
     """
     name = "status"
 
-    def fromResponseCode(clazz, code):
+    @classmethod
+    def fromResponseCode(cls, code):
         """
         code must be an integer response code in
-        twext.web2.responsecode.RESPONSES.keys()
+        txweb2.responsecode.RESPONSES.keys()
         """
         if code not in responsecode.RESPONSES:
-            raise ValueError("Invalid response code: %r" % (code,))
+            raise ValueError("Invalid response code: {0!r}".format(code,))
 
-        return clazz(PCDATAElement("HTTP/1.1 %d %s" % (code, responsecode.RESPONSES[code])))
+        return cls(PCDATAElement("HTTP/1.1 {0} {1}".format(code, responsecode.RESPONSES[code])))
 
-    fromResponseCode = classmethod(fromResponseCode)
 
     def __init__(self, *children, **attributes):
         super(Status, self).__init__(*children, **attributes)
 
         status = str(self)
         if not status.startswith("HTTP/1.1 "):
-            raise ValueError("Invalid WebDAV status: %s" % (status,))
+            raise ValueError("Invalid WebDAV status: {0}".format(status,))
 
         code = int(status[9:12])
         if code not in responsecode.RESPONSES:
-            raise ValueError("Invalid status code: %s" % (code,))
+            raise ValueError("Invalid status code: {0}".format(code,))
 
         self.code = code
 
 
+
 @registerElement
 @registerElementClass
 class ResponseDescription (WebDAVTextElement):
@@ -399,6 +436,7 @@ class ResponseDescription (WebDAVTextElement):
     name = "responsedescription"
 
 
+
 @registerElement
 @registerElementClass
 class Owner (WebDAVElement):
@@ -414,7 +452,8 @@ class Owner (WebDAVElement):
     hidden = True
     protected = True # may be protected, per RFC 3744, section 5.1
 
-    allowed_children = { WebDAVElement: (0, None) }
+    allowed_children = {WebDAVElement: (0, None)}
+
 
 
 @registerElement
@@ -425,7 +464,8 @@ class PropertyContainer (WebDAVElement):
     """
     name = "prop"
 
-    allowed_children = { WebDAVElement: (0, None) }
+    allowed_children = {WebDAVElement: (0, None)}
+
 
 
 @registerElement
@@ -438,22 +478,25 @@ class PropertyBehavior (WebDAVElement):
     name = "propertybehavior"
 
     allowed_children = {
-        (dav_namespace, "omit"     ): (0, 1),
+        (dav_namespace, "omit"): (0, 1),
         (dav_namespace, "keepalive"): (0, 1),
     }
 
+
     def __init__(self, *children, **attributes):
         super(PropertyBehavior, self).__init__(*children, **attributes)
 
         if len(self.children) != 1:
             raise ValueError(
-                "Exactly one of DAV:omit, DAV:keepalive required for %s, got: %s"
-                % (self.sname(), self.children)
+                "Exactly one of DAV:omit, DAV:keepalive required for {0}, got: {1}".format(
+                    self.sname(), self.children
+                )
             )
 
         self.behavior = children[0]
 
 
+
 @registerElement
 @registerElementClass
 class KeepAlive (WebDAVElement):
@@ -468,6 +511,7 @@ class KeepAlive (WebDAVElement):
         PCDATAElement: (0, 1),
     }
 
+
     def validate(self):
         super(KeepAlive, self).validate()
 
@@ -478,13 +522,15 @@ class KeepAlive (WebDAVElement):
                 type = child.qname()
             elif child.qname() != type:
                 raise ValueError(
-                    "Only one of DAV:href or PCDATA allowed for %s, got: %s"
-                    % (self.sname(), self.children)
+                    "Only one of DAV:href or PCDATA allowed for {0}, got: {1}".format(
+                        self.sname(), self.children
+                    )
                 )
 
         if type == "#PCDATA":
             if str(self) != "*":
-                raise ValueError("Invalid keepalive value: %r", (str(self),))
+                raise ValueError("Invalid keepalive value: {0!r}".format(str(self),))
+
 
 
 @registerElement
@@ -497,6 +543,7 @@ class Omit (WebDAVEmptyElement):
     name = "omit"
 
 
+
 @registerElement
 @registerElementClass
 class PropertyUpdate (WebDAVElement):
@@ -508,10 +555,11 @@ class PropertyUpdate (WebDAVElement):
 
     allowed_children = {
         (dav_namespace, "remove"): (0, None),
-        (dav_namespace, "set"   ): (0, None),
+        (dav_namespace, "set"): (0, None),
     }
 
 
+
 @registerElement
 @registerElementClass
 class Remove (WebDAVElement):
@@ -521,7 +569,8 @@ class Remove (WebDAVElement):
     """
     name = "remove"
 
-    allowed_children = { (dav_namespace, "prop"): (1, 1) }
+    allowed_children = {(dav_namespace, "prop"): (1, 1)}
+
 
 
 @registerElement
@@ -533,7 +582,8 @@ class Set (WebDAVElement):
     """
     name = "set"
 
-    allowed_children = { (dav_namespace, "prop"): (1, 1) }
+    allowed_children = {(dav_namespace, "prop"): (1, 1)}
+
 
 
 @registerElement
@@ -546,21 +596,24 @@ class PropertyFind (WebDAVElement):
     name = "propfind"
 
     allowed_children = {
-        (dav_namespace, "allprop" ): (0, 1),
+        (dav_namespace, "allprop"): (0, 1),
         (dav_namespace, "propname"): (0, 1),
-        (dav_namespace, "prop"    ): (0, 1),
+        (dav_namespace, "prop"): (0, 1),
     }
 
+
     def validate(self):
         super(PropertyFind, self).validate()
 
         if len(self.children) != 1:
             raise ValueError(
-                "Exactly one of DAV:allprop, DAV:propname or DAV:prop is required for %s, got: %r"
-                % (self.sname(), self.children)
+                "Exactly one of DAV:allprop, DAV:propname or DAV:prop is required for {0}, got: {1!r}".format(
+                    self.sname(), self.children
+                )
             )
 
 
+
 @registerElement
 @registerElementClass
 class AllProperties (WebDAVEmptyElement):
@@ -571,6 +624,7 @@ class AllProperties (WebDAVEmptyElement):
     name = "allprop"
 
 
+
 @registerElement
 @registerElementClass
 class PropertyName (WebDAVEmptyElement):
@@ -581,6 +635,7 @@ class PropertyName (WebDAVEmptyElement):
     name = "propname"
 
 
+
 ##
 # Section 13
 ##
@@ -597,6 +652,7 @@ class CreationDate (WebDAVDateTimeElement):
     protected = True
 
 
+
 @registerElement
 @registerElementClass
 class DisplayName (WebDAVTextElement):
@@ -607,6 +663,7 @@ class DisplayName (WebDAVTextElement):
     name = "displayname"
 
 
+
 @registerElement
 @registerElementClass
 class GETContentLanguage (WebDAVTextElement):
@@ -617,6 +674,7 @@ class GETContentLanguage (WebDAVTextElement):
     name = "getcontentlanguage"
 
 
+
 @registerElement
 @registerElementClass
 class GETContentLength (WebDAVTextElement):
@@ -628,6 +686,7 @@ class GETContentLength (WebDAVTextElement):
     protected = True
 
 
+
 @registerElement
 @registerElementClass
 class GETContentType (WebDAVTextElement):
@@ -641,6 +700,7 @@ class GETContentType (WebDAVTextElement):
         return MimeType.fromString(str(self))
 
 
+
 @registerElement
 @registerElementClass
 class GETETag (WebDAVTextElement):
@@ -652,6 +712,7 @@ class GETETag (WebDAVTextElement):
     protected = True
 
 
+
 @registerElement
 @registerElementClass
 class GETLastModified (DateTimeHeaderElement):
@@ -663,6 +724,7 @@ class GETLastModified (DateTimeHeaderElement):
     protected = True
 
 
+
 @registerElement
 @registerElementClass
 class LockDiscovery (WebDAVElement):
@@ -672,7 +734,8 @@ class LockDiscovery (WebDAVElement):
     name = "lockdiscovery"
     protected = True
 
-    allowed_children = { (dav_namespace, "activelock"): (0, None) }
+    allowed_children = {(dav_namespace, "activelock"): (0, None)}
+
 
 
 @registerElement
@@ -684,10 +747,10 @@ class ResourceType (WebDAVElement):
     name = "resourcetype"
     protected = True
 
-    allowed_children = { WebDAVElement: (0, None) }
+    allowed_children = {WebDAVElement: (0, None)}
 
 ResourceType.collection = ResourceType(Collection())
-ResourceType.empty      = ResourceType()
+ResourceType.empty = ResourceType()
 
 
 @registerElement
@@ -700,7 +763,8 @@ class Source (WebDAVElement):
     """
     name = "source"
 
-    allowed_children = { (dav_namespace, "link"): (0, None) }
+    allowed_children = {(dav_namespace, "link"): (0, None)}
+
 
 
 @registerElement
@@ -713,12 +777,12 @@ class SupportedLock (WebDAVElement):
     name = "supportedlock"
     protected = True
 
-    allowed_children = { (dav_namespace, "lockentry"): (0, None) }
+    allowed_children = {(dav_namespace, "lockentry"): (0, None)}
+
 
 
 # FIXME: Add preconditions codes defined in RFC4918
 
-
 @registerElement
 @registerElementClass
 class PropfindFiniteDepth (WebDAVEmptyElement):
