@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2014 Apple Computer, Inc. All rights reserved.
+# Copyright (c) 2005-2015 Apple Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -7,10 +7,10 @@
 # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
-# 
+#
 # The above copyright notice and this permission notice shall be included in all
 # copies or substantial portions of the Software.
-# 
+#
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -51,10 +51,12 @@ class Read (WebDAVEmptyElement):
     name = "read"
 
 
+
 # For DAV:write element (RFC 3744, section 3.2) see Write class in
 # rfc2518.py.
 
 
+
 @registerElement
 @registerElementClass
 class WriteProperties (WebDAVEmptyElement):
@@ -65,6 +67,7 @@ class WriteProperties (WebDAVEmptyElement):
     name = "write-properties"
 
 
+
 @registerElement
 @registerElementClass
 class WriteContent (WebDAVEmptyElement):
@@ -75,6 +78,7 @@ class WriteContent (WebDAVEmptyElement):
     name = "write-content"
 
 
+
 @registerElement
 @registerElementClass
 class Unlock (WebDAVEmptyElement):
@@ -85,6 +89,7 @@ class Unlock (WebDAVEmptyElement):
     name = "unlock"
 
 
+
 @registerElement
 @registerElementClass
 class ReadACL (WebDAVEmptyElement):
@@ -95,6 +100,7 @@ class ReadACL (WebDAVEmptyElement):
     name = "read-acl"
 
 
+
 @registerElement
 @registerElementClass
 class ReadCurrentUserPrivilegeSet (WebDAVEmptyElement):
@@ -106,6 +112,7 @@ class ReadCurrentUserPrivilegeSet (WebDAVEmptyElement):
     name = "read-current-user-privilege-set"
 
 
+
 @registerElement
 @registerElementClass
 class WriteACL (WebDAVEmptyElement):
@@ -116,6 +123,7 @@ class WriteACL (WebDAVEmptyElement):
     name = "write-acl"
 
 
+
 @registerElement
 @registerElementClass
 class Bind (WebDAVEmptyElement):
@@ -126,6 +134,7 @@ class Bind (WebDAVEmptyElement):
     name = "bind"
 
 
+
 @registerElement
 @registerElementClass
 class Unbind (WebDAVEmptyElement):
@@ -136,6 +145,7 @@ class Unbind (WebDAVEmptyElement):
     name = "unbind"
 
 
+
 @registerElement
 @registerElementClass
 class All (WebDAVEmptyElement):
@@ -147,6 +157,7 @@ class All (WebDAVEmptyElement):
     name = "all"
 
 
+
 ##
 # Section 4 (Principal Properties)
 ##
@@ -161,12 +172,12 @@ class Principal (WebDAVElement):
     name = "principal"
 
     allowed_children = {
-        (dav_namespace, "href"           ): (0, 1),
-        (dav_namespace, "all"            ): (0, 1),
-        (dav_namespace, "authenticated"  ): (0, 1),
+        (dav_namespace, "href"): (0, 1),
+        (dav_namespace, "all"): (0, 1),
+        (dav_namespace, "authenticated"): (0, 1),
         (dav_namespace, "unauthenticated"): (0, 1),
-        (dav_namespace, "property"       ): (0, 1),
-        (dav_namespace, "self"           ): (0, 1),
+        (dav_namespace, "property"): (0, 1),
+        (dav_namespace, "self"): (0, 1),
     }
 
     def validate(self):
@@ -176,11 +187,13 @@ class Principal (WebDAVElement):
             raise ValueError(
                 "Exactly one of DAV:href, DAV:all, DAV:authenticated, "
                 "DAV:unauthenticated, DAV:property or DAV:self is required for "
-                "%s, got: %r"
-                % (self.sname(), self.children)
+                "{0}, got: {1!r}".format(
+                    self.sname(), self.children
+                )
             )
 
 
+
 @registerElement
 @registerElementClass
 class AlternateURISet (WebDAVElement):
@@ -192,7 +205,8 @@ class AlternateURISet (WebDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (dav_namespace, "href"): (0, None) }
+    allowed_children = {(dav_namespace, "href"): (0, None)}
+
 
 
 @registerElement
@@ -206,7 +220,8 @@ class PrincipalURL (WebDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (dav_namespace, "href"): (0, 1) }
+    allowed_children = {(dav_namespace, "href"): (0, 1)}
+
 
 
 @registerElement
@@ -220,7 +235,8 @@ class GroupMemberSet (WebDAVElement):
     name = "group-member-set"
     hidden = True
 
-    allowed_children = { (dav_namespace, "href"): (0, None) }
+    allowed_children = {(dav_namespace, "href"): (0, None)}
+
 
 
 @registerElement
@@ -234,7 +250,8 @@ class GroupMembership (WebDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (dav_namespace, "href"): (0, None) }
+    allowed_children = {(dav_namespace, "href"): (0, None)}
+
 
 
 ##
@@ -245,6 +262,7 @@ class GroupMembership (WebDAVElement):
 # rfc2518.py.
 
 
+
 @registerElement
 @registerElementClass
 class Group (WebDAVElement):
@@ -256,7 +274,8 @@ class Group (WebDAVElement):
     hidden = True
     protected = True # may be protected, per RFC 3744, section 5.2
 
-    allowed_children = { (dav_namespace, "href"): (0, 1) }
+    allowed_children = {(dav_namespace, "href"): (0, 1)}
+
 
 
 @registerElement
@@ -270,7 +289,8 @@ class SupportedPrivilegeSet (WebDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (dav_namespace, "supported-privilege"): (0, None) }
+    allowed_children = {(dav_namespace, "supported-privilege"): (0, None)}
+
 
 
 @registerElement
@@ -282,13 +302,14 @@ class SupportedPrivilege (WebDAVElement):
     name = "supported-privilege"
 
     allowed_children = {
-        (dav_namespace, "privilege"          ): (1, 1),
-        (dav_namespace, "abstract"           ): (0, 1),
-        (dav_namespace, "description"        ): (1, 1),
+        (dav_namespace, "privilege"): (1, 1),
+        (dav_namespace, "abstract"): (0, 1),
+        (dav_namespace, "description"): (1, 1),
         (dav_namespace, "supported-privilege"): (0, None),
     }
 
 
+
 @registerElement
 @registerElementClass
 class Privilege (WebDAVElement):
@@ -297,7 +318,7 @@ class Privilege (WebDAVElement):
     """
     name = "privilege"
 
-    allowed_children = { WebDAVElement: (0, None) }
+    allowed_children = {WebDAVElement: (0, None)}
 
     def isAggregateOf(self, subprivilege, supportedPrivileges):
         """
@@ -339,6 +360,7 @@ class Privilege (WebDAVElement):
         else:
             return False
 
+
     def expandAggregate(self, supportedPrivileges):
         """
         Expand this privilege into the set of privileges aggregated under it
@@ -384,6 +406,7 @@ class Privilege (WebDAVElement):
         return aggregates
 
 
+
 @registerElement
 @registerElementClass
 class Abstract (WebDAVElement):
@@ -393,6 +416,7 @@ class Abstract (WebDAVElement):
     name = "abstract"
 
 
+
 @registerElement
 @registerElementClass
 class Description (WebDAVTextElement):
@@ -401,7 +425,8 @@ class Description (WebDAVTextElement):
     3744, sections 5.3 and 9.5)
     """
     name = "description"
-    allowed_attributes = { "xml:lang": True }
+    allowed_attributes = {"xml:lang": True}
+
 
 
 @registerElement
@@ -416,12 +441,14 @@ class CurrentUserPrivilegeSet (WebDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (dav_namespace, "privilege"): (0, None) }
+    allowed_children = {(dav_namespace, "privilege"): (0, None)}
+
 
 
 # For DAV:privilege element (RFC 3744, section 5.4) see Privilege class above.
 
 
+
 @registerElement
 @registerElementClass
 class ACL (WebDAVElement):
@@ -434,7 +461,8 @@ class ACL (WebDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (dav_namespace, "ace"): (0, None) }
+    allowed_children = {(dav_namespace, "ace"): (0, None)}
+
 
 
 @registerElement
@@ -448,9 +476,9 @@ class ACE (WebDAVElement):
 
     allowed_children = {
         (dav_namespace, "principal"): (0, 1),
-        (dav_namespace, "invert"   ): (0, 1),
-        (dav_namespace, "grant"    ): (0, 1),
-        (dav_namespace, "deny"     ): (0, 1),
+        (dav_namespace, "invert"): (0, 1),
+        (dav_namespace, "grant"): (0, 1),
+        (dav_namespace, "deny"): (0, 1),
         (dav_namespace, "protected"): (0, 1),
         (dav_namespace, "inherited"): (0, 1),
     }
@@ -458,12 +486,12 @@ class ACE (WebDAVElement):
     def __init__(self, *children, **attributes):
         super(ACE, self).__init__(*children, **attributes)
 
-        self.principal  = None
-        self.invert     = None
-        self.allow      = None
+        self.principal = None
+        self.invert = None
+        self.allow = None
         self.privileges = None
-        self.inherited  = None
-        self.protected  = False
+        self.inherited = None
+        self.protected = False
 
         my_children = []
 
@@ -477,28 +505,30 @@ class ACE (WebDAVElement):
                 if name in ("principal", "invert"):
                     if self.principal is not None:
                         raise ValueError(
-                            "Only one of DAV:principal or DAV:invert allowed in %s, got: %s"
-                            % (self.sname(), self.children)
+                            "Only one of DAV:principal or DAV:invert allowed in {0}, got: {1}".format(
+                                self.sname(), self.children
+                            )
                         )
                     if name == "invert":
-                        self.invert    = True
+                        self.invert = True
                         self.principal = child.children[0]
                     else:
-                        self.invert    = False
+                        self.invert = False
                         self.principal = child
-    
+
                 elif name in ("grant", "deny"):
                     if self.allow is not None:
                         raise ValueError(
-                            "Only one of DAV:grant or DAV:deny allowed in %s, got: %s"
-                            % (self.sname(), self.children)
+                            "Only one of DAV:grant or DAV:deny allowed in {0}, got: {1}".format(
+                                self.sname(), self.children
+                            )
                         )
-                    self.allow      = (name == "grant")
+                    self.allow = (name == "grant")
                     self.privileges = child.children
-    
+
                 elif name == "inherited":
                     self.inherited = str(child.children[0])
-    
+
                 elif name == "protected":
                     self.protected = True
 
@@ -508,15 +538,17 @@ class ACE (WebDAVElement):
 
         if self.principal is None:
             raise ValueError(
-                "One of DAV:principal or DAV:invert is required in %s, got: %s"
-                % (self.sname(), self.children)
+                "One of DAV:principal or DAV:invert is required in {0}, got: {1}".format(
+                    self.sname(), self.children
+                )
             )
         assert self.invert is not None
 
         if self.allow is None:
             raise ValueError(
-                "One of DAV:grant or DAV:deny is required in %s, got: %s"
-                % (self.sname(), self.children)
+                "One of DAV:grant or DAV:deny is required in {0}, got: {1}".format(
+                    self.sname(), self.children
+                )
             )
         assert self.privileges is not None
 
@@ -525,9 +557,11 @@ class ACE (WebDAVElement):
 # class above.
 
 
+
 # For DAV:all element (RFC 3744, section 5.5.1) see All class above.
 
 
+
 @registerElement
 @registerElementClass
 class Authenticated (WebDAVEmptyElement):
@@ -537,6 +571,7 @@ class Authenticated (WebDAVEmptyElement):
     name = "authenticated"
 
 
+
 @registerElement
 @registerElementClass
 class Unauthenticated (WebDAVEmptyElement):
@@ -546,10 +581,12 @@ class Unauthenticated (WebDAVEmptyElement):
     name = "unauthenticated"
 
 
+
 # For DAV:property element (RFC 3744, section 5.5.1) see Property
 # class above.
 
 
+
 @registerElement
 @registerElementClass
 class Self (WebDAVEmptyElement):
@@ -560,6 +597,7 @@ class Self (WebDAVEmptyElement):
     name = "self"
 
 
+
 @registerElement
 @registerElementClass
 class Invert (WebDAVElement):
@@ -569,7 +607,8 @@ class Invert (WebDAVElement):
     """
     name = "invert"
 
-    allowed_children = { (dav_namespace, "principal"): (1, 1) }
+    allowed_children = {(dav_namespace, "principal"): (1, 1)}
+
 
 
 @registerElement
@@ -580,7 +619,8 @@ class Grant (WebDAVElement):
     """
     name = "grant"
 
-    allowed_children = { (dav_namespace, "privilege"): (1, None) }
+    allowed_children = {(dav_namespace, "privilege"): (1, None)}
+
 
 
 @registerElement
@@ -591,13 +631,15 @@ class Deny (WebDAVElement):
     """
     name = "deny"
 
-    allowed_children = { (dav_namespace, "privilege"): (1, None) }
+    allowed_children = {(dav_namespace, "privilege"): (1, None)}
+
 
 
 # For DAV:privilege element (RFC 3744, section 5.5.2) see Privilege
 # class above.
 
 
+
 @registerElement
 @registerElementClass
 class Protected (WebDAVEmptyElement):
@@ -607,16 +649,18 @@ class Protected (WebDAVEmptyElement):
     name = "protected"
 
 
+
 @registerElement
 @registerElementClass
 class Inherited (WebDAVElement):
     """
-    Indicates that an ACE is inherited from the resource indentified by the
+    Indicates that an ACE is inherited from the resource identified by the
     contained DAV:href element. (RFC 3744, section 5.5.4)
     """
     name = "inherited"
 
-    allowed_children = { (dav_namespace, "href"): (1, 1) }
+    allowed_children = {(dav_namespace, "href"): (1, 1)}
+
 
 
 @registerElement
@@ -631,13 +675,14 @@ class ACLRestrictions (WebDAVElement):
     protected = True
 
     allowed_children = {
-        (dav_namespace, "grant-only"        ): (0, 1),
-        (dav_namespace, "no-invert"         ): (0, 1),
-        (dav_namespace, "deny-before-grant" ): (0, 1),
+        (dav_namespace, "grant-only"): (0, 1),
+        (dav_namespace, "no-invert"): (0, 1),
+        (dav_namespace, "deny-before-grant"): (0, 1),
         (dav_namespace, "required-principal"): (0, 1),
     }
 
 
+
 @registerElement
 @registerElementClass
 class GrantOnly (WebDAVEmptyElement):
@@ -648,6 +693,7 @@ class GrantOnly (WebDAVEmptyElement):
     name = "grant-only"
 
 
+
 @registerElement
 @registerElementClass
 class NoInvert (WebDAVEmptyElement):
@@ -658,6 +704,7 @@ class NoInvert (WebDAVEmptyElement):
     name = "no-invert"
 
 
+
 @registerElement
 @registerElementClass
 class DenyBeforeGrant (WebDAVEmptyElement):
@@ -668,6 +715,7 @@ class DenyBeforeGrant (WebDAVEmptyElement):
     name = "deny-before-grant"
 
 
+
 @registerElement
 @registerElementClass
 class RequiredPrincipal (WebDAVElement):
@@ -678,12 +726,12 @@ class RequiredPrincipal (WebDAVElement):
     name = "required-principal"
 
     allowed_children = {
-        (dav_namespace, "all"            ): (0, 1),
-        (dav_namespace, "authenticated"  ): (0, 1),
+        (dav_namespace, "all"): (0, 1),
+        (dav_namespace, "authenticated"): (0, 1),
         (dav_namespace, "unauthenticated"): (0, 1),
-        (dav_namespace, "self"           ): (0, 1),
-        (dav_namespace, "href"           ): (0, None),
-        (dav_namespace, "property"       ): (0, None),
+        (dav_namespace, "self"): (0, 1),
+        (dav_namespace, "href"): (0, None),
+        (dav_namespace, "property"): (0, None),
     }
 
     def validate(self):
@@ -697,11 +745,13 @@ class RequiredPrincipal (WebDAVElement):
             elif child.qname() != type:
                 raise ValueError(
                     "Only one of DAV:all, DAV:authenticated, DAV:unauthenticated, "
-                    "DAV:self, DAV:href or DAV:property allowed for %s, got: %s"
-                    % (self.sname(), self.children)
+                    "DAV:self, DAV:href or DAV:property allowed for {0}, got: {1}".format(
+                        self.sname(), self.children
+                    )
                 )
 
 
+
 @registerElement
 @registerElementClass
 class InheritedACLSet (WebDAVElement):
@@ -713,7 +763,8 @@ class InheritedACLSet (WebDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (dav_namespace, "href"): (0, None) }
+    allowed_children = {(dav_namespace, "href"): (0, None)}
+
 
 
 @registerElement
@@ -728,7 +779,8 @@ class PrincipalCollectionSet (WebDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (dav_namespace, "href"): (0, None) }
+    allowed_children = {(dav_namespace, "href"): (0, None)}
+
 
 
 ##
@@ -743,7 +795,8 @@ class NeedPrivileges (WebDAVElement):
     """
     name = "need-privileges"
 
-    allowed_children = { (dav_namespace, "resource"): (0, None) }
+    allowed_children = {(dav_namespace, "resource"): (0, None)}
+
 
 
 @registerElement
@@ -756,11 +809,12 @@ class Resource (WebDAVElement):
     name = "resource"
 
     allowed_children = {
-        (dav_namespace, "href"     ): (1, 1),
+        (dav_namespace, "href"): (1, 1),
         (dav_namespace, "privilege"): (1, 1),
     }
 
 
+
 ##
 # Section 9 (Access Control Reports)
 ##
@@ -776,23 +830,25 @@ class ACLPrincipalPropSet (WebDAVElement):
     """
     name = "acl-principal-prop-set"
 
-    allowed_children = { WebDAVElement: (0, None) }
+    allowed_children = {WebDAVElement: (0, None)}
 
     def validate(self):
         super(ACLPrincipalPropSet, self).validate()
 
         prop = False
-        
+
         for child in self.children:
             if child.qname() == (dav_namespace, "prop"):
                 if prop:
                     raise ValueError(
-                        "Only one DAV:prop allowed for %s, got: %s"
-                        % (self.sname(), self.children)
+                        "Only one DAV:prop allowed for {0}, got: {1}".format(
+                            self.sname(), self.children
+                        )
                     )
                 prop = True
 
 
+
 @registerElement
 @registerElementClass
 class PrincipalMatch (WebDAVElement):
@@ -805,8 +861,8 @@ class PrincipalMatch (WebDAVElement):
 
     allowed_children = {
         (dav_namespace, "principal-property"): (0, 1),
-        (dav_namespace, "self"              ): (0, 1),
-        (dav_namespace, "prop"              ): (0, 1),
+        (dav_namespace, "self"): (0, 1),
+        (dav_namespace, "prop"): (0, 1),
     }
 
     def validate(self):
@@ -824,18 +880,21 @@ class PrincipalMatch (WebDAVElement):
             if (namespace == dav_namespace) and name in ("principal-property", "self"):
                 if principalPropertyOrSelf:
                     raise ValueError(
-                        "Only one of DAV:principal-property or DAV:self allowed in %s, got: %s"
-                        % (self.sname(), self.children)
+                        "Only one of DAV:principal-property or DAV:self allowed in {0}, got: {1}".format(
+                            self.sname(), self.children
+                        )
                     )
                 principalPropertyOrSelf = True
 
         if not principalPropertyOrSelf:
             raise ValueError(
-                "One of DAV:principal-property or DAV:self is required in %s, got: %s"
-                % (self.sname(), self.children)
+                "One of DAV:principal-property or DAV:self is required in {0}, got: {1}".format(
+                    self.sname(), self.children
+                )
             )
 
 
+
 @registerElement
 @registerElementClass
 class PrincipalProperty (WebDAVElement):
@@ -844,12 +903,14 @@ class PrincipalProperty (WebDAVElement):
     """
     name = "principal-property"
 
-    allowed_children = { WebDAVElement: (0, None) }
+    allowed_children = {WebDAVElement: (0, None)}
+
 
 
 # For DAV:self element (RFC 3744, section 9.3) see Self class above.
 
 
+
 @registerElement
 @registerElementClass
 class PrincipalPropertySearch (WebDAVElement):
@@ -861,11 +922,12 @@ class PrincipalPropertySearch (WebDAVElement):
     name = "principal-property-search"
 
     allowed_children = {
-        (dav_namespace, "property-search"                  ): (0, None),    # This is required but this element must be empty in supported-report-set
-        (dav_namespace, "prop"                             ): (0, 1),
+        (dav_namespace, "property-search"): (0, None),    # This is required but this element must be empty in supported-report-set
+        (dav_namespace, "prop"): (0, 1),
         (dav_namespace, "apply-to-principal-collection-set"): (0, 1),
     }
-    allowed_attributes = { "test": False }
+    allowed_attributes = {"test": False}
+
 
 
 @registerElement
@@ -878,11 +940,12 @@ class PropertySearch (WebDAVElement):
     name = "property-search"
 
     allowed_children = {
-        (dav_namespace, "prop" ): (1, 1),
+        (dav_namespace, "prop"): (1, 1),
         (dav_namespace, "match"): (1, 1),
     }
 
 
+
 @registerElement
 @registerElementClass
 class Match (WebDAVTextElement):
@@ -892,6 +955,7 @@ class Match (WebDAVTextElement):
     name = "match"
 
 
+
 @registerElement
 @registerElementClass
 class PrincipalSearchPropertySet (WebDAVElement):
@@ -901,7 +965,8 @@ class PrincipalSearchPropertySet (WebDAVElement):
     """
     name = "principal-search-property-set"
 
-    allowed_children = { (dav_namespace, "principal-search-property"): (0, None) }
+    allowed_children = {(dav_namespace, "principal-search-property"): (0, None)}
+
 
 
 @registerElement
@@ -914,11 +979,12 @@ class PrincipalSearchProperty (WebDAVElement):
     name = "principal-search-property"
 
     allowed_children = {
-        (dav_namespace, "prop"       ): (1, 1),
+        (dav_namespace, "prop"): (1, 1),
         (dav_namespace, "description"): (1, 1),
     }
 
 
+
 @registerElement
 @registerElementClass
 class NumberOfMatchesWithinLimits (WebDAVEmptyElement):
